Package: rtdists
Type: Package
Title: Response Time Distributions
Depends: R (>= 3.0.0)
Suggests: testthat, glba, knitr, rmarkdown, dplyr, tidyr, purrr,
        lattice, latticeExtra, binom, RWiener
Imports: evd, msm, gsl, stats, Rcpp
LinkingTo: Rcpp
Description: Provides response time distributions (density/PDF,
       distribution function/CDF, quantile function, and random
       generation): (a) Ratcliff diffusion model (Ratcliff &
       McKoon, 2008, <doi:10.1162/neco.2008.12-06-420>) based on C
       code by Andreas and Jochen Voss and (b) linear ballistic
       accumulator (LBA; Brown & Heathcote, 2008,
       <doi:10.1016/j.cogpsych.2007.12.002>) with different
       distributions underlying the drift rate.
URL: https://github.com/rtdists/rtdists/
License: GPL (>= 3)
BugReports: https://github.com/rtdists/rtdists/issues
VignetteBuilder: knitr
Authors@R: c( person(given="Henrik", family="Singmann",
       email="singmann@gmail.com", role=c("aut", "cre"),
       comment=c(ORCID="0000-0002-4842-3657")),
       person(given="Scott", family="Brown", role=c("aut")),
       person(given="Matthew", family="Gretton", role=c("aut")),
       person(given="Andrew", family="Heathcote", role=c("aut")),
       person(given="Andreas", family="Voss", role=c("ctb")),
       person(given="Jochen", family="Voss", role=c("ctb")),
       person(given="Andrew", family="Terry", role=c("ctb")) )
LazyData: true
Version: 0.11-6
RoxygenNote: 7.3.3
NeedsCompilation: yes
Packaged: 2026-06-10 06:39:55 UTC; singmann
Author: Henrik Singmann [aut, cre] (ORCID:
    <https://orcid.org/0000-0002-4842-3657>),
  Scott Brown [aut],
  Matthew Gretton [aut],
  Andrew Heathcote [aut],
  Andreas Voss [ctb],
  Jochen Voss [ctb],
  Andrew Terry [ctb]
Maintainer: Henrik Singmann <singmann@gmail.com>
Repository: CRAN
Date/Publication: 2026-06-10 09:00:02 UTC
Built: R 4.5.2; x86_64-apple-darwin20; 2026-06-10 09:35:03 UTC; unix
Archs: rtdists.so.dSYM

Package: rtdists
Type: Package
Title: Response Time Distributions
Depends: R (>= 3.0.0)
Suggests: testthat, glba, knitr, rmarkdown, dplyr, tidyr, purrr,
        lattice, latticeExtra, binom, RWiener
Imports: evd, msm, gsl, stats, Rcpp
LinkingTo: Rcpp
Description: Provides response time distributions (density/PDF,
       distribution function/CDF, quantile function, and random
       generation): (a) Ratcliff diffusion model (Ratcliff &
       McKoon, 2008, <doi:10.1162/neco.2008.12-06-420>) based on C
       code by Andreas and Jochen Voss and (b) linear ballistic
       accumulator (LBA; Brown & Heathcote, 2008,
       <doi:10.1016/j.cogpsych.2007.12.002>) with different
       distributions underlying the drift rate.
URL: https://github.com/rtdists/rtdists/
License: GPL (>= 3)
BugReports: https://github.com/rtdists/rtdists/issues
VignetteBuilder: knitr
Authors@R: c( person(given="Henrik", family="Singmann",
       email="singmann@gmail.com", role=c("aut", "cre"),
       comment=c(ORCID="0000-0002-4842-3657")),
       person(given="Scott", family="Brown", role=c("aut")),
       person(given="Matthew", family="Gretton", role=c("aut")),
       person(given="Andrew", family="Heathcote", role=c("aut")),
       person(given="Andreas", family="Voss", role=c("ctb")),
       person(given="Jochen", family="Voss", role=c("ctb")),
       person(given="Andrew", family="Terry", role=c("ctb")) )
LazyData: true
Version: 0.11-6
RoxygenNote: 7.3.3
NeedsCompilation: yes
Packaged: 2026-06-10 06:39:55 UTC; singmann
Author: Henrik Singmann [aut, cre] (ORCID:
    <https://orcid.org/0000-0002-4842-3657>),
  Scott Brown [aut],
  Matthew Gretton [aut],
  Andrew Heathcote [aut],
  Andreas Voss [ctb],
  Jochen Voss [ctb],
  Andrew Terry [ctb]
Maintainer: Henrik Singmann <singmann@gmail.com>
Repository: CRAN
Date/Publication: 2026-06-10 09:00:02 UTC

# Generated by roxygen2: do not edit by hand

export(dLBA)
export(ddiffusion)
export(dlba_frechet)
export(dlba_gamma)
export(dlba_lnorm)
export(dlba_norm)
export(drd)
export(n1CDF)
export(n1PDF)
export(pLBA)
export(pdiffusion)
export(plba_frechet)
export(plba_gamma)
export(plba_lnorm)
export(plba_norm)
export(prd)
export(qLBA)
export(qdiffusion)
export(rLBA)
export(rdiffusion)
export(rlba_frechet)
export(rlba_gamma)
export(rlba_lnorm)
export(rlba_norm)
export(rrd)
importFrom(Rcpp,evalCpp)
importFrom(evd,dfrechet)
importFrom(evd,pfrechet)
importFrom(evd,rfrechet)
importFrom(gsl,gamma_inc)
importFrom(msm,rtnorm)
importFrom(stats,dgamma)
importFrom(stats,dlnorm)
importFrom(stats,dnorm)
importFrom(stats,integrate)
importFrom(stats,optimize)
importFrom(stats,pgamma)
importFrom(stats,plnorm)
importFrom(stats,pnorm)
importFrom(stats,rgamma)
importFrom(stats,rlnorm)
importFrom(stats,rnorm)
importFrom(stats,runif)
importFrom(stats,uniroot)
useDynLib(rtdists, .registration = TRUE)

require(msm)

# z = time, A=x0max, b=chi, v=driftrate, sv=sddrift
fptcdf0=function(z,x0max,chi,driftrate,sddrift) {
  if (x0max<1e-10) # LATER solution
    return( pnorm(chi/z,mean=driftrate,sd=sddrift,lower.tail=FALSE) ) 
  zs=z*sddrift ; zu=z*driftrate ; chiminuszu=chi-zu ; xx=chiminuszu-x0max
  chizu=chiminuszu/zs ; chizumax=xx/zs
  tmp1=zs*(dnorm(chizumax)-dnorm(chizu))
  tmp2=xx*pnorm(chizumax)-chiminuszu*pnorm(chizu)
  1+(tmp1+tmp2)/x0max 
}

# z = time, A=x0max, b=chi, v=driftrate, sv=sddrift
fptpdf0=function(z,x0max,chi,driftrate,sddrift) {
  if (x0max<1e-10) # LATER solution
    return( (chi/z^2)*dnorm(chi/z,mean=driftrate,sd=sddrift) ) 
  zs=z*sddrift ; zu=z*driftrate ; chiminuszu=chi-zu
  chizu=chiminuszu/zs ; chizumax=(chiminuszu-x0max)/zs
  (driftrate*(pnorm(chizu)-pnorm(chizumax)) +
             sddrift*(dnorm(chizumax)-dnorm(chizu)))/x0max
}

# protected normal desity and cdf
pnormP <- function(x,mean=0,sd=1,lower.tail=TRUE){
  ifelse(abs(x)<7,pnorm(x,mean=mean,sd=sd,lower.tail=lower.tail),ifelse(x<0,0,1))}  
dnormP <- function(x,mean=0,sd=1){
  ifelse(abs(x)<7,dnorm(x,mean=mean,sd=sd),0)}

# robust version, 3 times slower!
fptcdfR=function(z,x0max,chi,driftrate,sddrift) {
  if (x0max<1e-10) # LATER solution
    return( pmin(1,pmax(0,pnormP(chi/z,mean=driftrate,sd=sddrift,lower.tail=FALSE))) ) 
  zs=z*sddrift ; zu=z*driftrate ; chiminuszu=chi-zu ; xx=chiminuszu-x0max
  chizu=chiminuszu/zs ; chizumax=xx/zs
  tmp1=zs*(dnormP(chizumax)-dnormP(chizu))
  tmp2=xx*pnormP(chizumax)-chiminuszu*pnormP(chizu)
  pmin(pmax(0,1+(tmp1+tmp2)/x0max),1) 
}

# robust version, 3 times slower!
fptpdfR=function(z,x0max,chi,driftrate,sddrift) {
  if (x0max<1e-10) # LATER solution
    return( pmax(0,(chi/z^2)*dnormP(chi/z,mean=driftrate,sd=sddrift)) ) 
  zs=z*sddrift ; zu=z*driftrate ; chiminuszu=chi-zu
  chizu=chiminuszu/zs ; chizumax=(chiminuszu-x0max)/zs
  pmax(0,(driftrate*(pnormP(chizu)-pnormP(chizumax)) +
             sddrift*(dnormP(chizumax)-dnormP(chizu)))/x0max)
}


fptcdf=function(z,x0max,chi,driftrate,sddrift,posdrift=TRUE,robust=FALSE) {
  if ( robust ) {
    if (!posdrift) fptcdfR(z,x0max,chi,driftrate,sddrift) else
      fptcdfR(z,x0max,chi,driftrate,sddrift)/pmax(pnormP(driftrate/sddrift),1e-10)  
  } else {
    if (!posdrift) fptcdf0(z,x0max,chi,driftrate,sddrift) else
      fptcdf0(z,x0max,chi,driftrate,sddrift)/pmax(pnormP(driftrate/sddrift),1e-10)
  }
}

fptpdf=function(z,x0max,chi,driftrate,sddrift,posdrift=TRUE,robust=FALSE) {
  if ( robust ) {
    if (!posdrift) fptpdfR(z,x0max,chi,driftrate,sddrift) else
      fptpdfR(z,x0max,chi,driftrate,sddrift)/pmax(pnormP(driftrate/sddrift),1e-10)  
  } else {
    if (!posdrift) fptpdf0(z,x0max,chi,driftrate,sddrift) else
      fptpdf0(z,x0max,chi,driftrate,sddrift)/pmax(pnormP(driftrate/sddrift),1e-10)
  }
}

# n = number of samples, v = vs, t0 = minimum non-decision time
# st = width of uniform non-decision time
# If all rates are negative returns Inf and resp=1
rlba=function(n,b,A,vs,s,t0,st0=0,posdrift=TRUE){
  if (posdrift) {
    drifts=matrix(rtnorm(mean=vs,sd=s,n=n*length(vs),lower=0),ncol=length(vs),byrow=TRUE)    
  } else {
    drifts=matrix(rnorm(mean=vs,sd=s,n=n*length(vs)),ncol=length(vs),byrow=TRUE)
    drifts[drifts<0]=0
  }
  starts=matrix(runif(min=0,max=A,n=n*length(vs)),ncol=length(vs),byrow=TRUE)
  ttf=t((b-t(starts)))/drifts
  rt=apply(ttf,1,min)+t0+runif(min=0,max=st0,n=n)
  resp=apply(ttf,1,which.min)
  bad <- !is.finite(rt)
  if (any(bad)) {
    warning(paste(sum(bad),"infinite RTs removed"))
    resp <- resp[!bad]
    rt <- rt[!bad]
  }
  list(rt=rt,resp=resp)
}


# t = time, A=x0max, b=chi, v=drift, sv=sdI
.n1PDFfixedt0=function(t,x0max,chi,drift,sdI,posdrift=TRUE,robust=FALSE) {
  # Generates defective PDF for responses on node #1.
  N=length(drift) # Number of responses.
  if (N>2) {
    tmp=array(dim=c(length(t),N-1))
    for (i in 2:N) tmp[,i-1]=fptcdf(z=t,x0max=x0max[i],chi=chi[i],
      driftrate=drift[i],sddrift=sdI[i],posdrift=posdrift,robust=robust)
    G=apply(1-tmp,1,prod)
  } else {
    G=1-fptcdf(z=t,x0max=x0max[2],chi=chi[2],driftrate=drift[2],
      sddrift=sdI[2],posdrift=posdrift,robust=robust)
  }
  G*fptpdf(z=t,x0max=x0max[1],chi=chi[1],driftrate=drift[1],
    sddrift=sdI[1],posdrift=posdrift,robust=robust)
}

.n1PDF=function(t,x0max,chi,drift,sdI,st0=0,posdrift=TRUE,robust=FALSE) {
  N=length(drift) # Number of responses
  if (length(x0max)<N) x0max=rep(x0max,length.out=N)
  if (length(chi)<N) chi=rep(chi,length.out=N)
  if (length(sdI)<N) sdI=rep(sdI,length.out=N)
  if (length(st0)>1) st0=st0[1] # Only ONE non-decision time.
  if (st0==0) return(.n1PDFfixedt0(t,x0max,chi,drift,sdI,posdrift,robust))
  tmpf=function(t,x0max,chi,drift,sdI,st0,posdrift,robust)
    .n1PDFfixedt0(t,x0max,chi,drift,sdI,posdrift,robust)/st0
  outs=numeric(length(t))
  for (i in 1:length(outs)) outs[i]=integrate(f=tmpf,lower=t[i]-st0,upper=t[i],
    x0max=x0max,chi=chi,drift=drift,sdI=sdI,st0=st0,posdrift=posdrift,robust=robust)$value
  outs
}


.n1CDF=function(t,x0max,chi,drift,sdI,st0=0,posdrift=TRUE,robust=FALSE) {  #, browser=FALSE
  # Generates defective CDF for responses on node #1. 
  N=length(drift) # Number of responses
  if (length(x0max)<N) x0max=rep(x0max,length.out=N)
  if (length(chi)<N) chi=rep(chi,length.out=N)
  if (length(sdI)<N) sdI=rep(sdI,length.out=N)
  if (length(st0)>1) stop("Only one value of st0 allowed.")
  if (st0<1e-6) st0=0 # Integral can fail for small st0.
  outs=numeric(length(t)) ; bounds=c(-st0/2,t)
  for (i in 1:length(t)) {
    tmp="error"
    repeat {
      if (bounds[i]>=bounds[i+1]) {outs[i]=0;break}
      #if(i==1 && browser) browser()
      tmp=try(integrate(f=.n1PDF,lower=bounds[i],upper=bounds[i+1],subdivisions=1000,
        x0max=x0max,chi=chi,drift=drift,sdI=sdI,st0=st0,posdrift=posdrift,robust=robust)$value,silent=T)
      if (is.numeric(tmp)) {outs[i]=tmp;break}
      #browser()
      # Try smart lower bound.
      if (bounds[i]<=0) {
  bounds[i]=max(c((chi-0.98*x0max)/(max(mean(drift),drift[1])+2*sdI)[1],0))
	next
      }
      # Try smart upper bound.
      if (bounds[i+1]==Inf) {
	bounds[i+1]=0.02*max(chi)/(mean(drift)-2*mean(sdI))
	next
      }
      stop("Error in n1CDF that I could not catch.")
    }
  }
  cumsum(outs)
}

n1mean=function(x0max,chi,drift,sdI,posdrift=TRUE,robust=FALSE) {
  # Generates mean RT for responses on node #1. 
   pc=n1CDF(Inf,x0max,chi,drift,sdI,posdrift,robust)
   fn=function(t,x0max,chi,drift,sdI,st0=0,pc,posdrift,robust)
     t*n1PDF(t,x0max,chi,drift,sdI,st0,posdrift,robust)/pc
   tmp=integrate(f=fn,lower=0,upper=100*chi,x0max=x0max,chi=chi,pc=pc,
     drift=drift,sdI=sdI,st0=st0,posdrift=posdrift,robust=robust)$value
   list(mean=tmp,p=pc)
}

context("LBA-math agrees with current implementation")

runif(1)
x <- .Random.seed
set.seed(2)

test_that("PDF and CDF", {
  n <- 10
  samples_per_run <- 100
  source(system.file("extdata", "lba-math.R", package = "rtdists"))
        
  #source("inst/extdata//lba-math.r")
  for (i in seq_len(n)) {
    A <- runif(1, 0.3, 0.9)
    b <- A+runif(1, 0, 0.5)
    t0 <- runif(1, 0.1, 0.7)
    v1 <- runif(2, 0.5, 1.5)
    v2 <- runif(2, 0.1, 0.5)
    r_lba1 <- rLBA(samples_per_run, A=A, b=b, t0 = t0, 
                   mean_v=v1[1:2], sd_v=v2[1:2])
    
    expect_equal(
      dlba_norm(r_lba1$rt[r_lba1$response==1], A=A, b=b, t0 = t0, 
                mean_v=v1[1], sd_v=v2[1]), 
      fptpdf(pmax(r_lba1$rt[r_lba1$response==1]-t0[1], 0), x0max=A, chi=b, 
             driftrate=v1[1], sddrift=v2[1])
      )
    expect_equal(
      plba_norm(r_lba1$rt[r_lba1$response==1], A=A, b=b, t0 = t0, 
                mean_v=v1[1], sd_v=v2[1]), 
      fptcdf(pmax(r_lba1$rt[r_lba1$response==1]-t0[1], 0), x0max=A, chi=b, 
             driftrate=v1[1], sddrift=v2[1])
    )
    
  }
  
})

test_that("small A values for 'norm'", {
  n <- 10
  samples_per_run <- 100
  source(system.file("extdata", "lba-math.R", package = "rtdists"))
  
  #source("inst/extdata//lba-math.r")
  for (i in seq_len(n)) {
    A <- runif(1, 0, 1e-10)
    b <- A+runif(1, 0, 0.5)
    t0 <- runif(1, 0.1, 0.7)
    v1 <- runif(2, 0.5, 1.5)
    v2 <- runif(2, 0.1, 0.5)
    r_lba1 <- rlba_norm(samples_per_run, A=A, b=b, t0 = t0, 
                        mean_v=v1[1:2], sd_v=v2[1:2])
    
    expect_equal(
      dlba_norm(r_lba1[,"rt"][r_lba1[,"response"]==1], A=A, b=b, t0 = t0, 
                mean_v=v1[1], sd_v=v2[1]), 
      fptpdf(pmax(r_lba1[,"rt"][r_lba1[,"response"]==1]-t0[1], 0), 
             x0max=A, chi=b, driftrate=v1[1], sddrift=v2[1])
    )
    expect_equal(
      plba_norm(r_lba1[,"rt"][r_lba1[,"response"]==1], A=A, b=b, t0 = t0, 
                mean_v=v1[1], sd_v=v2[1]), 
      fptcdf(pmax(r_lba1[,"rt"][r_lba1[,"response"]==1]-t0[1], 0), 
             x0max=A, chi=b, driftrate=v1[1], sddrift=v2[1])
    )
    
  }
  
})

test_that("Random generation", {
  n <- 10
  samples_per_run <- 100
  source(system.file("extdata", "lba-math.R", package = "rtdists"))
  #source("inst/extdata//lba-math.r")
  for (i in seq_len(n)) {
    A <- runif(1, 0.3, 0.9)
    b <- A+runif(1, 0, 0.5)
    t0 <- runif(1, 0.1, 0.7)
    v1 <- runif(2, 0.5, 1.5)
    v2 <- runif(2, 0.1, 0.5)
    
    x <- .Random.seed   
    
    r_lba1 <- rlba_norm(samples_per_run, A=A, b=b, t0 = t0, 
                        mean_v=v1[1:2], sd_v=v2[1:2])
    
    .Random.seed <<- x
    
    #r_lba2 <- rlba_norm(samples_per_run, A=A, b=b, t0 = t0, mean_v=v1[1:2], sd_v=v2[1:2])
    r_lba2 <- rlba(samples_per_run, A=A, b=b, t0 = t0, vs=v1[1:2], s=v2[1:2])
    
    expect_equal(r_lba1[,"rt"], r_lba2$rt)
    expect_equal(r_lba1[,"response"], r_lba2$resp)
  }
})

test_that("n1CDF", {
  n <- 10
  samples_per_run <- 100
  source(system.file("extdata", "lba-math.R", package = "rtdists"))
  #source("inst/extdata//lba-math.r")
  for (i in seq_len(n)) {
    A <- runif(1, 0.3, 0.9)
    b <- A+runif(1, 0, 0.5)
    t0 <- runif(1, 0.1, 0.7)
    v1 <- runif(2, 0.5, 1.5)
    v2 <- runif(2, 0.1, 0.5)
    r_lba1 <- rlba_norm(samples_per_run, A=A, b=b, t0 = t0, 
                        mean_v=v1[1:2], sd_v=v2[1:2], posdrift = TRUE)
    #head(r_lba1)
    #if(!isTRUE(all.equal(n1CDF(r_lba1$rt[r_lba1$response==1], A=A, b=b, t0 = t0, mean_v=v1[1:2], sd_v=v2[1]),.n1CDF(pmax(r_lba1$rt[r_lba1$response==1]-t0[1], 0), x0max=A, chi=b, drift=v1[1:2], sdI=v2[1]) ))) browser()
      #n1CDF(r_lba1$rt[r_lba1$response==1], A=A, b=b, t0 = t0, mean_v=v1[1:2], sd_v=v2[1], browser = TRUE) 
      #n1CDF(pmax(r_lba1$rt[r_lba1$response==1]-t0[1], 0), A=A, b=b, t0 = 0, mean_v=v1[1:2], sd_v=v2[1])      
      #.n1CDF(pmax(r_lba1$rt[r_lba1$response==1]-t0[1], 0), x0max=A, chi=b, drift=v1[1:2], sdI=v2[1], browser=TRUE)     
    #save(r_lba1, A, b, t0, v1, v2, file = "n1CDF_no_diff_example_5.RData")
    
    expect_equal(
      n1CDF(sort(r_lba1[,"rt"][r_lba1[,"response"]==1]), A=A, b=b, t0 = t0, 
            mean_v=v1[1:2], sd_v=v2[1]), 
      .n1CDF(sort(pmax(r_lba1[,"rt"][r_lba1[,"response"]==1]-t0[1], 0)), 
             x0max=A, chi=b, drift=v1[1:2], sdI=v2[1]), tolerance = 0.0001
    )
    
  }
  
})

.Random.seed <<- x

## ----fig.height=4, fig.width=7, message=FALSE, warning=FALSE------------------
require(rtdists)
require(dplyr)   # for data manipulations and looping
require(tidyr)   # for data manipulations
require(purrr)   # for data manipulations
require(lattice) # for plotting and corresponding themes
require(latticeExtra)
lattice.options(default.theme = standard.theme(color = FALSE))
lattice.options(default.args = list(as.table = TRUE))
options(digits = 3) # only three decimal digits
require(binom)  # for binomial confidence intervals

data(rr98)
rr98 <- rr98[!rr98$outlier,]  #remove outliers

# aggregate data for first plot:
agg_rr98 <- rr98  %>% group_by(id, instruction, strength) %>% 
  summarise(prop = mean(response == "dark"), mean_rt = mean(rt), median_rt = mean(rt)) %>% 
  ungroup()

xyplot(prop ~ strength|id, agg_rr98, group = instruction, type = "b", 
       auto.key = list(lines = TRUE), ylab = "Proportion of 'dark' responses")



## ----fig.height=6, fig.width=7------------------------------------------------

quantiles <- c(0.1, 0.3, 0.5, 0.7, 0.9)
## aggregate data for quantile plot
quantiles_rr98 <- rr98  %>% 
  group_by(id, instruction, strength) %>% 
  nest() %>% 
  mutate(quantiles = map(data, ~ as.data.frame(t(quantile(.x$rt, probs = quantiles))))) %>% 
  unnest(quantiles) %>% 
  gather("quantile", "rt",`10%`:`90%`) %>% 
  arrange(id, instruction, strength)

xyplot(rt ~ strength|id + instruction, quantiles_rr98, group = quantile, type = "b", 
       auto.key = list(lines = TRUE), ylab = "RT (in seconds)", subset = instruction == "speed")

xyplot(rt ~ strength|id + instruction, quantiles_rr98, group = quantile, type = "b", 
       auto.key = FALSE, ylab = "RT (in seconds)", subset = instruction == "accuracy")



## ----fig.height=4, fig.width=7------------------------------------------------

#bins <- c(-0.5, 5.5, 10.5, 13.5, 16.5, 19.5, 25.5, 32.5) # seven bins like RR98
bins <- c(-0.5, 10.5, 13.5, 16.5, 19.5, 32.5)
rr98$strength_bin <- cut(rr98$strength, breaks = bins, include.lowest = TRUE)
levels(rr98$strength_bin) <- as.character(1:7)

# aggregate data for response probability plot:
agg_rr98_bin <- rr98 %>% 
  group_by(id, instruction, strength_bin) %>%
  summarise(n1 = n(), 
            dark = sum(response == "dark"),
            light = sum(response == "light")) %>%
  ungroup() %>%
  mutate(prop = map2(dark, n1, ~ binom.confint(.x, .y, methods = "agresti-coull"))) %>% 
  unnest(prop)
  

knitr::kable(
  rr98 %>% group_by(id, instruction, strength_bin, response) %>%
    summarise(n = n()) %>%
    spread(strength_bin, n)
)



## ----fig.height=4, fig.width=7------------------------------------------------
xyplot(mean ~ strength_bin|id, agg_rr98_bin, group = instruction, type = "b", 
       auto.key = list(lines = TRUE), ylab = "Proportion of 'dark' responses")


## ----fig.height=6, fig.width=7------------------------------------------------

## aggregate data for quantile plot
quantiles_rr98_bin <- rr98  %>% 
  group_by(id, instruction, strength_bin) %>% 
  nest() %>% 
  mutate(quantiles = map(data, ~ as.data.frame(t(quantile(.x$rt, probs = quantiles))))) %>% 
  unnest(quantiles) %>% 
  gather("quantile", "rt",`10%`:`90%`) %>% 
  arrange(id, instruction, strength_bin)

xyplot(rt ~ strength_bin|id + instruction, quantiles_rr98_bin, group = quantile, type = "b", 
       auto.key = list(lines = TRUE), ylab = "RT (in seconds)", subset = instruction == "speed")

xyplot(rt ~ strength_bin|id + instruction, quantiles_rr98_bin, group = quantile, type = "b", 
       auto.key = FALSE, ylab = "RT (in seconds)", subset = instruction == "accuracy")


## ----fig.height=6, fig.width=7------------------------------------------------

agg2_rr98_response <- rr98  %>% 
  group_by(id, instruction, strength_bin, response) %>% 
  nest() %>% 
  mutate(quantiles = map(data, ~ as.data.frame(t(quantile(.x$rt, probs = quantiles))))) %>% 
  unnest(quantiles) %>% 
  gather("quantile", "rt",`10%`:`90%`) %>% 
  arrange(id, instruction, response, strength_bin)

p1 <- xyplot(rt ~ strength_bin|id, agg2_rr98_response, group = quantile, type = "b", 
             auto.key = list(lines = TRUE), ylab = "RT (in seconds)", 
             subset = instruction == "speed" & response == "dark", layout = c(3,1))
p2 <- xyplot(rt ~ strength_bin|id, agg2_rr98_response, group = quantile, type = "b", 
             auto.key = list(lines = TRUE), ylab = "RT (in seconds)", 
             subset = instruction == "speed" & response == "light", col = "grey")
p1 + as.layer(p2)


p1 <- xyplot(rt ~ strength_bin|id, agg2_rr98_response, group = quantile, type = "b", 
             auto.key = list(lines = TRUE), ylab = "RT (in seconds)", 
             subset = instruction == "accuracy" & response == "dark", layout = c(3,1))
p2 <- xyplot(rt ~ strength_bin|id, agg2_rr98_response, group = quantile, type = "b", 
             auto.key = list(lines = TRUE), ylab = "RT (in seconds)", 
             subset = instruction == "accuracy" & response == "light", col = "grey")
p1 + as.layer(p2)



## -----------------------------------------------------------------------------
d_nested <- rr98 %>% 
  group_by(id, instruction) %>% # we loop across both, id and instruction
  nest()
d_nested

## -----------------------------------------------------------------------------
# objective function for diffusion with 1 a. loops over drift to assign drift rates to strength
objective_diffusion_separate <- function(pars, rt, response, drift, ...) {
  non_v_pars <- grep("^v", names(pars), invert = TRUE, value = TRUE)
  base_par <- length(non_v_pars)  # number of non-drift parameters
  densities <- vector("numeric", length(rt))
  for (i in seq_along(levels(drift))) {
    densities[drift == levels(drift)[i]] <- 
      ddiffusion(rt[drift == levels(drift)[i]], response=response[drift == levels(drift)[i]], 
                 a=pars["a"], t0=pars["t0"],  
                 sv=pars["sv"],
                 sz=if ("sz" %in% non_v_pars) pars["sz"] else 0.1,
                 z=if ("z" %in% non_v_pars) pars["z"]*pars["a"] else 0.5*pars["a"],
                 st0=if ("st0" %in% non_v_pars) pars["st0"] else 0, 
                 v=pars[base_par+i])
  }
  if (any(densities == 0)) return(1e6)
  return(-sum(log(densities)))
}


## -----------------------------------------------------------------------------

# function that creates random start values, also 
get_start <- function(base_par, n_drift = 5) {
  start1 <- c(
    a = runif(1, 0.5, 3),
    a_1 = runif(1, 0.5, 3), 
    a_2 = runif(1, 0.5, 3),
    t0 = runif(1, 0, 0.5), 
    z = runif(1, 0.4, 0.6), 
    sz = runif(1, 0, 0.5),
    sv = runif(1, 0, 0.5),
    st0 = runif(1, 0, 0.5),
    d = rnorm(1, 0, 0.05)
  )
  start2 <- sort(rnorm(n_drift), decreasing = FALSE)
  names(start2) <- paste0("v_", seq_len(n_drift))
  c(start1[base_par], start2)
}

# function that tries different random start values until it works:
ensure_fit <- 
  function(data, start_function, objective_function, 
           base_pars, n_drift = 5, n_fits = 1, 
           lower = c(rep(0, length(base_pars)), -Inf,
                     rep(-Inf,length(start_function(base_pars))-length(base_pars)))) {
    best_fit <- list(objective = 1e+06)
  for (i in seq_len(n_fits)) {
    start_ll <- 1e+06
    #browser()
    while(start_ll == 1e+06) {
      start <- start_function(base_pars, n_drift=n_drift)
      start_ll <- objective_function(start, 
                                     rt = data$rt, response = data$response_num, 
                                     drift = factor(data$strength_bin, seq_len(n_drift)), 
                                     instruction = data$instruction)
    }
    cat("\nstart fitting.\n") # just for information to see if it is stuck
    
    fit <- nlminb(start, objective_function, 
                  rt = data$rt, response = data$response_num, 
                  drift = factor(data$strength_bin, seq_len(n_drift)), 
                  instruction = data$instruction,
                  lower = lower)
    
    if (fit$objective < best_fit$objective) best_fit <- fit
  }
  out <- as.data.frame(t(unlist(best_fit[1:3])))
  colnames(out) <- sub("par.", "", colnames(out))
  out
}


## ----echo=FALSE---------------------------------------------------------------
load("rr98_full-diffusion_fits.rda")
load("rr98_full-lba_fits.rda")


## ----eval = FALSE-------------------------------------------------------------
# fit_diffusion <- d_nested %>%
#   mutate(fit =
#            map(data,
#                ~ensure_fit(data = ., start_function = get_start,
#                             objective_function = objective_diffusion_separate,
#                             base_pars = c("a", "t0", "sv", "sz", "z")))) %>%
#   unnest(fit)

## ----eval = FALSE-------------------------------------------------------------
# require(parallel)
# 
# fit_diffusion <- d_nested
# fit_diffusion$fit <-
#   mclapply(d_nested$data, function(x)
#     ensure_fit(data = x, start_function = get_start,
#                objective_function = objective_diffusion_separate,
#                base_pars = c("a", "t0", "sv", "sz", "z")),
#     mc.cores = 2)
# fit_diffusion <- unnest(fit_diffusion, fit)

## -----------------------------------------------------------------------------

fit_diffusion$data <- NULL
if (!("st0" %in% colnames(fit_diffusion))) fit_diffusion$st0 <- 0
if (!("z" %in% colnames(fit_diffusion))) fit_diffusion$z <- 0.5
if (!("sz" %in% colnames(fit_diffusion))) fit_diffusion$sz <- 0.1
knitr::kable(fit_diffusion)


## ----obtain_fits_not_run, eval = FALSE, include = FALSE-----------------------
# 
# require(parallel)
# 
# fit_diffusion <- d_nested %>%
#   mutate(fit =
#            map(data,
#                ~ensure_fit(data = ., start_function = get_start,
#                             objective_function = objective_diffusion_separate,
#                             base_pars = c("a", "t0", "sv", "sz", "z")))) %>%
#   unnest(fit)
# fit_diffusion$data <- NULL
# 
# fit_diffusion2 <- d_nested
# fit_diffusion2$fit <-
#   mclapply(d_nested$data, function(x)
#     ensure_fit(data = x, start_function = get_start,
#                objective_function = objective_diffusion_separate,
#                base_pars = c("a", "t0", "sv", "sz", "z")),
#     mc.cores = 3)
# fit_diffusion2 <- unnest(fit_diffusion2, fit)
# fit_diffusion2$data <- NULL
# 
# all.equal(as.data.frame(fit_diffusion), as.data.frame(fit_diffusion2), tolerance = 0.01)
# 
# save(fit_diffusion, fit_diffusion2, file = "rr98_full-diffusion_fits.rda")
# 
# 

## ----fig.height=5, fig.width=7, message=FALSE---------------------------------


# get predicted response proportions
pars_separate_l <- fit_diffusion %>% gather("strength_bin", "v", starts_with("v"))
pars_separate_l$strength_bin <- factor(substr(pars_separate_l$strength_bin, 3,3), 
                                       levels = as.character(seq_len(length(bins)-1)))
#pars_separate_l <- inner_join(pars_separate_l, agg_rr98_bin)
pars_separate_l <- pars_separate_l  %>% group_by(id, instruction, strength_bin) %>%
  mutate(resp_prop = pdiffusion(rt=Inf, response="lower", 
                                a=a, v=v, t0=t0, sz = sz, z=a*z, sv=sv, st0=st0))

p1 <- xyplot(mean ~ strength_bin|id + instruction, agg_rr98_bin, type = "b", auto.key = 
               list(lines = TRUE), ylab = "Proportion of 'dark' responses", col = "grey")
p2 <- segplot(strength_bin ~ upper+lower|id + instruction, agg_rr98_bin, 
              auto.key = list(lines = TRUE), ylab = "Proportion of 'dark' responses", 
              col = "grey", horizontal = FALSE, segments.fun = panel.arrows,  
              draw.bands = FALSE, angle = 90, length = 0.05, ends = "both")
p3 <- xyplot(resp_prop ~ strength_bin|id + instruction, pars_separate_l, type = "b", 
             auto.key = list(lines = TRUE), ylab = "Proportion of 'dark' responses", 
             col = "black")
p2 + as.layer(p1) + as.layer(p3)


## ----fig.height=6, fig.width=7, message=FALSE---------------------------------

# get predicted quantiles (uses predicted response proportions)
separate_pred_dark <- pars_separate_l %>% do(as.data.frame(t(
  qdiffusion(quantiles*.$resp_prop, response="lower", 
             a=.$a, v=.$v, t0=.$t0, sz = .$sz, z = .$z*.$a, sv=.$sv, st0=.$st0)))) %>% 
  ungroup() %>% gather("quantiles", "dark", V1:V5)
separate_pred_light <- pars_separate_l %>% do(as.data.frame(t(
  qdiffusion(quantiles*(1-.$resp_prop), response="upper", 
             a=.$a, v=.$v, t0=.$t0, sz = .$sz, z = .$z*.$a, sv=.$sv, st0=.$st0)))) %>% 
  ungroup() %>% gather("quantiles", "light", V1:V5)

#separate_pred_light %>% filter(is.na(light))
separate_pred <- inner_join(separate_pred_dark, separate_pred_light)
separate_pred$quantiles <- factor(separate_pred$quantiles, 
                                  levels = c("V5", "V4", "V3", "V2", "V1"), 
                                  labels = c("90%", "70%", "50%", "30%", "10%"))
separate_pred <- separate_pred %>% gather("response", "rt", dark, light)

# get SE for observed quantiles
agg2_rr98_response_se <- rr98  %>% group_by(id, instruction, strength_bin, response) %>% 
  summarise(se_median = sqrt(pi/2)*(sd(rt)/sqrt(n()))) %>%
  ungroup()

# calculate error bars for quantiles.
agg2_rr98_response <- left_join(agg2_rr98_response, agg2_rr98_response_se)
agg2_rr98_response <- agg2_rr98_response %>%
  mutate(lower = rt-se_median, upper = rt+se_median)


p1 <- xyplot(rt ~ strength_bin|id+response, agg2_rr98_response, type = "b", 
             auto.key = list(lines = TRUE), ylab = "RT (in seconds)", 
             subset = instruction == "speed" & quantile == "50%", 
             layout = c(3,2), col = "grey")
p1e <- segplot(strength_bin ~ upper+lower|id+response, agg2_rr98_response, 
               auto.key = list(lines = TRUE), ylab = "Proportion of 'dark' responses", 
               col = "grey", horizontal = FALSE, segments.fun = panel.arrows,  
               draw.bands = FALSE, angle = 90, length = 0.05, ends = "both", 
               subset = instruction == "speed" & quantile == "50%", layout = c(3,2))
p2 <- xyplot(rt ~ strength_bin|id + response, separate_pred, type = "b", 
             auto.key = list(lines = TRUE), ylab = "RT (in seconds)", 
             subset = instruction == "speed" & quantiles == "50%", 
             scales = list(y = list(limits = c(0.25, 0.5))))
p2 + as.layer(p1) + as.layer(p1e)


## ----fig.height=6, fig.width=7------------------------------------------------

p1 <- xyplot(rt ~ strength_bin|id+response, agg2_rr98_response, type = "b", 
             auto.key = list(lines = TRUE), ylab = "RT (in seconds)", 
             subset = instruction == "accuracy" & quantile == "50%", 
             layout = c(3,2), col = "grey")
p1e <- segplot(strength_bin ~ upper+lower|id+response, agg2_rr98_response, 
               auto.key = list(lines = TRUE), ylab = "Proportion of 'dark' responses", 
               col = "grey", horizontal = FALSE, segments.fun = panel.arrows,  
               draw.bands = FALSE, angle = 90, length = 0.05, ends = "both", 
               subset = instruction == "accuracy" & quantile == "50%", layout = c(3,2))
p2 <- xyplot(rt ~ strength_bin|id + response, separate_pred, type = "b", 
             auto.key = list(lines = TRUE), ylab = "RT (in seconds)", 
             subset = instruction == "accuracy" & quantiles == "50%", 
             scales = list(y = list(limits = c(0.2, 1.5))))
p2 + as.layer(p1) + as.layer(p1e)


## ----fig.height=7, fig.width=7------------------------------------------------

p1 <- xyplot(rt ~ strength_bin|id+response, agg2_rr98_response, group = quantile, type = "b", 
             auto.key = list(lines = TRUE), ylab = "RT (in seconds)", 
             subset = instruction == "speed", layout = c(3,2), col = "grey")
p1e <- segplot(strength_bin ~ upper+lower|id+response, agg2_rr98_response, 
               auto.key = list(lines = TRUE), ylab = "Proportion of 'dark' responses", 
               col = "grey", horizontal = FALSE, segments.fun = panel.arrows,  
               draw.bands = FALSE, angle = 90, length = 0.05, ends = "both", 
               subset = instruction == "speed")
p2 <- xyplot(rt ~ strength_bin|id + response, separate_pred, group = quantiles, type = "b", 
             auto.key = list(lines = TRUE), ylab = "RT (in seconds)", 
             subset = instruction == "speed", scales = list(y = list(limits = c(0.2, 0.9))))
p2 + as.layer(p1) + as.layer(p1e)


## ----fig.height=7, fig.width=7------------------------------------------------

p1 <- xyplot(rt ~ strength_bin|id+response, agg2_rr98_response, group = quantile, type = "b", 
             auto.key = list(lines = TRUE), ylab = "RT (in seconds)", 
             subset = instruction == "accuracy", layout = c(3,2), col = "grey")
p1e <- segplot(strength_bin ~ upper+lower|id+response, agg2_rr98_response, 
               auto.key = list(lines = TRUE), ylab = "Proportion of 'dark' responses", 
               col = "grey", horizontal = FALSE, segments.fun = panel.arrows,  
               draw.bands = FALSE, angle = 90, length = 0.05, ends = "both", 
               subset = instruction == "accuracy")
p2 <- xyplot(rt ~ strength_bin|id + response, separate_pred, group = quantiles, type = "b", 
             auto.key = list(lines = TRUE), ylab = "RT (in seconds)", 
             subset = instruction == "accuracy", scales = list(y = list(limits = c(0.1, 3.0))))
p2 + as.layer(p1) + as.layer(p1e)


## -----------------------------------------------------------------------------

# objective function for diffusion with 1 a. loops over drift to assign drift rates to strength
objective_lba_separate <- function(pars, rt, response, drift, ...) {
  non_v_pars <- grep("^v", names(pars), invert = TRUE, value = TRUE)
  base_par <- length(non_v_pars)  # number of non-drift parameters
  densities <- vector("numeric", length(rt))
  for (i in seq_along(levels(drift))) {
    if (sum(drift == levels(drift)[i]) == 0) next
    densities[drift == levels(drift)[i]] <- dLBA(
      rt[drift == levels(drift)[i]], 
      response=response[drift == levels(drift)[i]],
      A = list(pars["a_1"], pars["a_2"]), 
      b = max(pars["a_1"], pars["a_2"])+pars["b"], 
      t0 = pars["t0"], 
      mean_v = c(pars[i], 1-pars[i]), 
      sd_v = pars["sv"], silent=TRUE)
  }
  if (any(densities == 0)) return(1e6)
  return(-sum(log(densities)))
}

# function that creates random start values
get_start_lba <- function(base_par, n_drift = 10) {
  start1 <- c(
    a = runif(1, 0.5, 3),
    a_1 = runif(1, 0.5, 3), 
    a_2 = runif(1, 0.5, 3),
    t0 = runif(1, 0, 0.5), 
    b = runif(1, 0, 0.5), 
    sv = runif(1, 0.5, 1.5),
    st0 = runif(1, 0, 0.5)
  )
  start2 <- sort(rnorm(n_drift), decreasing = FALSE)
  names(start2) <- paste0("v_", seq_len(n_drift))
  c(start2, start1[base_par])
}


## ----eval=FALSE---------------------------------------------------------------
# 
# fit_lba <- d_nested %>%
#   mutate(fit =
#            map(data,
#                ~ensure_fit(data = ., start_function = get_start_lba,
#                       objective_function = objective_lba_separate,
#                       base_pars = c("a_1", "a_2", "t0", "b", "sv"),
#                       lower = c(rep(-Inf, 5), rep(0, 5)),
#                       n_drift = 5, n_fits = 10))) %>%
#   unnest(fit)
# 

## ----eval = FALSE-------------------------------------------------------------
# require(parallel)
# 
# fit_lba <- d_nested
# fit_lba$fit <-
#   mclapply(d_nested$data, function(x)
#     ensure_fit(data = x, start_function = get_start_lba,
#                       objective_function = objective_lba_separate,
#                       base_pars = c("a_1", "a_2", "t0", "b", "sv"),
#                       lower = c(rep(-Inf, 5), rep(0, 5)),
#                       n_drift = 5, n_fits = 10),
#     mc.cores = 2)
# fit_lba <- unnest(fit_lba, fit)

## -----------------------------------------------------------------------------
knitr::kable(fit_lba)


## ----obtain_fits_lba, eval = FALSE, include = FALSE---------------------------
# fit_lba <- d_nested %>%
#   mutate(fit =
#            map(data,
#                ~ensure_fit(data = ., start_function = get_start_lba,
#                       objective_function = objective_lba_separate,
#                       base_pars = c("a_1", "a_2", "t0", "b", "sv"),
#                       lower = c(rep(-Inf, 5), rep(0, 5)),
#                       n_drift = 5, n_fits = 10))) %>%
#   unnest(fit)
# fit_lba$data <- NULL
# 
# fit_lba2 <- d_nested
# fit_lba2$fit <-
#   mclapply(d_nested$data, function(x)
#     ensure_fit(data = x, start_function = get_start_lba,
#                       objective_function = objective_lba_separate,
#                       base_pars = c("a_1", "a_2", "t0", "b", "sv"),
#                       lower = c(rep(-Inf, 5), rep(0, 5)),
#                       n_drift = 5, n_fits = 10),
#     mc.cores = 2)
# fit_lba2 <- unnest(fit_lba2, fit)
# fit_lba2$data <- NULL
# 
# all.equal(as.data.frame(fit_lba), as.data.frame(fit_lba2), tolerance = 0.03)
# save(fit_lba, fit_lba2, file = "rr98_full-lba_fits.rda")
# 
# 
# 
# # objective function for LBA with 1 a. loops over drift to assign drift rates to strength
# objective_lba_separate <- function(pars, rt, response, drift, ...) {
#   non_v_pars <- grep("^v", names(pars), invert = TRUE, value = TRUE)
#   base_par <- length(non_v_pars)  # number of non-drift parameters
#   densities <- vector("numeric", length(rt))
#   for (i in seq_along(levels(drift))) {
#     if (sum(drift == levels(drift)[i]) == 0) next
#     densities[drift == levels(drift)[i]] <- dLBA(
#       rt[drift == levels(drift)[i]],
#       response=response[drift == levels(drift)[i]],
#       A = pars["a"], b = pars["a"]+pars["b"],
#       t0 = pars["t0"],
#       mean_v = pars[((i-1)*2+1):((i-1)*2+2)],
#       sd_v = c(1, pars["sv"]), silent=TRUE)
#   }
#   if (any(densities == 0)) return(1e6)
#   return(-sum(log(densities)))
# }
# 
# # objective function for diffusion with 1 a. loops over drift to assign drift rates to strength
# objective_lba_separate <- function(pars, rt, response, drift, ...) {
#   non_v_pars <- grep("^v", names(pars), invert = TRUE, value = TRUE)
#   base_par <- length(non_v_pars)  # number of non-drift parameters
#   densities <- vector("numeric", length(rt))
#   for (i in seq_along(levels(drift))) {
#     if (sum(drift == levels(drift)[i]) == 0) next
#     densities[drift == levels(drift)[i]] <- dLBA(
#       rt[drift == levels(drift)[i]],
#       response=response[drift == levels(drift)[i]],
#       A = list(pars["a_1"], pars["a_2"]), b = list(pars["a_1"]+pars["b"], pars["a_2"]+pars["b"]),
#       t0 = pars["t0"],
#       mean_v = c(pars[i], 1-pars[i]),
#       sd_v = pars["sv"], silent=TRUE)
#   }
#   if (any(densities == 0)) return(1e6)
#   return(-sum(log(densities)))
# }
# 

## ----fig.height=5, fig.width=7, message=FALSE---------------------------------
# get predicted response proportions
lba_pars_separate_l <- fit_lba %>% gather("strength_bin", "v", starts_with("v"))
lba_pars_separate_l$strength_bin <- factor(substr(lba_pars_separate_l$strength_bin, 3,3), 
                                       levels = as.character(seq_len(length(bins)-1)))
#pars_separate_l <- inner_join(pars_separate_l, agg_rr98_bin)
lba_pars_separate_l <- lba_pars_separate_l  %>% group_by(id, instruction, strength_bin) %>%
  mutate(resp_prop = pLBA(rt=Inf, response=1, A=list(a_1, a_2), sd_v=sv,
                          mean_v=c(v, 1-v), t0=t0, b=max(a_1, a_2)+b, silent=TRUE))

p1 <- xyplot(mean ~ strength_bin|id + instruction, agg_rr98_bin, type = "b", auto.key = 
               list(lines = TRUE), ylab = "Proportion of 'dark' responses", col = "grey")
p2 <- segplot(strength_bin ~ upper+lower|id + instruction, agg_rr98_bin, 
              auto.key = list(lines = TRUE), ylab = "Proportion of 'dark' responses", 
              col = "grey", horizontal = FALSE, segments.fun = panel.arrows,  
              draw.bands = FALSE, angle = 90, length = 0.05, ends = "both")
p3 <- xyplot(resp_prop ~ strength_bin|id + instruction, lba_pars_separate_l, type = "b", 
             auto.key = list(lines = TRUE), ylab = "Proportion of 'dark' responses", 
             col = "black")
p2 + as.layer(p1) + as.layer(p3)


## ----fig.height=6, fig.width=7, message=FALSE---------------------------------

# get predicted quantiles (uses predicted response proportions)
lba_separate_pred_dark <- lba_pars_separate_l %>% do(as.data.frame(t(
  qLBA(quantiles*.$resp_prop, response=1, A=list(.$a_1, .$a_2), sd_v=.$sv,
       mean_v=c(.$v, 1-.$v), t0=.$t0, b=max(.$a_1, .$a_2)+.$b, silent=TRUE)))) %>% 
  ungroup() %>% gather("quantiles", "dark", V1:V5)
lba_separate_pred_light <- lba_pars_separate_l %>% do(as.data.frame(t(
  qLBA(quantiles*(1-.$resp_prop), response=2, A=list(.$a_1, .$a_2), sd_v=.$sv,
       mean_v=c(.$v, 1-.$v), t0=.$t0, b=max(.$a_1, .$a_2)+.$b, silent=TRUE)))) %>% 
  ungroup() %>% gather("quantiles", "light", V1:V5)

#separate_pred_light %>% filter(is.na(light))
lba_separate_pred <- inner_join(lba_separate_pred_dark, lba_separate_pred_light)
lba_separate_pred$quantiles <- factor(lba_separate_pred$quantiles, 
                                  levels = c("V5", "V4", "V3", "V2", "V1"), 
                                  labels = c("90%", "70%", "50%", "30%", "10%"))
lba_separate_pred <- lba_separate_pred %>% gather("response", "rt", dark, light)

p1 <- xyplot(rt ~ strength_bin|id+response, agg2_rr98_response, type = "b", 
             auto.key = list(lines = TRUE), ylab = "RT (in seconds)", 
             subset = instruction == "speed" & quantile == "50%", 
             layout = c(3,2), col = "grey")
p1e <- segplot(strength_bin ~ upper+lower|id+response, agg2_rr98_response, 
               auto.key = list(lines = TRUE), ylab = "Proportion of 'dark' responses", 
               col = "grey", horizontal = FALSE, segments.fun = panel.arrows,  
               draw.bands = FALSE, angle = 90, length = 0.05, ends = "both", 
               subset = instruction == "speed" & quantile == "50%", layout = c(3,2))
p2 <- xyplot(rt ~ strength_bin|id + response, lba_separate_pred, type = "b", 
             auto.key = list(lines = TRUE), ylab = "RT (in seconds)", 
             subset = instruction == "speed" & quantiles == "50%", 
             scales = list(y = list(limits = c(0.25, 0.5))))
p2 + as.layer(p1) + as.layer(p1e)


## ----fig.height=6, fig.width=7------------------------------------------------

p1 <- xyplot(rt ~ strength_bin|id+response, agg2_rr98_response, type = "b", 
             auto.key = list(lines = TRUE), ylab = "RT (in seconds)", 
             subset = instruction == "accuracy" & quantile == "50%", 
             layout = c(3,2), col = "grey")
p1e <- segplot(strength_bin ~ upper+lower|id+response, agg2_rr98_response, 
               auto.key = list(lines = TRUE), ylab = "Proportion of 'dark' responses", 
               col = "grey", horizontal = FALSE, segments.fun = panel.arrows,  
               draw.bands = FALSE, angle = 90, length = 0.05, ends = "both", 
               subset = instruction == "accuracy" & quantile == "50%", layout = c(3,2))
p2 <- xyplot(rt ~ strength_bin|id + response, lba_separate_pred, type = "b", 
             auto.key = list(lines = TRUE), ylab = "RT (in seconds)", 
             subset = instruction == "accuracy" & quantiles == "50%", 
             scales = list(y = list(limits = c(0.2, 1.5))))
p2 + as.layer(p1) + as.layer(p1e)


## ----fig.height=7, fig.width=7------------------------------------------------

p1 <- xyplot(rt ~ strength_bin|id+response, agg2_rr98_response, group = quantile, type = "b", 
             auto.key = list(lines = TRUE), ylab = "RT (in seconds)", 
             subset = instruction == "speed", layout = c(3,2), col = "grey")
p1e <- segplot(strength_bin ~ upper+lower|id+response, agg2_rr98_response, 
               auto.key = list(lines = TRUE), ylab = "Proportion of 'dark' responses", 
               col = "grey", horizontal = FALSE, segments.fun = panel.arrows,  
               draw.bands = FALSE, angle = 90, length = 0.05, ends = "both", 
               subset = instruction == "speed")
p2 <- xyplot(rt ~ strength_bin|id + response, lba_separate_pred, group = quantiles, type = "b", 
             auto.key = list(lines = TRUE), ylab = "RT (in seconds)", 
             subset = instruction == "speed", scales = list(y = list(limits = c(0.2, 0.6))))
p2 + as.layer(p1) + as.layer(p1e)


## ----fig.height=7, fig.width=7------------------------------------------------

p1 <- xyplot(rt ~ strength_bin|id+response, agg2_rr98_response, group = quantile, type = "b", 
             auto.key = list(lines = TRUE), ylab = "RT (in seconds)", 
             subset = instruction == "accuracy", layout = c(3,2), col = "grey")
p1e <- segplot(strength_bin ~ upper+lower|id+response, agg2_rr98_response, 
               auto.key = list(lines = TRUE), ylab = "Proportion of 'dark' responses", 
               col = "grey", horizontal = FALSE, segments.fun = panel.arrows,  
               draw.bands = FALSE, angle = 90, length = 0.05, ends = "both", 
               subset = instruction == "accuracy")
p2 <- xyplot(rt ~ strength_bin|id + response, lba_separate_pred, group = quantiles, type = "b", 
             auto.key = list(lines = TRUE), ylab = "RT (in seconds)", 
             subset = instruction == "accuracy", scales = list(y = list(limits = c(0.1, 3.3))))
p2 + as.layer(p1) + as.layer(p1e)


## ----fig.height=6.5, fig.width=7, message=FALSE-------------------------------

key <- simpleKey(text = c("data", "LBA", "Diffusion"), lines = TRUE)
key$lines$col <- c("grey", "black", "black")
key$lines$lty <- c(1, 1, 2)
key$points$col <- c("grey", "black", "black")
key$points$pch <- c(1, 0, 4)

p1 <- xyplot(mean ~ strength_bin|id + instruction, agg_rr98_bin, type = "b", auto.key = 
               list(lines = TRUE), ylab = "Proportion of 'dark' responses", col = "grey")
p2 <- segplot(strength_bin ~ upper+lower|id + instruction, agg_rr98_bin, 
              auto.key = list(lines = TRUE), ylab = "Proportion of 'dark' responses", 
              col = "grey", horizontal = FALSE, segments.fun = panel.arrows,  
              draw.bands = FALSE, angle = 90, length = 0.05, ends = "both")
p3 <- xyplot(resp_prop ~ strength_bin|id + instruction, lba_pars_separate_l, type = "b", 
             auto.key = list(lines = TRUE), ylab = "Proportion of 'dark' responses", 
             col = "black", pch = 0)
p4 <- xyplot(resp_prop ~ strength_bin|id + instruction, pars_separate_l, type = "b", 
             auto.key = list(lines = TRUE), ylab = "Proportion of 'dark' responses", 
             col = "black", lty = 2, key = key, pch=4)
p4 + as.layer(p2) + as.layer(p1) + as.layer(p3)


## ----fig.height=6.5, fig.width=7, message=FALSE-------------------------------

p1 <- xyplot(rt ~ strength_bin|id+response, agg2_rr98_response, type = "b", 
             auto.key = list(lines = TRUE), ylab = "RT (in seconds)", 
             subset = instruction == "speed" & quantile == "50%", 
             layout = c(3,2), col = "grey")
p1e <- segplot(strength_bin ~ upper+lower|id+response, agg2_rr98_response, 
               auto.key = list(lines = TRUE), ylab = "Proportion of 'dark' responses", 
               col = "grey", horizontal = FALSE, segments.fun = panel.arrows,  
               draw.bands = FALSE, angle = 90, length = 0.05, ends = "both", 
               subset = instruction == "speed" & quantile == "50%", layout = c(3,2))
p2 <- xyplot(rt ~ strength_bin|id + response, lba_separate_pred, type = "b", 
             auto.key = list(lines = TRUE), ylab = "RT (in seconds)", 
             subset = instruction == "speed" & quantiles == "50%", 
             scales = list(y = list(limits = c(0.25, 0.5))), pch = 0)
p3 <- xyplot(rt ~ strength_bin|id + response, separate_pred, type = "b", 
             auto.key = list(lines = TRUE), ylab = "RT (in seconds)", 
             subset = instruction == "speed" & quantiles == "50%", 
             scales = list(y = list(limits = c(0.25, 0.5))),
             col = "black", lty = 2, key = key, pch=4)

p3 + as.layer(p2) + as.layer(p1) + as.layer(p1e)


## ----fig.height=6.5, fig.width=7, message=FALSE-------------------------------

p1 <- xyplot(rt ~ strength_bin|id+response, agg2_rr98_response, type = "b", 
             auto.key = list(lines = TRUE), ylab = "RT (in seconds)", 
             subset = instruction == "accuracy" & quantile == "50%", 
             layout = c(3,2), col = "grey")
p1e <- segplot(strength_bin ~ upper+lower|id+response, agg2_rr98_response, 
               auto.key = list(lines = TRUE), ylab = "Proportion of 'dark' responses", 
               col = "grey", horizontal = FALSE, segments.fun = panel.arrows,  
               draw.bands = FALSE, angle = 90, length = 0.05, ends = "both", 
               subset = instruction == "accuracy" & quantile == "50%", layout = c(3,2))
p2 <- xyplot(rt ~ strength_bin|id + response, lba_separate_pred, type = "b", 
             auto.key = list(lines = TRUE), ylab = "RT (in seconds)", 
             subset = instruction == "accuracy" & quantiles == "50%", 
             pch = 0)
p3 <- xyplot(rt ~ strength_bin|id + response, separate_pred, type = "b", 
             auto.key = list(lines = TRUE), ylab = "RT (in seconds)", 
             subset = instruction == "accuracy" & quantiles == "50%", 
             scales = list(y = list(limits = c(0.2, 1.5))),
             col = "black", lty = 2, key = key, pch=4)

p3 + as.layer(p2) + as.layer(p1) + as.layer(p1e)

#' Single accumulator of linear ballistic accumulator (LBA)
#' 
#' Density, distribution function, and random generation for a single accumulator of the LBA model with the following parameters: \code{A} (upper value of starting point), \code{b} (response threshold), \code{t0} (non-decision time), and driftrate (\code{v}). All functions are available with different distributions underlying the drift rate: Normal (\code{norm}), Gamma (\code{gamma}), Frechet (\code{frechet}), and log normal (\code{lnorm}).
#' 
#' @param rt a vector of RTs.
#' @param n desired number of observations (scalar integer).
#' @param A start point interval or evidence in accumulator before beginning of decision process. Start point varies from trial to trial in the interval [0, \code{A}] (uniform distribution). Average amount of evidence before evidence accumulation across trials is \code{A}/2.
#' @param b response threshold. (\code{b} - \code{A}/2) is a measure of "response caution". 
#' @param t0 non-decision time or response time constant (in seconds). Lower bound for the duration of all non-decisional processes (encoding and response execution).
#' @param st0 variability of non-decision time, such that \code{t0} is uniformly distributed between \code{t0} and \code{t0} + \code{st0}. Only available in random number generation functions \code{rlba_}.
#' 
#' @param mean_v,sd_v mean and standard deviation of normal distribution for drift rate (\code{norm}). See \code{\link{Normal}}
#' @param shape_v,rate_v,scale_v shape, rate, and scale of gamma (\code{gamma}) and scale and shape of Frechet (\code{frechet}) distributions for drift rate. See \code{\link{GammaDist}} or \code{\link[evd]{frechet}}. For Gamma, scale = 1/shape and shape = 1/scale.
#' @param meanlog_v,sdlog_v mean and standard deviation of lognormal distribution on the log scale for drift rate (\code{lnorm}). See \code{\link{Lognormal}}.
#' 
#' @param posdrift logical. Should driftrates be forced to be positive? Default is \code{TRUE}. (Uses truncated normal for random generation).
#' @param robust logical. Should robust normal distributions be used for \code{norm} and \code{lnorm}? Can be helpful in rare cases but is approximately three times slower than the non-robust versions. Default is \code{FALSE}.
#' 
#' 
#' @details These functions are mainly for internal purposes. We do not recommend to use them. Use the high-level functions described in \code{/link{LBA}} instead.
#' 
#' @return All functions starting with a \code{d} return the density (PDF), all functions starting with \code{p} return the distribution function (CDF), and all functions starting with \code{r} return random response times and responses (in a \code{matrix}).
#' 
#' @note Density (i.e., \code{dlba_}), distribution (i.e., \code{plba_}), and random derivative (i.e., \code{rlba_}) functions are vectorized for all parameters (i.e., in case parameters are not of the same length as \code{rt}, parameters are recycled). Furthermore, the random derivative functions also accept a matrix of length \code{n} in which each column corresponds to a accumulator specific value (see \code{\link{rLBA}} for a more user-friendly way).
#' 
#' @references 
#' 
#' Brown, S. D., & Heathcote, A. (2008). The simplest complete model of choice response time: Linear ballistic accumulation. \emph{Cognitive Psychology}, 57(3), 153-178. doi:10.1016/j.cogpsych.2007.12.002
#' 
#' Donkin, C., Averell, L., Brown, S., & Heathcote, A. (2009). Getting more from accuracy and response time data: Methods for fitting the linear ballistic accumulator. \emph{Behavior Research Methods}, 41(4), 1095-1110. doi:10.3758/BRM.41.4.1095
#' 
#' Heathcote, A., & Love, J. (2012). Linear deterministic accumulator models of simple choice. \emph{Frontiers in Psychology}, 3, 292. doi:10.3389/fpsyg.2012.00292
#' 
#' @importFrom evd rfrechet dfrechet pfrechet
#' @importFrom msm rtnorm
#' @importFrom gsl gamma_inc
#' @importFrom stats dgamma dlnorm dnorm pgamma plnorm pnorm rgamma rlnorm rnorm runif
#' 
#' 
#' @name single-LBA
#' 
#' @example examples/examples.slba.R
#' 
NULL

# protected normal desity and cdf
pnormP <- function(x,mean=0,sd=1,lower.tail=TRUE) ifelse(abs(x)<7,pnorm(x, mean=mean, sd=sd,lower.tail=lower.tail),ifelse(x<0,0,1))
dnormP <- function(x,mean=0,sd=1) ifelse(abs(x)<7,dnorm(x,mean=mean,sd=sd),0)

make_r <- function(drifts, n,b,A,n_v,t0,st0=0) {
  drifts <- drifts[1:n,]
  drifts[drifts<0] <- 0
  if (is.null(dim(A))) starts <- matrix(runif(min=0,max=A,n=n*n_v),ncol=n_v,byrow=TRUE)
  else starts <- apply(A, c(1,2), function(x) runif(min=0, max = x, 1))
  if (is.null(dim(b))) ttf <- t((b-t(starts)))/drifts
  else ttf <- (b-starts)/drifts
  rt <- apply(ttf+t0+runif(min=0,max=st0,n=n),1,min)
  resp <- apply(ttf+t0,1,which.min)
  bad <- !is.finite(rt)
  if (any(bad)) {
    warning(paste(sum(bad),"infinite RTs removed and less than", n, "rts returned"))
    resp <- resp[!bad]
    rt <- rt[!bad]
  }
  cbind(rt=rt,response=resp)
}

rem_t0 <- function(rt, t0) pmax(rt - t0, 0)

check_single_arg <- function(...) {
  mc <- match.call()
  vars <- all.vars(mc)
  arguments <- list(...)
  for(i in seq_along(arguments)) {
    if (length(arguments[[i]]) != 1) stop(paste(vars[i], "needs to be of length 1!"))
    if (!is.numeric(arguments[[i]]) | !is.finite(arguments[[i]])) stop(paste(vars[i], "needs to be numeric and finite!"))
  }
}

check_vector <- function(...) {
  mc <- match.call()
  vars <- all.vars(mc)
  dots <- list(...)
  for (i in seq_along(dots)) {
    if ((vars[i] == "rt") && (any(dots[[i]] < 0))) stop("rt needs to contain only positive values.") 
    if (!is.vector(dots[[i]], "numeric")) stop(paste(vars[[i]], "needs to be a numeric vector!"))
    if (length(dots[[i]]) < 1) stop(paste(vars[[i]], "needs to have a length >= 1."))
  }
}

error_message_b_smaller_A <- "b cannot be smaller than A!"

####### Normal:

#' @rdname single-LBA
#' @export dlba_norm
dlba_norm <- function(rt,A,b, t0, mean_v, sd_v, posdrift=TRUE, robust = FALSE) {
  #check_single_arg(A=A, b=b, t0=t0, mean_v=mean_v, sd_v=sd_v)
  check_vector(rt, A, b, t0, mean_v, sd_v)
  # bring all arguments to length of rt
  nn <- length(rt)
  A <- rep(A, length.out = nn)
  b <- rep(b, length.out = nn)
  t0 <- rep(t0, length.out = nn)
  mean_v <- rep(mean_v, length.out = nn)
  sd_v <- rep(sd_v, length.out = nn)
  if (any(b < A)) stop(error_message_b_smaller_A)
  dlba_norm_core(rt = rt, A = A, b = b, t0 = t0, mean_v = mean_v, sd_v = sd_v, posdrift = posdrift, robust = robust, nn = nn)
}

## this functions expects all arguments to have the samel length (which is nn)
dlba_norm_core <- function(rt,A,b, t0, mean_v, sd_v, posdrift=TRUE, robust = FALSE, nn) {
  if (robust) { # robust == TRUE uses robust versions of the normal distributions
    pnorm1 <- pnormP
    dnorm1 <- dnormP
  } else {
    pnorm1 <- pnorm
    dnorm1 <- dnorm
  }
  rt <- rem_t0(rt, t0) # rmove t0 from rt
  if (posdrift) denom <- pmax(pnorm1(mean_v/sd_v),1e-10) else denom <- rep(1, nn)
  
  if (any(A<1e-10, na.rm = TRUE)) {
    # for A<1e-10 save results in out_A
    A_small <- A<1e-10
    out_A <- pmax(0, ((b[A_small]/rt[A_small]^2)*dnorm1(b[A_small]/rt[A_small],mean_v[A_small],sd=sd_v[A_small]))/denom[A_small], na.rm = TRUE) 
    # calculate other results into out_o
    zs <- rt[!A_small]*sd_v[!A_small]
    zu <- rt[!A_small]*mean_v[!A_small]
    chiminuszu <- b[!A_small]-zu
    chizu <- chiminuszu/zs
    chizumax <- (chiminuszu-A[!A_small])/zs
    out_o <- pmax(0,(mean_v[!A_small]*(pnorm1(chizu)-pnorm1(chizumax)) + sd_v[!A_small]*(dnorm1(chizumax)-dnorm1(chizu)))/(A[!A_small]*denom[!A_small]))
    # combine out_A and out_o
    out <- numeric(nn)
    out[!A_small] <- out_o
    out[A_small] <- out_A
    return(out) 
  } else {
    zs <- rt*sd_v
    zu <- rt*mean_v
    chiminuszu <- b-zu
    chizu <- chiminuszu/zs
    chizumax <- (chiminuszu-A)/zs
    return(pmax(0,(mean_v*(pnorm1(chizu)-pnorm1(chizumax)) + sd_v*(dnorm1(chizumax)-dnorm1(chizu)))/(A*denom), na.rm=TRUE)) 
  }
}

#' @rdname single-LBA
#' @export plba_norm
plba_norm <- function(rt,A,b,t0,mean_v, sd_v,posdrift=TRUE, robust = FALSE) {
  check_vector(rt, A, b, t0, mean_v, sd_v)
  nn <- length(rt)
  A <- rep(A, length.out = nn)
  b <- rep(b, length.out = nn)
  t0 <- rep(t0, length.out = nn)
  mean_v <- rep(mean_v, length.out = nn)
  sd_v <- rep(sd_v, length.out = nn)
  if (any(b < A)) stop(error_message_b_smaller_A)
  plba_norm_core(rt = rt, A = A, b = b, t0 = t0, mean_v = mean_v, sd_v = sd_v, posdrift = posdrift, robust = robust, nn = nn)
}

plba_norm_core <- function(rt,A,b,t0,mean_v, sd_v,posdrift=TRUE, robust = FALSE, nn) {
  if (robust) { # robust == TRUE uses robust versions of the normal distributions
    pnorm1 <- pnormP
    dnorm1 <- dnormP
  } else {
    pnorm1 <- pnorm
    dnorm1 <- dnorm  
  }

  rt <- rem_t0(rt, t0)
  if (posdrift) denom <- pmax(pnorm1(mean_v/sd_v),1e-10) else denom <- 1
  
  if (any(A<1e-10, na.rm = TRUE)) {
    # for A<1e-10 save results in out_A
    A_small <- A<1e-10
    out_A <- pmin(1, pmax(0, (pnorm1(b[A_small]/rt[A_small],mean=mean_v[A_small],sd=sd_v[A_small],lower.tail=FALSE))/denom[A_small], na.rm=TRUE))

    # calculate other results into out_o
    zs <- rt[!A_small]*sd_v[!A_small]
    zu <- rt[!A_small]*mean_v[!A_small]
    chiminuszu <- b[!A_small]-zu
    xx <- chiminuszu-A[!A_small]
    chizu <- chiminuszu/zs
    chizumax <- xx/zs
    tmp1 <- zs*(dnorm1(chizumax)-dnorm1(chizu))
    tmp2 <- xx*pnorm1(chizumax)-chiminuszu*pnorm1(chizu)
    out_o <- pmin(pmax(0,(1+(tmp1+tmp2)/A[!A_small])/denom[!A_small]), 1)
    
    # combine out_A and out_o
    out <- numeric(nn)
    out[!A_small] <- out_o
    out[A_small] <- out_A
    return(out)
  } else {
    zs <- rt*sd_v
    zu <- rt*mean_v
    chiminuszu <- b-zu
    xx <- chiminuszu-A
    chizu <- chiminuszu/zs
    chizumax <- xx/zs
    tmp1 <- zs*(dnorm1(chizumax)-dnorm1(chizu))
    tmp2 <- xx*pnorm1(chizumax)-chiminuszu*pnorm1(chizu)
    return(pmin(pmax(0,(1+(tmp1+tmp2)/A)/denom, na.rm=TRUE), 1))
  }
}

#' @rdname single-LBA
#' @export rlba_norm
rlba_norm <- function(n,A,b,t0,mean_v, sd_v, st0=0,posdrift=TRUE) {
  #check_single_arg(n, A, b, t0, st0)
  check_single_arg(n)
  if (any(b < A)) stop(error_message_b_smaller_A)
  n_v <- max(length(mean_v), length(sd_v))
  if (posdrift) drifts <- matrix(rtnorm(n=n*n_v, mean=mean_v, sd=sd_v, lower=0),ncol=n_v,byrow=TRUE)  
  else drifts <- matrix(rnorm(n=n*n_v, mean=mean_v, sd=sd_v),ncol=n_v,byrow=TRUE)
  make_r(drifts=drifts, n=n, b=b,A=A, n_v=n_v, t0=t0, st0=st0)
}


####### Gamma:

#' @rdname single-LBA
#' @export dlba_gamma
dlba_gamma <- function(rt,A,b,t0,shape_v,rate_v, scale_v) {
  
  if (!missing(rate_v) && !missing(scale_v)) stop("specify 'rate_v' or 'scale_v', but not both")
  if (missing(rate_v)) rate_v <- 1/scale_v
  
  check_vector(rt, A, b=b, t0, shape_v, rate_v)
  
  nn <- length(rt)
  A <- rep(A, length.out = nn)
  b <- rep(b, length.out = nn)
  t0 <- rep(t0, length.out = nn)
  shape_v <- rep(shape_v, length.out = nn)
  rate_v <- rep(rate_v, length.out = nn)
  if (any(b < A)) stop(error_message_b_smaller_A)
  
  dlba_gamma_core(rt=rt,A=A,b=b,t0=t0, shape_v=shape_v, rate_v=rate_v, nn=nn)
  
}


dlba_gamma_core <- function(rt,A,b,t0,shape_v, rate_v, nn) {
  rt <- rem_t0(rt, t0)
  min <- (b-A)/rt
  max <- b/rt
  
  
  if (any(A<1e-10, na.rm = TRUE)) {
    # for A<1e-10 save results in out_A
    A_small<- A<1e-10
    
    out_A<-  pmax(0, (b[A_small]/rt[A_small]^2)*(dgamma(b[A_small]/rt[A_small],shape=shape_v[A_small],rate=rate_v[A_small])), na.rm = TRUE)
    
    min <- (b[!A_small]-A[!A_small])/rt[!A_small]
    max <- b[!A_small]/rt[!A_small]
    
    Gmax <- pgamma(max, shape_v[!A_small], rate=rate_v[!A_small])
    Gmin <- pgamma(min, shape_v[!A_small], rate=rate_v[!A_small])
    Gmax2 <- pgamma(max, (shape_v[!A_small]+1), rate=rate_v[!A_small])
    Gmin2 <- pgamma(min, (shape_v[!A_small]+1), rate=rate_v[!A_small])
    zgamma <- ( ((Gmax2-Gmin2)*gamma(shape_v[!A_small]+1))/((Gmax-Gmin)*rate_v[!A_small]*gamma(shape_v[!A_small])) )
    
    diffG <- function(rt,point,shape_v, rate_v) {
      (-point/(rt^2))*dgamma(point/rt,shape_v[!A_small],rate = rate_v)
    } #NB:point refers to the constants b OR b-A.
    u <- (Gmax2-Gmin2)
    v <- (Gmax-Gmin)
    udash <- (diffG(rt[!A_small], b[!A_small], shape_v[!A_small]+1, rate_v[!A_small])- diffG(rt[!A_small], (b[!A_small]-A[!A_small]), shape_v[!A_small]+1, rate_v[!A_small]))
    vdash <- (diffG(rt[!A_small], b[!A_small], shape_v[!A_small], rate_v[!A_small])- diffG(rt[!A_small], (b[!A_small]-A[!A_small]), shape_v[!A_small], rate_v[!A_small]))
    const <- gamma(shape_v[!A_small]+1)/(rate_v[!A_small]*gamma(shape_v[!A_small]))
    diffzgamma <- ((udash*v - vdash*u)/(v^2))*const #quotient rule
    term1 <- (Gmax - Gmin)*(zgamma + (rt*diffzgamma))
    term2 <- diffG(rt,b,shape_v[!A_small],rate_v[!A_small])*((zgamma*rt)-b)
    term3 <- diffG(rt,(b-A),shape_v[!A_small],rate_v[!A_small])*(b-A-(zgamma*rt))
    out_o <- ((term1+term2+term3)/A)
    out_o[!is.finite(out_o)] <- 0 # Set NaN or -Inf or Inf to pdf=0
    
    out<- numeric(nn)
    out[!A_small] <- out_o
    out[A_small] <- out_A
    
    return(pmax(0, out))
  } else{
    
    Gmax <- pgamma(max, shape_v, rate=rate_v)
    Gmin <- pgamma(min, shape_v, rate=rate_v)
    Gmax2 <- pgamma(max, (shape_v+1), rate=rate_v)
    Gmin2 <- pgamma(min, (shape_v+1), rate=rate_v)
    zgamma <- ( ((Gmax2-Gmin2)*gamma(shape_v+1))/((Gmax-Gmin)*rate_v*gamma(shape_v)) )
    
    diffG <- function(rt,point,shape_v, rate_v) {
      (-point/(rt^2))*dgamma(point/rt,shape_v,rate = rate_v)
    } #NB:point refers to the constants b OR b-A.
    u <- (Gmax2-Gmin2)
    v <- (Gmax-Gmin)
    udash <- (diffG(rt, b, shape_v+1, rate_v)- diffG(rt, (b-A), shape_v+1, rate_v))
    vdash <- (diffG(rt, b, shape_v, rate_v)- diffG(rt, (b-A), shape_v, rate_v))
    const <- gamma(shape_v+1)/(rate_v*gamma(shape_v))
    diffzgamma <- ((udash*v - vdash*u)/(v^2))*const #quotient rule
    term1 <- (Gmax - Gmin)*(zgamma + (rt*diffzgamma))
    term2 <- diffG(rt,b,shape_v,rate_v)*((zgamma*rt)-b)
    term3 <- diffG(rt,(b-A),shape_v,rate_v)*(b-A-(zgamma*rt))
    out.value <- ((term1+term2+term3)/A)
    out.value[!is.finite(out.value)] <- 0 # Set NaN or -Inf or Inf to pdf=0
    return(pmax(0, out.value))
  }
}


#' @rdname single-LBA
#' @export plba_gamma  
plba_gamma <- function(rt,A,b,t0,shape_v, rate_v, scale_v) {
  if (!missing(rate_v) && !missing(scale_v)) stop("specify 'rate_v' or 'scale_v', but not both")
  if (missing(rate_v)) rate_v <- 1/scale_v
  
  check_vector(rt, A, b=b, t0, shape_v, rate_v)
  
  nn <- length(rt)
  A <- rep(A, length.out = nn)
  b <- rep(b, length.out = nn)
  t0 <- rep(t0, length.out = nn)
  shape_v <- rep(shape_v, length.out = nn)
  rate_v <- rep(rate_v, length.out = nn)
  if (any(b < A)) stop(error_message_b_smaller_A)
  
  plba_gamma_core(rt=rt,A=A,b=b,t0=t0,shape_v=shape_v, rate_v=rate_v, nn=nn)
}

plba_gamma_core <- function(rt,A,b,t0,shape_v, rate_v, nn) {
  
  rt <- rem_t0(rt, t0)
  if (any(A<1e-10, na.rm = TRUE)) {
    # for A<1e-10 save results in out_A
    A_small<- A<1e-10
    
    out_A <- pmin(1, pmax(0, (pgamma(b[A_small]/rt[A_small],shape=shape_v[A_small],rate=rate_v[A_small],lower.tail=FALSE)), na.rm=TRUE))
    
    min <- (b[!A_small]-A[!A_small])/rt[!A_small]
    max <- b[!A_small]/rt[!A_small]
    Gmax <- pgamma(max, shape_v[!A_small], rate=rate_v[!A_small])
    Gmin <- pgamma(min, shape_v[!A_small], rate=rate_v[!A_small])
    Gmax2 <- pgamma(max, (shape_v[!A_small]+1), rate=rate_v[!A_small])
    Gmin2 <- pgamma(min, (shape_v[!A_small]+1), rate=rate_v[!A_small])
    zgamma <- ((Gmax2-Gmin2)*gamma(shape_v[!A_small]+1))/((Gmax-Gmin)*rate_v[!A_small]*gamma(shape_v[!A_small]))
    
    term1 <- ((rt*zgamma) - b)/A
    term2 <- (b-A-(rt*zgamma))/A
    pmax <- pgamma(max, shape_v[!A_small], rate = rate_v[!A_small])
    pmin <- pgamma(min, shape_v[!A_small], rate = rate_v[!A_small])
    out_o <- (1 + pmax*term1 + pmin*term2)
    out_o[rt==Inf] <- 1 # term1=Inf and term2=-Inf cancel in this case
    out_o[!is.finite(out_o)] <- 0 # Set NaN or -Inf to CDF=0
    
    
    out<- numeric(nn)
    out[!A_small] <- out_o
    out[A_small] <- out_A
    return(pmin(pmax(0, out), 1))
    
  } else {
    min <- (b-A)/rt
    max <- b/rt
    Gmax <- pgamma(max, shape_v, rate=rate_v)
    Gmin <- pgamma(min, shape_v, rate=rate_v)
    Gmax2 <- pgamma(max, (shape_v+1), rate=rate_v)
    Gmin2 <- pgamma(min, (shape_v+1), rate=rate_v)
    zgamma <- ((Gmax2-Gmin2)*gamma(shape_v+1))/((Gmax-Gmin)*rate_v*gamma(shape_v)) 
    
    term1 <- ((rt*zgamma) - b)/A
    term2 <- (b-A-(rt*zgamma))/A
    pmax <- pgamma(max, shape_v, rate = rate_v)
    pmin <- pgamma(min, shape_v, rate = rate_v)
    out.value <- (1 + pmax*term1 + pmin*term2)
    out.value[rt==Inf] <- 1 # term1=Inf and term2=-Inf cancel in this case
    out.value[!is.finite(out.value)] <- 0 # Set NaN or -Inf to CDF=0
    return(pmin(pmax(0, out.value), 1))
  }
}

#' @rdname single-LBA
#' @export rlba_gamma
rlba_gamma <- function(n,A,b,t0,shape_v, rate_v, scale_v, st0=0) {
  check_single_arg(n)
  if (any(b < A)) stop(error_message_b_smaller_A)
  if (!missing(rate_v) && !missing(scale_v)) stop("specify 'rate_v' or 'scale_v', but not both")
  if (missing(rate_v)) rate_v <- 1/scale_v
  n_v <- max(length(shape_v), length(rate_v))  
  drifts <- matrix(rgamma(n=n*n_v,shape = shape_v,rate = rate_v),ncol=n_v,byrow=TRUE)
  make_r(drifts=drifts, n=n, b=b,A=A, n_v = n_v, t0=t0, st0=st0)
}


####### Frechet:

#' @rdname single-LBA
#' @export dlba_frechet
dlba_frechet <- function(rt,A,b,t0,shape_v, scale_v) {

  check_vector(rt, A, b, t0, shape_v, scale_v)
  nn <- length(rt)
  A <- rep(A, length.out = nn)
  b <- rep(b, length.out = nn)
  t0 <- rep(t0, length.out = nn)
  shape_v <- rep(shape_v, length.out = nn)
  scale_v <- rep(scale_v, length.out = nn)
  if (any(b < A)) stop(error_message_b_smaller_A)
  
  dlba_frechet_core(rt=rt,A=A,b=b,t0=t0,shape_v=shape_v, scale_v=scale_v, nn=nn)
}

dlba_frechet_core <- function(rt,A,b,t0,shape_v, scale_v, nn) {
  
  rt <- rem_t0(rt, t0)
  
  ps <- cbind(b, b-A, scale_v,shape_v)
  ps_below_zero <- apply(ps, 1, function(x) any(x <= 0))
  
  # rt <- pmax(rt,0) #not needed, see rem_t0
  t_old <- rt
  
  out <- numeric(nn)
  
  if (sum(!ps_below_zero) > 0) {
    rt <- rt[!ps_below_zero]
    A <- A[!ps_below_zero]
    b <- b[!ps_below_zero]
    t0 <- t0[!ps_below_zero]
    shape_v <- shape_v[!ps_below_zero]
    scale_v <- scale_v[!ps_below_zero]
  
    min <- (b-A)/rt
    max <- b/rt
    Gmax <- pfrechet(max, loc=0, scale=scale_v, shape=shape_v)
    Gmin <- pfrechet(min, loc=0, scale=scale_v, shape=shape_v)
    D <- Gmax - Gmin
    gam <- gamma_inc(1-(1/shape_v), (1/scale_v*max)^(-shape_v))-gamma_inc(1-(1/shape_v), (1/scale_v*min)^(-shape_v))
    zfrechet <- gam/(1/scale_v*D)
    diffG1 <- ((-b/(rt^2))*dfrechet(b/rt, loc=0, scale=scale_v, shape=shape_v))
    diffG2 <- ((-(b-A)/(rt^2))*dfrechet((b-A)/rt, loc=0, scale=scale_v, shape=shape_v))    
    diffD <- diffG1 - diffG2    
    diffgam <- (-shape_v*(((1/scale_v*b)^(-shape_v+1))/(rt^(-shape_v+2)))*exp(-(1/scale_v*b/rt)^(-shape_v))) - (-shape_v*(((1/scale_v*(b-A))^(-shape_v+1))/(rt^(-shape_v+2)))*exp(-(1/scale_v*(b-A)/rt)^(-shape_v)))
    diffzfrechet <- ((1/scale_v)^(-1))*(((-D^(-2))*diffD)*gam + (diffgam*(D^(-1))))
    term1 <- (Gmax - Gmin)*(zfrechet + (rt*diffzfrechet))
    term2 <- diffG1*((zfrechet*rt)-b)
    term3 <- diffG2*(b-A-(zfrechet*rt))
    out.value <- ((term1+term2+term3)/A)
    out.value[!is.finite(out.value)] <- 0 # Set NaN or -Inf or Inf to pdf=0
    out[!ps_below_zero] <- out.value
  }
  return(pmax(0, out))
}

#' @rdname single-LBA
#' @export plba_frechet
plba_frechet <- function(rt,A,b,t0,shape_v, scale_v) {
  check_vector(rt, A, b, t0, shape_v, scale_v)
  nn <- length(rt)
  A <- rep(A, length.out = nn)
  b <- rep(b, length.out = nn)
  t0 <- rep(t0, length.out = nn)
  shape_v <- rep(shape_v, length.out = nn)
  scale_v <- rep(scale_v, length.out = nn)
  if (any(b < A)) stop(error_message_b_smaller_A)
  
  plba_frechet_core(rt=rt,A=A,b=b,t0=t0,shape_v=shape_v, scale_v=scale_v, nn=nn)
}

plba_frechet_core <- function(rt,A,b,t0,shape_v, scale_v, nn) {  
  rt <- rem_t0(rt, t0)
  
  ps <- cbind(b, b-A, scale_v,shape_v)
  ps_below_zero <- apply(ps, 1, function(x) any(x <= 0))
  
  # rt <- pmax(rt,0) #not needed, see rem_t0
  t_old <- rt
  
  out <- numeric(nn)
  
  if (sum(!ps_below_zero) > 0) {
    
    rt <- rt[!ps_below_zero]
    A <- A[!ps_below_zero]
    b <- b[!ps_below_zero]
    t0 <- t0[!ps_below_zero]
    shape_v <- shape_v[!ps_below_zero]
    scale_v <- scale_v[!ps_below_zero]
    
    # rt <- pmax(rt,0) #not needed, see rem_t0
    min <- (b-A)/rt
    max <- b/rt
    pmax <- pfrechet(max, loc=0, scale=scale_v, shape=shape_v)
    pmin <- pfrechet(min, loc=0, scale=scale_v, shape=shape_v)
    zfrechet <- (gamma_inc(1-(1/shape_v),(1/scale_v*max)^(-shape_v))-gamma_inc(1-(1/shape_v),(1/scale_v*min)^(-shape_v)))/(1/scale_v*(pmax-pmin))    
    term1 <- ((rt*zfrechet) - b)/A
    term2 <- (b-A-(rt*zfrechet))/A 
    out.value <- (1 + pmax*term1 + pmin*term2)
    out.value[rt==Inf] <- 1 # term1=Inf and term2=-Inf cancel in this case
    out.value[!is.finite(out.value)] <- 0 # Set NaN or -Inf to CDF=0
    out[!ps_below_zero] <- out.value
  }
  
  return(pmin(pmax(0, out), 1))
}


#' @rdname single-LBA
#' @export rlba_frechet
rlba_frechet <- function(n,A,b,t0,shape_v, scale_v,st0=0){
  check_single_arg(n)
  if (any(b < A)) stop(error_message_b_smaller_A)
  n_v <- max(length(shape_v), length(scale_v))
  drifts <- matrix(rfrechet(n=n*n_v, loc=0, scale=scale_v, shape=shape_v),ncol=n_v,byrow=TRUE)
  
  make_r(drifts=drifts, n=n, b=b,A=A, n_v=n_v, t0=t0, st0=st0)
}


####### Log-Normal:

#' @rdname single-LBA
#' @export dlba_lnorm
dlba_lnorm <- function(rt,A,b,t0,meanlog_v, sdlog_v, robust = FALSE) {
  check_vector(rt, A, b, t0, meanlog_v, sdlog_v)
  nn <- length(rt)
  A <- rep(A, length.out = nn)
  b <- rep(b, length.out = nn)
  t0 <- rep(t0, length.out = nn)
  meanlog_v <- rep(meanlog_v, length.out = nn)
  sdlog_v <- rep(sdlog_v, length.out = nn)
  if (any(b < A)) stop(error_message_b_smaller_A)
  
  dlba_lnorm_core(rt=rt,A=A,b=b,t0=t0,meanlog_v=meanlog_v, sdlog_v=sdlog_v, robust = robust, nn=nn)
}

dlba_lnorm_core <- function(rt,A,b,t0,meanlog_v, sdlog_v, robust=FALSE, nn) {
  if (robust) { # robust == TRUE uses robust versions of the normal distributions
    pnorm1 <- pnormP
    dnorm1 <- dnormP
  } else {
    pnorm1 <- pnorm
    dnorm1 <- dnorm  
  }
  
  rt <- rem_t0(rt, t0)
  
  if (any(A<1e-10, na.rm = TRUE)) {
    # for A<1e-10 save results in out_A
    A_small<- A<1e-10
    out_A <- pmax(0, (b[A_small]/rt[A_small]^2)*(dlnorm(b[A_small]/rt[A_small],meanlog_v[A_small],sdlog=sdlog_v[A_small])), na.rm = TRUE)
    
    # calculate other results into out_o
    #Should there also be a check that A_small has at least one FALSE to bother running this code? -Angus; Probably not necessary, Henrik.
    min <- (b-A)/rt
    max <- b/rt
    
    zlognorm <- (exp(meanlog_v[!A_small]+(sdlog_v[!A_small]^2)/2)*(pnorm1((log(max)-meanlog_v[!A_small]-(sdlog_v[!A_small]^2))/sdlog_v[!A_small])-pnorm1((log(max)-meanlog_v[!A_small]-(sdlog_v[!A_small]^2))/sdlog_v[!A_small]))) / (pnorm1((log(max)-meanlog_v[!A_small])/sdlog_v[!A_small])-pnorm1((log(max)-meanlog_v[!A_small])/sdlog_v[!A_small]))
    Gmax <- plnorm(max,meanlog=meanlog_v[!A_small],sdlog=sdlog_v[!A_small])
    Gmin <- plnorm(max,meanlog=meanlog_v[!A_small],sdlog=sdlog_v[!A_small])
    
    u <- (pnorm1((log(max)-meanlog_v[!A_small]-(sdlog_v[!A_small])^2)/sdlog_v[!A_small])-pnorm1((log(max)-meanlog_v[!A_small]-(sdlog_v[!A_small])^2)/sdlog_v[!A_small]))
    v <- (pnorm1((log(max)-meanlog_v[!A_small])/sdlog_v[!A_small])-pnorm1((log(max)-meanlog_v[!A_small])/sdlog_v[!A_small]))
    
    udash <- (((-1/(sdlog_v[!A_small]*rt[!A_small]))*dnorm1((log(b[!A_small]/rt[!A_small])-meanlog_v[!A_small]-(sdlog_v[!A_small])^2)/sdlog_v[!A_small])) - ((-1/(sdlog_v[!A_small]*rt[!A_small]))*dnorm1((log((b[!A_small]-A[!A_small])/rt[!A_small])-meanlog_v[!A_small]-(sdlog_v[!A_small])^2)/sdlog_v[!A_small])))
    vdash <- (((-1/(sdlog_v[!A_small]*rt[!A_small]))*dnorm1((log(b[!A_small]/rt[!A_small])-meanlog_v[!A_small])/sdlog_v[!A_small])) - ((-1/(sdlog_v[!A_small]*rt[!A_small]))*dnorm1((log((b[!A_small]-A[!A_small])/rt[!A_small])-meanlog_v[!A_small])/sdlog_v[!A_small])))
    const <- exp(meanlog_v[!A_small]+((sdlog_v[!A_small])^2)/2)
    
    diffzlognorm <- ((udash*v - vdash*u)/(v^2))*const #quotient rule
    term1 <- (Gmax - Gmin)*(zlognorm + (rt[!A_small]*diffzlognorm))
    term2 <- ((-b[!A_small]/(rt[!A_small]^2))*dlnorm(b[!A_small]/rt[!A_small],meanlog=meanlog_v[!A_small],sdlog=sdlog_v[!A_small]))*((zlognorm*rt[!A_small])-b[!A_small])
    term3 <- (b[!A_small]-A[!A_small]-(zlognorm*rt[!A_small]))*((-(b[!A_small]-A[!A_small])/(rt[!A_small]^2))*dlnorm((b[!A_small]-A[!A_small])/rt[!A_small],meanlog=meanlog_v[!A_small],sdlog=sdlog_v[!A_small]))
    out_o<- ((term1+term2+term3)/A[!A_small])
    out_o[!is.finite(out_o)] <- 0 # Set NaN or -Inf or Inf to pdf=0
    
    out<- numeric(nn)
    out[!A_small] <- out_o
    out[A_small] <- out_A
    
    return(pmax(0, out))
    
  } else{
    
    min <- (b-A)/rt
    max <- b/rt
    
    zlognorm <- (exp(meanlog_v+(sdlog_v^2)/2)*(pnorm1((log(max)-meanlog_v-(sdlog_v^2))/sdlog_v)-pnorm1((log(min)-meanlog_v-(sdlog_v^2))/sdlog_v))) / (pnorm1((log(max)-meanlog_v)/sdlog_v)-pnorm1((log(min)-meanlog_v)/sdlog_v))
    Gmax <- plnorm(max,meanlog=meanlog_v,sdlog=sdlog_v) 
    Gmin <- plnorm(min,meanlog=meanlog_v,sdlog=sdlog_v)
    
    u <- (pnorm1((log(max)-meanlog_v-(sdlog_v)^2)/sdlog_v)-pnorm1((log(min)-meanlog_v-(sdlog_v)^2)/sdlog_v))
    v <- (pnorm1((log(max)-meanlog_v)/sdlog_v)-pnorm1((log(min)-meanlog_v)/sdlog_v))
    
    udash <- (((-1/(sdlog_v*rt))*dnorm1((log(b/rt)-meanlog_v-(sdlog_v)^2)/sdlog_v)) - ((-1/(sdlog_v*rt))*dnorm1((log((b-A)/rt)-meanlog_v-(sdlog_v)^2)/sdlog_v)))
    vdash <- (((-1/(sdlog_v*rt))*dnorm1((log(b/rt)-meanlog_v)/sdlog_v)) - ((-1/(sdlog_v*rt))*dnorm1((log((b-A)/rt)-meanlog_v)/sdlog_v)))
    const <- exp(meanlog_v+((sdlog_v)^2)/2)
    
    diffzlognorm <- ((udash*v - vdash*u)/(v^2))*const #quotient rule
    term1 <- (Gmax - Gmin)*(zlognorm + (rt*diffzlognorm))
    term2 <- ((-b/(rt^2))*dlnorm(b/rt,meanlog=meanlog_v,sdlog=sdlog_v))*((zlognorm*rt)-b)
    term3 <- (b-A-(zlognorm*rt))*((-(b-A)/(rt^2))*dlnorm((b-A)/rt,meanlog=meanlog_v,sdlog=sdlog_v))
    out.value <- ((term1+term2+term3)/A)
    out.value[!is.finite(out.value)] <- 0 # Set NaN or -Inf or Inf to pdf=0
    return(pmax(0, out.value))
  }
}

#' @rdname single-LBA
#' @export plba_lnorm
plba_lnorm <- function(rt,A,b,t0,meanlog_v, sdlog_v, robust = FALSE) {
  check_vector(rt, A, b, t0, meanlog_v, sdlog_v)
  nn <- length(rt)
  A <- rep(A, length.out = nn)
  b <- rep(b, length.out = nn)
  t0 <- rep(t0, length.out = nn)
  meanlog_v <- rep(meanlog_v, length.out = nn)
  sdlog_v <- rep(sdlog_v, length.out = nn)
  if (any(b < A)) stop(error_message_b_smaller_A)
  
  plba_lnorm_core(rt=rt,A=A,b=b,t0=t0,meanlog_v=meanlog_v, sdlog_v=sdlog_v, robust=robust, nn=nn)
}

plba_lnorm_core <- function(rt,A,b,t0,meanlog_v, sdlog_v, robust = FALSE, nn) {
  if (robust) { # robust == TRUE uses robust versions of the normal distributions
    pnorm1 <- pnormP
  } else {
    pnorm1 <- pnorm 
  }
  
  if (any(A<1e-10, na.rm = TRUE)) {
    # for A<1e-10 save results in out_A
    A_small<- A<1e-10

    #Pretty sure it is log(b[A_small]/rt[A_small]), not b[A_small]/log(rt[A_small])- Angus
    #Should this be lower.tail=TRUE?
    out_A <- pmin(1, pmax(0, (plnorm(b[A_small]/rt[A_small],meanlog=meanlog_v[A_small],sdlog=sdlog_v[A_small],lower.tail=FALSE)), na.rm=TRUE))
    
    pmin(1, pmax(0, (plnorm(b[A_small]/rt[A_small],meanlog=meanlog_v[A_small],sdlog=sdlog_v[A_small],lower.tail=FALSE)), na.rm=TRUE))

    min <- (b[!A_small]-A[!A_small])/rt[!A_small]
    max <- b[!A_small]/rt[!A_small]
    zlognorm <- (exp(meanlog_v[!A_small]+(sdlog_v[!A_small]^2)/2)*(pnorm1((log(max)-meanlog_v[!A_small]-(sdlog_v[!A_small]^2))/sdlog_v[!A_small])-pnorm1((log(max)-meanlog_v[!A_small]-(sdlog_v[!A_small]^2))/sdlog_v[!A_small]))) / (pnorm1((log(max)-meanlog_v[!A_small])/sdlog_v[!A_small])-pnorm1((log(max)-meanlog_v[!A_small])/sdlog_v[!A_small]))
    term1 <- ((rt[!A_small]*zlognorm) - b[!A_small])/A[!A_small]
    term2 <- (b[!A_small]-A[!A_small]-(rt[!A_small]*zlognorm))/A [!A_small]
    pmax <- plnorm(max, meanlog=meanlog_v[!A_small], sdlog=sdlog_v[!A_small])
    pmin <- plnorm(max, meanlog=meanlog_v[!A_small], sdlog=sdlog_v[!A_small])
    out_o <- (1 + pmax*term1 + pmin*term2)

    #not sure about this next line-Angus
    out_o[rt[!A_small]==Inf] <- 1 # term1=Inf and term2=-Inf cancel in this case
    out_o[!is.finite(out_o)] <- 0 # Set NaN or -Inf to CDF=0

    out<- numeric(nn)
    out[!A_small] <- out_o
    out[A_small] <- out_A
    return(pmin(pmax(0, out), 1))

  }else{
    
    
    rt <- rem_t0(rt, t0)
    min <- (b-A)/rt
    max <- b/rt
    zlognorm <- (exp(meanlog_v+(sdlog_v^2)/2)*(pnorm1((log(max)-meanlog_v-(sdlog_v^2))/sdlog_v)-pnorm1((log(min)-meanlog_v-(sdlog_v^2))/sdlog_v))) / (pnorm1((log(max)-meanlog_v)/sdlog_v)-pnorm1((log(min)-meanlog_v)/sdlog_v))
    term1 <- ((rt*zlognorm) - b)/A
    term2 <- (b-A-(rt*zlognorm))/A 
    pmax <- plnorm(max, meanlog=meanlog_v, sdlog=sdlog_v) 
    pmin <- plnorm(min, meanlog=meanlog_v, sdlog=sdlog_v)
    out.value <- (1 + pmax*term1 + pmin*term2)
    out.value[rt==Inf] <- 1 # term1=Inf and term2=-Inf cancel in this case
    out.value[!is.finite(out.value)] <- 0 # Set NaN or -Inf to CDF=0
    return(pmin(pmax(0, out.value), 1))
  }
}

#' @rdname single-LBA
#' @export rlba_lnorm
rlba_lnorm <- function(n,A,b,t0,meanlog_v, sdlog_v, st0=0){
  check_single_arg(n)
  if (any(b < A)) stop(error_message_b_smaller_A)
  n_v <- max(length(meanlog_v), length(sdlog_v))
  drifts=matrix(rlnorm(n=n*n_v,meanlog = meanlog_v,sdlog=sdlog_v),ncol=n_v,byrow=TRUE)
  make_r(drifts=drifts, n=n, b=b, A=A, n_v=n_v, t0=t0, st0=st0)
}

context("LBA race functions= f*(1-F)")

test_that("dLBA with norm and posdrift=FALSE works as expected", {
  rt <- 2
  vs <- c(0.8, 0.4)
  sd <- 0.2
  
  f <- dlba_norm(rt, A=0.5, b=1, t0 = 0.5, mean_v=vs[1], sd_v=sd, posdrift = FALSE)
  F <- plba_norm(rt, A=0.5, b=1, t0 = 0.5, mean_v=vs[2], sd_v=sd, posdrift = FALSE)
  
  expect_equivalent(n1PDF(rt, A=0.5, b=1, t0 = 0.5, mean_v=vs, sd_v=sd, 
                          args.dist = list(posdrift = FALSE)), f*(1-F))
})

test_that("dLBA with lnorm works as expected", {
  rt <- 2
  vs <- c(0.8, 0.4)
  sd <- 1
  
  f <- dlba_lnorm(rt, A=0.5, b=1, t0 = 0.5, meanlog_v = vs[1], sdlog_v = sd)
  
  F <- plba_lnorm(rt, A=0.5, b=1, t0 = 0.5, meanlog_v = vs[2], sdlog_v = sd)
  
  expect_equivalent(f*(1-F), 
                    n1PDF(rt, A=0.5, b=1, t0 = 0.5,distribution = "lnorm", 
                          meanlog_v = vs, sdlog_v = sd))
})

test_that("dLBA with gamma works as expected", {
  rt <- 2
  vs <- c(1, 1.2)
  sd <- 1
  
  f <- dlba_gamma(rt, A=0.5, b=1, t0 = 0.5, shape_v = vs[1], scale_v = sd)
  
  F <- plba_gamma(rt, A=0.5, b=1, t0 = 0.5, shape_v = vs[2], scale_v = sd)
  
  expect_equivalent(f*(1-F), 
                    n1PDF(rt, A=0.5, b=1, t0 = 0.5,distribution = "gamma", 
                          shape_v = vs, scale_v = sd))
})

test_that("dLBA with frechet works as expected", {
  rt <- 2
  vs <- c(1, 1.2)
  sd <- 1
  
  f <- dlba_frechet(rt, A=0.5, b=1, t0 = 0.5, shape_v = vs[1], scale_v = sd)
  
  F <- plba_frechet(rt, A=0.5, b=1, t0 = 0.5, shape_v = vs[2], scale_v = sd)
  
  expect_equivalent(f*(1-F), 
                    n1PDF(rt, A=0.5, b=1, t0 = 0.5,distribution = "frechet", 
                          shape_v = vs, scale_v = sd))
})

context("dLBA: Known Bugs")

test_that("dLBA: List and trialwise input for A and b", {
  samples <- 2
  A <- runif(4, 0.3, 0.9)
  b <- A+runif(4, 0, 0.5)
  t0 <- runif(2, 0.1, 0.7)
  v1 <- runif(4, 0.5, 1.5)
  v2 <- runif(4, 0.1, 0.5)
  st0 <- runif(1, 0.1, 0.5)
  r_lba <- rLBA(samples, A=A[1], b=b[1], t0 = t0[1], mean_v=v1[1:2], 
                sd_v=v2[1:2])
  
  p1 <- dLBA(rt = r_lba$rt, response = c(1, 2), 
             A=list(A[1:2],A[3:4]), 
             b=b[1], t0 = t0[1], mean_v=v1[1:2], 
             sd_v=v2[1:2], silent = TRUE)
  p2 <- dLBA(rt = r_lba$rt[1], response = 1, 
             A=list(A[1],A[3]), 
             b=b[1], t0 = t0[1], mean_v=v1[1:2], 
             sd_v=v2[1:2], silent = TRUE)
  p3 <- dLBA(rt = r_lba$rt[2], response = 2, 
             A=list(A[2],A[4]), 
             b=b[1], t0 = t0[1], mean_v=v1[1:2], 
             sd_v=v2[1:2], silent = TRUE)
  expect_identical(p1, c(p2, p3))
  
  p2n1 <- n1PDF(rt = r_lba$rt[1], 
             A=list(A[1],A[3]), 
             b=b[1], t0 = t0[1], mean_v=v1[1:2], 
             sd_v=v2[1:2], silent = TRUE)
  expect_identical(p2, p2n1)
  
  p3n1 <- n1PDF(rt = r_lba$rt[2], 
             A=list(A[4],A[2]), 
             b=b[1], t0 = t0[1], mean_v=v1[2:1], 
             sd_v=v2[2:1], silent = TRUE)
  expect_identical(p3, p3n1)
  
  pb1 <- dLBA(r_lba$rt, c(1, 2), 
              A=A[1:2], 
              b=list(b[1:2],b[3:4]), 
              t0 = t0[1], mean_v=v1[1:2], 
              sd_v=v2[1:2], silent = TRUE)
  pb2 <- dLBA(r_lba$rt[1], 1, A=A[1], 
              b=list(b[1],b[3]), 
              t0 = t0[1], mean_v=v1[1:2], 
              sd_v=v2[1:2], silent = TRUE)
  pb3 <- dLBA(r_lba$rt[2], 2, A=A[2], 
              b = list(b[2],b[4]), 
              t0 = t0[1], 1, mean_v=v1[1:2], 
              sd_v=v2[1:2], silent = TRUE)
  expect_identical(pb1, c(pb2, pb3))
  
  pb2n1 <- n1PDF(rt = r_lba$rt[1], 
                A=A[1], 
                b=list(b[1],b[3]),
                t0 = t0[1], mean_v=v1[1:2], 
                sd_v=v2[1:2], silent = TRUE)
  expect_identical(pb2, pb2n1)
  
  pb3n1 <- n1PDF(rt = r_lba$rt[2], 
                A=A[2], 
                b=list(b[4],b[2]),
                t0 = t0[1], mean_v=v1[2:1], 
                sd_v=v2[2:1], silent = TRUE)
  expect_identical(pb3, pb3n1)
})

context("n1PDF: Known Bugs")

test_that("n1PDF and n1CDF pass arguments correctly", {
  skip_if_not_installed("glba")
  data(bh08, package = "glba")
  bh08 <- bh08[bh08$rt>.180&bh08$rt<2,]
  ny <- dim(bh08)[1]
  
  set.seed(3)
  sddr <- rep(0.2,ny)
  sp <- rep(rnorm(1,.3,.02),ny)
  bound <- rep(rnorm(1,.1,.02),ny)
  nond  <- rep(rnorm(1,.2,.02),ny)
  drift1 <- rep(rnorm(1,.75,.05),ny)
  drift2 <- 1-drift1
  
  parsMat <- matrix(c(sddr,sp,bound,nond,drift1,drift2),ncol=6,nrow=ny)
  o1 <- n1PDF(bh08$rt,A=sp[1],b=bound[1]+sp[1], t0=nond[1], 
              mean_v=c(drift1[1],drift2[1]), sd_v=sddr[1],dist="norm")
  o2 <- n1PDF(bh08$rt,A=sp[1],b=bound[1]+sp[1], t0=nond[1], 
              mean_v=c(drift1[1],drift2[1]), sd_v=sddr[1],dist="norm", 
              args.dist = list(posdrift = TRUE))
  expect_identical(o1, o2)
  o3 <- n1PDF(bh08$rt,A=sp[1],b=bound[1]+sp[1], t0=nond[1], 
              mean_v=c(drift1[1],drift2[1]), sd_v=sddr[1],dist="norm", 
              args.dist = list(posdrift = FALSE))
  expect_false(all(o1 == o3))
  o4 <- n1PDF(bh08$rt,A=sp[1],b=bound[1]+sp[1], t0=nond[1], 
              mean_v=c(drift1[1],drift2[1]), sd_v=sddr[1],dist="norm", 
              args.dist = list(posdrift = FALSE, robust = TRUE))
  expect_false(all(o1 == o4))
  
  c1 <- n1CDF(bh08$rt,A=sp[1],b=bound[1]+sp[1], t0=nond[1], 
              mean_v=c(drift1[1],drift2[1]), sd_v=sddr[1],dist="norm")
  c2 <- n1CDF(bh08$rt,A=sp[1],b=bound[1]+sp[1], t0=nond[1], 
              mean_v=c(drift1[1],drift2[1]), sd_v=sddr[1],dist="norm", 
              args.dist = list(posdrift = TRUE))
  expect_identical(c1, c2)
  c3 <- n1CDF(bh08$rt,A=sp[1],b=bound[1]+sp[1], t0=nond[1], 
              mean_v=c(drift1[1],drift2[1]), sd_v=sddr[1],dist="norm", 
              args.dist = list(posdrift = FALSE))
  expect_false(all(c1 == c3))
  c4 <- n1CDF(bh08$rt,A=sp[1],b=bound[1]+sp[1], t0=nond[1], 
              mean_v=c(drift1[1],drift2[1]), sd_v=sddr[1],dist="norm", 
              args.dist = list(posdrift = FALSE, robust = TRUE))
  expect_false(all(c1 == c4))
  set.seed(NULL)
})

test_that("named parameter vectors do not cause havoc", {
  xx <- rLBA(10, A=0.5, b=1, t0 = 0.5, mean_v=1.2, sd_v=0.2)
  expect_is(n1PDF(xx$rt, A=0.5, b=1, t0 = 0.5, 
                  mean_v=c(1.2, 1.0), sd_v=0.2, 
                  st0 = c(xx = 0.1), silent =TRUE), 
            "numeric")
  expect_is(n1PDF(xx$rt, A=0.5, b=1, t0 = c(aa=0.5), 
                  mean_v=c(1.2, 1.0), sd_v=c(xx=0.2), 
                  silent =TRUE), 
            "numeric")
  expect_is(n1PDF(xx$rt, A=c(xx=0.5), b=c(A = 1), t0 = 0.5, 
                  mean_v=c(1.2, 1.0), sd_v=c(xx=0.2), 
                  silent =TRUE), 
            "numeric")
  expect_is(n1PDF(xx$rt, A=0.5, b=1, t0 = c(aa=0.5), 
                  mean_v=c(1.2, 1.0), sd_v=0.2, st0 = 0.1, 
                  silent =TRUE), 
            "numeric")
  
  expect_is(n1CDF(xx$rt, A=0.5, b=1, t0 = 0.5, 
                  mean_v=c(1.2, 1.0), sd_v=0.2, 
                  st0 = c(xx = 0.1), silent =TRUE), 
            "numeric")
  expect_is(n1CDF(xx$rt, A=0.5, b=1, t0 = c(aa=0.5), 
                  mean_v=c(1.2, 1.0), sd_v=c(xx=0.2), silent =TRUE), 
            "numeric")
  expect_is(n1CDF(xx$rt, A=c(xx=0.5), b=c(A = 1), t0 = 0.5, 
                  mean_v=c(1.2, 1.0), sd_v=c(xx=0.2), silent =TRUE), 
            "numeric")
  expect_is(n1CDF(xx$rt, A=0.5, b=1, t0 = c(aa=0.5), 
                  mean_v=c(1.2, 1.0), sd_v=0.2, st0 = 0.1, silent =TRUE), 
            "numeric")
})


test_that("PDFs and CDFs do not return NaN for A = 0", {
  expect_true(all(is.finite(dlba_norm(rt = c(0, 0.0000001, 0.5),  
                                      A=0, b=1, t0 = 0, 
                                      mean_v=1.2, sd_v=0.2))))
  expect_true(all(is.finite(dlba_gamma(rt = c(0, 0.0000001, 0.5), 
                                       A=0, b=1, t0 = 0, 
                                       shape_v=1.2, rate_v=0.2))))
  expect_true(all(is.finite(dlba_frechet(rt = c(0, 0.0000001, 0.5),  
                                         A=0, b=1, t0 = 0, 
                                         shape_v=1.2, scale_v=0.2))))
  expect_true(all(is.finite(dlba_lnorm(rt = c(0, 0.0000001, 0.5),  
                                       A=0, b=1, t0 = 0, 
                                       meanlog_v = 1.2, sdlog_v = 0.2))))
  
  expect_true(all(is.finite(plba_norm(rt = c(0, 0.0000001, 0.5),  
                                      A=0, b=1, t0 = 0, 
                                      mean_v=1.2, sd_v=0.2))))
  expect_true(all(is.finite(plba_gamma(rt = c(0, 0.0000001, 0.5),  
                                       A=0, b=1, t0 = 0, 
                                       shape_v=1.2, rate_v=0.2))))
  expect_true(all(is.finite(plba_frechet(rt = c(0, 0.0000001, 0.5),  
                                         A=0, b=1, t0 = 0, 
                                         shape_v=1.2, scale_v=0.2))))
  expect_true(all(is.finite(plba_lnorm(rt = c(0, 0.0000001, 0.5),  
                                       A=0, b=1, t0 = 0, 
                                       meanlog_v = 1.2, sdlog_v = 0.2))))
  
})


test_that("LBA-norm: PDF and CDF work with various parameter values", {
  testthat::skip_on_cran()
  rts <- c(0, 0.0000001, 0.5, 1.5, 2)
  seq_parameters <- seq(0, 1, length.out = 5)
  
  for (A in seq_parameters) {
    for (b in seq_parameters) {
      for (t0 in seq_parameters) {
        for (d1 in seq_parameters) {
          for (d2 in c(0.0001, seq_parameters[-1])) {
            expect_true(all(is.finite(
              dlba_norm(
                rt = rts,
                A = A,
                b = (A + b),
                t0 = t0,
                mean_v = d1,
                sd_v = d2
              )
            )),
            info = paste0("A=", A, ", b=", b, ", t0=", t0, 
                          ", mean_v=", d1, ", sd_v=", d2))
            expect_true(all(is.finite(
              dlba_norm(
                rt = rts,
                A = A,
                b = (A + b),
                t0 = t0,
                mean_v = d1,
                sd_v = d2,
                posdrift = FALSE
              )
            )),
            info = paste0("A=", A, ", b=", b, ", t0=", t0, 
                          ", mean_v=", d1, ", sd_v=", d2))
            expect_true(all(is.finite(
              dlba_norm(
                rt = rts,
                A = A,
                b = (A + b),
                t0 = t0,
                mean_v = d1,
                sd_v = d2,
                robust = TRUE
              )
            )),
            info = paste0("A=", A, ", b=", b, ", t0=", t0, 
                          ", mean_v=", d1, ", sd_v=", d2))
            expect_true(all(is.finite(
              dlba_norm(
                rt = rts,
                A = A,
                b = (A + b),
                t0 = t0,
                mean_v = d1,
                sd_v = d2,
                robust = TRUE,
                posdrift = FALSE
              )
            )),
            info = paste0("A=", A, ", b=", b, ", t0=", t0, 
                          ", mean_v=", d1, ", sd_v=", d2))
            
            expect_true(all(is.finite(
              plba_norm(
                rt = rts,
                A = A,
                b = (A + b),
                t0 = t0,
                mean_v = d1,
                sd_v = d2
              )
            )),
            info = paste0("A=", A, ", b=", b, ", t0=", t0, 
                          ", mean_v=", d1, ", sd_v=", d2))
            expect_true(all(is.finite(
              plba_norm(
                rt = rts,
                A = A,
                b = (A + b),
                t0 = t0,
                mean_v = d1,
                sd_v = d2,
                posdrift = FALSE
              )
            )),
            info = paste0("A=", A, ", b=", b, ", t0=", t0, 
                          ", mean_v=", d1, ", sd_v=", d2))
            expect_true(all(is.finite(
              plba_norm(
                rt = rts,
                A = A,
                b = (A + b),
                t0 = t0,
                mean_v = d1,
                sd_v = d2,
                robust = TRUE
              )
            )),
            info = paste0("A=", A, ", b=", b, ", t0=", t0, 
                          ", mean_v=", d1, ", sd_v=", d2))
            expect_true(all(is.finite(
              plba_norm(
                rt = rts,
                A = A,
                b = (A + b),
                t0 = t0,
                mean_v = d1,
                sd_v = d2,
                robust = TRUE,
                posdrift = FALSE
              )
            )),
            info = paste0("A=", A, ", b=", b, ", t0=", t0, 
                          ", mean_v=", d1, ", sd_v=", d2))
          }
        }
      }
    }
  }
  
})

test_that("LBA-gamma: PDF and CDF work with various parameter values", {
  testthat::skip_on_cran()
  rts <- c(0, 0.0000001, 0.5, 1.5, 2)
  seq_parameters <- seq(0, 1, length.out = 5)
  
  for (A in seq_parameters) {
    for (b in seq_parameters) {
      for (t0 in seq_parameters) {
        for (d1 in seq_parameters) {
          for (d2 in c(0.0001, seq_parameters[-1])) {
            suppressWarnings(expect_true(
              all(is.finite(
                dlba_gamma(
                  rt = rts,
                  A = A,
                  b = (A + b),
                  t0 = t0,
                  shape_v = d1,
                  scale_v = d2
                )
              )),
              info = paste0("A=", A, ", b=", b, ", t0=", t0, 
                            ", shape_v=", d1, ", scale_v=", d2)
            ))
            
            suppressWarnings(expect_true(
              all(is.finite(
                plba_gamma(
                  rt = rts,
                  A = A,
                  b = (A + b),
                  t0 = t0,
                  shape_v = d1,
                  scale_v = d2
                )
              )),
              info = paste0("A=", A, ", b=", b, ", t0=", t0, 
                            ", shape_v=", d1, ", scale_v=", d2)
            ))
          }
        }
      }
    }
  }
  
})

test_that("LBA-frechet: PDF and CDF work with various parameter values", {
  testthat::skip_on_cran()
  rts <- c(0, 0.0000001, 0.5, 1.5, 2)
  seq_parameters <- seq(0, 1, length.out = 5)
  
  for (A in seq_parameters) {
    for (b in seq_parameters) {
      for (t0 in seq_parameters) {
        for (d1 in c(0.0001, seq_parameters[-1])) {
          for (d2 in c(0.0001, seq_parameters[-1])) {
            expect_true(all(is.finite(
              dlba_frechet(
                rt = rts,
                A = A,
                b = (A + b),
                t0 = t0,
                shape_v = d1,
                scale_v = d2
              )
            )),
            info = paste0("A=", A, ", b=", b, ", t0=", t0, 
                          ", shape_v=", d1, ", scale_v=", d2))
            
            expect_true(all(is.finite(
              plba_frechet(
                rt = rts,
                A = A,
                b = (A + b),
                t0 = t0,
                shape_v = d1,
                scale_v = d2
              )
            )),
            info = paste0("A=", A, ", b=", b, ", t0=", t0, 
                          ", shape_v=", d1, ", scale_v=", d2))
          }
        }
      }
    }
  }
  
})

test_that("LBA-lnorm: PDF and CDF work with various parameter values", {
  testthat::skip_on_cran()
  rts <- c(0, 0.0000001, 0.5, 1.5, 2)
  seq_parameters <- seq(0, 1, length.out = 5)
  
  for (A in seq_parameters) {
    for (b in seq_parameters) {
      for (t0 in seq_parameters) {
        for (d1 in c(0.0001, seq_parameters[-1])) {
          for (d2 in c(0.0001, seq_parameters[-1])) {
            expect_true(
              all(is.finite(
                dlba_lnorm(
                  rt = rts,
                  A = A,
                  b = (A + b),
                  t0 = t0,
                  meanlog_v = d1,
                  sdlog_v = d2
                )
              )),
              info = paste0("A=", A, ", b=", b, ", t0=", t0, 
                            ", meanlog_v=", d1, ", sdlog_v=", d2)
            )
            expect_true(
              all(is.finite(
                dlba_lnorm(
                  rt = rts,
                  A = A,
                  b = (A + b),
                  t0 = t0,
                  meanlog_v = d1,
                  sdlog_v = d2
                )
              ), robust = TRUE),
              info = paste0("A=", A, ", b=", b, ", t0=", t0, 
                            ", meanlog_v=", d1, ", sdlog_v=", d2)
            )
            
            expect_true(
              all(is.finite(
                plba_lnorm(
                  rt = rts,
                  A = A,
                  b = (A + b),
                  t0 = t0,
                  meanlog_v = d1,
                  sdlog_v = d2
                )
              )),
              info = paste0("A=", A, ", b=", b, ", t0=", t0, 
                            ", meanlog_v=", d1, ", sdlog_v=", d2)
            )
            expect_true(
              all(is.finite(
                plba_lnorm(
                  rt = rts,
                  A = A,
                  b = (A + b),
                  t0 = t0,
                  meanlog_v = d1,
                  sdlog_v = d2,
                  robust = TRUE
                )
              )),
              info = paste0("A=", A, ", b=", b, ", t0=", t0, 
                            ", meanlog_v=", d1, ", sdlog_v=", d2)
            )
          }
        }
      }
    }
  }
  
})

context("glba and rtdists are in agreement")

test_that("glba and rtdists agree", {
  obj <-
    function(rt,pars,loglink,weights) {
      # vectorized loglike function
      # rt: a vector with response times
      # pars: matrix with 4+nrcat parameters on each row to model each rt
      # the drift pars are ordered: the drift for the given response first, the others 
      # after that (order in the remaining drifts does not make a difference)
      for(i in 1:4) if(loglink[i]) pars[,i]=exp(pars[,i])
      ndrift <- dim(pars)[2]-4
      if(ndrift<2) stop("nr of drift pars should at least be two")
      ll <- numeric(length(rt))
      
      ll <- glba:::n1PDF(t=rt-pars[,4], x0max=pars[,2],
                         chi=pars[,2]+pars[,3], sdI=pars[,1], # sdI=0.15, # Scott: I fit chi-x0max.
                         drift=pars[,5:(4+ndrift)])	
      
      # 	return(logl=-sum(log(pmax(weights*ll,1e-10)))) # this has weird effects due to the contaminant model ...
      return(logl=log(weights*ll))
    }
  
  skip_if_not_installed("glba")
  data(bh08, package = "glba")
  # remove extreme RTs
  bh08 <- bh08[bh08$rt>.180&bh08$rt<2,]
  
  ny <- dim(bh08)[1]
  
  set.seed(3)
  sddr <- rep(0.2,ny)
  sp <- rep(rnorm(1,.3,.02),ny)
  bound <- rep(rnorm(1,.1,.02),ny)
  nond  <- rep(rnorm(1,.2,.02),ny)
  drift1 <- rep(rnorm(1,.75,.05),ny)
  drift2 <- 1-drift1
  
  parsMat <- matrix(c(sddr,sp,bound,nond,drift1,drift2),ncol=6,nrow=ny)
  
  head(parsMat)
  
  ll1 <- obj(bh08$rt,parsMat,loglink = c(FALSE,FALSE,FALSE,FALSE),rep(1,ny))
  
  
  ll2 <- log(n1PDF(bh08$rt,A=sp[1],b=bound[1]+sp[1], t0=nond[1], 
                   mean_v=c(drift1[1],drift2[1]), sd_v=sddr[1],
                   dist="norm", args.dist = list(posdrift = FALSE)))
  
  expect_identical(ll1, ll2)
})


test_that("n1PDF works with named lists", {
  rt1 <- rLBA(500, A=0.5, b=1, t0 = 0.5, 
              mean_v=list(a=2.4, b=1.6), sd_v=list(v=1,A=1.2))
  expect_is(sum(log(n1PDF(rt1$rt, A=list(r1=0.5,r2=.5), b=1, t0 = 0.5, 
                          mean_v=list(b=seq(2.0, 2.4, length.out = 500), c=1.6), 
                          sd_v=c(xx=1,hans=1.2)))), 
            "numeric")
  expect_is(sum(log(n1PDF(rt1$rt, A=.5, b=list(r1=0.5,r2=.5), t0 = 0.5, 
                          mean_v=list(b=seq(2.0, 2.4, length.out = 500), c=1.6), 
                          sd_v=c(xx=1,hans=1.2)))), 
            "numeric")
})

test_that("lba_lnorm work with A = 0", {
  A <- 0
  b <- 1  #Can compare to log-normal if b=1
  t0 <- 0
  meanlog_v=0
  sdlog_v=.5
  
  ##########
  set.seed(1)
  check<-rlba_lnorm(1000, A=0, b=1, t0 = 0, meanlog_v=0, sdlog_v=.5)
  rt<-check[,"rt"]
  
  expect_equal(
    sum(log(dlba_lnorm(rt=rt,A=A,b=b,t0=0,meanlog_v = 0,sdlog_v=.5))),
    sum(log(dlnorm(rt,0,.5)))
  )
  
  set.seed(2)
  check<-rlba_lnorm(1000, A=0, b=1, t0 = 0, meanlog_v=.5, sdlog_v=.5)
  rt<-check[,"rt"]
  
  expect_equal(
    sum(log(dlba_lnorm(rt=rt,A=A,b=b,t0=0,meanlog_v = .5,sdlog_v=.5))),
    sum(log(dlnorm(rt,-.5,.5)))  
  )
  
  #x<-  plba_lnorm(rt=rt,A=A,b=b,t0=0,meanlog_v = .5,sdlog_v=.5)- plnorm(rt,-.5,.5)
  expect_equal(
    plba_lnorm(rt=rt,A=A,b=b,t0=0,meanlog_v = .5,sdlog_v=.5),
    plnorm(rt,-.5,.5)
    )
  #CDF is working too
  
  
  #what about b=.5
  set.seed(3)
  check2<-rlba_lnorm(1000, A=0, b=.5, t0 = 0, meanlog_v=0, sdlog_v=.5)
  rt<-check[,"rt"]
  
  expect_equal(
  sum(log(dlba_lnorm(rt=rt,A=A,b=.5,t0=0,meanlog_v = 0,sdlog_v=.5))),
  # [1] -Inf
  sum(log(dlnorm(rt/.5,0,.5)/.5))
  # [1] -261.9191
  )
  
  expect_equal(
    plba_lnorm(rt=rt,A=A,b=.5,t0=0,meanlog_v = 0,sdlog_v=.5),
    plnorm(rt/.5,0,.5)
  )
  
})

test_that("lba_gamma works with A=0", {
  check_gamma <- rlba_gamma(10, A=0.5, b=1, t0 = 0.5, 
                            shape_v=c(1.2, 1), scale_v=c(0.2,0.3))
  rt<-check_gamma[,"rt"]
  expect_equal(
    sum(log(dlba_gamma(rt=rt,A=0.00001, b=1, t0 = 0.5, 
                       shape_v=1.2, scale_v=0.2))),
    sum(log(dlba_gamma(rt=rt,A=0, b=1, t0 = 0.5, 
                       shape_v=1.2, scale_v=0.2)))  
  , tolerance = 0.00001)
  
  expect_equal(
    plba_gamma(rt=rt,A=0.00001, b=1, t0 = 0.5, 
               shape_v=1.2, scale_v=0.2),
    plba_gamma(rt=rt,A=0, b=1, t0 = 0.5, 
               shape_v=1.2, scale_v=0.2)  
  , tolerance = 0.00001)
  
  A <- runif(1, 0.3, 0.9)
  b <- A+runif(1, 0, 0.5)
  t0 <- runif(1, 0.1, 0.7)
  v1 <- runif(2, 0.5, 1.5)
  v2 <- runif(2, 0.1, 0.5)
  
  expect_equal(
    sum(log(dlba_gamma(rt=rt,A=0.00001, b=b, t0 = t0, shape_v=v1, scale_v=v2))),
    sum(log(dlba_gamma(rt=rt,A=0, b=b, t0 = t0, shape_v=v1, scale_v=v2)))  
    , tolerance = 0.00001)
  
  expect_equal(
    sum(log(dlba_gamma(rt=rt,A=0.00001, b=b, t0 = t0, shape_v=v1, scale_v=v2))),
    sum(log(dlba_gamma(rt=rt,A=0, b=b, t0 = t0, shape_v=v1, scale_v=v2)))  
  , tolerance = 0.00001)
  
})

test_that("args.dist is passed through correctly for dLBA, pLBA, qLBA", {
  # see: https://github.com/rtdists/rtdists/issues/7
  d1 <- dLBA(100,1, 10, 100, 0,  
             mean_v=c(3,1), sd_v=c(1,1),
             args.dist = list(posdrift = FALSE))  
  d2 <- dlba_norm(100, 10, 100, 0, 3, 1, posdrift = F, robust = FALSE) * 
    (1-plba_norm(100, 10, 100, 0, 1, 1, posdrift = F, robust = FALSE))
  d3 <- n1PDF(100, 10, 100, 0,  
              mean_v=c(3,1), sd_v=c(1,1), 
              args.dist = list(posdrift = FALSE))
  
  expect_identical(d1, d2)
  expect_identical(d1, d3)
  
  p1 <- pLBA(100,1, 10, 100, 0,  
             mean_v=c(3,1), sd_v=c(1,1),
             args.dist = list(posdrift = FALSE))
  p2 <- n1CDF(100, 10, 100, 0,  
              mean_v=c(3,1), sd_v=c(1,1),
              args.dist = list(posdrift = FALSE))  
  
  expect_identical(p1, p2)
  
  q1 <- qLBA(0.5, 1, 10, 100, 0,  
             mean_v=c(3,1), sd_v=c(1,1), scale_p = TRUE, interval = c(0, 100))
  q2 <- qLBA(0.5, 1, 10, 100, 0,  
             mean_v=c(3,1), sd_v=c(1,1), scale_p = TRUE, interval = c(0, 100), 
             args.dist = list(posdrift = FALSE))
  expect_true(q1 != q2)
  
})

test_that("another args.dist bug (Glen Livingston Jr, 19/11/2018)", {
  N_choices = 3
  N_data = 1000
  
  # Simulated Data--------------------------------------------------------------
  A_actual = 1
  b_actual = 1.4
  t0_actual = 0.3
  v_actual = c(3, 1, 1)
  s_actual = c(1, 0.7, 0.65)
  
  rt1 <- rLBA(N_data, A=A_actual, b=b_actual, t0 = t0_actual, mean_v=v_actual, 
              sd_v=s_actual, posdrift=FALSE)
  
  # Density function------------------------------------------------------------
  
  expect_is(dLBA(rt1$rt,rt1$response, A=A_actual, b=b_actual, t0 = t0_actual, 
                 mean_v=v_actual, sd_v=s_actual,
       args.dist = list(posdrift=FALSE), silent = TRUE), class = "numeric")
})

context("LBA vectorization")

test_that("_norm vectorizes", {
  n <- 10
  x <- rlba_norm(n, A=0.5, b=1, t0 = 0.5, mean_v=c(1.2, 1), sd_v=c(0.2,0.3))
  
  as <- seq(0.2, 0.5, length.out = n)
  bs <- seq(0.5, 1.5, length.out = n)
  t0s <- seq(0, 0.5, length.out = n/2)
  vs <- seq(0.8, 1.2, length.out = 10)
  
  o1 <- plba_norm(x[,"response"], A=as, b=bs, t0 = t0s, mean_v=vs, sd_v=0.2)
  o2 <- mapply(plba_norm, rt = x[,"response"], A = as, b = bs, t0 = t0s, 
               mean_v=vs, MoreArgs = list(sd_v=0.2))
  expect_identical(o1, o2)
  p1 <- dlba_norm(x[,"response"], A=as, b=bs, t0 = t0s, mean_v=1.2, sd_v=vs)
  p2 <- mapply(dlba_norm, rt = x[,"response"], A = as, b = bs, t0 = t0s, 
               sd_v=vs, MoreArgs = list(mean_v=1.2))
  expect_identical(p1, p2)
})

test_that("_norm with small A", {
  n <- 10
  x <- rlba_norm(n, A=0.5, b=1, t0 = 0.5, mean_v=c(1.2, 1), sd_v=c(0.2,0.3))
  
  as <- seq(0.2, 0.5, length.out = n)
  bs <- seq(0.5, 1.5, length.out = n)
  t0s <- seq(0, 0.5, length.out = n/2)
  vs <- seq(0.8, 1.2, length.out = 10)
  
  o1 <- plba_norm(x[,"response"], 
                  A=c(as[1:4], 0.1e-10, 0.5e-10, as[7:10]), 
                  b=bs, t0 = t0s, mean_v=vs, sd_v=0.2)
  o2_a <- plba_norm(x[,"response"][1:4], 
                    A=as[1:4], b=bs, t0 = t0s, 
                    mean_v=vs, sd_v=0.2)
  o2_b <- plba_norm(x[,"response"][5:6], 
                    A=c(0.1e-10, 0.5e-10), 
                    b=bs[5:6], t0 = c(t0s[5], t0s[1]), 
                    mean_v=vs[5:6], sd_v=0.2)
  o2_c <- plba_norm(x[,"response"][7:10], 
                    A=as[7:10], b=bs[7:10], t0 = t0s[2:5], 
                    mean_v=vs[7:10], sd_v=0.2)
  expect_identical(o1, c(o2_a, o2_b, o2_c))
})

test_that("_gamma vectorizes", {
  n <- 10
  x <- rlba_norm(n, A=0.5, b=1, t0 = 0.5, mean_v=c(1.2, 1), sd_v=c(0.2,0.3))
  
  as <- seq(0.2, 0.5, length.out = n)
  bs <- seq(0.5, 1.5, length.out = n)
  t0s <- seq(0, 0.5, length.out = n/2)
  vs <- seq(0.8, 1.2, length.out = 10)
  
  o1 <- dlba_gamma(x[,"response"], A=as, b=bs, t0 = t0s, 
                   shape_v=vs, scale_v=0.2)
  o2 <- mapply(dlba_gamma, rt = x[,"response"], A = as, b = bs, t0 = t0s, 
               shape_v=vs, MoreArgs = list(scale_v=0.2))
  expect_identical(o1, o2)
  p1 <- plba_gamma(x[,"response"], A=as, b=bs, t0 = t0s, 
                   shape_v=1.2, scale_v=vs)
  p2 <- mapply(plba_gamma, rt = x[,"response"], A = as, b = bs, t0 = t0s, 
               scale_v=vs, MoreArgs = list(shape_v=1.2))
  expect_identical(p1, p2)
})

test_that("_frechet vectorizes", {
  n <- 10
  x <- rlba_norm(n, A=0.5, b=1, t0 = 0.5, mean_v=c(1.2, 1), sd_v=c(0.2,0.3))
  
  as <- seq(0.2, 0.5, length.out = n)
  bs <- seq(0.5, 1.5, length.out = n)
  t0s <- seq(0, 0.5, length.out = n/2)
  vs <- seq(0.8, 1.2, length.out = 10)
  
  o1 <- dlba_frechet(x[,"response"], A=as, b=bs, t0 = t0s, 
                     shape_v=vs, scale_v=0.2)
  o2 <- mapply(dlba_frechet, rt = x[,"response"], A = as, b = bs, t0 = t0s, 
               shape_v=vs, MoreArgs = list(scale_v=0.2))
  expect_identical(o1, o2)
  p1 <- plba_frechet(x[,"response"], A=as, b=bs, t0 = t0s, 
                     shape_v=1.2, scale_v=vs)
  p2 <- mapply(plba_frechet, rt = x[,"response"], A = as, b = bs, t0 = t0s, 
               scale_v=vs, MoreArgs = list(shape_v=1.2))
  expect_identical(p1, p2)
})

test_that("_lnorm vectorizes", {
  n <- 10
  x <- rlba_norm(n, A=0.5, b=1, t0 = 0.5, mean_v=c(1.2, 1), sd_v=c(0.2,0.3))
  
  as <- seq(0.2, 0.5, length.out = n)
  bs <- seq(0.5, 1.5, length.out = n)
  t0s <- seq(0, 0.5, length.out = n/2)
  vs <- seq(0.8, 1.2, length.out = 10)
  
  o1 <- dlba_lnorm(x[,"response"], A=as, b=bs, t0 = t0s, 
                   meanlog_v=vs, sdlog_v=0.2)
  o2 <- mapply(dlba_lnorm, rt = x[,"response"], A = as, b = bs, t0 = t0s, 
               meanlog_v=vs, MoreArgs = list(sdlog_v=0.2))
  expect_identical(o1, o2)
  p1 <- plba_lnorm(x[,"response"], A=as, b=bs, t0 = t0s, 
                   meanlog_v=1.2, sdlog_v=vs)
  p2 <- mapply(plba_lnorm, rt = x[,"response"], A = as, b = bs, t0 = t0s, 
               sdlog_v=vs, MoreArgs = list(meanlog_v=1.2))
  expect_identical(p1, p2)
})

context("LBA input")

test_that("_norm input works as they should", {
  expect_error(rlba_norm("10", A=0.5, b=1, t0 = 0.5, 
                         mean_v=c(1.2, 1), sd_v=c(0.2,0.3)))
  expect_is(rlba_norm(10, A=c(0.5, 0.6), b=1, t0 = 0.5, 
                      mean_v=c(1.2, 1), sd_v=c(0.2,0.3)), 
            "matrix")    
  x <- rlba_norm(10, A=0.5, b=1, t0 = 0.5, mean_v=c(1.2, 1), sd_v=c(0.2,0.3))
  expect_error(plba_norm(x[,"response"], 
                         A=as.character(seq(0.2, 0.5, length.out = 10)), 
                         b=0.5, t0 = 0.5, 
                         mean_v=1.2, sd_v=0.2))
  expect_is(plba_norm(x[,"response"], 
                      A=seq(0.2, 0.5, length.out = 10), 
                      b=0.5, t0 = seq(0.5, 0.7, 0.1), 
                      mean_v=1.2, sd_v=0.2), 
            "numeric")
})

test_that("_gamma input works as they should", {
  expect_error(rlba_gamma("10", A=0.5, b=1, t0 = 0.5, 
                          shape_v =c(1.2, 1), scale_v =c(0.2,0.3)), 
               regexp = "numeric and finite")
  expect_is(rlba_gamma(10, A=c(0.5, 0.6), b=1, t0 = 0.5, 
                       shape_v=c(1.2, 1), scale_v=c(0.2,0.3)), 
            "matrix") 
  x <- rlba_gamma(10, A=0.5, b=1, t0 = 0.5, 
                  shape_v=c(1.2, 1), scale_v=c(0.2,0.3))
  expect_error(plba_gamma(x[,"response"], 
                          A=as.character(seq(0.2, 0.5, length.out = 10)), 
                          b=0.5, t0 = 0.5, shape_v=1.2, scale_v=0.2), 
               "numeric")
  expect_is(plba_gamma(x[,"response"], 
                       A=seq(0.2, 0.5, length.out = 10), b=0.5, 
                       t0 = seq(0.5, 0.7, 0.1), 
                       shape_v=1.2, scale_v=0.2), 
            "numeric")
})

test_that("_frechet input works as they should", {
  expect_error(rlba_frechet("10", A=0.5, b=1, t0 = 0.5, 
                            shape_v =c(1.2, 1), scale_v =c(0.2,0.3)), 
               regexp = "numeric and finite")
  expect_is(rlba_frechet(10, A=c(0.5, 0.6), b=1, t0 = 0.5, 
                         shape_v=c(1.2, 1), scale_v=c(0.2,0.3)), 
            "matrix") 
  x <- rlba_frechet(10, A=0.5, b=1, t0 = 0.5, 
                    shape_v=c(1.2, 1), scale_v=c(0.2,0.3))
  expect_error(plba_frechet(x[,"response"], 
                            A=as.character(seq(0.2, 0.5, length.out = 10)), 
                            b=0.5, t0 = 0.5, 
                            shape_v=1.2, scale_v=0.2), 
               "numeric")
  expect_is(plba_frechet(x[,"response"], 
                         A=seq(0.2, 0.5, length.out = 10), b=0.5, 
                         t0 = seq(0.5, 0.7, 0.1), 
                         shape_v=1.2, scale_v=0.2), 
            "numeric")
})

test_that("_lnorm input works as they should", {
  expect_error(rlba_lnorm("10", A=0.5, b=1, t0 = 0.5, 
                          meanlog_v =c(1.2, 1), sdlog_v =c(0.2,0.3)), 
               regexp = "numeric and finite")
  expect_is(rlba_lnorm(10, A=c(0.5, 0.6), b=1, t0 = 0.5, 
                       meanlog_v=c(1.2, 1), sdlog_v=c(0.2,0.3)), "matrix") 
  x <- rlba_lnorm(10, A=0.5, b=1, t0 = 0.5, meanlog_v=c(1.2, 1), sdlog_v=c(0.2,0.3))
  expect_error(plba_lnorm(x[,"response"], 
                          A=as.character(seq(0.2, 0.5, length.out = 10)), 
                          b=0.5, t0 = 0.5, meanlog_v=1.2, sdlog_v=0.2), 
               "numeric")
  expect_is(plba_lnorm(x[,"response"], 
                       A=seq(0.2, 0.5, length.out = 10), 
                       b=0.5, t0 = seq(0.5, 0.7, 0.1), 
                       meanlog_v=1.2, sdlog_v=0.2), 
            "numeric")
})

test_that("LBA functions are identical with all input options", {
  rt1 <- rLBA(500, A=0.5, b=1, t0 = 0.5, mean_v=c(1.2, 1), sd_v=c(0.2,0.3))
  # get density for random RTs:
  ref <- sum(log(dLBA(rt1$rt, rt1$response, A=0.5, b = 1, t0 = 0.5, 
                      mean_v=c(1.2, 1), sd_v=c(0.2,0.3))))  # response is factor
  #expect_identical(sum(log(dLBA(rt1$rt, as.numeric(rt1$response), A=0.5, b = 1, t0 = 0.5, mean_v=c(1.2, 1), sd_v=c(0.2,0.3)))), ref)
  expect_identical(sum(log(dLBA(rt1, A=0.5, b = 1, t0 = 0.5, 
                                mean_v=c(1.2, 1), sd_v=c(0.2,0.3)))), ref)
  
  rt2 <- rt1[order(rt1$rt),]
  
  ref2 <- pLBA(rt2$rt, rt2$response, A=0.5, b = 1, t0 = 0.5, 
               mean_v=c(1.2, 1), sd_v=c(0.2,0.3))
  #expect_identical(pLBA(rt2$rt, as.numeric(rt2$response), A=0.5, b = 1, t0 = 0.5, mean_v=c(1.2, 1), sd_v=c(0.2,0.3)), ref2)
  expect_identical(pLBA(rt2, A=0.5, b = 1, t0 = 0.5, 
                        mean_v=c(1.2, 1), sd_v=c(0.2,0.3)), ref2)
  
  rt3 <- data.frame(p = rep(c(0.05, 0.1), 2),
                    response = rep(1:2, each = 2))
  ref3 <- qLBA(rt3$p, rt3$response, A=0.5, b = 1, t0 = 0.5, 
               mean_v=c(1.2, 1), sd_v=c(0.2,0.3))
  #expect_identical(qLBA(rt3$p, as.character(rt3$response), A=0.5, b = 1, t0 = 0.5, mean_v=c(1.2, 1), sd_v=c(0.2,0.3)), ref3)
  expect_identical(qLBA(rt3, A=0.5, b = 1, t0 = 0.5, 
                        mean_v=c(1.2, 1), sd_v=c(0.2,0.3)), ref3)
  
})

test_that("scale_p works as expected", {
  (max_p <- pLBA(Inf, response = 1, A=0.5, b=1, t0 = 0.5, 
                 mean_v=c(2.4, 1.6), sd_v=c(1,1.2)))
  # [1] 0.6604696
  # to get predicted quantiles, scale required quantiles by maximally predicted response rate:
  qs <- c(.1, .3, .5, .7, .9)
  expect_identical(qLBA(qs*max_p, response = 1, A=0.5, b=1, t0 = 0.5, 
                        mean_v=c(2.4, 1.6), sd_v=c(1,1.2)),
                    qLBA(qs, response = 1, A=0.5, b=1, t0 = 0.5, 
                         mean_v=c(2.4, 1.6), sd_v=c(1,1.2), scale_p=TRUE))
  
})

test_that("pLBA and dLBA work with vectorized accumulators", {
  v1 <- seq(2, 4, length.out = 6)
  v2 <- seq(0, 2, length.out = 6)
  
  res1p <- vector("numeric", 6)
  res1d <- vector("numeric", 6)
  for (i in 1:6) {
    res1p[i]  <- pLBA(1, response = 1, A=0.5, b=1, t0 = 0.5, 
                      mean_v=c(v1[i], v2[i]), sd_v=1, silent=TRUE)
    res1d[i]  <- dLBA(1, response = 1, A=0.5, b=1, t0 = 0.5, 
                      mean_v=c(v1[i], v2[i]), sd_v=1, silent=TRUE)
  }
  res2p <- pLBA(rep(1, 6), response = 1, A=0.5, b=1, t0 = 0.5, 
                mean_v=list(v1, v2), sd_v=1, silent=TRUE)
  res2d <- dLBA(rep(1, 6), response = 1, A=0.5, b=1, t0 = 0.5, 
                mean_v=list(v1, v2), sd_v=1, silent=TRUE)
  expect_identical(res1p, res2p)
  expect_identical(res1d, res2d)
})

#' Speed-Accuracy Data from Wagenmakers, Ratcliff, Gomez, & McKoon (2008, Experiment 1)
#' 
#' Responses and response times from an experiment in which instruction manipulated speed and accuracy between blocks. This data was also analyzed by Heathcote and Love (2012) who were the first to use the 17 participants also included here. 
#' 
#' @details The data excludes the practice blocks but includes all trials. Variable \code{censor} can be used for excluding all trials also excluded from the papers using it namely uninterpretable response, too fast response (<180 ms), too slow response (>3 sec). Heathcote and Love (2012, p. 7) describe the data as follows:
#' 
#' We fit the LBA and LNR models to data from Wagenmaker et al.'s (2008) experiment one, where participants made decisions about whether a string of letters constituted a word. These lexical decisions were made about four types of stimuli, non-words (nw) and high-frequency (hf), low-frequency (lf), and very low-frequency (vlf) words. Participants made decisions either under speed or accuracy emphasis instructions in different experimental blocks. Accuracy blocks were preceded by the message "Try to respond accurately" and "ERROR" was displayed after each wrong response. Speed blocks were preceded by the message "Try to respond accurately" and "TOO SLOW" was displayed after each response slower than 0.75 s.We report analyses of data from 17 participants (31,412 data points) in their Experiment 1, including the 15 participants analyzed in Wagenmakers et al. (2008) and two extras (we thank Eric-Jan Wagenmakers for supplying this data).
#' 
#' @docType data
#' @keywords dataset
#' @name speed_acc
#' @usage speed_acc
#' @format A \code{data.frame} with 31,522 obs. and 9 variables:
#' \describe{
#'  \item{id}{participant id}
#'  \item{block}{block number}
#'  \item{condition}{\code{accuracy} for blocks with accuracy instructions; \code{speed} for blocks with speed instruction}
#'  \item{stim}{unique identifier of stimulus, stimuli are nested in frequency conditions}
#'  \item{stim_cat}{category of stimulus, either word or non-word}
#'  \item{frequency}{"high frequency word", "low frequency word", "very low frequency word", or non-words derived from the first three categories}
#'  \item{response}{\code{word}, \code{nonword}, or not interpretable response (\code{error}, i.e., pushed a button, but not the right one and also not the one next to the right button)}
#'  \item{rt}{response time in seconds}
#'  \item{censor}{boolean indicating whether or not a response should be eliminated prior to analysis; uninterpretable response, too fast response (<180 ms), too slow response (>3 sec)}
#' }
#'
#' @source Wagenmakers, E.-J., Ratcliff, R., Gomez, P., & McKoon, G. (2008). A diffusion model account of criterion shifts in the lexical decision task. \emph{Journal of Memory and Language}, 58(1), 140-159.
#' 
#' @references Heathcote, A., & Love, J. (2012). Linear deterministic accumulator models of simple choice. \emph{Frontiers in Psychology: Cognitive Science}, 3, 292. doi:10.3389/fpsyg.2012.00292
#' 
#' @example examples/examples.speed_acc.R
#'
NULL

#' LBA race functions: Likelihood for first accumulator to win.
#' 
#' n1PDF and n1CDF take RTs, the distribution functions of the \link{LBA}, and
#' corresponding parameter values and put them throughout the race equations and
#' return the likelihood for the first accumulator winning (hence n1) in a set
#' of accumulators.
#' 
#' @param rt a vector of RTs.
#' @param A,b,t0 LBA parameters, see \code{\link{LBA}}. Can either be a single
#'   numeric vector (which will be recycled to reach \code{length(rt)} for
#'   trialwise parameters) \emph{or} a \code{list} of such vectors in which each
#'   list element corresponds to the parameters for this accumulator (i.e., the
#'   list needs to be of the same length as there are accumulators). Each list
#'   will also be recycled to reach \code{length(rt)} for trialwise parameters
#'   per accumulator.
#' @param st0 parameter specifying the variability of \code{t0} (which varies
#'   uniformly from \code{t0} to \code{t0} + \code{st0}). Can be trialwise, and
#'   will be recycled to length of \code{rt}.
#' @param ... two \emph{named} drift rate parameters depending on
#'   \code{distribution} (e.g., \code{mean_v} and \code{sd_v} for
#'   \code{distribution=="norm"}). The parameters can either be given as a
#'   numeric vector or a list. If a numeric vector is passed each element of the
#'   vector corresponds to one accumulator. If a list is passed each list
#'   element corresponds to one accumulator allowing again trialwise driftrates.
#'   The shorter parameter will be recycled as necessary (and also the elements
#'   of the list to match the length of \code{rt}). See examples.
#' @param distribution character specifying the distribution of the drift rate.
#'   Possible values are \code{c("norm", "gamma", "frechet", "lnorm")}, default
#'   is \code{"norm"}.
#' @param args.dist list of optional further arguments to the distribution
#'   functions (i.e., \code{posdrift} or \code{robust} for
#'   \code{distribution=="norm"}).
#' @param silent logical. Should the number of accumulators used be suppressed?
#'   Default is \code{FALSE} which prints the number of accumulators.
#'   
#'   
#' @details For a set of \eqn{N} independent accumulators \eqn{i = 1...N}, the
#'   race likelihood for a given accumulator \eqn{i} is given by 
#'   \deqn{L(\mbox{unit }i \mbox{ wins}) = f_i(t) \times \prod_{j<>i} [ S_j(t)
#'   ]}{L(unit i wins) = f_i(t) * prod_j<>i [ S_j(t) ]} where \eqn{f(t)} is the
#'   PDF (\code{dlba_...}) and \eqn{S_j(t) = 1 - F_j(t)} is the survivor
#'   function, that is the complement of the CDF \eqn{F(t)} (\code{plba_...}) at
#'   time \eqn{t}.
#'   
#'   In other words, this is just the PDF/CDF for the winning accumulator at
#'   time \eqn{t} times the probability that no other accumulators have finished
#'   at time \eqn{t}.
#'   
#' @seealso For more user-friendly functions that return the PDF or CDF for the
#'   corresponding (and not first) accumulator winning see \code{\link{LBA}}.
#'   
#' @name LBA-race
#' @importFrom stats integrate
#'   
#' @example examples/examples.lba-race.R
#'   
NULL

## note, this functions does not check parameters, it is only called internally (i.e., passed correctly).
n1PDFfixedt0 <- function(rt,A,b, t0, ..., pdf, cdf, args.dist = list()) {
  # Generates defective PDF for responses on node #1.
  dots <- list(...)
  nn <- length(rt)
  #if (length(A) != nn) browser()
  #browser()
  n_v <- max(vapply(dots, length, 0))  # Number of responses
  if (n_v>2) {
    tmp=array(dim=c(length(rt),n_v-1))
    for (i in 2:n_v) 
      tmp[,i-1] <- do.call(cdf, 
                           args = c(
                             rt=list(rt), 
                             A=if(is.list(A)) A[i] else list(A), 
                             b=if(is.list(b)) b[i] else list(b), 
                             t0 = if(is.list(t0)) t0[i] else list(t0), 
                             sapply(dots, "[[", i = i, simplify = FALSE), 
                             args.dist, nn=nn))
    G <- apply(1-tmp,1,prod)
  } else {
    G <- 1-do.call(cdf, 
                   args = c(
                     rt=list(rt), 
                     A=if(is.list(A)) A[2] else list(A), 
                     b=if(is.list(b)) b[2] else list(b), 
                     t0 = if(is.list(t0)) t0[2] else list(t0), 
                     sapply(dots, "[[", i = 2, simplify = FALSE), 
                     args.dist, nn=nn))
  }
  G*do.call(pdf, 
            args = c(
              rt=list(rt), 
              A=if(is.list(A)) A[1] else list(A), 
              b=if(is.list(b)) b[1] else list(b), 
              t0 = if(is.list(t0)) t0[1] else list(t0), 
              sapply(dots, "[[", i = 1, simplify = FALSE), 
              args.dist, nn=nn))
}

#sapply(dots, rep_dots, which = 1, nn = nn, simplify = FALSE)
rep_dots <- function(arg, which, nn) {
  rep(arg[[which]], length.out=nn)
}

## functions which checks if argument is numeric and 
check_n1_arguments <- function(arg, nn, n_v, dots = FALSE) {
  mc <- match.call()
  varname <- sub("dots$", "", deparse(mc[["arg"]]), fixed = TRUE)
  if (!is.list(arg)) {
    if ((!is.vector(arg, "numeric")) || (length(arg) < 1)) 
      stop(paste(varname, "needs to be a numeric vector of length >= 1!"))
    if (dots) {
      arg <- as.list(arg)
      arg <- lapply(arg, rep, length.out=nn)
    } else arg <- rep(arg, length.out=nn)
  } else {
    if (!dots && (length(arg) != n_v)) 
      stop(paste("if", varname, "is a list, its length needs to correspond to the number of accumulators."))
    for (i in seq_along(arg)) {
      if ((!is.vector(arg[[i]], "numeric")) || (length(arg[[i]]) < 1)) 
        stop(paste0(varname, "[[", i, "]] needs to be a numeric vector of length >= 1!"))
      arg[[i]] <- rep(arg[[i]], length.out=nn)
    }
  }
  return(unname(arg))
}

#' @rdname LBA-race
#' @export
n1PDF <- function(rt, A, b, t0, ..., st0=0, 
                  distribution = c("norm", "gamma", "frechet", "lnorm"), 
                  args.dist = list(), silent = FALSE) {
  dots <- list(...)
  #browser()
  if (is.null(names(dots))) stop("... arguments need to be named.")
  if (any(names(dots) == "")) stop("all ... arguments need to be named.")
  
  n_v <- max(vapply(dots, length, 0))  # Number of responses
  if(!silent) message(paste("Results based on", n_v, "accumulators/drift rates."))
  if (n_v < 2) stop("There need to be at least two accumulators/drift rates.")
  distribution <- match.arg(distribution)
  #check_single_arg(t0 = t0)
  nn <- length(rt)
  #browser()
  A <- check_n1_arguments(A, nn=nn, n_v=n_v)
  b <- check_n1_arguments(b, nn=nn, n_v=n_v)
  t0 <- check_n1_arguments(t0, nn=nn, n_v=n_v)
  st0 <- rep(unname(st0), length.out = nn)
  switch(distribution, 
         norm = {
           pdf <- dlba_norm_core
           cdf <- plba_norm_core
           if (any(!(c("mean_v","sd_v") %in% names(dots)))) 
             stop("mean_v and sd_v need to be passed for distribution = \"norm\"")
           dots$mean_v <- check_n1_arguments(dots$mean_v, nn=nn, n_v=n_v, dots = TRUE)
           dots$sd_v <- check_n1_arguments(dots$sd_v, nn=nn, n_v=n_v, dots = TRUE)
           dots <- dots[c("mean_v","sd_v")]
         },
         gamma = {
           pdf <- dlba_gamma_core
           cdf <- plba_gamma_core
           if (!("shape_v" %in% names(dots))) 
             stop("shape_v needs to be passed for distribution = \"gamma\"")
           if ((!("rate_v" %in% names(dots))) & (!("scale_v" %in% names(dots)))) 
             stop("rate_v or scale_v need to be passed for distribution = \"gamma\"")
           dots$shape_v <- check_n1_arguments(dots$shape_v, nn=nn, n_v=n_v, dots = TRUE)
           if ("scale_v" %in% names(dots)) {
             dots$scale_v <- check_n1_arguments(dots$scale_v, nn=nn, n_v=n_v, dots = TRUE)
             if (is.list(dots$scale_v)) {
               dots$rate_v <- lapply(dots$scale_v, function(x) 1/x)
             } else dots$rate_v <- 1/dots$scale_v
           } else dots$rate_v <- check_n1_arguments(dots$rate_v, nn=nn, n_v=n_v, dots = TRUE)
           dots <- dots[c("shape_v","rate_v")]
         },
         frechet = {
           pdf <- dlba_frechet_core
           cdf <- plba_frechet_core
           if (any(!(c("shape_v","scale_v") %in% names(dots)))) 
             stop("shape_v and scale_v need to be passed for distribution = \"frechet\"")
           dots$shape_v <- check_n1_arguments(dots$shape_v, nn=nn, n_v=n_v, dots = TRUE)
           dots$scale_v <- check_n1_arguments(dots$scale_v, nn=nn, n_v=n_v, dots = TRUE)
           dots <- dots[c("shape_v","scale_v")]
         },
         lnorm = {
           pdf <- dlba_lnorm_core
           cdf <- plba_lnorm_core
           if (any(!(c("meanlog_v","sdlog_v") %in% names(dots)))) 
             stop("meanlog_v and sdlog_v need to be passed for distribution = \"lnorm\"")
           dots$meanlog_v <- check_n1_arguments(dots$meanlog_v, nn=nn, n_v=n_v, dots = TRUE)
           dots$sdlog_v <- check_n1_arguments(dots$sdlog_v, nn=nn, n_v=n_v, dots = TRUE)
           dots <- dots[c("meanlog_v","sdlog_v")]
         }
  )
  #browser()
  for (i in seq_len(length(dots))) {
    if (length(dots[[i]]) < n_v) dots[[i]] <- rep(dots[[i]],length.out=n_v)
  }
#   if (length(st0)>1) {
#     warning("st0 set to st0[1]. Only one non-decision time variability permitted.")
#     st0 <- st0[1] # Only ONE non-decision time.
#   }
  #browser()
  do.call(n1PDF_core, 
          args = c(
            rt = list(rt),
            A = list(A),
            b = list(b),
            t0 = list(t0),
            st0 = list(st0),
            dots,
            pdf = pdf,
            cdf = cdf,
            args.dist = list(args.dist)
          ))
}


n1PDF_core <- function(rt, A, b, t0, ..., st0, pdf, cdf, args.dist = list()) {
  dots <- list(...)
  #browser()
  if (all(st0==0)) return(do.call(n1PDFfixedt0, 
                                  args = c(
                                    rt = list(rt),
                                    A = list(A),
                                    b = list(b),
                                    t0 = list(t0),
                                    dots,
                                    pdf = pdf,
                                    cdf = cdf,
                                    args.dist = list(args.dist)
                                  )))
  else {
    tmpf <- function(rt, A, b, t0, st0, ..., pdf, cdf, args.dist = list()) {
      #browser()
      dots2 <- list(...)
      do.call(n1PDFfixedt0, 
              args = c(
                rt = list(rt),
                A = list(A),
                b = list(b),
                t0 = list(t0),
                dots2,
                pdf = pdf,
                cdf = cdf,
                args.dist = list(args.dist)
              )) / st0
      #rt=list(pmax(rt-t0, 0))
    }
    outs <- vector("numeric", length = length(rt))
    if (length(st0) == 1) st0 <- rep(st0, length.out = length(rt))
    for (i in 1:length(rt)) {
      if (st0[i] != 0) {
        tmp <- do.call(integrate, 
                       args = c(
                         f = tmpf,
                         lower = unname(rt[i] - st0[i]),
                         upper = unname(rt[i]),
                         A = ret_arg(A, i),
                         b = ret_arg(b, i),
                         t0 = ret_arg(t0, i),
                         sapply(dots, function(z, i)
                           sapply(z, ret_arg2, which = i, simplify = FALSE), 
                           i = i, simplify = FALSE),
                         pdf = pdf,
                         cdf = cdf,
                         args.dist = list(args.dist),
                         stop.on.error = FALSE,
                         st0 = list(st0[i])
                       ))
        if (tmp$message != "OK") warning(paste("n1PDF:", tmp$message))
        outs[i] <- tmp$value
      } else outs[i] <- do.call(n1PDFfixedt0, 
                                args = c(
                                  rt = list(rt[i]),
                                  A = ret_arg(A, i),
                                  b = ret_arg(b, i),
                                  t0 = ret_arg(t0, i),
                                  sapply(dots, function(z, i)
                                    sapply(z, ret_arg2, which = i, 
                                           simplify = FALSE), 
                                    i = i, simplify = FALSE),
                                  pdf = pdf,
                                  cdf = cdf,
                                  args.dist = list(args.dist)
                                ))
    }
    return(outs)
  }
}

# n1PDF_single <- function(rt, A, b, t0, ..., st0, pdf, cdf, args.dist = list()) {
#   dots <- list(...)
#   #browser()
#   if (st0==0) return(do.call(n1PDFfixedt0, args = c(rt=list(rt), A=list(A), b=list(b), t0 = list(t0), dots, pdf=pdf, cdf=cdf, args.dist = args.dist)))
#   else {
#     tmpf <- function(rt, A, b, t0, ..., pdf, cdf, args.dist = list()) {
#       #browser()
#       do.call(n1PDFfixedt0, args = c(rt=list(pmax(rt-t0, 0)), A=list(A), b=list(b), t0 = list(0), dots, pdf=pdf, cdf=cdf, args.dist = args.dist))/st0
#     }
#     outs=numeric(length(rt))
#     #browser()
#     for (i in 1:length(outs)) {
#       tmp <- do.call(integrate, args=c(f=tmpf, lower=rt[i]-t0[1]-st0, upper=rt[i]-t0[1], A=list(A), b=list(b), t0=list(0), dots, pdf=pdf, cdf=cdf, args.dist = args.dist, stop.on.error = FALSE))
#       if (tmp$message != "OK") warning(paste("n1PDF:", tmp$message))
#       outs[i] <- tmp$value
#     }
#     return(outs)
#   }
# }

ret_arg <- function(arg, which) {
  list(if(is.list(arg)) {
    if (which <= min(sapply(arg, length))) sapply(arg, "[[", i = which, simplify = FALSE) else arg 
    } else {
      if (which <= length(arg)) arg[which]
      else arg
      })
}

ret_arg2 <- function(arg, which) {
  if(is.list(arg)) {
    if (which <= min(sapply(arg, length))) sapply(arg, "[[", i = which, simplify = FALSE) else arg 
    } else {
      if (which <= length(arg)) arg[which]
      else arg
      }
}

# rt = time, A=x0max, b=chi, v=drift, sv=sdI
#' @rdname LBA-race
#' @export
n1CDF <- function(rt,A,b, t0, ..., st0=0, 
                  distribution = c("norm", "gamma", "frechet", "lnorm"), 
                  args.dist = list(), silent = FALSE) {  #, browser=FALSE
  # Generates defective CDF for responses on node #1. 
  dots <- list(...)
  if (is.null(names(dots))) stop("... arguments need to be named.")
  
  n_v <- max(vapply(dots, length, 0))  # Number of responses
  if(!silent) message(paste("Results based on", n_v, "accumulators/drift rates."))
  if (n_v < 2) stop("There need to be at least two accumulators/drift rates.")
  distribution <- match.arg(distribution)
  #check_single_arg(t0 = t0)
  nn <- length(rt)
  #browser()
  A <- check_n1_arguments(A, nn=nn, n_v=n_v)
  b <- check_n1_arguments(b, nn=nn, n_v=n_v)
  t0 <- check_n1_arguments(t0, nn=nn, n_v=n_v)
  st0 <- rep(unname(st0), length.out = nn)
  switch(distribution, 
         norm = {
           pdf <- dlba_norm_core
           cdf <- plba_norm_core
           if (any(!(c("mean_v","sd_v") %in% names(dots)))) 
             stop("mean_v and sd_v need to be passed for distribution = \"norm\"")
           dots$mean_v <- check_n1_arguments(dots$mean_v, nn=nn, n_v=n_v, dots = TRUE)
           dots$sd_v <- check_n1_arguments(dots$sd_v, nn=nn, n_v=n_v, dots = TRUE)
           dots <- dots[c("mean_v","sd_v")]
         },
         gamma = {
           pdf <- dlba_gamma_core
           cdf <- plba_gamma_core
           if (!("shape_v" %in% names(dots))) 
             stop("shape_v needs to be passed for distribution = \"gamma\"")
           if ((!("rate_v" %in% names(dots))) & (!("scale_v" %in% names(dots)))) 
             stop("rate_v or scale_v need to be passed for distribution = \"gamma\"")
           dots$shape_v <- check_n1_arguments(dots$shape_v, nn=nn, n_v=n_v, dots = TRUE)
           if ("scale_v" %in% names(dots)) {
             dots$scale_v <- check_n1_arguments(dots$scale_v, nn=nn, n_v=n_v, dots = TRUE)
             if (is.list(dots$scale_v)) {
               dots$rate_v <- lapply(dots$scale_v, function(x) 1/x)
             } else dots$rate_v <- 1/dots$scale_v
           } else dots$rate_v <- check_n1_arguments(dots$rate_v, nn=nn, n_v=n_v, dots = TRUE)
           dots <- dots[c("shape_v","rate_v")]
         },
         frechet = {
           pdf <- dlba_frechet_core
           cdf <- plba_frechet_core
           if (any(!(c("shape_v","scale_v") %in% names(dots)))) 
             stop("shape_v and scale_v need to be passed for distribution = \"frechet\"")
           dots$shape_v <- check_n1_arguments(dots$shape_v, nn=nn, n_v=n_v, dots = TRUE)
           dots$scale_v <- check_n1_arguments(dots$scale_v, nn=nn, n_v=n_v, dots = TRUE)
           dots <- dots[c("shape_v","scale_v")]
         },
         lnorm = {
           pdf <- dlba_lnorm_core
           cdf <- plba_lnorm_core
           if (any(!(c("meanlog_v","sdlog_v") %in% names(dots)))) 
             stop("meanlog_v and sdlog_v need to be passed for distribution = \"lnorm\"")
           dots$meanlog_v <- check_n1_arguments(dots$meanlog_v, nn=nn, n_v=n_v, dots = TRUE)
           dots$sdlog_v <- check_n1_arguments(dots$sdlog_v, nn=nn, n_v=n_v, dots = TRUE)
           dots <- dots[c("meanlog_v","sdlog_v")]
         }
  )
  
  for (i in seq_len(length(dots))) {
    if (length(dots[[i]]) < n_v) dots[[i]] <- rep(dots[[i]],length.out=n_v)
  }
#   if (length(st0)>1) {
#     warning("st0 set to st0[1]. Only one non-decision time variability permitted.")
#     st0 <- st0[1] # Only ONE non-decision time.
#   }
  if (any(st0<1e-6)) {
    if (any(sapply(st0[st0<1e-6], function(x) !isTRUE(all.equal(x, 0))))) 
      warning("st0 set to 0 for values < 1e-6. Integral can fail for small st0.")
    st0[st0<1e-6] <- 0
  } # 
  outs <- numeric(length(rt))
  #bounds <- c(0,rt)
  #browser()
  for (i in 1:length(rt)) {
    tmp_obj <- 
      do.call(integrate, 
              args = c(
                f = n1PDF_core,
                lower = 0,
                upper = rt[i],
                subdivisions = 1000,
                A = ret_arg(A, i),
                b = ret_arg(b, i),
                t0 = ret_arg(t0, i),
                st0 = list(st0[i]),
                sapply(dots, function(z, i)
                  sapply(z, "[[", i = i, simplify = FALSE), 
                  i = i, simplify = FALSE),
                pdf = pdf,
                cdf = cdf,
                stop.on.error = FALSE,
                args.dist = list(args.dist)
              ))
    if (tmp_obj$message != "OK") {
      warning(tmp_obj$message)
    }
    outs[i] <- tmp_obj$value
  }
  outs
}

context("LBA race functions: RNG is equivalent to n1")

x <- .Random.seed
set.seed(5)

tryCatch.W.E <- function(expr)
{
  mc <- match.call()
  mc2 <- match.call(definition = ks.test, call =  as.call(mc[[2]]))
  mc2[[1]] <- list
  
  W <- NULL
  w.handler <- function(w){ # warning handler
    W <<- w
    invokeRestart("muffleWarning")
  }
  list(value = withCallingHandlers(tryCatch(expr, error = function(e) e), 
                                   warning = w.handler), warning = W, 
       data = eval(mc2, envir = parent.frame()))
}

conditional_save_t <- function(t, distribution) {
  mc <- match.call()
  ex_data <- t$data
  #if (!is.null(t$warning)) save(ex_data, file = paste0(mc[[2]], "_", distribution, "_problem.Rdata"))
  #browser()
  #str(t)  
}

test_that("Norm: n1CDF corresponds to random derivates", {
  testthat::skip_on_cran()
  testthat::skip_on_travis()
  normalised_n1CDF = function(rt,...) n1CDF(rt,...)/n1CDF(rt=Inf,...) 
  samples <- 1e3
  p_min <- 0.001
  p_max <- 0.05
  A <- runif(2, 0.3, 0.9)
  b <- A+runif(2, 0, 0.5)
  t0 <- runif(2, 0.1, 0.7)
  v1 <- runif(4, 0.5, 1.5)
  v2 <- runif(4, 0.1, 0.5)
  st0 <- runif(1, 0.1, 0.5)
  r_lba1 <- rLBA(samples, A=A[1], b=b[1], t0 = t0[1], 
                 mean_v=v1[1:2], sd_v=v2[1:2])
  r_lba2 <- rLBA(samples, A=A[2], b=b[2], t0 = t0[2], 
                 mean_v=v1[3:4], sd_v=v2[3:4], st0 = st0[1])
  t1 <- tryCatch.W.E(ks.test(r_lba1$rt[r_lba1$response==1], normalised_n1CDF, 
                             A=A[1], b=b[1]+0.1, t0 = t0[1], 
                             mean_v=v1[1:2], sd_v=v2[1:2]))
  expect_lt(t1$value$p.value, p_min)
  conditional_save_t(t1, "norm")
  
  t2 <- tryCatch.W.E(ks.test(r_lba2$rt[r_lba2$response==1], normalised_n1CDF, 
                             A=A[2], b=b[2], t0 = t0[2], 
                             mean_v=v1[3:4], sd_v=v2[3:4], st0 = 0))
  expect_lt(t2$value$p.value, p_min)
  conditional_save_t(t2, "norm")
  
  t3 <- tryCatch.W.E(ks.test(r_lba2$rt[r_lba2$response==1], normalised_n1CDF, 
                             A=A[2], b=b[2], t0 = t0[2], 
                             mean_v=v1[3:4], sd_v=v2[3:4], st0 = st0[1]+0.1))
  expect_lt(t3$value$p.value, p_min)
  conditional_save_t(t3, "norm")
  
  t4 <- tryCatch.W.E(ks.test(r_lba1$rt[r_lba1$response==1], normalised_n1CDF, 
                             A=A[1], b=b[1], t0 = t0[1], 
                             mean_v=v1[1:2], sd_v=v2[1:2]))
  expect_gt(t4$value$p.value, p_max)
  conditional_save_t(t4, "norm")
  
  t5 <- tryCatch.W.E(ks.test(r_lba2$rt[r_lba2$response==1], normalised_n1CDF, 
                             A=A[2], b=b[2], t0 = t0[2], 
                             mean_v=v1[3:4], sd_v=v2[3:4], st0 = st0[1]))
  conditional_save_t(t5, "norm")
  
  expect_gt(t5$value$p.value, p_max)
  
  #if (any(sapply(list(t1, t2, t3, t4, t5), function(x) !is.null(x$warning)))) browser()
})

test_that("Gamma: n1CDF corresponds to random derivates", {
  testthat::skip_on_cran()
  normalised_n1CDF = function(rt,...) n1CDF(rt,...)/n1CDF(rt=Inf,...) 
  samples <- 1e3
  p_min <- 0.01
  p_max <- 0.05
  A <- runif(2, 0.3, 0.9)
  b <- A+runif(2, 0, 0.5)
  t0 <- runif(2, 0.1, 0.7)
  v1 <- runif(4, 0.5, 1.5)
  v2 <- runif(4, 0.1, 0.5)
  st0 <- runif(1, 0.25, 0.75)
  r_lba1 <- rLBA(samples, A=A[1], b=b[1], t0 = t0[1], 
                 shape_v=v1[1:2], scale_v=v2[1:2], distribution = "gamma")
  r_lba2 <- rLBA(samples, A=A[2], b=b[2], t0 = t0[2], 
                 shape_v=v1[3:4], scale_v=v2[3:4], 
                 st0 = st0[1], distribution = "gamma")
  
  t1 <- tryCatch.W.E(ks.test(r_lba1$rt[r_lba1$response==1], normalised_n1CDF, 
                             A=A[1]+0.1, b=b[1]+0.2, t0 = t0[1], 
                             shape_v=v1[1:2], scale_v=v2[1:2], 
                             distribution = "gamma"))
  expect_lt(t1$value$p.value, p_min)
  conditional_save_t(t1, "gamma")
  
  t2 <- tryCatch.W.E(ks.test(r_lba2$rt[r_lba2$response==1], normalised_n1CDF, 
                             A=A[2], b=b[2], t0 = t0[2], 
                             shape_v=v1[3:4], scale_v=v2[3:4], 
                             st0 = 0, distribution = "gamma"))
  expect_lt(t2$value$p.value, p_min)
  conditional_save_t(t2, "gamma")
  
  t3 <- tryCatch.W.E(ks.test(r_lba2$rt[r_lba2$response==1], normalised_n1CDF, 
                             A=A[2], b=b[2], t0 = t0[2], 
                             shape_v=v1[3:4], scale_v=v2[3:4], 
                             st0 = st0[1]+0.5, distribution = "gamma"))
  expect_lt(t3$value$p.value, p_min)
  conditional_save_t(t3, "gamma")
  
  t4 <- tryCatch.W.E(ks.test(r_lba1$rt[r_lba1$response==1], normalised_n1CDF, 
                             A=A[1], b=b[1], t0 = t0[1], 
                             shape_v=v1[1:2], scale_v=v2[1:2], 
                             distribution = "gamma"))
  expect_gt(t4$value$p.value, p_max)
  conditional_save_t(t4, "gamma")
  
  t5 <- tryCatch.W.E(ks.test(r_lba2$rt[r_lba2$response==1], normalised_n1CDF, 
                             A=A[2], b=b[2], t0 = t0[2], 
                             shape_v=v1[3:4], scale_v=v2[3:4], 
                             st0 = st0[1], distribution = "gamma"))
  expect_gt(t5$value$p.value, p_max)
  conditional_save_t(t5, "gamma")
  
  #if (any(sapply(list(t1, t2, t3, t4, t5), function(x) !is.null(x$warning)))) browser()
})

test_that("Frechet: n1CDF corresponds to random derivates", {
  testthat::skip_on_cran()
  testthat::skip_on_travis()
  normalised_n1CDF = function(rt,...) n1CDF(rt,...)/n1CDF(rt=Inf,...) 
  samples <- 2e2
  p_min <- 0.001
  p_max <- 0.05
  A <- runif(2, 0.3, 0.9)
  b <- A+runif(2, 0, 0.5)
  t0 <- runif(2, 0.1, 0.7)
  v1 <- runif(4, 0.5, 1.5)
  v2 <- runif(4, 0.5, 1.5)
  st0 <- runif(1, 0.25, 0.5)
  r_lba1 <- rLBA(samples, A=A[1], b=b[1], t0 = t0[1], 
                 shape_v=v1[1:2], scale_v=v2[1:2], distribution = "frechet")
  r_lba2 <- rLBA(samples, A=A[2], b=b[2], t0 = t0[2], 
                 shape_v=v1[3:4], scale_v=v2[3:4], 
                 st0 = st0[1], distribution = "frechet")

  t1 <- tryCatch.W.E(ks.test(r_lba1$rt[r_lba1$response==1], normalised_n1CDF, 
                             A=A[1]+0.4, b=b[1]+0.8, t0 = t0[1], 
                             shape_v=v1[1:2], scale_v=v2[1:2], 
                             distribution = "frechet"))
  expect_lt(t1$value$p.value, p_min)
  conditional_save_t(t1, "frechet")
  
  t2 <- tryCatch.W.E(ks.test(r_lba2$rt[r_lba2$response==1], normalised_n1CDF, 
                             A=A[2], b=b[2], t0 = t0[2], 
                             shape_v=v1[3:4], scale_v=v2[3:4], 
                             st0 = 0, distribution = "frechet"))
  expect_lt(t2$value$p.value, p_min)
  conditional_save_t(t2, "frechet")
  
  #browser()
  t3 <- tryCatch.W.E(ks.test(r_lba2$rt[r_lba2$response==1], normalised_n1CDF, 
                             A=A[2], b=b[2], t0 = t0[2], 
                             shape_v=v1[3:4], scale_v=v2[3:4], 
                             st0 = st0[1]+0.2, distribution = "frechet"))
  expect_lt(t3$value$p.value, p_min+0.01)
  conditional_save_t(t3, "frechet")  
  
  t4 <- tryCatch.W.E(ks.test(r_lba1$rt[r_lba1$response==1], normalised_n1CDF, 
                             A=A[1], b=b[1], t0 = t0[1], 
                             shape_v=v1[1:2], scale_v=v2[1:2], 
                             distribution = "frechet"))
  expect_gt(t4$value$p.value, p_max)
  conditional_save_t(t4, "frechet")
  
  t5 <- tryCatch.W.E(ks.test(r_lba2$rt[r_lba2$response==1], normalised_n1CDF, 
                             A=A[2], b=b[2], t0 = t0[2], 
                             shape_v=v1[3:4], scale_v=v2[3:4], 
                             st0 = st0[1], distribution = "frechet"))
  #t5 <- tryCatch.W.E(ks.test(r_lba2$rt[r_lba2$response==1]-t0[2], normalised_n1CDF, A=A[2], b=b[2], t0 = 0, shape_v=v1[3:4], scale_v=v2[3:4], st0 = st0[1], distribution = "frechet"))
  expect_gt(t5$value$p.value, p_max)
  conditional_save_t(t5, "frechet")
  
  #if (any(sapply(list(t1, t2, t3, t4, t5), function(x) !is.null(x$warning)))) browser()
})

test_that("lnorm: n1CDF corresponds to random derivates", {
  testthat::skip_on_cran()
  testthat::skip_on_travis()
  normalised_n1CDF = function(rt,...) n1CDF(rt,...)/n1CDF(rt=Inf,...) 
  samples <- 1e3
  p_min <- 0.0001
  p_max <- 0.05
  A <- runif(2, 0.3, 0.9)
  b <- A+runif(2, 0, 0.5)
  t0 <- runif(2, 0.1, 0.7)
  v1 <- runif(4, 0.5, 1.5)
  v2 <- runif(4, 0.1, 0.5)
  st0 <- runif(1, 0.1, 0.5)
  r_lba1 <- rLBA(samples, A=A[1], b=b[1], t0 = t0[1], 
                 meanlog_v=v1[1:2], sdlog_v=v2[1:2], 
                 distribution = "lnorm")
  r_lba2 <- rLBA(samples, A=A[2], b=b[2], t0 = t0[2], 
                 meanlog_v=v1[3:4], sdlog_v=v2[3:4], st0 = st0[1], 
                 distribution = "lnorm")
  
  t1 <- tryCatch.W.E(ks.test(r_lba1$rt[r_lba1$response==1], normalised_n1CDF, 
                             A=A[1], b=b[1]+0.1, t0 = t0[1], 
                             meanlog_v=v1[1:2], sdlog_v=v2[1:2], 
                             distribution = "lnorm"))
  expect_lt(t1$value$p.value, p_min)
  conditional_save_t(t1, "lnorm")
  
  t2 <- tryCatch.W.E(ks.test(r_lba2$rt[r_lba2$response==1], normalised_n1CDF, 
                             A=A[2], b=b[2], t0 = t0[2], 
                             meanlog_v=v1[3:4], sdlog_v=v2[3:4], 
                             st0 = 0, distribution = "lnorm"))
  expect_lt(t2$value$p.value, p_min)
  conditional_save_t(t2, "lnorm")
  
  t3 <- tryCatch.W.E(ks.test(r_lba2$rt[r_lba2$response==1], normalised_n1CDF, 
                             A=A[2], b=b[2], t0 = t0[2], 
                             meanlog_v=v1[3:4], sdlog_v=v2[3:4], 
                             st0 = st0[1]+0.2, distribution = "lnorm"))
  expect_lt(t3$value$p.value, p_min)
  conditional_save_t(t3, "lnorm")
  
  #t4 <- tryCatch.W.E(ks.test(r_lba1$rt[r_lba1$response==1], normalised_n1CDF, A=A[1], b=b[1], t0 = t0[1], meanlog_v=v1[1:2], sdlog_v=v2[1:2], distribution = "lnorm"))
  t4 <- tryCatch.W.E(ks.test(r_lba1$rt[r_lba1$response==1]-t0[1], 
                             normalised_n1CDF, 
                             A=A[1], b=b[1], t0 = 0, 
                             meanlog_v=v1[1:2], sdlog_v=v2[1:2], 
                             distribution = "lnorm"))
  expect_gt(t4$value$p.value, p_max)
  conditional_save_t(t4, "lnorm")
  
  #t5 <- tryCatch.W.E(ks.test(r_lba2$rt[r_lba2$response==1], normalised_n1CDF, A=A[2], b=b[2], t0 = t0[2], meanlog_v=v1[3:4], sdlog_v=v2[3:4], st0 = st0[1], distribution = "lnorm"))
  t5 <- tryCatch.W.E(ks.test(pmax(r_lba2$rt[r_lba2$response==1]-t0[2],0), 
                             normalised_n1CDF, 
                             A=A[2], b=b[2], t0 = 0, 
                             meanlog_v=v1[3:4], sdlog_v=v2[3:4], 
                             st0 = st0[1], distribution = "lnorm"))
  expect_gt(t5$value$p.value, p_max)
  conditional_save_t(t5, "lnorm")
  
  #if (any(sapply(list(t1, t2, t3, t4, t5), function(x) !is.null(x$warning)))) browser()
})


test_that("Norm: n1CDF corresponds to random derivates with accumulatorwise parameters", {
  testthat::skip_on_cran()
  testthat::skip_on_travis()
  normalised_n1CDF = function(rt,...) n1CDF(rt,...)/n1CDF(rt=Inf,...) 
  samples <- 1e3
  p_min <- 0.001
  p_max <- 0.05
  A <- runif(4, 0.3, 0.9)
  b <- A+runif(4, 0, 0.5)
  t0 <- runif(4, 0.1, 0.7)
  v1 <- runif(4, 0.5, 1.5)
  v2 <- runif(4, 0.1, 0.5)
  st0 <- runif(1, 0.1, 0.5)
  r_lba1 <- rLBA(samples, A=list(A[1], A[2]), b=list(b[1], b[2]), 
                 t0 = list(t0[1], t0[2]), mean_v=v1[1:2], sd_v=v2[1:2])
  r_lba2 <- rLBA(samples, A=list(A[3], A[4]), b=list(b[3], b[4]), 
                 t0 = list(t0[3], t0[4]), mean_v=v1[3:4], sd_v=v2[3:4], 
                 st0 = st0[1])
  t1 <- tryCatch.W.E(ks.test(r_lba1$rt[r_lba1$response==1], normalised_n1CDF, 
                             A=list(A[1], A[2]), b=list(b[1], b[2]), 
                             t0 = list(t0[1]+0.1, t0[2]+0.1), 
                             mean_v=v1[1:2], sd_v=v2[1:2]))
  expect_lt(t1$value$p.value, p_min)
  conditional_save_t(t1, "norm")
  
  t2 <- tryCatch.W.E(ks.test(r_lba2$rt[r_lba2$response==1], normalised_n1CDF, 
                             A=list(A[3], A[4]), b=list(b[3], b[4]), 
                             t0 = list(t0[3], t0[4]), mean_v=v1[3:4], 
                             sd_v=v2[3:4], st0 = 0))
  expect_lt(t2$value$p.value, p_min)
  conditional_save_t(t2, "norm")
  
  t3 <- tryCatch.W.E(ks.test(r_lba2$rt[r_lba2$response==1], normalised_n1CDF, 
                             A=list(A[3], A[4]), b=list(b[3], b[4]), 
                             t0 = list(t0[3], t0[4]), 
                             mean_v=v1[3:4], sd_v=v2[3:4], st0 = st0[1]+0.1))
  expect_lt(t3$value$p.value, p_min) #+0.003
  conditional_save_t(t3, "norm")
  
  t4 <- tryCatch.W.E(ks.test(r_lba1$rt[r_lba1$response==1], normalised_n1CDF, 
                             A=list(A[1], A[2]), b=list(b[1], b[2]), 
                             t0 = list(t0[1], t0[2]), 
                             mean_v=v1[1:2], sd_v=v2[1:2]))
  expect_gt(t4$value$p.value, p_max)
  conditional_save_t(t4, "norm")
  
  t5 <- tryCatch.W.E(ks.test(r_lba2$rt[r_lba2$response==1], normalised_n1CDF, 
                             A=list(A[3], A[4]), b=list(b[3], b[4]), 
                             t0 = list(t0[3], t0[4]), 
                             mean_v=v1[3:4], sd_v=v2[3:4], st0 = st0[1]))
  conditional_save_t(t5, "norm")
  
  expect_gt(t5$value$p.value, p_max)
  
  #if (any(sapply(list(t1, t2, t3, t4, t5), function(x) !is.null(x$warning)))) browser()
})

test_that("rLBA works with trial wise input", {
  set.seed(1)
  rt1 <- rLBA(100, A=rep(c(0.5, 0.4), each = 50), b=rep(c(1, 1.4), each = 50), 
              t0 = rep(c(0.2, 0.4), each = 50), 
              mean_v=list(rep(c(1.5, 0.9), each = 50), 
                          rep(c(0.9, 1.5), each = 50)), 
              sd_v=list(rep(c(0.2, 0.4), each = 50), 
                        rep(c(0.3, 0.5), each = 50)), 
              st0 = rep(c(0.1, 0.2), each = 50))
  set.seed(1)
  rt2a <- rLBA(50, A=0.5, b=1, t0 = 0.2, 
               mean_v=c(1.5, 0.9), sd_v=list(0.2, 0.3), st0 = 0.1)
  rt2b <- rLBA(50, A=0.4, b=1.4, t0 = 0.4, 
               mean_v=c(0.9, 1.5), sd_v=c(0.4, 0.5), st0 = 0.2)
  expect_identical(rt1, rbind(rt2a, rt2b))
  
})

.Random.seed <<- x

#' Deprecated functions
#'
#' These functions have been renamed and deprecated in \pkg{rtdists}:
#' \code{drd()} (use \code{\link{ddiffusion}()}), 
#' \code{prd()} (use \code{\link{pdiffusion}()}),
#' \code{rrd()} (use \code{\link{rdiffusion}()}).
#' @rdname deprecated
#' @keywords internal
#' @aliases rtdists-deprecated
#' @param ... arguments passed from the old functions to the new functions
#' @export
drd <- function(...) {
  .Deprecated("ddiffusion", "rtdists", "drd was renamed to ddiffusion and is now deprecated.")
  ddiffusion(...)
}
#' @rdname deprecated
#' @export
prd <- function(...) {
  .Deprecated("pdiffusion", "rtdists", "prd was renamed to pdiffusion and is now deprecated.")
  pdiffusion(...)
}
#' @rdname deprecated
#' @export
rrd <- function(...) {
  .Deprecated("rdiffusion", "rtdists", "rrd was renamed to rdiffusion and is now deprecated.")
  rdiffusion(...)
}

#' The Ratcliff Diffusion Model
#' 
#' Density, distribution function, quantile function, and random generation for the Ratcliff diffusion model with following parameters: \code{a} (threshold separation), \code{z} (starting point), \code{v} (drift rate), \code{t0} (non-decision time/response time constant), \code{d} (differences in speed of response execution), \code{sv} (inter-trial-variability of drift), \code{st0} (inter-trial-variability of non-decisional components), \code{sz} (inter-trial-variability of relative starting point), and \code{s} (diffusion constant). \strong{Note that the parameterization or defaults of non-decision time variability \code{st0} and diffusion constant \code{s} differ from what is often found in the literature and that the parameterization of \code{z} and \code{sz} have changed compared to previous versions (now absolute and not relative).}
#'
#' @param rt a vector of RTs. Or for convenience also a \code{data.frame} with columns \code{rt} and \code{response} (such as returned from \code{rdiffusion} or \code{\link{rLBA}}). See examples.
#' @param n is a desired number of observations.
#' @param response character vector. Which response boundary should be tested? Possible values are \code{c("upper", "lower")}, possibly abbreviated and \code{"upper"} being the default. Alternatively, a numeric vector with values 1=lower and 2=upper. For convenience, \code{response} is converted via \code{as.numeric} also allowing factors (see examples). Ignored if the first argument is a \code{data.frame}.
#' @param p vector of probabilities. Or for convenience also a \code{data.frame} with columns \code{p} and \code{response}. See examples.
#' 
#' @param a threshold separation. Amount of information that is considered for a decision. Large values indicate a conservative decisional style. Typical range: 0.5 < \code{a} < 2
#' @param v drift rate. Average slope of the information accumulation process. The drift gives information about the speed and direction of the accumulation of information. Large (absolute) values of drift indicate a good performance. If received information supports the response linked to the upper threshold the sign will be positive and vice versa. Typical range: -5 < \code{v} < 5
#' @param t0 non-decision time or response time constant (in seconds). Lower bound for the duration of all non-decisional processes (encoding and response execution). Typical range: 0.1 < \code{t0} < 0.5
#' @param z starting point. Indicator of an a priori bias in decision making. When the relative starting point \code{z} deviates from \code{0.5*a}, the amount of information necessary for a decision differs between response alternatives. Default is \code{0.5*a} (i.e., no bias).
#' @param d differences in speed of response execution (in seconds). Positive values indicate that response execution is faster for responses linked to the upper threshold than for responses linked to the lower threshold. Typical range: -0.1 < \code{d} < 0.1. Default is 0.
#' @param sz inter-trial-variability of starting point. Range of a uniform distribution with mean \code{z} describing the distribution of actual starting points from specific trials. Values different from 0 can predict fast errors (but can slow computation considerably). Typical range: 0 < \code{sz} < 0.5. Default is 0.
#' @param sv inter-trial-variability of drift rate. Standard deviation of a normal distribution with mean \code{v} describing the distribution of actual drift rates from specific trials. Values different from 0 can predict slow errors. Typical range: 0 < \code{sv} < 2. Default is 0.
#' @param st0 inter-trial-variability of non-decisional components. Range of a uniform distribution with mean \code{t0 + st0/2} describing the distribution of actual \code{t0} values across trials. Accounts for response times below \code{t0}. Reduces skew of predicted RT distributions. Values different from 0 can slow computation considerably. Typical range: 0 < \code{st0} < 0.2. Default is 0.
#' @param s diffusion constant; standard deviation of the random noise of the diffusion process (i.e., within-trial variability), scales \code{a}, \code{v}, and \code{sv}. Needs to be fixed to a constant in most applications. Default is 1. Note that the default used by Ratcliff and in other applications is often 0.1. 
#' 
#' @param precision \code{numerical} scalar value. Precision of calculation. Corresponds roughly to the number of decimals of the predicted CDFs that are calculated accurately. Default is 3.
#' @param maxt maximum \code{rt} allowed, used to stop integration problems. Larger values lead to considerably longer calculation times.
#' @param interval a vector containing the end-points of the interval to be searched for the desired quantiles (i.e., RTs) in \code{qdiffusion}. Default is \code{c(0, 10)}.
#' @param scale_p logical. Should entered probabilities automatically be scaled by maximally predicted probability? Default is \code{FALSE}. Convenience argument for obtaining predicted quantiles. Can be slow as the maximally predicted probability is calculated individually for each \code{p}.
#' @param scale_max numerical scalar. Value at which maximally predicted RT should be calculated if \code{scale_p} is \code{TRUE}. 
#' @param stop_on_error Should the diffusion functions return 0 if the parameters values are outside the allowed range (= \code{FALSE}) or produce an error in this case (= \code{TRUE}).
#' @param use_precise boolean. Should \code{pdiffusion} use the precise version for calculating the CDF? The default is \code{TRUE} which is highly recommended. Using \code{FALSE} (i.e., the imprecise version) is hardly any faster and produces clearly wrong results for most parameter settings.
#' @param max_diff numeric. Maximum acceptable difference between desired and observed probability of the quantile function (\code{qdiffusion}). 
#' @param method character. Experimentally implementation of an alternative way of generating random variates via the quantile function (\code{qdiffusion}) and random uniform value. For simple calls, the default method \code{"fastdm"} is dramatically faster.
#'
#' @return \code{ddiffusion} gives the density, \code{pdiffusion} gives the distribution function, \code{qdiffusion} gives the quantile function (i.e., predicted RTs), and \code{rdiffusion} generates random response times and decisions (returning a \code{data.frame} with columns \code{rt} (numeric) and \code{response} (factor)).
#' 
#' The length of the result is determined by \code{n} for \code{rdiffusion}, equal to the length of \code{rt} for \code{ddiffusion} and \code{pdiffusion}, and equal to the length of \code{p} for \code{qdiffusion}.
#' 
#' The distribution parameters (as well as \code{response}) are recycled to the length of the result. In other words, the functions are completely vectorized for all parameters and even the response boundary.
#'
#' @details The Ratcliff diffusion model (Ratcliff, 1978) is a mathematical model for two-choice discrimination tasks. It is based on the assumption that information is accumulated continuously until one of two decision thresholds is hit. For introductions see Ratcliff and McKoon (2008), Voss, Rothermund, and Voss (2004), Voss, Nagler, and Lerche (2013), or Wagenmakers (2009).
#' 
#' All functions are fully vectorized across all parameters as well as the response to match the length or \code{rt} (i.e., the output is always of length equal to \code{rt}). This allows for trialwise parameters for each model parameter. 
#' 
#' For convenience, all functions (with the exception of \code{rdiffusion}) allow that the first argument is a \code{data.frame} containing the information of the first and second argument in two columns (i.e., \code{rt}/\code{p} and \code{response}). Other columns (as well as passing \code{response} separately argument) will be ignored. This allows, for example, to pass the \code{data.frame} generated by \code{rdiffusion} directly to \code{pdiffusion}. See examples.
#' 
#' \subsection{Quantile Function}{
#' Due to the bivariate nature of the diffusion model, the diffusion processes reaching each response boundary only return the defective CDF that does not reach 1. Only the sum of the CDF for both boundaries reaches 1. Therefore, \code{qdiffusion} can only return quantiles/RTs for any accumulator up to the maximal probability of that accumulator's CDF. This can be obtained by evaluating the CDF at \code{Inf}. 
#' 
#' As a convenience for the user, if \code{scale_p = TRUE} in the call to \code{qdiffusion} the desired probabilities are automatically scaled by the maximal probability for the corresponding response. Note that this can be slow as the maximal probability is calculated separately for each desired probability. See examples.
#' 
#' Also note that quantiles (i.e., predicted RTs) are obtained by numerically minimizing the absolute difference between desired probability and the value returned from \code{pdiffusion} using \code{\link{optimize}}. If the difference between the desired probability and probability corresponding to the returned quantile is above a certain threshold (currently 0.0001) no quantile is returned but \code{NA}. This can be either because the desired quantile is above the maximal probability for this accumulator or because the limits for the numerical integration are too small (default is \code{c(0, 10)}).
#' }
#' 
#' @note The parameterization of the non-decisional components, \code{t0} and \code{st0}, differs from the parameterization used by, for example, Andreas Voss or Roger Ratcliff. In the present case \code{t0} is the lower bound of the uniform distribution of length \code{st0}, but \emph{not} its midpoint. The parameterization employed here is in line with the parametrization for the \link{LBA} code (where \code{t0} is also the lower bound).
#' 
#' The default diffusion constant \code{s} is 1 and not 0.1 as in most applications of Roger Ratcliff and others.
#' 
#' We have changed the parameterization of the start point \code{z} which is now the absolute start point in line with most published literature (it was the relative start point in previous versions of \pkg{rtdists}). 
#' 
#' @references Ratcliff, R. (1978). A theory of memory retrieval. \emph{Psychological Review}, 85(2), 59-108.
#' 
#' Ratcliff, R., & McKoon, G. (2008). The diffusion decision model: Theory and data for two-choice decision tasks. \emph{Neural Computation}, 20(4), 873-922.
#' 
#' Voss, A., Rothermund, K., & Voss, J. (2004). Interpreting the parameters of the diffusion model: An empirical validation. \emph{Memory & Cognition}. Vol 32(7), 32, 1206-1220.
#' 
#' Voss, A., Nagler, M., & Lerche, V. (2013). Diffusion Models in Experimental Psychology: A Practical Introduction. \emph{Experimental Psychology}, 60(6), 385-402. doi:10.1027/1618-3169/a000218
#' 
#' Wagenmakers, E.-J., van der Maas, H. L. J., & Grasman, R. P. P. P. (2007). An EZ-diffusion model for response time and accuracy. \emph{Psychonomic Bulletin & Review}, 14(1), 3-22.
#' 
#' Wagenmakers, E.-J. (2009). Methodological and empirical developments for the Ratcliff diffusion model of response times and accuracy. \emph{European Journal of Cognitive Psychology}, 21(5), 641-671.
#' 
#' 
#' @author Underlying C code by Jochen Voss and Andreas Voss. Porting and R wrapping by Matthew Gretton, Andrew Heathcote, Scott Brown, and Henrik Singmann. \code{qdiffusion} by Henrik Singmann.
#'
#' @useDynLib rtdists, .registration = TRUE
#'
#' @name Diffusion
# @importFrom utils head
#' @importFrom stats optimize uniroot runif
# @importFrom pracma integral
#' @aliases diffusion
#' @importFrom Rcpp evalCpp
#' 
#' @example examples/examples.diffusion.R
#' 


# [MG 20150616]
# In line with LBA, adjust t0 to be the lower bound of the non-decision time distribution rather than the average 
# Called from prd, drd, rrd 
recalc_t0 <- function (t0, st0) { t0 <- t0 + st0/2 }


prepare_diffusion_parameter <- function(response, 
                                        a, v, t0, z, d, 
                                        sz, sv, st0, s, 
                                        nn, 
                                        z_absolute = TRUE, 
                                        stop_on_error) {
  if(any(missing(a), missing(v), missing(t0))) stop("a, v, and/or t0 must be supplied")
  if ( (length(s) == 1) & 
       (length(a) == 1) & 
       (length(v) == 1) & 
       (length(t0) == 1) & 
       (length(z) == 1) & 
       (length(d) == 1) & 
       (length(sz) == 1) & 
       (length(sv) == 1) & 
       (length(st0) == 1)) {
    skip_checks <- TRUE
  } else {
    skip_checks <- FALSE
  }
  
  # Build parameter matrix  
  # Convert boundaries to numeric if necessary
  if (is.character(response)) {
    response <- match.arg(response, choices=c("upper", "lower"),several.ok = TRUE)
    numeric_bounds <- ifelse(response == "upper", 2L, 1L)
  }
  else {
    response <- as.numeric(response)
    if (any(!(response %in% 1:2))) 
      stop("response needs to be either 'upper', 'lower', or as.numeric(response) %in% 1:2!")
    numeric_bounds <- as.integer(response)
  }
  
  numeric_bounds <- rep(numeric_bounds, length.out = nn)
  if (!skip_checks) {
    # all parameters brought to length of rt
    s <- rep(s, length.out = nn)
    a <- rep(a, length.out = nn)
    v <- rep(v, length.out = nn)
    t0 <- rep(t0, length.out = nn)
    z <- rep(z, length.out = nn)
    d <- rep(d, length.out = nn)
    sz <- rep(sz, length.out = nn)
    sv <- rep(sv, length.out = nn)
    st0 <- rep(st0, length.out = nn)
  }
  if (z_absolute) {
    z <- z/a  # transform z from absolute to relative scale (which is currently required by the C code)
    sz <- sz/a # transform sz from absolute to relative scale (which is currently required by the C code)
  }
  t0 <- recalc_t0 (t0, st0) 
  
  # Build parameter matrix (and divide a, v, and sv, by s)
  params <- cbind (a/s, v/s, t0, d, sz, sv/s, st0, z, numeric_bounds)
  
  # Check for illegal parameter values
  if(ncol(params)<9) stop("Not enough parameters supplied: probable attempt to pass NULL values?")
  if(!is.numeric(params)) stop("Parameters need to be numeric.")
  if (any(is.na(params)) || !all(is.finite(params))) {
    if (stop_on_error) stop("Parameters need to be numeric and finite.")
  }
  
  
  if (!skip_checks) {
    parameter_char <- apply(params, 1, paste0, collapse = "\t")
    parameter_factor <- factor(parameter_char, levels = unique(parameter_char))
    parameter_indices <- split(seq_len(nn), f = parameter_factor)
  } else {
    if (all(numeric_bounds == 2L) | all(numeric_bounds == 1L)) {
      parameter_indices <- list(
        seq_len(nn)
      )
    } else {
      parameter_indices <- list(
        seq_len(nn)[numeric_bounds == 2L], 
        seq_len(nn)[numeric_bounds == 1L]
      )  
    }
  }
  list(
    params = params
    , parameter_indices = parameter_indices
  )
}


#' @rdname Diffusion
#' @export
ddiffusion <- function (rt, response = "upper", 
                 a, v, t0, z = 0.5*a, d = 0, sz = 0, sv = 0, st0 = 0, s = 1,
                 precision = 3, stop_on_error = FALSE)
{
  # for convenience accept data.frame as first argument.
  if (is.data.frame(rt)) {
    response <- rt$response
    rt <- rt$rt
  }
  
  nn <- length(rt)
  
  pars <- prepare_diffusion_parameter(response = response, 
                              a = a, v = v, t0 = t0, z = z, 
                              d = d, sz = sz, sv = sv, st0 = st0, s = s, 
                              nn = nn, stop_on_error = stop_on_error)
  
  densities <- vector("numeric",length=nn)  
  for (i in seq_len(length(pars$parameter_indices))) {
    ok_rows <- pars$parameter_indices[[i]]

    densities[ok_rows] <- d_fastdm (rt[ok_rows], 
                                pars$params[ok_rows[1],1:8], 
                                precision, 
                                pars$params[ok_rows[1],9], 
                                stop_on_error)
  }
  abs(densities)
}


## @param stop.on.error logical. If true (the default) an error stops the \code{integration} of \code{pdiffusion}. If false some errors will give a result with a warning in the message component.
#' @rdname Diffusion
#' @export
pdiffusion <- function (rt, response = "upper",
                 a, v, t0, z = 0.5*a, d = 0, sz = 0, sv = 0, st0 = 0, s = 1,
                 precision = 3, maxt = 20, stop_on_error = FALSE, use_precise = TRUE)
{
  if(any(missing(a), missing(v), missing(t0))) stop("a, v, and/or t0 must be supplied")
  # for convenience accept data.frame as first argument.
  if (is.data.frame(rt)) {
    response <- rt$response
    rt <- rt$rt
  }
  
  rt[rt>maxt] <- maxt
  # if(!all(rt == sort(rt)))  stop("rt needs to be sorted")
  
  # Convert boundaries to numeric
  nn <- length(rt)
  
  pars <- prepare_diffusion_parameter(response = response, 
                              a = a, v = v, t0 = t0, z = z, 
                              d = d, sz = sz, sv = sv, st0 = st0, s = s, 
                              nn = nn, stop_on_error = stop_on_error)
  
  pvalues <- vector("numeric",length=nn)  
  
  if (use_precise) {
    for (i in seq_len(length(pars$parameter_indices))) {
      ok_rows <- pars$parameter_indices[[i]]
      pvalues[ok_rows] <- p_precise_fastdm (rt[ok_rows], 
                                        pars$params[ok_rows[1],1:8], 
                                        precision, 
                                        pars$params[ok_rows[1],9], 
                                        stop_on_error)
    }
  } else {
    for (i in seq_len(length(pars$parameter_indices))) {
      ok_rows <- pars$parameter_indices[[i]]
      pvalues[ok_rows] <- p_fastdm (rt[ok_rows], 
                                pars$params[ok_rows[1],1:8], 
                                precision, 
                                pars$params[ok_rows[1],9], 
                                stop_on_error)
    }
  }
  #pvalues <- unsplit(densities, f = parameter_factor)
  pvalues
}


inv_cdf_diffusion <- function(x, response, a, v, t0, z, d, sz, sv, st0, s, precision, maxt, value, abs = TRUE, stop_on_error = TRUE) {
  if (abs) abs(value - pdiffusion(rt=x, response=response, a=a, v=v, t0=t0, z=z, d=d, sz=sz, sv=sv, s=s, st0=st0, precision=precision, maxt=maxt, stop_on_error))
  else (value - pdiffusion(rt=x, response=response, a=a, v=v, t0=t0, z=z, d=d, sz=sz, sv=sv, st0=st0, s=s, precision=precision, maxt=maxt, stop_on_error))
}

#' @rdname Diffusion
#' @export
qdiffusion <- function (p, response = "upper", 
                 a, v, t0, z = 0.5*a, d = 0, sz = 0, sv = 0, st0 = 0, s = 1,
                 precision = 3, maxt = 20, interval = c(0, 10),
                 scale_p = FALSE, scale_max = Inf, stop_on_error = FALSE, 
                 max_diff = 0.0001)
{
  if(any(missing(a), missing(v), missing(t0))) stop("a, v, and t0 must be supplied")

  # for convenience accept data.frame as first argument.
  if (is.data.frame(p)) {
    response <- p$response
    p <- p$p
  }
  
  nn <- length(p)
  response <- rep(unname(response), length.out = nn)
  s <- rep(s, length.out = nn) # pass s to other functions for correct handling! No division here.
  a <- rep(unname(a), length.out = nn)
  v <- rep(unname(v), length.out = nn)
  t0 <- rep(unname(t0), length.out = nn)
  z <- rep(unname(z), length.out = nn)
  d <- rep(unname(d), length.out = nn)
  sz <- rep(unname(sz), length.out = nn)
  sv <- rep(unname(sv), length.out = nn)
  st0 <- rep(unname(st0), length.out = nn)
  p <- unname(p)
  
  op <- order(p)
  
  max_interval <- interval[2] - interval[1]
  steps <- max_interval/nn * 4
  
  out <- vector("numeric", nn)
  for (i in seq_len(nn)) {
    if (scale_p) max_p <- pdiffusion(scale_max, response=response[op[i]], 
                                     a=a[op[i]], v=v[op[i]], t0=t0[op[i]], z=z[op[i]], 
                                     d=d[op[i]], sz=sz[op[i]], sv=sv[op[i]], st0=st0[op[i]], 
                                     s=s[op[i]], 
                                     precision=precision, maxt=maxt, 
                                     stop_on_error=stop_on_error)
    else max_p <- 1
    tmp <- list(objective = 1)
    if (i > 1  && !is.na(out[op[i-1]])) {
      tmp <- do.call(optimize, args = 
                       c(f = inv_cdf_diffusion, 
                         interval = list(c(out[op[i-1]], out[op[i-1]]+steps)), 
                         response=response[op[i]], 
                         a=a[op[i]], v=v[op[i]], t0=t0[op[i]], 
                         z=z[op[i]], d=d[op[i]], sz=sz[op[i]], 
                         sv=sv[op[i]], st0=st0[op[i]], s=s[op[i]], 
                         precision=precision, maxt=maxt, 
                         stop_on_error=stop_on_error, 
                         value =p[op[i]]*max_p, tol = .Machine$double.eps^0.5))
    }

    if (tmp$objective > max_diff) {
      tmp <- do.call(optimize, args = 
                       c(f = inv_cdf_diffusion, 
                         interval = list(c(max(interval[1], t0), interval[2])), 
                         response=response[op[i]], 
                         a=a[op[i]], v=v[op[i]], t0=t0[op[i]], z=z[op[i]], d=d[op[i]], sz=sz[op[i]], 
                         sv=sv[op[i]], st0=st0[op[i]], s=s[op[i]], 
                         precision=precision, maxt=maxt, 
                         stop_on_error=stop_on_error, 
                         value =p[op[i]]*max_p, tol = .Machine$double.eps^0.5))
    }
    if (tmp$objective > max_diff) {
      tmp <- do.call(optimize, args = 
                       c(f=inv_cdf_diffusion, 
                         interval = list(c(max(interval[1], t0),max(interval)/2)), 
                         response=response[op[i]], a=a[op[i]], v=v[op[i]], t0=t0[op[i]], 
                         z=z[op[i]], d=d[op[i]], sz=sz[op[i]], sv=sv[op[i]], 
                         st0=st0[op[i]], s=s[op[i]], precision=precision, 
                         maxt=maxt, stop_on_error=stop_on_error, 
                         value =p[op[i]]*max_p, tol = .Machine$double.eps^0.5))
    }
    if (tmp$objective > max_diff) {
      try({
        uni_tmp <- do.call(uniroot, args = 
                             c(f=inv_cdf_diffusion, 
                               interval = list(c(max(interval[1], t0), interval[2])), 
                               response=response[op[i]], 
                               a=a[op[i]], v=v[op[i]], t0=t0[op[i]], z=z[op[i]], d=d[op[i]], 
                               sz=sz[op[i]], sv=sv[op[i]], st0=st0[op[i]], 
                               precision=precision, s=s[op[i]], maxt=maxt, 
                               stop_on_error=stop_on_error, 
                               value =p[op[i]]*max_p, 
                               tol = .Machine$double.eps^0.5, abs = FALSE))
      tmp$objective <- uni_tmp$f.root
      tmp$minimum <- uni_tmp$root
      }, silent = TRUE)
    }
    if (tmp$objective > max_diff) {
      tmp[["minimum"]] <- NA
      warning("Cannot obtain RT that is less than ", max_diff, 
              " away from desired p = ", p[op[i]], 
              ".\nIncrease/decrease interval or obtain for different response.", 
              call. = FALSE)
    }
    out[op[i]] <- tmp[["minimum"]]
  }
  return(out)
}

## When given vectorised parameters, n is the number of replicates for each parameter set
#' @rdname Diffusion
#' @export
rdiffusion <- function (n, 
                 a, v, t0, z = 0.5*a, d = 0, sz = 0, sv = 0, st0 = 0, s = 1,
                 precision = 3, stop_on_error = TRUE, 
                 maxt = 20, interval = c(0, 10), 
                 method = c("fastdm", "qdiffusion"))
{
  if(any(missing(a), missing(v), missing(t0))) stop("a, v, and/or t0 must be supplied")
  method <- match.arg(method)
  
  if (method == "fastdm") {
    pars <- prepare_diffusion_parameter(response = 1L, 
                              a = a, v = v, t0 = t0, z = z, 
                              d = d, sz = sz, sv = sv, st0 = st0, s = s, 
                              nn = n, stop_on_error = stop_on_error)
    
    randRTs    <- vector("numeric",length=n)
    randBounds <- vector("numeric",length=n)
    
    for (i in seq_len(length(pars$parameter_indices))) {
      ok_rows <- pars$parameter_indices[[i]]
      
      # Calculate n for this row
      current_n <- length(ok_rows)
      
      out <- r_fastdm (current_n, 
                       pars$params[ok_rows[1],1:8], 
                       precision, 
                       stop_on_error=stop_on_error)
      #current_n, uniques[i,1:8], precision, stop_on_error=stop_on_error)
      
      randRTs[ok_rows]    <- out$rt       
      randBounds[ok_rows] <- out$boundary 
    }
    # badrt <- randRTs <=  pars$params[,3]
    # if (any(badrt)) {
    #   randRTs[badrt] <- pars$params[badrt,3] + 1.5e-8
    # }
    response <- factor(randBounds, levels = 0:1, labels = c("lower", "upper"))
    return(data.frame(rt = randRTs, response))
  } else if (method == "qdiffusion") {
    s <- rep(s, length.out = n)
    a <- rep(a, length.out = n)
    v <- rep(v, length.out = n)
    t0 <- rep(t0, length.out = n)
    z <- rep(z, length.out = n)
    #z <- z/a  # transform z from absolute to relative scale (which is currently required by the C code)
    d <- rep(d, length.out = n)
    sz <- rep(sz, length.out = n)
    #sz <- sz/a # transform sz from absolute to relative scale (which is currently required by the C code)
    sv <- rep(sv, length.out = n)
    st0 <- rep(st0, length.out = n)
    t0 <- recalc_t0 (t0, st0) 
    
    # Build parameter matrix (and divide a, v, and sv, by s)
    params <- cbind (a, v, t0, d, sz, sv, st0, z)
    
    # Check for illegal parameter values
    if(ncol(params)<8) stop("Not enough parameters supplied: probable attempt to pass NULL values?")
    if(!is.numeric(params)) stop("Parameters need to be numeric.")
    if (any(is.na(params)) || !all(is.finite(params))) stop("Parameters need to be numeric and finite.")
    
    randRTs    <- vector("numeric",length=n)
    randBounds <- vector("numeric",length=n)
    
    #uniques <- unique(params)
    parameter_char <- apply(params, 1, paste0, collapse = "\t")
    parameter_factor <- factor(parameter_char, levels = unique(parameter_char))
    parameter_indices <- split(seq_len(n), f = parameter_factor)
    
    for (i in seq_len(length(parameter_indices))) {
      ok_rows <- parameter_indices[[i]]
      
      # Calculate n for this row
      current_n <- length(ok_rows)
      
      mu <- pdiffusion(rt = Inf, response = "upper", 
                       a = a[ok_rows[1]], v = v[ok_rows[1]], 
                       t0 = t0[ok_rows[1]], z = z[ok_rows[1]], 
                       d = d[ok_rows[1]], sz = sz[ok_rows[1]], 
                       sv = sv[ok_rows[1]], st0 = st0[ok_rows[1]], 
                       s = s[ok_rows[1]], precision = precision, 
                       maxt = maxt)
      
      unif_variates <- runif(current_n)
      
      sel_u <- unif_variates < mu
      sel_l <- !sel_u
      
      unif_variates_u <- unif_variates[sel_u]
      unif_variates_l <- 1-unif_variates[sel_l]
      
      qdiffusion(p = unif_variates_u, response = "upper", 
                 a = a[ok_rows[1]], v = v[ok_rows[1]], 
                 t0 = t0[ok_rows[1]], z = z[ok_rows[1]], 
                 d = d[ok_rows[1]], sz = sz[ok_rows[1]], 
                 sv = sv[ok_rows[1]], st0 = st0[ok_rows[1]], 
                 s = s[ok_rows[1]], precision = precision, 
                 maxt = maxt, interval = interval, 
                 scale_p = FALSE)
      
      randRTs[ok_rows[sel_u]] <- 
        qdiffusion(p = unif_variates_u, response = "upper", 
                   a = a[ok_rows[1]], v = v[ok_rows[1]], 
                   t0 = t0[ok_rows[1]], z = z[ok_rows[1]], 
                   d = d[ok_rows[1]], sz = sz[ok_rows[1]], 
                   sv = sv[ok_rows[1]], st0 = st0[ok_rows[1]], 
                   s = s[ok_rows[1]], precision = precision, 
                   maxt = maxt, interval = interval, 
                   scale_p = FALSE)
      randRTs[ok_rows[sel_l]] <- 
        qdiffusion(p = unif_variates_l, response = "lower", 
                   a = a[ok_rows[1]], v = v[ok_rows[1]], 
                   t0 = t0[ok_rows[1]], z = z[ok_rows[1]], 
                   d = d[ok_rows[1]], sz = sz[ok_rows[1]], 
                   sv = sv[ok_rows[1]], st0 = st0[ok_rows[1]], 
                   s = s[ok_rows[1]], precision = precision, 
                   maxt = maxt, interval = interval, 
                   scale_p = FALSE)
      randBounds[ok_rows] <- ifelse(sel_u, 1, 0)
    }
    response <- factor(randBounds, levels = 0:1, labels = c("lower", "upper"))
    return(data.frame(rt = randRTs, response))
  }
}

context("diffusion parameter input (via rdiffusion)")

test_that("check individual parameters:", {
  expect_that(rdiffusion(10, a=1, z=0.5, v=2, t0=0.5, d=0, sz = 0, sv = 0, st0 = 0), is_a("data.frame"))
  expect_that(suppressWarnings(rdiffusion(10, a=1, z=0.5, v=2, t0=0.5, d=0, sz = 0, sv = 0, st0 = NULL)), throws_error("Not enough parameters"))
  expect_that(rdiffusion(10, a=1, z=0.5, v=2, t0=0.5, d=0, sz = 0, sv = Inf, st0 = 0), throws_error())
  expect_that(suppressWarnings(rdiffusion(10, a=1, z=NA, v=2, t0=0.5,  d=0, sz = 0, sv = 0, st0 = 0)), throws_error())  
})

# test_that("check parameters:", {
#   p1 <- c(1, 0.5, 2, 0.5, 0, 0, 0, 0)
#   expect_that(rdiffusion(10, parameters = p1), is_a("data.frame"))
#   expect_that(rdiffusion(10, parameters = p1[1:7]), throws_error())
#   names(p1) <- c("a", "z", "v","t0","d", "sz","sv","st0")
#   expect_that(rdiffusion(10, parameters = p1), is_a("data.frame"))
#   names(p1) <- c(c("a","v","t0","z"), sample(c("sz","sv","st0", "d")))
#   expect_that(rdiffusion(10, parameters = p1), is_a("data.frame"))
#   names(p1)[3] <- "xx"
#   expect_that(rdiffusion(10, parameters = p1), throws_error())
#   names(p1) <- NULL
#   p1[1] <- NA
#   expect_that(rdiffusion(10, parameters = p1), throws_error())
#   p1[1] <- Inf
#   expect_that(rdiffusion(10, parameters = p1), throws_error())
# })


context("rdiffusion: random number generation for the diffusion model")
test_that("rdiffusion works", {
  rrds1 <- rdiffusion(10, a=1, z=0.5, v=2, t0=0.5, d=0, sz = 0, sv = 0, st0 = 0)
  rrds2 <- rdiffusion(10, a=1, z=0.5, v=2, t0=0.5, d=0, sz = 0, sv = 0, st0 = 0)
  expect_that(rrds1, is_a("data.frame"))
  expect_false(isTRUE(all.equal(rrds1, rrds2)))
  
  set.seed(1)
  rrds1 <- rdiffusion(10, a=1, z=0.5, v=2, t0=0.5, d=0, sz = 0, sv = 0, st0 = 0)
  set.seed(1)
  rrds2 <- rdiffusion(10, a=1, z=0.5, v=2, t0=0.5, d=0, sz = 0, sv = 0, st0 = 0)
  expect_that(rrds1, equals(rrds2))
  set.seed(NULL)
})

context("Diffusion pdiffusion and ddiffusion functions.")

test_that("diffusion functions work with numeric and factor boundaries", {
  n_test <- 20
  rts <- rdiffusion(n_test, a=1, z=0.5, v=2, t0=0.5, d=0, sz = 0, sv = 0, st0 = 0)
  expect_is(ddiffusion(rts$rt, response = rts$response, a=1, z=0.5, v=2, t0=0.5, d=0, sz = 0, sv = 0, st0 = 0), "numeric")
  expect_is(pdiffusion(sort(rts$rt), response = rts$response, a=1, z=0.5, v=2, t0=0.5, d=0, sz = 0, sv = 0, st0 = 0), "numeric")
  expect_is(ddiffusion(rts$rt, response = sample(1:2, 20, replace = TRUE), a=1, z=0.5, v=2, t0=0.5, d=0, sz = 0, sv = 0, st0 = 0), "numeric")
  expect_is(pdiffusion(sort(rts$rt), response = sample(1:2, 20, replace = TRUE), a=1, z=0.5, v=2, t0=0.5, d=0, sz = 0, sv = 0, st0 = 0), "numeric")
  expect_error(ddiffusion(rts$rt, rep_len(1:3, length.out=20), a=1, z=0.5, v=2, t0=0.5, d=0, sz = 0, sv = 0, st0 = 0), "response")
  expect_error(pdiffusion(sort(rts$rt), rep_len(1:3, length.out=20), a=1, z=0.5, v=2, t0=0.5, d=0, sz = 0, sv = 0, st0 = 0), "response")
})

test_that("diffusion functions are identical with all input options", {
  rt1 <- rdiffusion(500, a=1, v=2, t0=0.5)
  # get density for random RTs:
  ref <- sum(log(ddiffusion(rt1$rt, rt1$response, a=1, v=2, t0=0.5)))  # response is factor
  expect_identical(sum(log(ddiffusion(rt1$rt, as.numeric(rt1$response), a=1, v=2, t0=0.5))),
                   ref)
  expect_identical(sum(log(ddiffusion(rt1$rt, as.character(rt1$response), a=1, v=2, t0=0.5))),
                   ref)
  expect_identical(sum(log(ddiffusion(rt1, a=1, v=2, t0=0.5))), ref)

  rt2 <- rt1[order(rt1$rt),]

  ref2 <- pdiffusion(rt2$rt, rt2$response, a=1, v=2, t0=0.5)
  expect_identical(pdiffusion(rt2$rt, as.numeric(rt2$response), a=1, v=2, t0=0.5), ref2)
  expect_identical(pdiffusion(rt2$rt, as.character(rt2$response), a=1, v=2, t0=0.5), ref2)
  expect_identical(pdiffusion(rt2, a=1, v=2, t0=0.5), ref2)

#   rt3 <- data.frame(p = rep(seq(0.1, 0.9, 0.2), 2),
#                     response = rep(c("upper", "lower"), each = 5))

  rt3 <- data.frame(p = rep(c(0.05, 0.1), 2),
                    response = factor(rep(c("upper", "lower"), each = 2)))
  ref3 <- qdiffusion(rt3$p, rt3$response, a=1, v=2, t0=0.5)
  expect_identical(qdiffusion(rt3$p, as.numeric(rt3$response), a=1, v=2, t0=0.5), ref3)
  expect_identical(qdiffusion(rt3$p, as.character(rt3$response), a=1, v=2, t0=0.5), ref3)
  expect_identical(qdiffusion(rt3, a=1, v=2, t0=0.5), ref3)

})


test_that("qdiffusion is equivalent to manual calculation",{
  p11_fit <- structure(list(par = structure(c(1.32060063610882, 3.27271614698074, 0.338560144920614, 0.34996447540773, 0.201794924457386, 1.05516829794661), .Names = c("a", "v", "t0", "sz", "st0", "sv"))))
  q <- c(0.1, 0.3, 0.5, 0.7, 0.9)

  i_pdiffusion <- function(x, args, value, response) {
    abs(value - do.call(pdiffusion, args = c(rt = x, args, response = response)))
  }
  prop_correct <- pdiffusion(Inf, response = "upper", a=p11_fit$par["a"], v=p11_fit$par["v"], t0=p11_fit$par["t0"], sz=p11_fit$par["sz"], st0=p11_fit$par["st0"], sv=p11_fit$par["sv"])
  pred_dir <- sapply(q*prop_correct, function(y) optimize(i_pdiffusion, c(0, 5), args = as.list(p11_fit$par), value = y, response = "upper")[[1]])
  
  expect_equal(qdiffusion(q, response = "upper", a=p11_fit$par["a"], v=p11_fit$par["v"], t0=p11_fit$par["t0"], sz=p11_fit$par["sz"], st0=p11_fit$par["st0"], sv=p11_fit$par["sv"], scale_p = TRUE),pred_dir, tolerance=0.001)

  expect_equal(suppressWarnings(qdiffusion(q, response = "lower", a=p11_fit$par["a"], v=p11_fit$par["v"], t0=p11_fit$par["t0"], sz=p11_fit$par["sz"]*p11_fit$par["a"], st0=p11_fit$par["st0"], sv=p11_fit$par["sv"])),as.numeric(rep(NA, 5)))
})

test_that("s works as expected", {
  set.seed(1)
  x <- rdiffusion(n = 100, a = 1, v = 2, t0 = 0.3, z = 0.5, s = 1)
  set.seed(1)
  y <- rdiffusion(n = 100, a = 0.1, v = 0.2, t0 = 0.3, z = 0.05, s = 0.1)
  expect_identical(x, y)
  set.seed(1)
  z <- rdiffusion(n = 100, a = 0.1, v = 0.2, t0 = 0.3, s = 0.1)
  expect_identical(x, z)
  expect_identical(
    ddiffusion(x[x$response == "upper", "rt"], a = 1, v = 2, t0 = 0.3, z = 0.5, s=1),
    ddiffusion(x[x$response == "upper", "rt"], a = 0.1, v = 0.2, t0 = 0.3, z = 0.05, s=0.1)
    )
  expect_identical(
    ddiffusion(x[x$response == "upper", "rt"], a = 1, v = 2, t0 = 0.3, z = 0.5, s=1),
    ddiffusion(x[x$response == "upper", "rt"], a = 0.1, v = 0.2, t0 = 0.3, s=0.1)
    )
  expect_identical(
    pdiffusion(sort(x[x$response == "upper", "rt"]), a = 1, v = 2, t0 = 0.3, z = 0.5, s=1),
    pdiffusion(sort(x[x$response == "upper", "rt"]), a = 0.1, v = 0.2, t0 = 0.3, z = 0.05, s=0.1)
  )
  expect_identical(
    pdiffusion(sort(x[x$response == "upper", "rt"]), a = 1, v = 2, t0 = 0.3, z = 0.5, s=1),
    pdiffusion(sort(x[x$response == "upper", "rt"]), a = 0.1, v = 0.2, t0 = 0.3, s=0.1)
  )
  expect_identical(
    qdiffusion(0.6, a = 1, v = 2, t0 = 0.3, z = 0.5, s=1),
    qdiffusion(0.6, a = 0.1, v = 0.2, t0 = 0.3, z = 0.05, s=0.1)
  )
  expect_identical(
    qdiffusion(0.6, a = 1, v = 2, t0 = 0.3, z = 0.5, s=1),
    qdiffusion(0.6, a = 0.1, v = 0.2, t0 = 0.3, s=0.1)
  )
})

test_that("scale_p works as expected", {
  (max_p <- pdiffusion(20, a=1, v=2, t0=0.5, st0=0.2, sz = 0.1, sv = 0.5, response="u"))
  # [1] 0.8705141
  # to get predicted quantiles, scale required quantiles by maximally predicted response rate:
  qs <- c(.1, .3, .5, .7, .9)
  expect_equal(qdiffusion(qs*max_p, a=1, v=2, t0=0.5, st0=0.2, sz = 0.1, sv = 0.5, response="u"),
                    qdiffusion(qs, a=1, v=2, t0=0.5, st0=0.2, sz = 0.1, sv = 0.5, response="u", scale_p = TRUE))

})

test_that("rdiffusion recovers Table 1 from Wagenmakers et al. (2007)", {
  set.seed(4)
  n <- 1e4 # number of samples
  # take parameter valeus from Table 2 and set s to 0.1
  george <- rdiffusion(n, a = 0.12, v = 0.25, t0 = 0.3, s = 0.1)
  rich   <- rdiffusion(n, a = 0.12, v = 0.25, t0 = 0.25, s = 0.1)
  amy    <- rdiffusion(n, a = 0.08, v = 0.25, t0 = 0.3, s = 0.1)
  mark   <- rdiffusion(n, a = 0.08, v = 0.25, t0 = 0.25, s = 0.1)

  george$id <- "george"
  rich$id <- "rich"
  amy$id <- "amy"
  mark$id <- "mark"

  wag <- rbind(george, rich, amy, mark)
  wag$id <- factor(wag$id, levels = c("george", "rich", "amy", "mark"))

  expect_equal(aggregate(rt ~ id, wag, mean)$rt, c(0.517, 0.467, 0.422, 0.372), tolerance = 0.003)

  expect_equal(aggregate(as.numeric(response)-1 ~ id, wag, mean)[,2], c(0.953, 0.953, 0.881, 0.881), tolerance = 0.01)

  expect_equal(aggregate(rt ~ id, wag, var)$rt, c(0.024, 0.024, 0.009, 0.009), tolerance = 0.01)
})


test_that("pdiffusion recovers proportions of Table 1 from Wagenmakers et al. (2007)", {

  props <- pdiffusion(rep(Inf, 4), a = rep(c(0.12, 0.08), each = 2), v = 0.25, t0 = c(0.3, 0.25), s = 0.1)
  expect_equal(props, c(0.953, 0.953, 0.881, 0.881), tolerance = 0.001)

  props <- pdiffusion(rep(Inf, 4), a = rep(c(0.12, 0.08), each = 2), v = 0.25, t0 = c(0.3, 0.25), z = rep(c(0.06, 0.04), each = 2), s = 0.1)
  expect_equal(props, c(0.953, 0.953, 0.881, 0.881), tolerance = 0.001)
})

test_that("ddiffusion and dwiener give same log-likelihoods when fitted to RNG data", {
  skip_if_not_installed("RWiener")
  library("RWiener")
  
  set.seed(1)
  ## identical calls (but different random values)
  rt1 <- rdiffusion(500, a=1, v=2, t0=0.5)
  
  ll_diffusion <- function(pars, rt, response) 
  {
    densities <- ddiffusion(rt, response=response, 
                            a=pars[1], v=pars[2], t0=pars[3])
    if (any(densities == 0)) return(1e6)
    return(-sum(log(densities)))
  }
  
  ll_wiener <- function(pars, rt, response) 
  {
    densities <- dwiener(q = rt, resp=response, 
                         alpha = pars[1], delta = pars[2], tau = pars[3], 
                         beta = 0.5)
    if (any(densities == 0)) return(1e6)
    return(-sum(log(densities)))
  }
  
  start <- c(runif(2, 0.5, 3), 0.1, runif(3, 0, 0.5))
  names(start) <- c("a", "v", "t0")
  recov_rtdists <- nlminb(start, ll_diffusion, lower = 0.1, 
                          rt=rt1$rt, response=rt1$response)
  recov_rwiener <- nlminb(start, ll_wiener, lower = 0.1, 
                          rt=rt1$rt, response=rt1$response)
  expect_equivalent(recov_rtdists, recov_rwiener)
})

context("pdiffusion functions: RNG is equivalent to pdiffusion")

#x <- .Random.seed
set.seed(3)

tryCatch.W.E <- function(expr)
{
  mc <- match.call()
  mc2 <- match.call(definition = ks.test, call =  as.call(mc[[2]]))
  mc2[[1]] <- list
  
  W <- NULL
  w.handler <- function(w){ # warning handler
    W <<- w
    invokeRestart("muffleWarning")
  }
  list(value = withCallingHandlers(tryCatch(expr, error = function(e) e), warning = w.handler),warning = W, data = eval(mc2, envir = parent.frame()))
}

conditional_save_t <- function(t, distribution) {
  mc <- match.call()
  ex_data <- t$data
  #if (!is.null(t$warning)) save(ex_data, file = paste0(mc[[2]], "_", distribution, "_problem.Rdata"))
  #browser()
  #str(t)  
}

test_that("Norm: pdiffusion corresponds to random derivates with specific values", {
  testthat::skip_on_cran()
  #testthat::skip_on_travis()
  normalised_pdiffusion = function(rt,...) pdiffusion(rt,...)/pdiffusion(rt=Inf,...) 
  samples <- 1e3
  p_min <- 0.01
  p_max <- 0.01
  diffusion_pars <- structure(list(par = structure(c(1.32060063610882, 3.27271614698074, 0.338560144920614, 0.34996447540773, 0.201794924457386, 1.05516829794661), .Names = c("a", "v", "t0", "sz", "st0", "sv"))))
  
  r_diff1 <- rdiffusion(samples, a=diffusion_pars$par["a"], v=diffusion_pars$par["v"], t0=diffusion_pars$par["t0"], sz=diffusion_pars$par["sz"], st0=diffusion_pars$par["st0"], sv=diffusion_pars$par["sv"])
  t1 <- tryCatch.W.E(ks.test(r_diff1$rt[r_diff1$response=="upper"], normalised_pdiffusion, a=1.1, v=diffusion_pars$par["v"], t0=diffusion_pars$par["t0"], sz=diffusion_pars$par["sz"], st0=diffusion_pars$par["st0"], sv=diffusion_pars$par["sv"]))
  expect_lt(t1$value$p.value, p_min)
  conditional_save_t(t1, "norm")
  
  t2 <- tryCatch.W.E(ks.test(r_diff1$rt[r_diff1$response=="upper"], normalised_pdiffusion, a=diffusion_pars$par["a"], v=diffusion_pars$par["v"]-0.5, t0=diffusion_pars$par["t0"], sz=diffusion_pars$par["sz"], st0=diffusion_pars$par["st0"], sv=diffusion_pars$par["sv"]))
  expect_lt(t2$value$p.value, p_min)
  conditional_save_t(t2, "norm")
  
  t3 <- tryCatch.W.E(ks.test(r_diff1$rt[r_diff1$response=="upper"], normalised_pdiffusion, a=diffusion_pars$par["a"], v=diffusion_pars$par["v"]-0.5, t0=diffusion_pars$par["t0"], sz=diffusion_pars$par["sz"], st0=diffusion_pars$par["st0"], sv=diffusion_pars$par["sv"]))
  expect_lt(t3$value$p.value, p_min)
  conditional_save_t(t3, "norm")
  
  t4 <- tryCatch.W.E(ks.test(r_diff1$rt[r_diff1$response=="upper"], normalised_pdiffusion, a=diffusion_pars$par["a"], v=diffusion_pars$par["v"], t0=diffusion_pars$par["t0"]-0.1, sz=diffusion_pars$par["sz"], st0=diffusion_pars$par["st0"], sv=diffusion_pars$par["sv"]))
  expect_lt(t4$value$p.value, p_min)
  conditional_save_t(t4, "norm")
  
  t5 <- tryCatch.W.E(ks.test(r_diff1$rt[r_diff1$response=="upper"], normalised_pdiffusion, a=diffusion_pars$par["a"], v=diffusion_pars$par["v"], t0=diffusion_pars$par["t0"], sz=diffusion_pars$par["sz"], st0=diffusion_pars$par["st0"], sv=diffusion_pars$par["sv"]))
  conditional_save_t(t5, "norm")
  
  expect_gt(t5$value$p.value, p_max)
  
  #if (any(sapply(list(t1, t2, t3, t4, t5), function(x) !is.null(x$warning)))) browser()
})



test_that("Norm: pdiffusion corresponds to random derivates with random values", {
  testthat::skip_on_cran()
  #testthat::skip_on_travis()
  normalised_pdiffusion = function(rt,...) pdiffusion(rt,...)/pdiffusion(rt=Inf,...) 
  samples <- 5e3
  p_min <- 0.01
  p_max <- 0.01
  diffusion_pars <- list(
    a = runif(1, 0.5, 1.5),
    v = runif(1, 2, 3.5),
    t0 = runif(1, 0.2, 0.4),
    sz = runif(1, 0.1, 0.2),
    st0 = runif(1, 0.1, 0.2),
    sv = runif(1, 0.5, 1.5)
  )
    
  r_diff1 <- rdiffusion(samples, a=diffusion_pars[["a"]], v=diffusion_pars[["v"]], t0=diffusion_pars[["t0"]], sz=diffusion_pars[["sz"]], st0=diffusion_pars[["st0"]], sv=diffusion_pars[["sv"]])
  t1 <- tryCatch.W.E(ks.test(r_diff1$rt[r_diff1$response=="upper"], normalised_pdiffusion, a=diffusion_pars[["a"]]-0.3, v=diffusion_pars[["v"]], t0=diffusion_pars[["t0"]], sz=diffusion_pars[["sz"]], st0=diffusion_pars[["st0"]], sv=diffusion_pars[["sv"]]))
  expect_lt(t1$value$p.value, p_min)
  conditional_save_t(t1, "norm")
  
  t2 <- tryCatch.W.E(ks.test(r_diff1$rt[r_diff1$response=="upper"], normalised_pdiffusion, a=diffusion_pars[["a"]], v=pmin(0, diffusion_pars[["v"]]-1), t0=diffusion_pars[["t0"]], sz=diffusion_pars[["sz"]], st0=diffusion_pars[["st0"]], sv=diffusion_pars[["sv"]]))
  expect_lt(t2$value$p.value, p_min)
  conditional_save_t(t2, "norm")
  
  t3 <- tryCatch.W.E(ks.test(r_diff1$rt[r_diff1$response=="upper"], normalised_pdiffusion, a=diffusion_pars[["a"]], v=diffusion_pars[["v"]], t0=diffusion_pars[["t0"]], sz=diffusion_pars[["sz"]]+0.25, st0=0, sv=diffusion_pars[["sv"]]-0.5))
  expect_lt(t3$value$p.value, p_min)
  conditional_save_t(t3, "norm")
  
  t4 <- tryCatch.W.E(ks.test(r_diff1$rt[r_diff1$response=="upper"], normalised_pdiffusion, a=diffusion_pars[["a"]], v=diffusion_pars[["v"]], t0=diffusion_pars[["t0"]]-0.08, sz=diffusion_pars[["sz"]], st0=diffusion_pars[["st0"]], sv=diffusion_pars[["sv"]]))
  expect_lt(t4$value$p.value, p_max)
  conditional_save_t(t4, "norm")
  
  t5 <- tryCatch.W.E(ks.test(r_diff1$rt[r_diff1$response=="upper"], normalised_pdiffusion, a=diffusion_pars[["a"]], v=diffusion_pars[["v"]], t0=diffusion_pars[["t0"]], sz=diffusion_pars[["sz"]], st0=diffusion_pars[["st0"]], sv=diffusion_pars[["sv"]]))
  conditional_save_t(t5, "norm")
  expect_gt(t5$value$p.value, p_max)
  
  #if (any(sapply(list(t1, t2, t3, t4, t5), function(x) !is.null(x$warning)))) browser()
})


test_that("Norm: pdiffusion corresponds to random derivates with random values 2", {
  testthat::skip_on_cran()
  #testthat::skip_on_travis()
  normalised_pdiffusion = function(rt,...) pdiffusion(rt,...)/pdiffusion(rt=Inf,...) 
  samples <- 5e3
  p_min <- 0.01
  p_max <- 0.01
  diffusion_pars <- list(
    a = runif(1, 0.5, 1.5),
    v = runif(1, 2, 3.5),
    t0 = runif(1, 0.2, 0.4),
    sz = runif(1, 0, 0.4),
    st0 = runif(1, 0, 0.2),
    sv = runif(1, 0, 1)
  )
    
  r_diff1 <- rdiffusion(samples, a=diffusion_pars[["a"]], v=diffusion_pars[["v"]], t0=diffusion_pars[["t0"]], sz=diffusion_pars[["sz"]], st0=diffusion_pars[["st0"]], sv=diffusion_pars[["sv"]])
  t1 <- tryCatch.W.E(ks.test(r_diff1$rt[r_diff1$response=="upper"], normalised_pdiffusion, a=diffusion_pars[["a"]]-0.3, v=diffusion_pars[["v"]], t0=diffusion_pars[["t0"]], sz=diffusion_pars[["sz"]], st0=diffusion_pars[["st0"]], sv=diffusion_pars[["sv"]]))
  expect_lt(t1$value$p.value, p_min)
  conditional_save_t(t1, "norm")
  
  t2 <- tryCatch.W.E(ks.test(r_diff1$rt[r_diff1$response=="upper"], normalised_pdiffusion, a=diffusion_pars[["a"]], v=diffusion_pars[["v"]]+4, t0=diffusion_pars[["t0"]], sz=diffusion_pars[["sz"]], st0=diffusion_pars[["st0"]], sv=diffusion_pars[["sv"]]))
  expect_lt(t2$value$p.value, p_min)
  conditional_save_t(t2, "norm")
  
  t3 <- tryCatch.W.E(ks.test(r_diff1$rt[r_diff1$response=="upper"], normalised_pdiffusion, a=diffusion_pars[["a"]], v=diffusion_pars[["v"]], t0=diffusion_pars[["t0"]], sz=diffusion_pars[["sz"]]+0.25, st0=0, sv=0))
  expect_lt(t3$value$p.value, p_min)
  conditional_save_t(t3, "norm")
  
  t4 <- tryCatch.W.E(ks.test(r_diff1$rt[r_diff1$response=="upper"], normalised_pdiffusion, a=diffusion_pars[["a"]], v=diffusion_pars[["v"]], t0=diffusion_pars[["t0"]]-0.08, sz=diffusion_pars[["sz"]], st0=diffusion_pars[["st0"]], sv=diffusion_pars[["sv"]]))
  expect_lt(t4$value$p.value, p_max)
  conditional_save_t(t4, "norm")
  
  t5 <- tryCatch.W.E(ks.test(r_diff1$rt[r_diff1$response=="upper"], normalised_pdiffusion, a=diffusion_pars[["a"]], v=diffusion_pars[["v"]], t0=diffusion_pars[["t0"]], sz=diffusion_pars[["sz"]], st0=diffusion_pars[["st0"]], sv=diffusion_pars[["sv"]]))
  conditional_save_t(t5, "norm")
  expect_gt(t5$value$p.value, p_max)
  
  #if (any(sapply(list(t1, t2, t3, t4, t5), function(x) !is.null(x$warning)))) browser()
})

context("Diffusion Model: bugs")

test_that("ddiffusion passes NAs if all are NA", {
  expect_true(
    isTRUE(is.na(ddiffusion(rt = 1, response = 1, a = NA, v = NA, t0 = NA)))
  )
  # expect_true(
  #   isTRUE(is.na(ddiffusion(rt = 1, response = 1, a = 0, v = NA, t0 = NA)))
  # )
  expect_true(
    isTRUE(is.na(ddiffusion(rt = 1, response = 1, a = NA, v = 0, t0 = NA)))
  )
  expect_true(
    isTRUE(is.na(ddiffusion(rt = 1, response = 1, a = NA, v = NA, t0 = 0)))
  )
})

test_that("qdiffusion does not fail for certain values", {
  qex <- c(a = 1.97092532512193, t0 = 0.22083875893466, sv = 0.462630056494015, v = -2.59881372383245, resp_prop = 0.99877722499984, z = 0.5, sz= 0.1)
  
  #pdiffusion(0.28165, response = "lower", a=qex["a"], v=qex["v"], t0 = qex["t0"], sv = qex["sv"], sz = 0.1, z = 0.5)
  
  expect_true(!is.na(
    qdiffusion(p = 0.0998, response = "lower", a=qex["a"], v=qex["v"], t0 = qex["t0"], sv = qex["sv"], sz = 0.1, z = 0.5)
  ))

})

test_that("qdiffusion does not fail for named probability", {
   qex2 <- c( t0 = 0.194096266864241, sv = 0.867039443746426, v = -3.8704467331985, a = 0.819960332004152, resp_prop = 0.923810293538986, z = 0.5, sz= 0.1)
   #pdiffusion(20, response = "upper", a=qex2["a"], v=qex2["v"], t0 = qex2["t0"], sv = qex2["sv"], sz = 0.1, z = 0.5)
  
  expect_true(!is.na(
    qdiffusion(p = qex2["resp_prop"], response = "lower", a=qex2["a"], v=qex2["v"], t0 = qex2["t0"], sv = qex2["sv"], sz = 0.1, z = 0.5*qex2["a"])
    ))

})

test_that("pdiffusion is equal to pwiener", {
  if (require(RWiener)) {
    
    expect_equal(pdiffusion(2, response = "upper", a = 1.63, v = 1.17, t0 = 0.22, z = 0.517*1.63), pwiener(2, resp = "upper", alpha = 1.63, delta = 1.17, tau = 0.22, beta =  0.517),
                      tolerance = 0.0001)
  }
})

test_that("numerical integration does not fail in certain cases", {
  # fails
  expect_equal(pdiffusion(0.640321329425053, response = "upper", a = 1.1, v=1.33, t0 = 0.3, z = 0.55*1.1, st0 = 0.2, sz = 0.7*1.1, precision = 3),
               0.5345, tolerance = 0.001)
  expect_equal(pdiffusion(0.640321329425053, response = "upper", a = 1.1, v=1.33, t0 = 0.3, z = 0.55*1.1, st0 = 0.2, sz = 0.7*1.1, precision = 2.9),
               0.5345517, tolerance = 0.001)

})


test_that("pdiffusion does not add to 1 for both responses in case sz goes towards max.", {
  
  expect_equal(sum(pdiffusion(rep(Inf, 2), a=1, v=2, t0=0.5, st0=0.2, sz = 0.1, sv = 0.5, response=c("l", "u"))), 1, tolerance = 0.001)
  
  expect_equal(sum(pdiffusion(rep(Inf, 2), a=1, v=2, z = 0.4, t0=0.5, st0=0.2, sz = 0.1, sv = 0.5, response=c("l", "u"))), 1, tolerance = 0.001)
  
  expect_equal(sum(pdiffusion(rep(Inf, 2), response=c("l", "u"), a=1, v = 3.69, t0 = 0.3, sz = 0.1, sv = 1.2, st0 = 0)), 1, tolerance = 0.001)
  
  expect_equal(sum(pdiffusion(rep(Inf, 2), response=c("l", "u"), a=1, v = 3.69, t0 = 0.3, sz = 0.5, sv = 1.2, st0 = 0)), 1, tolerance = 0.001)

  #testthat::skip("currently pdiffusion does not add up to 1 fo rthe following tests.")
  
  expect_equal(sum(pdiffusion(rep(Inf, 2), response=c("l", "u"), a=1, v = 3.69, t0 = 0.3, sz = 0.9, sv = 1.2, st0 = 0)), 1, tolerance = 0.001)
  expect_equal(sum(pdiffusion(rep(Inf, 2), response=c("l", "u"), a=0.08, v = 0.369, t0 = 0.3, sz = 0.07, sv = 0.12, st0 = 0, s=0.1, precision = 2)), 1, tolerance = 0.001)
  expect_equal(sum(pdiffusion(rep(Inf, 2), response=c("l", "u"), a=0.08, v = 0.369, t0 = 0.3, sz = 0.07, sv = 0.12, st0 = 0, s=0.1, precision = 3)), 1, tolerance = 0.001)
})

test_that("ddiffusion does not go crazy if sz, sv, and st0 goes to 0", {
  y1 <- ddiffusion(0.53, a = 1.84, t0 = 0.14, sv =0 , sz =0, z = 0.879, v = -2.4, st0 = 0, response = 1)
  x1 <- ddiffusion(0.53, a = 1.84, t0 = 0.14, sv =0 , sz =0.0000001, z = 0.879, v = -2.4, response = 1)
  x2 <- ddiffusion(0.53, a = 1.84, t0 = 0.14, sv =0 , sz =0.000001, z = 0.879, v = -2.4, response = 1)
  x3 <- ddiffusion(0.53, a = 1.84, t0 = 0.14, sv =0 , sz =0.00001, z = 0.879, v = -2.4, response = 1)
  
  st00 <- ddiffusion(0.53, a = 1.84, t0 = 0.14, sv =0 , st0 =0.0000007, z = 0.879, v = -2.4, response = 1)
  st01 <- ddiffusion(0.53, a = 1.84, t0 = 0.14, sv =0 , st0 =0.0000001, z = 0.879, v = -2.4, response = 1)
  st02 <- ddiffusion(0.53, a = 1.84, t0 = 0.14, sv =0 , st0 =0.000001, z = 0.879, v = -2.4, response = 1)
  st03 <- ddiffusion(0.53, a = 1.84, t0 = 0.14, sv =0 , st0 =0.00001, z = 0.879, v = -2.4, response = 1)
  
  sv0 <- ddiffusion(0.53, a = 1.84, t0 = 0.14, sv =0.0000007, z = 0.879, v = -2.4, response = 1)
  sv1 <- ddiffusion(0.53, a = 1.84, t0 = 0.14, sv =0.0000001, z = 0.879, v = -2.4, response = 1)
  sv2 <- ddiffusion(0.53, a = 1.84, t0 = 0.14, sv =0.000001, z = 0.879, v = -2.4, response = 1)
  sv3 <- ddiffusion(0.53, a = 1.84, t0 = 0.14, sv =0.00001, z = 0.879, v = -2.4, response = 1)
  
  
  tolerance <- 0.0001
  expect_equal(y1, x1, tolerance=tolerance)
  expect_equal(y1, x2, tolerance=tolerance)
  expect_equal(y1, x3, tolerance=tolerance)
  
  expect_equal(y1, st00, tolerance=tolerance)
  expect_equal(y1, st01, tolerance=tolerance)
  expect_equal(y1, st02, tolerance=tolerance)
  expect_equal(y1, st03, tolerance=tolerance)
  
  expect_equal(y1, sv0, tolerance=tolerance)
  expect_equal(y1, sv1, tolerance=tolerance)
  expect_equal(y1, sv2, tolerance=tolerance)
  expect_equal(y1, sv3, tolerance=tolerance)
  
  ### shows the bug:
  # curve(ddiffusion(rep(0.53, 101), a = 1.84, t0 = 0.14, sv =0 , sz =x, z = 0.879, v = -2.4, response = 1), from= 0, to = 0.00001, xlab = "sz")
  # curve(ddiffusion(rep(0.53, 101), a = 1.84, t0 = 0.14, sv =x , sz =0, z = 0.879, v = -2.4, response = 1), from= 0, to = 0.00001, xlab = "sv")
  # curve(ddiffusion(rep(0.53, 101), a = 1.84, t0 = 0.14, sv =0 , sz =0, z = 0.879, v = -2.4, st0 = x, response = 1), from= 0, to = 0.00001, xlab = "st0")
  # curve(ddiffusion(rep(0.53, 101), a = 2, t0 = 0.3, sv =0 , sz =0, z = x, v = 1, response = 1), from= 0, to = 0.1, xlab = "sz")
})

context("Diffusion Model: Compare with RWiener")

test_that("ddiffusion is equal to dwiener", {
  if (require(RWiener)) {
    for (a in seq(0.5, 2.0, length.out = 6)) {
      for (v in seq(0.5, 2.0, length.out = 6)) {
        for (t0 in seq(0.05, 0.5, length.out = 6)) {
          for (z in seq(0.4, 0.6, length.out = 6)) {
            expect_equivalent(
              ddiffusion(seq(0, 3, length.out = 15), a=a, v=v, t0=t0, z = z*a)
              ,
              dwiener(seq(0, 3, length.out = 15), resp = rep("upper", 15), alpha=a, delta=v, tau = t0, beta = z)
            )
            expect_equivalent(
              ddiffusion(seq(0, 3, length.out = 16), c("upper", "lower"), a=a, v=v, t0=t0, z = z*a)
              ,
              dwiener(seq(0, 3, length.out = 16), resp = rep(c("upper", "lower"), 8), alpha=a, delta=v, tau = t0, beta = z)
            )
          }
        }
      }
    }
  }
  
})

test_that("pdiffusion is equal to pwiener", {
  testthat::skip_on_cran()
  if (require(RWiener)) {
    for (a in seq(0.5, 2.0, length.out = 6)) {
      for (v in seq(0.5, 2.0, length.out = 6)) {
        for (t0 in seq(0.05, 0.5, length.out = 6)) {
          for (z in seq(0.4, 0.6, length.out = 6)) {
            expect_equal(
              pdiffusion(seq(0, 3, length.out = 15), a=a, v=v, t0=t0, z = z*a)
              ,
              pwiener(seq(0, 3, length.out = 15), resp = rep("upper", 15), alpha=a, delta=v, tau = t0, beta = z)
            , tolerance = 0.01)
            expect_equivalent(
              pdiffusion(seq(0, 3, length.out = 16), c("upper", "lower"), a=a, v=v, t0=t0, z = z*a)
              ,
              pwiener(seq(0, 3, length.out = 16), resp = rep(c("upper", "lower"), 8), alpha=a, delta=v, tau = t0, beta = z)
            , tolerance = 0.01)
          }
        }
      }
    }
  }
})


tryCatch.W.E <- function(expr)
{
  mc <- match.call()
  mc2 <- match.call(definition = ks.test, call =  as.call(mc[[2]]))
  mc2[[1]] <- list
  
  W <- NULL
  w.handler <- function(w){ # warning handler
    W <<- w
    invokeRestart("muffleWarning")
  }
  list(value = withCallingHandlers(tryCatch(expr, error = function(e) e), warning = w.handler),warning = W, data = eval(mc2, envir = parent.frame()))
}


test_that("Norm: pdiffusion corresponds to random derivates", {
  testthat::skip_on_cran()
  #testthat::skip_on_travis()
  normalised_pdiffusion <- function(rt,...) pdiffusion(rt,...)/pdiffusion(rt=10, ...) 
  normalised_pwiener <- function(q,...) pwiener(q,  resp = rep("upper", length(q)), ...)/pwiener(q=10, resp = "upper", ...)
  samples <- 1e4
  p_min <- 0.001
  p_max <- 0.01
  a <- runif(1, 0.3, 0.9)
  t0 <- runif(1, 0.1, 0.5)
  v <- runif(1, 0.5, 2.5)
  z <- runif(1, 0.5, 0.6)
  r_diffusion <- rdiffusion(samples, a=a, t0=t0, v=v, z=z*a)
  t1 <- tryCatch.W.E(ks.test(r_diffusion$rt[r_diffusion$response=="upper"], normalised_pdiffusion, a=a*2, t0=t0, v=v*2, z=z*a))
  expect_lt(t1$value$p.value, p_min)
  
  t2 <- tryCatch.W.E(ks.test(r_diffusion$rt[r_diffusion$response=="upper"], normalised_pdiffusion, a=a, t0=t0, v=v, z=z*a))
  expect_gt(t2$value$p.value, p_max)
  
  t3 <- tryCatch.W.E(ks.test(r_diffusion$rt[r_diffusion$response=="upper"], normalised_pwiener, alpha=a, delta=v, tau = t0, beta = z))
  expect_gt(t3$value$p.value, p_max)
  
})

test_that("Norm: pdiffusion corresponds to random derivates (with variabilities)", {
  testthat::skip_on_cran()
  #testthat::skip_on_travis()
  normalised_pdiffusion <- function(rt,...) pdiffusion(rt,...)/pdiffusion(rt=10, ...) 
  samples <- 1e4
  p_min <- 0.001
  p_max <- 0.01
  a <- runif(1, 0.3, 0.9)
  t0 <- runif(1, 0.1, 0.5)
  v <- runif(1, 0.5, 2.5)
  sv <- runif(1, 0.1, 0.5)
  sz <- runif(1, 0.05, 0.2)
  z <- runif(1, 0.5, 0.6)
  r_diffusion <- rdiffusion(samples, a=a, t0=t0, v=v, z=z*a, sz=sz, sv = sv)
  t1 <- tryCatch.W.E(ks.test(r_diffusion$rt[r_diffusion$response=="upper"], normalised_pdiffusion, a=a, t0=t0, v=v, z=z*a, sv=1, sz = 0.6*a))
  expect_lt(t1$value$p.value, p_min)
  
  t2 <- tryCatch.W.E(ks.test(r_diffusion$rt[r_diffusion$response=="upper"], normalised_pdiffusion, a=a, t0=t0, v=v, z=z*a,sv=sv, sz=sz))
  expect_gt(t2$value$p.value, p_max)
  
})

#' Response Time Distributions.
#'
#' \tabular{ll}{
#' Package: \tab rtdists\cr
#' Type: \tab Package\cr
#' Version: \tab 0.8-3\cr
#' Date: \tab 2018-06-23\cr
#' Depends: \tab R (>= 3.0.0)\cr
#' License: \tab GPL (>=3)\cr
#' URL: \tab https://github.com/rtdists/rtdists/\cr
#' }
#'
#' Provides response time distributions (density/PDF, distribution function/CDF, quantile
#' function, and random generation): (a) Ratcliff diffusion model (Ratcliff & McKoon, 2008,
#' <doi:10.1162/neco.2008.12-06-420>) based on C code by Andreas and Jochen Voss and (b) linear
#' ballistic accumulator (LBA; Brown & Heathcote, 2008, <doi:10.1016/j.cogpsych.2007.12.002>)
#' with different distributions underlying the drift rate.
#'
#' @aliases rtdists-package
#' @name rtdists-package
#' @docType package
#' @title The rtdists Package
#' @author Henrik Singmann, Scott Brown, Matthew Gretton, Andrew Heathcote, with contributions from Andreas Voss, Jochen Voss, Andrew Terry
#' @keywords package
NULL

#' Ratcliff and Rouder (1998, Exp. 1) Luminance Discrimination Data 
#' 
#' Responses and response times from an experiment in which three participants were asked to decide whether the overall brightness of pixel arrays displayed on a computer monitor was "high" or "low". In addition, instruction manipulated speed and accuracy between blocks. 
#' 
#' @details The Experiment is described in the following by Ratcliff and Rouder (1998, pp. 349):
#' 
#' In Experiment 1, subjects were asked to decide whether the overall
#' brightness of pixel arrays displayed on a computer monitor was "high"
#' or "low". The brightness of a display was controlled by the
#' proportion of the pixels that were white. For each trial, the proportion
#' of white pixels was chosen from one of two distributions, a high
#' distribution [i.e., light] or a low [i.e., dark] distribution, each with fixed mean and standard
#' deviation. Feedback was given after each trial to tell the subject whether his or her decision had correctly indicated the distribution
#' from which the stimulus had been chosen. Other than this feedback, a
#' subject had no information about the distributions. Because the distributions overlapped substantially, a subject could not be highly accurate. A display with 50% white pixels, for example, might have come
#' from the high distribution on one trial and the low distribution on
#' another.
#' 
#' \subsection{Stimuli}{ 
#' The stimulus display for Experiment 1 was a square that was 64
#' pixels on each side and subtended 3.8 degree of visual angle on a PC-VGA
#' monitor. [...] In each square, 3,072 randomly
#' chosen pixels were neutral gray, like the background, and the remaining 
#' 1,024 pixels were either black or white; the proportion of white to
#' black pixels provided the brightness manipulation. There were 33
#' equally spaced proportions from zero (all 1,024 pixels were black) to
#' 1 (all 1,024 pixels were white). The two distributions from which the
#' bright and dark stimuli were chosen were centered at .375 (low brightness) 
#' and .625 (high brightness), and they each had a standard deviation of .1875.
#' }
#' 
#' \subsection{Procedure}{
#' A subject's task was to decide, on each trial, from which distribution, 
#' high or low brightness in Experiment 1, the observed
#' stimulus (stimuli) had been sampled. Subjects made their decision by
#' pressing one of two response keys. On each trial, a 500-ms foreperiod, 
#' during which the display consisted solely of neutral gray, was followed 
#' by presentation of the stimulus; presentation was terminated by
#' the subject's response. In Experiment 1, speed-versus-accuracy
#' instructions were manipulated. For some blocks of trials, subjects
#' were instructed to respond as quickly as possible, and a "too slow"
#' message followed every response longer than 550 ms. For other
#' blocks of trials, subjects were instructed to be as accurate as possible,
#' and a "bad error" message followed incorrect responses to stimuli
#' from the extreme ends of the distributions. Experiment 1 had ten 35-min 
#' sessions, and Experiments 2 and 3 had four sessions. In Experiment 1, 
#' subjects switched from emphasis on speed to emphasis on
#' accuracy every 204 trials. Each session consisted of eight blocks of
#' 102 trials per block, for a total of 8,160 trials per subject. Each 
#' session consisted of eight blocks of 102 trials, for a total of 3,264 trials
#' per subject in each experiment. For all trials in each experiment, subjects 
#' were instructed to maintain a high level of accuracy while
#' responding quickly, and an "error" message indicated incorrect
#' responses. Responses were followed by a 300-ms blank interval, and
#' the error message was displayed for 300 ms after the blank interval.
#' }
#' 
#' @note The data is already prepared following Ratcliff and Rouder (1998) by removing the following trials:
#' \itemize{
#' \item{the first session for each participant}
#' \item{the first 20 trials of each session}
#' \item{the first trial of each block (each change in speed accuracy starts a new block)}
#' }
#' To fully replicate the data used by Ratcliff and Rouder (1998) one only needs to remove the trials that are \code{TRUE} in column \code{outlier} (i.e., RTs outside of (200ms, 2500ms)). The full raw data is also available as part of this package, see:\cr
#' \code{system.file("extdata", "rr98-data", package = "rtdists")} and \code{system.file("extdata", "rr98-data.codes", package = "rtdists")}
#' 
#' @docType data
#' @keywords dataset
#' @name rr98
#' @usage rr98
#' @format A \code{data.frame} with 24,358 obs. and 12 variables:
#' \describe{
#'  \item{id}{participant id, factor with three levels}
#'  \item{session}{session number, integer}
#'  \item{block}{block number, integer}
#'  \item{trial}{trial number within a block, integer}
#'  \item{instruction}{factor with two levels: \code{"accuracy"} for blocks with accuracy instructions; \code{"speed"} for blocks with speed instruction}
#'  \item{source}{factor with two levels: distribution strength was drawn from, \code{"dark"} and \code{"light"}}
#'  \item{strength}{proportion of white to black pixels were varied by 33 equally spaced proportions from zero (all 1,024 pixels were black) to 1 (all 1,024 pixels were white). with 0 darkest and 32 lightest. Integer.}
#'  \item{response}{factor with two levels: \code{"dark"} and \code{"light"}}
#'  \item{response_num}{numeric response variable such that \code{1="dark"} and \code{2="light"}}
#'  \item{correct}{boolean indicating whether or not \code{source==response}. (Does not seem to be used in the original analysis.)}
#'  \item{rt}{response time in seconds}
#'  \item{outlier}{boolean indicating whether or not the response was considered an outlier by Ratcliff and Rouder (1998), i.e., RTs outside of (200ms, 2500ms)}
#' }
#'
#' @source Ratcliff, R., & Rouder, J. N. (1998). Modeling Response Times for Two-Choice Decisions. \emph{Psychological Science}, 9(5), 347-356. http://doi.org/10.1111/1467-9280.00067
#' 
#' @example examples/examples.rr98.R
#'
NULL

# Generated by using Rcpp::compileAttributes() -> do not edit by hand
# Generator token: 10BE3573-1514-4C36-9D1C-5A225CD40393

d_fastdm <- function(rts, params, precision = 3, boundary = 2L, stop_on_error = TRUE) {
    .Call(`_rtdists_d_fastdm`, rts, params, precision, boundary, stop_on_error)
}

p_fastdm <- function(rts, params, precision = 3, boundary = 2L, stop_on_error = TRUE) {
    .Call(`_rtdists_p_fastdm`, rts, params, precision, boundary, stop_on_error)
}

p_precise_fastdm <- function(rts, params, precision = 3, boundary = 2L, stop_on_error = TRUE) {
    .Call(`_rtdists_p_precise_fastdm`, rts, params, precision, boundary, stop_on_error)
}

r_fastdm <- function(num_values, params, precision = 3, stop_on_error = TRUE) {
    .Call(`_rtdists_r_fastdm`, num_values, params, precision, stop_on_error)
}

#' The Linear Ballistic Accumulator (LBA)
#' 
#' Density, distribution function, quantile function, and random generation for
#' the LBA model with the following parameters: \code{A} (upper value of
#' starting point), \code{b} (response threshold), \code{t0} (non-decision
#' time), and driftrate (\code{v}). All functions are available with different
#' distributions underlying the drift rate: Normal (\code{norm}), Gamma
#' (\code{gamma}), Frechet (\code{frechet}), and log normal (\code{lnorm}). The
#' functions return their values conditional on the accumulator given in the
#' response argument winning.
#' 
#' @param rt vector of RTs. Or for convenience also a \code{data.frame} with
#'   columns \code{rt} and \code{response} (such as returned from \code{rLBA} or
#'   \code{\link{rdiffusion}}). See examples.
#' @param response integer vector of winning accumulators/responses
#'   corresponding to the vector of RTs/p (i.e., used for specifying the
#'   response for a given RT/probability). Will be recycled if necessary. Cannot
#'   contain values larger than the number of accumulators. First
#'   response/accumulator must receive value 1, second 2, and so forth. For
#'   conmvenience, \code{response} is converted via \code{as.numeric} thereby
#'   allowing factors to be passed as well (such as returned from
#'   \code{\link{rdiffusion}}). Ignored if \code{rt} or \code{p} is a
#'   \code{data.frame}.
#' @param p vector of probabilities. Or for convenience also a \code{data.frame}
#'   with columns \code{p} and \code{response}. See examples.
#' @param n desired number of observations (scalar integer).
#' @param A start point interval or evidence in accumulator before beginning of
#'   decision process. Start point varies from trial to trial in the interval
#'   [0, \code{A}] (uniform distribution). Average amount of evidence before
#'   evidence accumulation across trials is \code{A}/2.
#' @param b response threshold. (\code{b} - \code{A}/2) is a measure of
#'   "response caution".
#' @param t0 non-decision time or response time constant (in seconds). Lower
#'   bound for the duration of all non-decisional processes (encoding and
#'   response execution).
#' @param st0 variability of non-decision time, such that \code{t0} is uniformly
#'   distributed between \code{t0} and \code{t0} + \code{st0}. Default is 0. Can
#'   be trialwise, and will be recycled to length of \code{rt}.
#' @param ... two \emph{named} drift rate parameters depending on
#'   \code{distribution} (e.g., \code{mean_v} and \code{sd_v} for
#'   \code{distribution=="norm"}). The parameters can either be given as a
#'   numeric vector or a list. If a numeric vector is passed each element of the
#'   vector corresponds to one accumulator. If a list is passed each list
#'   element corresponds to one accumulator allowing again trialwise driftrates.
#'   The shorter parameter will be recycled as necessary (and also the elements
#'   of the list to match the length of \code{rt}). See details.
#' @param distribution character specifying the distribution of the drift rate.
#'   Possible values are \code{c("norm", "gamma", "frechet", "lnorm")}, default
#'   is \code{"norm"}.
#' @param args.dist list of optional further arguments to the distribution
#'   functions (i.e., \code{posdrift} or \code{robust} for
#'   \code{distribution=="norm"}, see \code{\link{single-LBA}}).
#' @param silent logical. Should the number of accumulators used be suppressed?
#'   Default is \code{FALSE} which prints the number of accumulators.
#' @param interval a vector containing the end-points of the interval to be
#'   searched for the desired quantiles (i.e., RTs) in \code{qLBA}. Default is
#'   \code{c(0, 10)}.
#' @param scale_p logical. Should entered probabilities automatically be scaled
#'   by maximally predicted probability? Default is \code{FALSE}. Convenience
#'   argument for obtaining predicted quantiles. Can be slow as the maximally
#'   predicted probability is calculated individually for each \code{p}.
#' @param scale_max numerical scalar. Value at which maximally predicted RT
#'   should be calculated if \code{scale_p} is \code{TRUE}.
#'   
#' @details
#' 
#' For convenience, all functions (with the exception of \code{rdiffusion})
#' allow that the first argument is a \code{data.frame} containing the
#' information of the first and second argument in two columns (i.e.,
#' \code{rt}/\code{p} and \code{response}). Other columns will be ignored. This
#' allows, for example, to pass the \code{data.frame} generated by \code{rLBA}
#' directly to \code{pLBA}. See examples.
#' 
#' \subsection{Parameters}{ The following arguments are allowed as \code{...}
#' drift rate parameters: \itemize{ \item \code{mean_v,sd_v} mean and standard
#' deviation of normal distribution for drift rate (\code{norm}). See
#' \code{\link{Normal}} \item \code{shape_v,rate_v,scale_v} shape, rate, and
#' scale of gamma (\code{gamma}) and scale and shape of Frechet (\code{frechet})
#' distributions for drift rate. See \code{\link{GammaDist}} or
#' \code{\link[evd]{frechet}}. For Gamma, scale = 1/shape and shape = 1/scale. 
#' \item \code{meanlog_v,sdlog_v} mean and standard deviation of lognormal
#' distribution on the log scale for drift rate (\code{lnorm}). See
#' \code{\link{Lognormal}}. }
#' 
#' As described above, the accumulator parameters can either be given as a
#' numeric vector or a list. If a numeric vector is passed each element of the
#' vector corresponds to one accumulator. If a list is passed each list element
#' corresponds to one accumulator allowing trialwise driftrates. The shorter
#' parameter will be recycled as necessary (and also the elements of the list to
#' match the length of \code{rt}).
#' 
#' The other LBA parameters (i.e., \code{A}, \code{b}, and \code{t0}, with the
#' exception of \code{st0}) can either be a single numeric vector (which will be
#' recycled to reach \code{length(rt)} or \code{length(n)} for trialwise
#' parameters) \emph{or} a \code{list} of such vectors in which each list
#' element corresponds to the parameters for this accumulator (i.e., the list
#' needs to be of the same length as there are accumulators). Each list will
#' also be recycled to reach \code{length(rt)} for trialwise parameters per
#' accumulator.
#' 
#' To make the difference between both paragraphs clear: Whereas for the
#' accumulators both a single vector or a list corresponds to different
#' accumulators, only the latter is true for the other parameters. For those
#' (i.e., \code{A}, \code{b}, and \code{t0}) a single vector always corresponds
#' to trialwise values and a list must be used for accumulator wise values.
#' 
#' \code{st0} can only vary trialwise (via a vector). And it should be noted
#' that \code{st0} not equal to zero will considerably slow done everything. }
#' 
#' \subsection{Quantile Function}{ Due to the bivariate nature of the LBA,
#' single accumulators only return defective CDFs that do not reach 1. Only the
#' sum of all accumulators reaches 1. Therefore, \code{qLBA} can only return
#' quantiles/RTs for any accumulator up to the maximal probability of that
#' accumulator's CDF. This can be obtained by evaluating the CDF at \code{Inf}.
#' 
#' As a conveniece for the user, if \code{scale_p = TRUE} in the call to
#' \code{qLBA} the desired probabilities are automatically scaled by the maximal
#' probability for the corresponding response. Note that this can be slow as the
#' maximal probability is calculated separately for each desired probability.
#' See examples.
#' 
#' Also note that quantiles (i.e., predicted RTs) are obtained by numerically
#' minimizing the absolute difference between desired probability and the value
#' returned from \code{pLBA} using \code{\link{optimize}}. If the difference
#' between the desired probability and probability corresponding to the returned
#' quantile is above a certain threshold (currently 0.0001) no quantile is
#' returned but \code{NA}. This can be either because the desired quantile is
#' above the maximal probability for this accumulator or because the limits for
#' the numerical integration are too small (default is \code{c(0, 10)}). }
#' 
#' \subsection{RNG}{ For random number generation at least one of the
#' distribution parameters (i.e., \code{mean_v}, \code{sd_v}, \code{shape_v},
#' \code{scale_v}, \code{rate_v}, \code{meanlog_v}, and \code{sdlog_v}) should
#' be of length > 1 to receive RTs from multiple responses. Shorter vectors are
#' recycled as necessary.\cr Note that for random number generation from a
#' normal distribution for the driftrate the number of returned samples may be
#' less than the number of requested samples if \code{posdrifts==FALSE}. }
#' 
#' 
#' @return \code{dLBA} returns the density (PDF), \code{pLBA} returns the
#'   distribution function (CDF), \code{qLBA} returns the quantile/RT,
#'   \code{rLBA} return random response times and responses (in a
#'   \code{data.frame}).
#'   
#'   The length of the result is determined by \code{n} for \code{rLBA}, equal
#'   to the length of \code{rt} for \code{dLBA} and \code{pLBA}, and equal to
#'   the length of \code{p} for \code{qLBA}.
#'   
#'   The distribution parameters (as well as \code{response}) are recycled to
#'   the length of the result. In other words, the functions are completely
#'   vectorized for all parameters and even the response.
#'   
#' @note These are the top-level functions intended for end-users. To obtain the
#'   density and cumulative density the race functions are called for each
#'   response time with the corresponding winning accumulator as first
#'   accumulator (see \code{\link{LBA-race}}).
#'   
#' @references
#' 
#' Brown, S. D., & Heathcote, A. (2008). The simplest complete model of choice
#' response time: Linear ballistic accumulation. \emph{Cognitive Psychology},
#' 57(3), 153-178. doi:10.1016/j.cogpsych.2007.12.002
#' 
#' Donkin, C., Averell, L., Brown, S., & Heathcote, A. (2009). Getting more from
#' accuracy and response time data: Methods for fitting the linear ballistic
#' accumulator. \emph{Behavior Research Methods}, 41(4), 1095-1110.
#' doi:10.3758/BRM.41.4.1095
#' 
#' Heathcote, A., & Love, J. (2012). Linear deterministic accumulator models of
#' simple choice. \emph{Frontiers in Psychology}, 3, 292.
#' doi:10.3389/fpsyg.2012.00292
#' 
#' @name LBA
#' @importFrom stats optimize uniroot
#'   
#' @example examples/examples.lba.R
#'   
NULL

# this functions takes the argument as entered by the user and always returns a
# list of length n_v in which each element is of length nn. This ensures that
# for each parameter we have a list of length equal to the number of
# accumulators with each element of length equal to the number of trials. This
# consistency will be exploited when passing to the n1functions.
check_i_arguments <- function(arg, nn, n_v, dots = FALSE) {
  mc <- match.call()
  varname <- sub("dots$", "", deparse(mc[["arg"]]), fixed = TRUE)
  if (!is.list(arg)) {
    if ((!is.vector(arg, "numeric")) || (length(arg) < 1)) 
      stop(paste(varname, "needs to be a numeric vector of length >= 1!"))
    if (dots) {
      arg <- as.list(arg)
      arg <- lapply(arg, rep, length.out=nn)
    } else arg <- lapply(seq_len(n_v), function(x) rep(arg, length.out=nn))
  } else {
    if (!dots && (length(arg) != n_v)) 
      stop(paste("if", varname, "is a list, its length needs to correspond to the number of accumulators."))
    for (i in seq_along(arg)) {
      if ((!is.vector(arg[[i]], "numeric")) || (length(arg[[i]]) < 1)) 
        stop(paste0(varname, "[[", i, "]] needs to be a numeric vector of length >= 1!"))
      arg[[i]] <- rep(arg[[i]], length.out=nn)
    }
  }
  #if (length(arg) != n_v) stop(paste("size of", varname, "does not correspond to number of accumulators."))
  return(arg)
}

#' @rdname LBA
#' @export 
dLBA <-  function(rt, response, A, b, t0, ..., st0=0, 
                  distribution = c("norm", "gamma", "frechet", "lnorm"), 
                  args.dist = list(), silent = FALSE) {
  dots <- list(...)
  if (is.null(names(dots))) stop("... arguments need to be named.")
  
  # for convenience accept data.frame as first argument.
  if (is.data.frame(rt)) {
    response <- rt$response
    rt <- rt$rt
  }
  
  response <- as.numeric(response)
  nn <- length(rt)
  n_v <- max(vapply(dots, length, 0))  # Number of responses
  if(!silent) 
    message(paste("Results based on", n_v, "accumulators/drift rates."))
  if (!is.numeric(response) || max(response) > n_v) 
    stop("response needs to be a numeric vector of integers up to number of accumulators.")
  if (any(response < 1)) 
    stop("the first response/accumulator must have value 1.")
  if (n_v < 2) 
    stop("There need to be at least two accumulators/drift rates.")
  distribution <- match.arg(distribution)
  response <- rep(response, length.out = nn)
  A <- check_i_arguments(A, nn=nn, n_v=n_v)
  b <- check_i_arguments(b, nn=nn, n_v=n_v)
  t0 <- check_i_arguments(t0, nn=nn, n_v=n_v)
  switch(distribution, 
         norm = {
           if (any(!(c("mean_v","sd_v") %in% names(dots)))) 
             stop("mean_v and sd_v need to be passed for distribution = \"norm\"")
           dots$mean_v <- check_i_arguments(dots$mean_v, nn=nn, n_v=n_v, dots = TRUE)
           dots$sd_v <- check_i_arguments(dots$sd_v, nn=nn, n_v=n_v, dots = TRUE)
           dots <- dots[c("mean_v","sd_v")]
         },
         gamma = {
           if (!("shape_v" %in% names(dots))) 
             stop("shape_v needs to be passed for distribution = \"gamma\"")
           if ((!("rate_v" %in% names(dots))) & (!("scale_v" %in% names(dots)))) 
             stop("rate_v or scale_v need to be passed for distribution = \"gamma\"")
           dots$shape_v <- check_i_arguments(dots$shape_v, nn=nn, n_v=n_v, dots = TRUE)
           if ("scale_v" %in% names(dots)) {
             dots$scale_v <- check_i_arguments(dots$scale_v, nn=nn, n_v=n_v, dots = TRUE)
             if (is.list(dots$scale_v)) {
               dots$rate_v <- lapply(dots$scale_v, function(x) 1/x)
             } else dots$rate_v <- 1/dots$scale_v
           } else dots$rate_v <- check_i_arguments(dots$rate_v, nn=nn, n_v=n_v, dots = TRUE)
           dots <- dots[c("shape_v","rate_v")]
         },
         frechet = {
           if (any(!(c("shape_v","scale_v") %in% names(dots)))) 
             stop("shape_v and scale_v need to be passed for distribution = \"frechet\"")
           dots$shape_v <- check_i_arguments(dots$shape_v, nn=nn, n_v=n_v, dots = TRUE)
           dots$scale_v <- check_i_arguments(dots$scale_v, nn=nn, n_v=n_v, dots = TRUE)
           dots <- dots[c("shape_v","scale_v")]
         },
         lnorm = {
           if (any(!(c("meanlog_v","sdlog_v") %in% names(dots)))) 
             stop("meanlog_v and sdlog_v need to be passed for distribution = \"lnorm\"")
           dots$meanlog_v <- check_i_arguments(dots$meanlog_v, nn=nn, n_v=n_v, dots = TRUE)
           dots$sdlog_v <- check_i_arguments(dots$sdlog_v, nn=nn, n_v=n_v, dots = TRUE)
           dots <- dots[c("meanlog_v","sdlog_v")]
         }
  )
  #browser()
  for (i in seq_len(length(dots))) {
    if (length(dots[[i]]) < n_v) dots[[i]] <- rep(dots[[i]],length.out=n_v)
  }
  out <- vector("numeric", nn)
  for (i in unique(response)) {
    sel <- response == i
    out[sel] <- do.call(n1PDF, 
                        args = c(rt=list(rt[sel]), 
                                 A = list(lapply(A, "[", i = sel)[c(i, seq_len(n_v)[-i])]), 
                                 b = list(lapply(b, "[", i = sel)[c(i, seq_len(n_v)[-i])]),
                                 t0 = list(lapply(t0, "[", i = sel)[c(i, seq_len(n_v)[-i])]),
                                 lapply(dots, function(x) 
                                   lapply(x, "[", i = sel)[c(i, seq_len(n_v)[-i])]), 
                                 distribution=distribution, 
                                 args.dist=list(args.dist), 
                                 silent=TRUE, st0 = list(st0)))
  }
  return(out)
}


#' @rdname LBA
#' @export 
pLBA <-  function(rt, response, A, b, t0, ..., st0=0, 
                  distribution = c("norm", "gamma", "frechet", "lnorm"), 
                  args.dist = list(), silent = FALSE) {
  dots <- list(...)
  if (is.null(names(dots))) 
    stop("... arguments need to be named.")
  
  # for convenience accept data.frame as first argument.
  if (is.data.frame(rt)) {
    response <- rt$response
    rt <- rt$rt
  }
  
  response <- as.numeric(response)
  nn <- length(rt)
  n_v <- max(vapply(dots, length, 0))  # Number of responses
  if(!silent) message(paste("Results based on", n_v, "accumulators/drift rates."))
  if (!is.numeric(response) || max(response) > n_v) 
    stop("response needs to be a numeric vector of integers up to number of accumulators.")
  if (n_v < 2) 
    stop("There need to be at least two accumulators/drift rates.")
  distribution <- match.arg(distribution)
  response <- rep(response, length.out = nn)
  A <- check_i_arguments(A, nn=nn, n_v=n_v)
  b <- check_i_arguments(b, nn=nn, n_v=n_v)
  t0 <- check_i_arguments(t0, nn=nn, n_v=n_v)
  switch(distribution, 
         norm = {
           if (any(!(c("mean_v","sd_v") %in% names(dots)))) 
             stop("mean_v and sd_v need to be passed for distribution = \"norm\"")
           dots$mean_v <- check_i_arguments(dots$mean_v, nn=nn, n_v=n_v, dots = TRUE)
           dots$sd_v <- check_i_arguments(dots$sd_v, nn=nn, n_v=n_v, dots = TRUE)
           dots <- dots[c("mean_v","sd_v")]
         },
         gamma = {
           if (!("shape_v" %in% names(dots))) 
             stop("shape_v needs to be passed for distribution = \"gamma\"")
           if ((!("rate_v" %in% names(dots))) & (!("scale_v" %in% names(dots)))) 
             stop("rate_v or scale_v need to be passed for distribution = \"gamma\"")
           dots$shape_v <- check_i_arguments(dots$shape_v, nn=nn, n_v=n_v, dots = TRUE)
           if ("scale_v" %in% names(dots)) {
             dots$scale_v <- check_i_arguments(dots$scale_v, nn=nn, n_v=n_v, dots = TRUE)
             if (is.list(dots$scale_v)) {
               dots$rate_v <- lapply(dots$scale_v, function(x) 1/x)
             } else dots$rate_v <- 1/dots$scale_v
           } else dots$rate_v <- check_i_arguments(dots$rate_v, nn=nn, n_v=n_v, dots = TRUE)
           dots <- dots[c("shape_v","rate_v")]
         },
         frechet = {
           if (any(!(c("shape_v","scale_v") %in% names(dots)))) 
             stop("shape_v and scale_v need to be passed for distribution = \"frechet\"")
           dots$shape_v <- check_i_arguments(dots$shape_v, nn=nn, n_v=n_v, dots = TRUE)
           dots$scale_v <- check_i_arguments(dots$scale_v, nn=nn, n_v=n_v, dots = TRUE)
           dots <- dots[c("shape_v","scale_v")]
         },
         lnorm = {
           if (any(!(c("meanlog_v","sdlog_v") %in% names(dots)))) 
             stop("meanlog_v and sdlog_v need to be passed for distribution = \"lnorm\"")
           dots$meanlog_v <- check_i_arguments(dots$meanlog_v, nn=nn, n_v=n_v, dots = TRUE)
           dots$sdlog_v <- check_i_arguments(dots$sdlog_v, nn=nn, n_v=n_v, dots = TRUE)
           dots <- dots[c("meanlog_v","sdlog_v")]
         }
  )
  #browser()
  for (i in seq_len(length(dots))) {
    if (length(dots[[i]]) < n_v) dots[[i]] <- rep(dots[[i]],length.out=n_v)
  }
  out <- vector("numeric", nn)
  for (i in unique(response)) {
    sel <- response == i
    #if(!all(rt[sel] == sort(rt[sel])))  stop("rt needs to be sorted (per response)")
    out[sel] <- do.call(n1CDF, 
                        args = c(rt=list(rt[sel]), 
                                 A = list(lapply(A, "[", i = sel)[c(i, seq_len(n_v)[-i])]), 
                                 b = list(lapply(b, "[", i = sel)[c(i, seq_len(n_v)[-i])]),
                                 t0 = list(lapply(t0, "[", i = sel)[c(i, seq_len(n_v)[-i])]),
                                 lapply(dots, function(x) 
                                   lapply(x, "[", i = sel)[c(i, seq_len(n_v)[-i])]), 
                                 distribution=distribution, 
                                 args.dist=list(args.dist), 
                                 silent=TRUE, st0 = list(st0)))
  }
  return(out)
}

# rt, response, A, b, t0, ..., st0=0, distribution = c("norm", "gamma", "frechet", "lnorm"), args.dist = list(), silent = FALSE
inv_cdf_lba <- function(x, response, A, b, t0, ..., st0, 
                        distribution, args.dist, value, abs = TRUE) {
  if (abs) abs(value - pLBA(rt=x, response=response, 
                            A=A, b = b, t0 = t0, ..., st0=st0, 
                            distribution=distribution, 
                            args.dist=args.dist, 
                            silent=TRUE))
  else (value - pLBA(rt=x, response=response, A=A, b = b, t0 = t0, ..., st0=st0, 
                     distribution=distribution, 
                     args.dist=args.dist, 
                     silent=TRUE))
}


#' @rdname LBA
#' @export 
qLBA <-  function(p, response, A, b, t0, ..., st0=0, 
                  distribution = c("norm", "gamma", "frechet", "lnorm"), 
                  args.dist = list(), silent = FALSE, interval = c(0, 10),
                  scale_p = FALSE, scale_max = Inf) {
  dots <- list(...)
  if (is.null(names(dots))) stop("... arguments need to be named.")
  
  # for convenience accept data.frame as first argument. 
  if (is.data.frame(p)) {
    response <- p$response
    p <- p$p
  }
  
  response <- as.numeric(response)
  nn <- length(p)
  n_v <- max(vapply(dots, length, 0))  # Number of responses
  if(!silent) 
    message(paste("Results based on", n_v, "accumulators/drift rates."))
  if (!is.numeric(response) || max(response) > n_v) 
    stop("response needs to be a numeric vector of integers up to number of accumulators.")
  if (any(response < 1)) 
    stop("the first response/accumulator must have value 1.")
  if (n_v < 2) 
    stop("There need to be at least two accumulators/drift rates.")
  distribution <- match.arg(distribution)
  response <- rep(response, length.out = nn)
  A <- check_i_arguments(A, nn=nn, n_v=n_v)
  b <- check_i_arguments(b, nn=nn, n_v=n_v)
  t0 <- check_i_arguments(t0, nn=nn, n_v=n_v)
  st0 <- rep(st0, length.out = nn)
  out <- vector("numeric", nn)
  p <- unname(p)
  
  for (i in seq_len(nn)) {
    if (scale_p)
      max_p <- do.call(
        pLBA,
        args = c(
          rt = list(scale_max),
          response = list(response[i]),
          A = ret_arg(A, i),
          b = ret_arg(b, i),
          t0 = ret_arg(t0, i),
          sapply(dots, function(z, i)
            sapply(z, ret_arg2, which = i,
                   simplify = FALSE), i =
              i,
            simplify = FALSE),
          args.dist = list(args.dist),
          distribution = distribution,
          st0 = list(st0[i]),
          silent = TRUE
        )
      )
    else
      max_p <- 1
    tmp <- do.call(
      optimize,
      args = c(
        f = inv_cdf_lba,
        interval = list(interval),
        response = list(response[i]),
        A = ret_arg(A, i),
        b = ret_arg(b, i),
        t0 = ret_arg(t0, i),
        sapply(dots, function(z, i) 
          sapply(z, ret_arg2, which = i, simplify = FALSE), 
          i = i, simplify = FALSE),
        args.dist = list(args.dist),
        distribution = distribution,
        st0 = list(st0[i]),
        value = p[i] * max_p,
        tol = .Machine$double.eps ^ 0.5
      )
    )
    if (tmp$objective > 0.0001) {
      tmp <-
        do.call(
          optimize,
          args = c(
            f = inv_cdf_lba,
            interval = list(c(min(interval), max(interval) / 2)),
            response = list(response[i]),
            A = ret_arg(A, i),
            b = ret_arg(b, i),
            t0 = ret_arg(t0, i),
            sapply(dots, function(z, i)
              sapply(z, ret_arg2, which = i, simplify = FALSE), 
              i = i, simplify = FALSE),
            args.dist = list(args.dist),
            distribution = distribution,
            st0 = list(st0[i]),
            value = p[i] * max_p,
            tol = .Machine$double.eps ^ 0.5
          )
        )
    }
    if (tmp$objective > 0.0001) {
      try({
        uni_tmp <-
          do.call(
            uniroot,
            args = c(
              f = inv_cdf_lba,
              interval = list(c(min(interval), max(interval) / 2)),
              response = list(response[i]),
              A = ret_arg(A, i),
              b = ret_arg(b, i),
              t0 = ret_arg(t0, i),
              sapply(dots, function(z, i)
                sapply(z, ret_arg2, which = i, simplify = FALSE), 
                i = i, simplify = FALSE),
              args.dist = list(args.dist),
              distribution = distribution,
              st0 = list(st0[i]),
              value = p[i] * max_p,
              tol = .Machine$double.eps ^ 0.5,
              abs = FALSE
            )
          )
        tmp$objective <- uni_tmp$f.root
        tmp$minimum <- uni_tmp$root
      }, silent = TRUE)
    }
    if (tmp$objective > 0.0001) {
      tmp[["minimum"]] <- NA
      warning(
        "Cannot obtain RT that is less than 0.0001 away from desired p = ",
        p[i],
        ".\nIncrease/decrease interval or obtain for different boundary.",
        call. = FALSE
      )
    }
    out[i] <- tmp[["minimum"]]
  }
  return(out
  )
}



#' @rdname LBA
#' @export
rLBA <- function(n,A,b,t0, ..., st0=0, distribution = c("norm", "gamma", "frechet", "lnorm"), args.dist = list(), silent = FALSE) {
  dots <- list(...)
  if (is.null(names(dots))) stop("... arguments need to be named.")
  if (any(names(dots) == "")) stop("all ... arguments need to be named.")
  n_v <- max(vapply(dots, length, 0))  # Number of responses
  if(!silent) message(paste("Results based on", n_v, "accumulators/drift rates."))
  #if (n_v < 2) stop("There need to be at least two accumulators/drift rates.")
  nn <- n
  distribution <- match.arg(distribution)
  A <- check_i_arguments(A, nn=nn, n_v=n_v)
  b <- check_i_arguments(b, nn=nn, n_v=n_v)
  t0 <- check_i_arguments(t0, nn=nn, n_v=n_v)
  st0 <- rep(unname(st0), length.out = nn)
  switch(distribution, 
         norm = {
           rng <- rlba_norm
           if (any(!(c("mean_v","sd_v") %in% names(dots)))) 
             stop("mean_v and sd_v need to be passed for distribution = \"norm\"")
           dots$mean_v <- check_i_arguments(dots$mean_v, nn=nn, n_v=n_v, dots = TRUE)
           dots$sd_v <- check_i_arguments(dots$sd_v, nn=nn, n_v=n_v, dots = TRUE)
           dots <- dots[c("mean_v","sd_v")]
         },
         gamma = {
           rng <- rlba_gamma
           if (!("shape_v" %in% names(dots))) 
             stop("shape_v needs to be passed for distribution = \"gamma\"")
           if ((!("rate_v" %in% names(dots))) & (!("scale_v" %in% names(dots)))) 
             stop("rate_v or scale_v need to be passed for distribution = \"gamma\"")
           dots$shape_v <- check_i_arguments(dots$shape_v, nn=nn, n_v=n_v, dots = TRUE)
           if ("scale_v" %in% names(dots)) {
             dots$scale_v <- check_i_arguments(dots$scale_v, nn=nn, n_v=n_v, dots = TRUE)
             if (is.list(dots$scale_v)) {
               dots$rate_v <- lapply(dots$scale_v, function(x) 1/x)
             } else dots$rate_v <- 1/dots$scale_v
           } else dots$rate_v <- check_i_arguments(dots$rate_v, nn=nn, n_v=n_v, dots = TRUE)
           dots <- dots[c("shape_v","rate_v")]
         },
         frechet = {
           rng <- rlba_frechet
           if (any(!(c("shape_v","scale_v") %in% names(dots)))) 
             stop("shape_v and scale_v need to be passed for distribution = \"frechet\"")
           dots$shape_v <- check_i_arguments(dots$shape_v, nn=nn, n_v=n_v, dots = TRUE)
           dots$scale_v <- check_i_arguments(dots$scale_v, nn=nn, n_v=n_v, dots = TRUE)
           dots <- dots[c("shape_v","scale_v")]
         },
         lnorm = {
           rng <- rlba_lnorm
           if (any(!(c("meanlog_v","sdlog_v") %in% names(dots)))) 
             stop("meanlog_v and sdlog_v need to be passed for distribution = \"lnorm\"")
           dots$meanlog_v <- check_i_arguments(dots$meanlog_v, nn=nn, n_v=n_v, dots = TRUE)
           dots$sdlog_v <- check_i_arguments(dots$sdlog_v, nn=nn, n_v=n_v, dots = TRUE)
           dots <- dots[c("meanlog_v","sdlog_v")]
         }
  )
  for (i in seq_len(length(dots))) {
    if (length(dots[[i]]) < n_v) dots[[i]] <- rep(dots[[i]],length.out=n_v)
  }
  
  tmp_acc <- as.data.frame(dots, optional = TRUE)
  colnames(tmp_acc) <- sub("\\.c\\(.+", "", colnames(tmp_acc))
  
  parameter_char <- apply(tmp_acc, 1, paste0, collapse = "\t")
  parameter_factor <- factor(parameter_char, levels = unique(parameter_char))
  parameter_indices <- split(seq_len(nn), f = parameter_factor)
  
  out <- vector("list", length(parameter_indices))
  for (i in seq_len(length(parameter_indices))) {
    ok_rows <- parameter_indices[[i]]
    tmp_dots <- lapply(dots, function(x) sapply(x, "[[", i = ok_rows[1]))
    out[[i]] <- do.call(rng, 
                        args = c(n=list(length(ok_rows)), 
                                 A = list(sapply(A, "[", i = ok_rows)), 
                                 b = list(sapply(b, "[", i = ok_rows)), 
                                 t0 = list(sapply(t0, "[", i = ok_rows)), 
                                 st0 = list(st0[ok_rows]), 
                                 tmp_dots, args.dist))
  }
  out <- do.call("rbind", out)
  as.data.frame(out)
}

context("LBA works correctly")

test_that("dLBA norm is identical to n1PDF", {
  n <- 100
  x <- rLBA(n, A=0.5, b=1, t0 = 0.5, mean_v=c(1.2, 1), sd_v=c(0.2,0.3))
  
  ex1_n1pdf <- vector("numeric", n)
  ex1_n1pdf[x$response == 1] <- n1PDF(x$rt[x$response == 1], A=0.5, b=1, 
                                      t0 = 0.5, 
                                      mean_v=c(1.2, 1), 
                                      sd_v=c(0.2,0.3), 
                                      distribution = "norm", silent = TRUE)
  ex1_n1pdf[x$response == 2] <- n1PDF(x$rt[x$response == 2], 
                                      A=0.5, b=1, t0 = 0.5, 
                                      mean_v=c(1, 1.2), 
                                      sd_v=c(0.3,0.2), 
                                      distribution = "norm", silent = TRUE)
  ex1_dLBA <- dLBA(x$rt, x$response, A=0.5, b=1, t0 = 0.5, 
                   mean_v=c(1.2, 1), sd_v=c(0.2,0.3), distribution = "norm")
  expect_identical(ex1_n1pdf, ex1_dLBA)
  
  ex2_n1pdf <- vector("numeric", n)
  ex2_n1pdf[x$response == 1] <- n1PDF(x$rt[x$response == 1], 
                                      A=list(0.5, 0.6), 
                                      b=1, t0 = 0.5, 
                                      mean_v=c(1.2, 1), 
                                      sd_v=c(0.2,0.3), 
                                      distribution = "norm", silent = TRUE)
  ex2_n1pdf[x$response == 2] <- n1PDF(x$rt[x$response == 2], 
                                      A=list(0.6, 0.5), 
                                      b=1, t0 = 0.5, 
                                      mean_v=c(1, 1.2), 
                                      sd_v=c(0.3,0.2), 
                                      distribution = "norm", silent = TRUE)
  ex2_dLBA <- dLBA(x$rt, x$response, 
                   A=list(0.5, 0.6), b=1, t0 = 0.5, 
                   mean_v=c(1.2, 1), sd_v=c(0.2,0.3), 
                   distribution = "norm")
  expect_identical(ex2_n1pdf, ex2_dLBA)
  
  ex3_n1pdf <- vector("numeric", n)
  ex3_n1pdf[x$response == 1] <- n1PDF(x$rt[x$response == 1], 
                                      A=rep(c(0.5, 0.6), n/2)[x$response == 1], 
                                      b=1, t0 = 0.5, 
                                      mean_v=c(1.2, 1), 
                                      sd_v=c(0.2,0.3), 
                                      distribution = "norm", silent = TRUE)
  ex3_n1pdf[x$response == 2] <- n1PDF(x$rt[x$response == 2], 
                                      A=rep(c(0.5, 0.6), n/2)[x$response == 2], 
                                      b=1, t0 = 0.5, 
                                      mean_v=c(1, 1.2), 
                                      sd_v=c(0.3,0.2), 
                                      distribution = "norm", silent = TRUE)
  ex3_dLBA <- dLBA(x$rt, x$response, A=c(0.5, 0.6), b=1, t0 = 0.5, 
                   mean_v=c(1.2, 1), sd_v=c(0.2,0.3), 
                   distribution = "norm")
  expect_identical(ex3_n1pdf, ex3_dLBA)
  
  ex4_n1pdf <- vector("numeric", n)
  ex4_n1pdf[x$response == 1] <- n1PDF(x$rt[x$response == 1], 
                                      A=list(
                                        rep(c(0.5, 0.6), n/2)[x$response == 1], 
                                        rep(c(0.6, 0.5), n/2)[x$response == 1]
                                      ), 
                                      b=1, t0 = 0.5, 
                                      mean_v=c(1.2, 1), 
                                      sd_v=c(0.2,0.3), 
                                      distribution = "norm", silent = TRUE)
  ex4_n1pdf[x$response == 2] <- n1PDF(x$rt[x$response == 2], 
                                      A=list(
                                        rep(c(0.6, 0.5), n/2)[x$response == 2], 
                                        rep(c(0.5, 0.6), n/2)[x$response == 2]
                                      ),  
                                      b=1, t0 = 0.5, 
                                      mean_v=c(1, 1.2), 
                                      sd_v=c(0.3,0.2), 
                                      distribution = "norm", silent = TRUE)
  ex4_dLBA <- dLBA(x$rt, x$response, 
                   A=list(c(0.5, 0.6), c(0.6, 0.5)), b=1, t0 = 0.5, 
                   mean_v=c(1.2, 1), sd_v=c(0.2,0.3), 
                   distribution = "norm")
  expect_identical(ex4_n1pdf, ex4_dLBA)
  
  ex5_n1pdf <- vector("numeric", n)
  ex5_n1pdf[x$response == 1] <- n1PDF(x$rt[x$response == 1], 
                                      A=list(
                                        rep(c(0.5, 0.6), n/2)[x$response == 1], 
                                        rep(c(0.6, 0.5), n/2)[x$response == 1]
                                      ),
                                      b=1, 
                                      t0 = list(0.5, 0.3), 
                                      mean_v=c(1.2, 1), 
                                      sd_v=c(0.2,0.3), 
                                      distribution = "norm", silent = TRUE)
  ex5_n1pdf[x$response == 2] <- n1PDF(x$rt[x$response == 2], 
                                      A=list(
                                        rep(c(0.6, 0.5), n/2)[x$response == 2], 
                                        rep(c(0.5, 0.6), n/2)[x$response == 2]
                                      ), 
                                      b=1, 
                                      t0 = list(0.3, 0.5), 
                                      mean_v=c(1, 1.2), 
                                      sd_v=c(0.3,0.2), 
                                      distribution = "norm", silent = TRUE)
  ex5_dLBA <- dLBA(x$rt, x$response, 
                   A=list(c(0.5, 0.6), c(0.6, 0.5)), 
                   b=1, t0 = list(0.5, 0.3), 
                   mean_v=c(1.2, 1), 
                   sd_v=c(0.2,0.3), 
                   distribution = "norm")
  expect_identical(ex5_n1pdf, ex5_dLBA)
  
  ex6_n1pdf <- vector("numeric", n)
  ex6_n1pdf[x$response == 1] <- n1PDF(x$rt[x$response == 1], 
                                      A=list(
                                        rep(c(0.5, 0.6), n/2)[x$response == 1], 
                                        rep(c(0.6, 0.5), n/2)[x$response == 1]
                                      ), 
                                      b=1, t0 = list(0.5, 0.3), 
                                      mean_v=c(1.2, 1), 
                                      sd_v=c(0.2,0.3), 
                                      distribution = "norm", 
                                      silent = TRUE, st0 = 0.2)
  ex6_n1pdf[x$response == 2] <- n1PDF(x$rt[x$response == 2], 
                                      A=list(
                                        rep(c(0.6, 0.5), n/2)[x$response == 2], 
                                        rep(c(0.5, 0.6), n/2)[x$response == 2]
                                      ), 
                                      b=1, t0 = list(0.3, 0.5), 
                                      mean_v=c(1, 1.2), 
                                      sd_v=c(0.3,0.2), 
                                      distribution = "norm", silent = TRUE, 
                                      st0 = 0.2)
  ex6_dLBA <- dLBA(x$rt, x$response, 
                   A=list(c(0.5, 0.6), c(0.6, 0.5)), 
                   b=1, t0 = list(0.5, 0.3), 
                   mean_v=c(1.2, 1), 
                   sd_v=c(0.2,0.3), 
                   distribution = "norm", st0 = 0.2)
  expect_identical(ex6_n1pdf, ex6_dLBA)
  
  
  drifts1 <- rnorm(n/2, 1, 0.1)
  drifts2 <- rnorm(n/2, 1.5, 0.2)
  
  sddrift1 <- runif(n/5, 0.1, 0.3)
  sddrift2 <- runif(n/5, 0.2, 0.4)
    
  ex7_n1pdf <- vector("numeric", n)
  ex7_n1pdf[x$response == 1] <- n1PDF(x$rt[x$response == 1], 
                                      A=list(
                                        rep(c(0.5, 0.6), n/2)[x$response == 1], 
                                        rep(c(0.6, 0.5), n/2)[x$response == 1]
                                      ), 
                                      b = list(
                                        rep(c(1, 1.1, 1.2, 1.3, 1.4), n/5)[x$response == 1], 
                                        rep(c(1, 1.1, 1.2, 1.3, 1.4) - 0.2, n/5)[x$response == 1]
                                      ),
                                      t0 = list(
                                        seq(0.1, 0.5, length.out = n)[x$response == 1], 
                                        seq(0.3, 0.8, length.out = n)[x$response == 1]), 
                                      mean_v = list(
                                        rep(drifts1, 2)[x$response == 1], 
                                        rep(drifts2, 2)[x$response == 1]
                                      ), 
                                      sd_v = list(
                                        rep(sddrift1, 5)[x$response == 1], 
                                        rep(sddrift2, 5)[x$response == 1]
                                      ), 
                                      distribution = "norm", 
                                      silent = TRUE, st0 = 0.2)
  ex7_n1pdf[x$response == 2] <- n1PDF(x$rt[x$response == 2], 
                                      A=list(
                                        rep(c(0.6, 0.5), n/2)[x$response == 2], 
                                        rep(c(0.5, 0.6), n/2)[x$response == 2]
                                      ), 
                                      b = list(
                                        rep(c(1, 1.1, 1.2, 1.3, 1.4) - 0.2, n/5)[x$response == 2],
                                        rep(c(1, 1.1, 1.2, 1.3, 1.4), n/5)[x$response == 2]
                                      ),
                                      t0 = list(
                                        seq(0.3, 0.8, length.out = n)[x$response == 2],
                                        seq(0.1, 0.5, length.out = n)[x$response == 2]
                                      ), 
                                      mean_v = list(
                                        rep(drifts2, 2)[x$response == 2], 
                                        rep(drifts1, 2)[x$response == 2]
                                      ), 
                                      sd_v = list(
                                        rep(sddrift2, 5)[x$response == 2], 
                                        rep(sddrift1, 5)[x$response == 2]
                                      ), 
                                      distribution = "norm", silent = TRUE, 
                                      st0 = 0.2)
  ex7_dLBA <- dLBA(x$rt, x$response, 
                   A = list(
                     c(0.5, 0.6), 
                     c(0.6, 0.5)
                   ),
                   b = list(
                     c(1, 1.1, 1.2, 1.3, 1.4), 
                     c(1, 1.1, 1.2, 1.3, 1.4) - 0.2
                   ),
                   t0 = list(
                     seq(0.1, 0.5, length.out = n), 
                     seq(0.3, 0.8, length.out = n)), 
                   mean_v = list(
                     drifts1, 
                     drifts2
                   ), 
                   sd_v = list(
                     sddrift1, 
                     sddrift2
                   ), 
                   distribution = "norm", st0 = 0.2)
  expect_identical(ex7_n1pdf, ex7_dLBA)
  
})

test_that("pLBA norm is identical to n1CDF", {
  n <- 100
  x <- rLBA(n, A=0.5, b=1, t0 = 0.5, mean_v=c(1.2, 1), sd_v=c(0.2,0.3))
  
  ex1_n1cdf <- vector("numeric", n)
  ex1_n1cdf[x$response == 1] <- n1CDF(x$rt[x$response == 1], A=0.5, b=1, 
                                      t0 = 0.5, 
                                      mean_v=c(1.2, 1), 
                                      sd_v=c(0.2,0.3), 
                                      distribution = "norm", silent = TRUE)
  ex1_n1cdf[x$response == 2] <- n1CDF(x$rt[x$response == 2], 
                                      A=0.5, b=1, t0 = 0.5, 
                                      mean_v=c(1, 1.2), 
                                      sd_v=c(0.3,0.2), 
                                      distribution = "norm", silent = TRUE)
  ex1_pLBA <- pLBA(x$rt, x$response, A=0.5, b=1, t0 = 0.5, 
                   mean_v=c(1.2, 1), sd_v=c(0.2,0.3), distribution = "norm")
  expect_identical(ex1_n1cdf, ex1_pLBA)
  
  ex2_n1cdf <- vector("numeric", n)
  ex2_n1cdf[x$response == 1] <- n1CDF(x$rt[x$response == 1], 
                                      A=list(0.5, 0.6), 
                                      b=1, t0 = 0.5, 
                                      mean_v=c(1.2, 1), 
                                      sd_v=c(0.2,0.3), 
                                      distribution = "norm", silent = TRUE)
  ex2_n1cdf[x$response == 2] <- n1CDF(x$rt[x$response == 2], 
                                      A=list(0.6, 0.5), 
                                      b=1, t0 = 0.5, 
                                      mean_v=c(1, 1.2), 
                                      sd_v=c(0.3,0.2), 
                                      distribution = "norm", silent = TRUE)
  ex2_pLBA <- pLBA(x$rt, x$response, 
                   A=list(0.5, 0.6), b=1, t0 = 0.5, 
                   mean_v=c(1.2, 1), sd_v=c(0.2,0.3), 
                   distribution = "norm")
  expect_identical(ex2_n1cdf, ex2_pLBA)
  
  ex3_n1cdf <- vector("numeric", n)
  ex3_n1cdf[x$response == 1] <- n1CDF(x$rt[x$response == 1], 
                                      A=rep(c(0.5, 0.6), n/2)[x$response == 1], 
                                      b=1, t0 = 0.5, 
                                      mean_v=c(1.2, 1), 
                                      sd_v=c(0.2,0.3), 
                                      distribution = "norm", silent = TRUE)
  ex3_n1cdf[x$response == 2] <- n1CDF(x$rt[x$response == 2], 
                                      A=rep(c(0.5, 0.6), n/2)[x$response == 2], 
                                      b=1, t0 = 0.5, 
                                      mean_v=c(1, 1.2), 
                                      sd_v=c(0.3,0.2), 
                                      distribution = "norm", silent = TRUE)
  ex3_pLBA <- pLBA(x$rt, x$response, A=c(0.5, 0.6), b=1, t0 = 0.5, 
                   mean_v=c(1.2, 1), sd_v=c(0.2,0.3), 
                   distribution = "norm")
  expect_identical(ex3_n1cdf, ex3_pLBA)
  
  ex4_n1cdf <- vector("numeric", n)
  ex4_n1cdf[x$response == 1] <- n1CDF(x$rt[x$response == 1], 
                                      A=list(
                                        rep(c(0.5, 0.6), n/2)[x$response == 1], 
                                        rep(c(0.6, 0.5), n/2)[x$response == 1]
                                      ), 
                                      b=1, t0 = 0.5, 
                                      mean_v=c(1.2, 1), 
                                      sd_v=c(0.2,0.3), 
                                      distribution = "norm", silent = TRUE)
  ex4_n1cdf[x$response == 2] <- n1CDF(x$rt[x$response == 2], 
                                      A=list(
                                        rep(c(0.6, 0.5), n/2)[x$response == 2], 
                                        rep(c(0.5, 0.6), n/2)[x$response == 2]
                                      ),  
                                      b=1, t0 = 0.5, 
                                      mean_v=c(1, 1.2), 
                                      sd_v=c(0.3,0.2), 
                                      distribution = "norm", silent = TRUE)
  ex4_pLBA <- pLBA(x$rt, x$response, 
                   A=list(c(0.5, 0.6), c(0.6, 0.5)), b=1, t0 = 0.5, 
                   mean_v=c(1.2, 1), sd_v=c(0.2,0.3), 
                   distribution = "norm")
  expect_identical(ex4_n1cdf, ex4_pLBA)
  
  ex5_n1cdf <- vector("numeric", n)
  ex5_n1cdf[x$response == 1] <- n1CDF(x$rt[x$response == 1], 
                                      A=list(
                                        rep(c(0.5, 0.6), n/2)[x$response == 1], 
                                        rep(c(0.6, 0.5), n/2)[x$response == 1]
                                      ),
                                      b=1, 
                                      t0 = list(0.5, 0.3), 
                                      mean_v=c(1.2, 1), 
                                      sd_v=c(0.2,0.3), 
                                      distribution = "norm", silent = TRUE)
  ex5_n1cdf[x$response == 2] <- n1CDF(x$rt[x$response == 2], 
                                      A=list(
                                        rep(c(0.6, 0.5), n/2)[x$response == 2], 
                                        rep(c(0.5, 0.6), n/2)[x$response == 2]
                                      ), 
                                      b=1, 
                                      t0 = list(0.3, 0.5), 
                                      mean_v=c(1, 1.2), 
                                      sd_v=c(0.3,0.2), 
                                      distribution = "norm", silent = TRUE)
  ex5_pLBA <- pLBA(x$rt, x$response, 
                   A=list(c(0.5, 0.6), c(0.6, 0.5)), 
                   b=1, t0 = list(0.5, 0.3), 
                   mean_v=c(1.2, 1), 
                   sd_v=c(0.2,0.3), 
                   distribution = "norm")
  expect_identical(ex5_n1cdf, ex5_pLBA)
  
  ex6_n1cdf <- vector("numeric", n)
  ex6_n1cdf[x$response == 1] <- n1CDF(x$rt[x$response == 1], 
                                      A=list(
                                        rep(c(0.5, 0.6), n/2)[x$response == 1], 
                                        rep(c(0.6, 0.5), n/2)[x$response == 1]
                                      ), 
                                      b=1, t0 = list(0.5, 0.3), 
                                      mean_v=c(1.2, 1), 
                                      sd_v=c(0.2,0.3), 
                                      distribution = "norm", 
                                      silent = TRUE, st0 = 0.2)
  ex6_n1cdf[x$response == 2] <- n1CDF(x$rt[x$response == 2], 
                                      A=list(
                                        rep(c(0.6, 0.5), n/2)[x$response == 2], 
                                        rep(c(0.5, 0.6), n/2)[x$response == 2]
                                      ), 
                                      b=1, t0 = list(0.3, 0.5), 
                                      mean_v=c(1, 1.2), 
                                      sd_v=c(0.3,0.2), 
                                      distribution = "norm", silent = TRUE, 
                                      st0 = 0.2)
  ex6_pLBA <- pLBA(x$rt, x$response, 
                   A=list(c(0.5, 0.6), c(0.6, 0.5)), 
                   b=1, t0 = list(0.5, 0.3), 
                   mean_v=c(1.2, 1), 
                   sd_v=c(0.2,0.3), 
                   distribution = "norm", st0 = 0.2)
  expect_identical(ex6_n1cdf, ex6_pLBA)
  
  
  drifts1 <- rnorm(n/2, 1, 0.1)
  drifts2 <- rnorm(n/2, 1.5, 0.2)
  
  sddrift1 <- runif(n/5, 0.1, 0.3)
  sddrift2 <- runif(n/5, 0.2, 0.4)
    
  ex7_n1cdf <- vector("numeric", n)
  ex7_n1cdf[x$response == 1] <- n1CDF(x$rt[x$response == 1], 
                                      A=list(
                                        rep(c(0.5, 0.6), n/2)[x$response == 1], 
                                        rep(c(0.6, 0.5), n/2)[x$response == 1]
                                      ), 
                                      b = list(
                                        rep(c(1, 1.1, 1.2, 1.3, 1.4), n/5)[x$response == 1], 
                                        rep(c(1, 1.1, 1.2, 1.3, 1.4) - 0.2, n/5)[x$response == 1]
                                      ),
                                      t0 = list(
                                        seq(0.1, 0.5, length.out = n)[x$response == 1], 
                                        seq(0.3, 0.8, length.out = n)[x$response == 1]), 
                                      mean_v = list(
                                        rep(drifts1, 2)[x$response == 1], 
                                        rep(drifts2, 2)[x$response == 1]
                                      ), 
                                      sd_v = list(
                                        rep(sddrift1, 5)[x$response == 1], 
                                        rep(sddrift2, 5)[x$response == 1]
                                      ), 
                                      distribution = "norm", 
                                      silent = TRUE, st0 = 0.2)
  ex7_n1cdf[x$response == 2] <- n1CDF(x$rt[x$response == 2], 
                                      A=list(
                                        rep(c(0.6, 0.5), n/2)[x$response == 2], 
                                        rep(c(0.5, 0.6), n/2)[x$response == 2]
                                      ), 
                                      b = list(
                                        rep(c(1, 1.1, 1.2, 1.3, 1.4) - 0.2, n/5)[x$response == 2],
                                        rep(c(1, 1.1, 1.2, 1.3, 1.4), n/5)[x$response == 2]
                                      ),
                                      t0 = list(
                                        seq(0.3, 0.8, length.out = n)[x$response == 2],
                                        seq(0.1, 0.5, length.out = n)[x$response == 2]
                                      ), 
                                      mean_v = list(
                                        rep(drifts2, 2)[x$response == 2], 
                                        rep(drifts1, 2)[x$response == 2]
                                      ), 
                                      sd_v = list(
                                        rep(sddrift2, 5)[x$response == 2], 
                                        rep(sddrift1, 5)[x$response == 2]
                                      ), 
                                      distribution = "norm", silent = TRUE, 
                                      st0 = 0.2)
  ex7_pLBA <- pLBA(x$rt, x$response, 
                   A = list(
                     c(0.5, 0.6), 
                     c(0.6, 0.5)
                   ),
                   b = list(
                     c(1, 1.1, 1.2, 1.3, 1.4), 
                     c(1, 1.1, 1.2, 1.3, 1.4) - 0.2
                   ),
                   t0 = list(
                     seq(0.1, 0.5, length.out = n), 
                     seq(0.3, 0.8, length.out = n)), 
                   mean_v = list(
                     drifts1, 
                     drifts2
                   ), 
                   sd_v = list(
                     sddrift1, 
                     sddrift2
                   ), 
                   distribution = "norm", st0 = 0.2)
  expect_identical(ex7_n1cdf, ex7_pLBA)
  
})


test_that("pLBA norm is identical to n1CDF", {
  x <- seq(0, 3, by =0.1)
  
  o1a <- n1CDF(x, A=0.5, b=1, t0 = 0.5, 
               mean_v=c(1.2, 1), 
               sd_v=c(0.2,0.3), 
               distribution = "norm", silent = TRUE)
  o1b <- n1CDF(x, A=0.5, b=1, t0 = 0.5, 
               mean_v=c(1, 1.2), 
               sd_v=c(0.3,0.2), 
               distribution = "norm", silent = TRUE)
  o1c <- pLBA(c(x, x), rep(1:2, each = length(x)), 
              A=0.5, b=1, t0 = 0.5, 
              mean_v=c(1.2, 1), 
              sd_v=c(0.2,0.3), 
              distribution = "norm", silent = TRUE)
  expect_identical(c(o1a, o1b), o1c)
  
  o2a <- n1CDF(x, A=0.5, b=1, t0 = list(0.5, 0.2), 
               mean_v=c(1.2, 1), 
               sd_v=c(0.2,0.3), 
               distribution = "norm", silent = TRUE)
  o2b <- n1CDF(x, A=0.5, b=1, t0 = list(0.2, 0.5), 
               mean_v=c(1, 1.2), 
               sd_v=c(0.3,0.2), 
               distribution = "norm", silent = TRUE)
  o2c <- pLBA(c(x, x), rep(1:2, each = length(x)), 
              A=0.5, b=1, t0 = list(0.5, 0.2), 
              mean_v=c(1.2, 1), 
              sd_v=c(0.2,0.3), 
              distribution = "norm", silent = TRUE)
  expect_identical(c(o2a, o2b), o2c)
  
  o3a <- n1CDF(x, A=list(seq(0.5, 0.6, length.out = length(x)), 0.2), 
               b=1, t0 = list(0.5, 0.2), 
               mean_v=c(1.2, 1), 
               sd_v=c(0.2,0.3), 
               distribution = "norm", silent = TRUE)
  o3b <- n1CDF(x, A=list(0.2, seq(0.5, 0.6, length.out = length(x))), 
               b=1, t0 = list(0.2, 0.5), 
               mean_v=c(1, 1.2), 
               sd_v=c(0.3,0.2), 
               distribution = "norm", silent = TRUE)
  o3c <- pLBA(c(x, x), rep(1:2, each = length(x)), 
              A=list(seq(0.5, 0.6, length.out = length(x)), 0.2), 
              b=1, t0 = list(0.5, 0.2), 
              mean_v=c(1.2, 1), 
              sd_v=c(0.2,0.3), 
              distribution = "norm", silent = TRUE)
  expect_identical(c(o2a, o2b), o2c)
  
})

test_that("qLBA is equivalent to manual calculation",{
  p11_norm <- structure(list(par =
                               structure(
                                 c(
                                   0.11829564514883,
                                   -2.74097628458775,
                                   1.04498350371894,
                                   0.451351158199909,
                                   0.124346087574482,
                                   0.260994169758609
                                 ),
                                 .Names = c("A", "v1", "v2", "b", "t0", "sv")
                               )))
  q <- c(0.1, 0.3, 0.5, 0.7, 0.9)
  pred_prop_correct <- pLBA(Inf, 2, 
                            A = p11_norm$par["A"], 
                            b = p11_norm$par["A"]+p11_norm$par["b"], 
                            t0 = p11_norm$par["t0"], 
                            mean_v = c(p11_norm$par["v1"], p11_norm$par["v2"]), 
                            sd_v = c(1, p11_norm$par["sv"]), 
                            silent = TRUE)

  expect_equal(qLBA(pred_prop_correct*q, 2, 
                    A = p11_norm$par["A"], 
                    b = p11_norm$par["A"]+p11_norm$par["b"], 
                    t0 = p11_norm$par["t0"], 
                    mean_v = c(p11_norm$par["v1"], p11_norm$par["v2"]), 
                    sd_v = c(1, p11_norm$par["sv"]), silent = TRUE),
               c(0.487170939752454, 0.551026400837336, 0.608185370581083, 
                 0.680979476696082, 0.830128589908231))
  
  expect_equal(suppressWarnings(qLBA(pred_prop_correct*q, 1, 
                                     A = p11_norm$par["A"], 
                                     b = p11_norm$par["A"]+p11_norm$par["b"], 
                                     t0 = p11_norm$par["t0"], 
                                     mean_v = c(p11_norm$par["v1"], 
                                                p11_norm$par["v2"]), 
                                     sd_v = c(1, p11_norm$par["sv"]), 
                                     silent = TRUE)),
               as.numeric(rep(NA, 5)))
})

test_that("rLBA works wit all distributions", {
  expect_is(rLBA(100, A=c(0.5, 0.6), b=1, t0 = 0.5, 
                 meanlog_v=c(1.2, 1), sdlog_v=c(0.2,0.3), 
                 distribution = "lnorm"), 
            "data.frame")
  expect_is(rLBA(100, A=c(0.5, 0.6), b=1, t0 = 0.5, 
                 shape_v=c(1.2, 1), scale_v=c(0.2,0.3), 
                 distribution = "frechet"), 
            "data.frame")
  expect_is(rLBA(100, A=c(0.5, 0.6), b=1, t0 = 0.5, 
                 shape_v=c(1.2, 1), scale_v=c(0.2,0.3), 
                 distribution = "gamma"), 
            "data.frame")
})

---
title: "Reanalysis of Ratcliff and Rouder (1998) with Diffusion Model and LBA"
author: "Henrik Singmann"
date: "`r Sys.Date()`"
output: 
  rmarkdown::html_vignette:
    toc: true 
vignette: >
  %\VignetteIndexEntry{Reanalysis of Ratcliff and Rouder (1998) with Diffusion Model and LBA}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

This vignette provides the `R` scripts for a reanalysis of Experiment 1 of Ratcliff and Rouder (1998). In contrast to the original analysis, which used RT bins, we will employ trial-wise maximum likelihood estimation. 

The code heavily uses [`dplyr`](https://cran.r-project.org/package=dplyr) ([vignette](https://CRAN.R-project.org/package=dplyr/vignettes/dplyr.html)),  [`tidyr`](https://cran.r-project.org/package=tidyr) ([vignette](https://CRAN.R-project.org/package=tidyr/vignettes/tidy-data.html)), and [`purrr`](https://cran.r-project.org/package=purrr) for data handling and `lattice` (see `?Lattice`) and `latticeExtra` (specifically `as.layer`) for plotting. A throrough introduction to the former three packages is provided in [R for Data Science](https://r4ds.had.co.nz/) by Wickham and Gorlemund, see especially Chapter 25.

# Description of the Experiment

In the experiment, three participants were asked to decide whether the overall brightness of pixel arrays displayed on a computer monitor was "high" or "low". To this end, the number of white versus black pixels (i.e., the brightness `strength`) was manipulated in 33 levels from 0% white pixels (level 0) to 100% white pixels (level 32). In addition, instruction manipulated speed and accuracy between blocks. In total, each participant contributed around 4000 trials per instruction condition.

The experiment contained another manipulation, the distribution (or brightness `source`) from which the pixel array was drawn. One distribution mean was on the "high" brightness side and one distribution mean was on the "low" brightness side. However, as the distributions were unbounded and overlapping, the same strength level could come from either distribution. Participant also received feedback whether or not they had picked the correct distribution (e.g., for the middle strength level 16 probability of belonging to either source was 50%). We do not further consider this manipulation in the following, which seems to be in line with the analysis of Ratcliff and Rouder (1998).

# Descriptive data

As a first step, we load the data and then plot the probability with which each response (i.e., "dark" or "light") is given as a function of strength and instruction condition. This clearly shows that there is a massive effect of strength on which response is given while at the same time the instruction only seems to have a minor effect and more on the extremes than in the middle.

```{r, fig.height=4, fig.width=7, message=FALSE, warning=FALSE}
require(rtdists)
require(dplyr)   # for data manipulations and looping
require(tidyr)   # for data manipulations
require(purrr)   # for data manipulations
require(lattice) # for plotting and corresponding themes
require(latticeExtra)
lattice.options(default.theme = standard.theme(color = FALSE))
lattice.options(default.args = list(as.table = TRUE))
options(digits = 3) # only three decimal digits
require(binom)  # for binomial confidence intervals

data(rr98)
rr98 <- rr98[!rr98$outlier,]  #remove outliers

# aggregate data for first plot:
agg_rr98 <- rr98  %>% group_by(id, instruction, strength) %>% 
  summarise(prop = mean(response == "dark"), mean_rt = mean(rt), median_rt = mean(rt)) %>% 
  ungroup()

xyplot(prop ~ strength|id, agg_rr98, group = instruction, type = "b", 
       auto.key = list(lines = TRUE), ylab = "Proportion of 'dark' responses")


```

Next, we want to get an overview of the response time distributions. For this we look at the response times of the five quantiles (i.e., 0.1, 0.3, 0.5/median, 0.7, 0.9) across the strength manipulations. This time, we also separate the plots by condition as the speed condition resulted in, as expected, vastly shorter response times. These two plots reveal considerable differences between the two instruction conditions. 

```{r, fig.height=6, fig.width=7}

quantiles <- c(0.1, 0.3, 0.5, 0.7, 0.9)
## aggregate data for quantile plot
quantiles_rr98 <- rr98  %>% 
  group_by(id, instruction, strength) %>% 
  nest() %>% 
  mutate(quantiles = map(data, ~ as.data.frame(t(quantile(.x$rt, probs = quantiles))))) %>% 
  unnest(quantiles) %>% 
  gather("quantile", "rt",`10%`:`90%`) %>% 
  arrange(id, instruction, strength)

xyplot(rt ~ strength|id + instruction, quantiles_rr98, group = quantile, type = "b", 
       auto.key = list(lines = TRUE), ylab = "RT (in seconds)", subset = instruction == "speed")

xyplot(rt ~ strength|id + instruction, quantiles_rr98, group = quantile, type = "b", 
       auto.key = FALSE, ylab = "RT (in seconds)", subset = instruction == "accuracy")


```


In the speed conditions, response times were, as expected, generally fast and there seemed to be hardly any effect of strength. Only for one participant, `nh`, we can see a small increase in RTs for the higher quantiles for strength values near the middle. In contrast, in the accuracy condition strength has a considerable effect on response times for all participants. Again, this increase was especially strong for the slower responses (i.e., the higher quantiles). For those we see a strong inverse u-shaped effect, symmetrically around the middle -- where the probability for each response is 50% -- with very high response times for strength values near the middle. 

However, as this plot is getting a little bit messy, we now bin the strength levels to remove noise which should provide a clearer overview. For this, we will construct five separate strength bins with approximately equal response behavior and comparable numbers of trials. This is similar to what was done originally by Ratcliff and Rouder (1998). The next table shows the number of trials per participant, bin, and response.


```{r, fig.height=4, fig.width=7}

#bins <- c(-0.5, 5.5, 10.5, 13.5, 16.5, 19.5, 25.5, 32.5) # seven bins like RR98
bins <- c(-0.5, 10.5, 13.5, 16.5, 19.5, 32.5)
rr98$strength_bin <- cut(rr98$strength, breaks = bins, include.lowest = TRUE)
levels(rr98$strength_bin) <- as.character(1:7)

# aggregate data for response probability plot:
agg_rr98_bin <- rr98 %>% 
  group_by(id, instruction, strength_bin) %>%
  summarise(n1 = n(), 
            dark = sum(response == "dark"),
            light = sum(response == "light")) %>%
  ungroup() %>%
  mutate(prop = map2(dark, n1, ~ binom.confint(.x, .y, methods = "agresti-coull"))) %>% 
  unnest(prop)
  

knitr::kable(
  rr98 %>% group_by(id, instruction, strength_bin, response) %>%
    summarise(n = n()) %>%
    spread(strength_bin, n)
)


```

We first look again and the response proportions and see more clearly the difference between the strength conditions at the outer bins.

```{r, fig.height=4, fig.width=7}
xyplot(mean ~ strength_bin|id, agg_rr98_bin, group = instruction, type = "b", 
       auto.key = list(lines = TRUE), ylab = "Proportion of 'dark' responses")

```

Now we also look again at the RT quantiles and see more clearly the symmetrical inverse u-shaped increase in RTs for the middle bins described above.

```{r, fig.height=6, fig.width=7}

## aggregate data for quantile plot
quantiles_rr98_bin <- rr98  %>% 
  group_by(id, instruction, strength_bin) %>% 
  nest() %>% 
  mutate(quantiles = map(data, ~ as.data.frame(t(quantile(.x$rt, probs = quantiles))))) %>% 
  unnest(quantiles) %>% 
  gather("quantile", "rt",`10%`:`90%`) %>% 
  arrange(id, instruction, strength_bin)

xyplot(rt ~ strength_bin|id + instruction, quantiles_rr98_bin, group = quantile, type = "b", 
       auto.key = list(lines = TRUE), ylab = "RT (in seconds)", subset = instruction == "speed")

xyplot(rt ~ strength_bin|id + instruction, quantiles_rr98_bin, group = quantile, type = "b", 
       auto.key = FALSE, ylab = "RT (in seconds)", subset = instruction == "accuracy")

```

With this clear pattern we now take a look at the RT distributions separately for both responses to see if they are simply mirror images of each other or not. For this, we overlay the two RT quantile plots for all trials in which the responses was "dark" in black (there are more "dark" pixels for the bins on the left side of the plot) with the same plot in which the responses was "light" in grey (there are more "light" pixels for the bins on the right side of the plot).

```{r, fig.height=6, fig.width=7}

agg2_rr98_response <- rr98  %>% 
  group_by(id, instruction, strength_bin, response) %>% 
  nest() %>% 
  mutate(quantiles = map(data, ~ as.data.frame(t(quantile(.x$rt, probs = quantiles))))) %>% 
  unnest(quantiles) %>% 
  gather("quantile", "rt",`10%`:`90%`) %>% 
  arrange(id, instruction, response, strength_bin)

p1 <- xyplot(rt ~ strength_bin|id, agg2_rr98_response, group = quantile, type = "b", 
             auto.key = list(lines = TRUE), ylab = "RT (in seconds)", 
             subset = instruction == "speed" & response == "dark", layout = c(3,1))
p2 <- xyplot(rt ~ strength_bin|id, agg2_rr98_response, group = quantile, type = "b", 
             auto.key = list(lines = TRUE), ylab = "RT (in seconds)", 
             subset = instruction == "speed" & response == "light", col = "grey")
p1 + as.layer(p2)


p1 <- xyplot(rt ~ strength_bin|id, agg2_rr98_response, group = quantile, type = "b", 
             auto.key = list(lines = TRUE), ylab = "RT (in seconds)", 
             subset = instruction == "accuracy" & response == "dark", layout = c(3,1))
p2 <- xyplot(rt ~ strength_bin|id, agg2_rr98_response, group = quantile, type = "b", 
             auto.key = list(lines = TRUE), ylab = "RT (in seconds)", 
             subset = instruction == "accuracy" & response == "light", col = "grey")
p1 + as.layer(p2)


```

These two plots reveal an interesting pattern. In the speed condition (upper plot), we particularly see fast "errors" (i.e., responses to "dark" when there are more light pixels or the other way round). When "dark" is the more likely response (i.e. on the left side) the "light" responses in grey are faster and this is especially true for the lower quantiles. The opposite pattern seems to hold on the opposite side where "dark" responses in black are faster than "light" responses in grey. At intermediate bins the difference seems to be rather at the higher quantiles. This is particularly noticeable for participant `kr` for which for there seem to be slow "light"-"errors" just to the left to the middle bin and slow "right"-"errors" just to the right of the middle bin.

For the accuracy condition in the lower plot the pattern is noticeably different. First of all, there are only very few or no "error" responses in the extreme bins. Consequently, there does not seem to be any evidence for fast errors at the extremes (and also not at intermediate strength levels). However, we here more clearly see the slow errors at the intermediate bins. When "dark" is somewhat more probably (i.e., to the left of the middle) "light" responses are noticeably slower than "dark" responses. The same holds for "dark" responses if "light" is more probable. Importantly, this shows that the symmetrical inverse u-shaped increase for the middle bins described above is actually a consequence of a mixture of slow "errors", two asymmetric increases for the two different responses.



# Diffusion Model Analysis

We will follow Ratcliff and Rouder (1998) and analyze the data with the diffusion model. For this, we will fit a separate model to each participant and instruction condition. To do so, we will first create a new data set we will use for fitting. This data set will be `nested` with one row for each combinations of the variables:

```{r}
d_nested <- rr98 %>% 
  group_by(id, instruction) %>% # we loop across both, id and instruction
  nest()
d_nested
```

Like Ratcliff and Rouder we will fit the data to the strength bins instead of the full strength manipulation. We fit basically the full diffusion model (with the exception of $s_{t0}$) to each instruction condition which results in 10 parameters per participant and instruction condition:

- 5 drift rates $v$ (i.e., one per strength bin)
- 1 boundary separation $a$
- 1 non-decision time $t_0$
- 1 drift rate variability $s_v$
- 1 start point $z$ (for ease in interpretation we parameterize this as the *relative* start point so that values different from 0.5 indicate a bias towards one of the responses)
- 1 start point variability $s_z$

Like in Ratcliff and Rouder (1998), the two response boundaries are the two response options "dark" and "light". To estimate the model we diverge from Ratcliff and Rouder and employ trial wise maximum likelihood estimation (i.e., no binning of responses). 

For this, we simply need to have a wrapper function which returns us the negative summed log-likelihood of the data (i.e., RTs and corresponding responses) given a set of parameters. We need the negativ sum because most optimization function minimize whereas we want to obtain the maximum likelihood value. The following function for which we simply loop across drift rates will do so:

```{r}
# objective function for diffusion with 1 a. loops over drift to assign drift rates to strength
objective_diffusion_separate <- function(pars, rt, response, drift, ...) {
  non_v_pars <- grep("^v", names(pars), invert = TRUE, value = TRUE)
  base_par <- length(non_v_pars)  # number of non-drift parameters
  densities <- vector("numeric", length(rt))
  for (i in seq_along(levels(drift))) {
    densities[drift == levels(drift)[i]] <- 
      ddiffusion(rt[drift == levels(drift)[i]], response=response[drift == levels(drift)[i]], 
                 a=pars["a"], t0=pars["t0"],  
                 sv=pars["sv"],
                 sz=if ("sz" %in% non_v_pars) pars["sz"] else 0.1,
                 z=if ("z" %in% non_v_pars) pars["z"]*pars["a"] else 0.5*pars["a"],
                 st0=if ("st0" %in% non_v_pars) pars["st0"] else 0, 
                 v=pars[base_par+i])
  }
  if (any(densities == 0)) return(1e6)
  return(-sum(log(densities)))
}

```

Note that the function is written in such a way that we could easily fix certain parameters without the necessity to change it (using `if`-`then` on the parameters names passed via `pars`).

Additionally, we also need a function that generates a set of random starting values. And, as any random set of starting values may be impossible, another wrapper function that generates starting values until a set of valid starting values is found and then passes those to the optimization routine. As optimization routine we will be using `nlminb`. These functions are given next and are specified in a way that they will be usable for other model variants for this data (e.g., fixing parameters). 

```{r}

# function that creates random start values, also 
get_start <- function(base_par, n_drift = 5) {
  start1 <- c(
    a = runif(1, 0.5, 3),
    a_1 = runif(1, 0.5, 3), 
    a_2 = runif(1, 0.5, 3),
    t0 = runif(1, 0, 0.5), 
    z = runif(1, 0.4, 0.6), 
    sz = runif(1, 0, 0.5),
    sv = runif(1, 0, 0.5),
    st0 = runif(1, 0, 0.5),
    d = rnorm(1, 0, 0.05)
  )
  start2 <- sort(rnorm(n_drift), decreasing = FALSE)
  names(start2) <- paste0("v_", seq_len(n_drift))
  c(start1[base_par], start2)
}

# function that tries different random start values until it works:
ensure_fit <- 
  function(data, start_function, objective_function, 
           base_pars, n_drift = 5, n_fits = 1, 
           lower = c(rep(0, length(base_pars)), -Inf,
                     rep(-Inf,length(start_function(base_pars))-length(base_pars)))) {
    best_fit <- list(objective = 1e+06)
  for (i in seq_len(n_fits)) {
    start_ll <- 1e+06
    #browser()
    while(start_ll == 1e+06) {
      start <- start_function(base_pars, n_drift=n_drift)
      start_ll <- objective_function(start, 
                                     rt = data$rt, response = data$response_num, 
                                     drift = factor(data$strength_bin, seq_len(n_drift)), 
                                     instruction = data$instruction)
    }
    cat("\nstart fitting.\n") # just for information to see if it is stuck
    
    fit <- nlminb(start, objective_function, 
                  rt = data$rt, response = data$response_num, 
                  drift = factor(data$strength_bin, seq_len(n_drift)), 
                  instruction = data$instruction,
                  lower = lower)
    
    if (fit$objective < best_fit$objective) best_fit <- fit
  }
  out <- as.data.frame(t(unlist(best_fit[1:3])))
  colnames(out) <- sub("par.", "", colnames(out))
  out
}

```

```{r, echo=FALSE}
load("rr98_full-diffusion_fits.rda")
load("rr98_full-lba_fits.rda")

```

With these functions in place, we now simply need to loop over participants and items to obtain the fit. The simplest way is perhaps to use the combination of `purrr:map` and `dplyr::mutate` as shown here:


```{r, eval = FALSE}
fit_diffusion <- d_nested %>% 
  mutate(fit = 
           map(data, 
               ~ensure_fit(data = ., start_function = get_start, 
                            objective_function = objective_diffusion_separate, 
                            base_pars = c("a", "t0", "sv", "sz", "z")))) %>% 
  unnest(fit)
```

On Unix-like systems (i.e., Linux and Mac) we can easily use the inbuild multicore functionality using `parallel::mclapply` to distribute fitting of the different parts across different cores:

```{r, eval = FALSE}
require(parallel)

fit_diffusion <- d_nested
fit_diffusion$fit <- 
  mclapply(d_nested$data, function(x) 
    ensure_fit(data = x, start_function = get_start,
               objective_function = objective_diffusion_separate, 
               base_pars = c("a", "t0", "sv", "sz", "z")),  
    mc.cores = 2)
fit_diffusion <- unnest(fit_diffusion, fit)
```


The following table gives the parameter values, the negative summed log-likelihoods (i.e., `objective`, where smaller is better), and the convergence code of the optimization algorithm (where 0 indicates no problem) obtained from this fit:

```{r}

fit_diffusion$data <- NULL
if (!("st0" %in% colnames(fit_diffusion))) fit_diffusion$st0 <- 0
if (!("z" %in% colnames(fit_diffusion))) fit_diffusion$z <- 0.5
if (!("sz" %in% colnames(fit_diffusion))) fit_diffusion$sz <- 0.1
knitr::kable(fit_diffusion)

```

We can see from these values that there is a large effect of instruction on $a$. However, instruction also has effects on other parameters:

- $t_0$ is consistently larger in the accuracy compared to the speed condition, although this effect is small.
- $s_v$ is estimated at 0 or very low in the speed condition, but 0.5 or 1 in the accuracy condition. This is consistent with the absence of slow "errors" in the speed condition.
- $s_z$ is consistently larger in the speed conditions consistent with the presence or more fast "errors" in the speed than in the accuracy condition.
- $v$ appears to be more extreme (i.e., smaller for $v_1$ and $v_2$ and larger for $v_4$ and $v_5$) in the speed compared to the accuracy condition.


```{r obtain_fits_not_run, eval = FALSE, include = FALSE}

require(parallel)

fit_diffusion <- d_nested %>% 
  mutate(fit = 
           map(data, 
               ~ensure_fit(data = ., start_function = get_start, 
                            objective_function = objective_diffusion_separate, 
                            base_pars = c("a", "t0", "sv", "sz", "z")))) %>% 
  unnest(fit)
fit_diffusion$data <- NULL

fit_diffusion2 <- d_nested
fit_diffusion2$fit <- 
  mclapply(d_nested$data, function(x) 
    ensure_fit(data = x, start_function = get_start,
               objective_function = objective_diffusion_separate, 
               base_pars = c("a", "t0", "sv", "sz", "z")),  
    mc.cores = 3)
fit_diffusion2 <- unnest(fit_diffusion2, fit)
fit_diffusion2$data <- NULL

all.equal(as.data.frame(fit_diffusion), as.data.frame(fit_diffusion2), tolerance = 0.01)

save(fit_diffusion, fit_diffusion2, file = "rr98_full-diffusion_fits.rda")


```

## Graphical Model Fit

### Predicted Response Rates

To evaluate the fits graphically we first compare the actual response rates for the two responses with the predicted responses rates. The grey lines and points show the observed data and the error bars are binomial confidence intervals. The black lines and points show the predicted response rates.


```{r, fig.height=5, fig.width=7, message=FALSE}


# get predicted response proportions
pars_separate_l <- fit_diffusion %>% gather("strength_bin", "v", starts_with("v"))
pars_separate_l$strength_bin <- factor(substr(pars_separate_l$strength_bin, 3,3), 
                                       levels = as.character(seq_len(length(bins)-1)))
#pars_separate_l <- inner_join(pars_separate_l, agg_rr98_bin)
pars_separate_l <- pars_separate_l  %>% group_by(id, instruction, strength_bin) %>%
  mutate(resp_prop = pdiffusion(rt=Inf, response="lower", 
                                a=a, v=v, t0=t0, sz = sz, z=a*z, sv=sv, st0=st0))

p1 <- xyplot(mean ~ strength_bin|id + instruction, agg_rr98_bin, type = "b", auto.key = 
               list(lines = TRUE), ylab = "Proportion of 'dark' responses", col = "grey")
p2 <- segplot(strength_bin ~ upper+lower|id + instruction, agg_rr98_bin, 
              auto.key = list(lines = TRUE), ylab = "Proportion of 'dark' responses", 
              col = "grey", horizontal = FALSE, segments.fun = panel.arrows,  
              draw.bands = FALSE, angle = 90, length = 0.05, ends = "both")
p3 <- xyplot(resp_prop ~ strength_bin|id + instruction, pars_separate_l, type = "b", 
             auto.key = list(lines = TRUE), ylab = "Proportion of 'dark' responses", 
             col = "black")
p2 + as.layer(p1) + as.layer(p3)

```

This figure show that overall the model can predict the actual response rates very accurately. There are only a few minor deviations.


### Predicted Median RTs

Next we compare the central tendency of the RTs with the prediction. For this we evaluate the CDF at the quantiles of the predicted response proportions. Again, the data is shown in grey (and error bars show the standard errors of the median) and the predicted RTs in black. We first show the predictions for the speed condition, separated by response.

```{r, fig.height=6, fig.width=7, message=FALSE}

# get predicted quantiles (uses predicted response proportions)
separate_pred_dark <- pars_separate_l %>% do(as.data.frame(t(
  qdiffusion(quantiles*.$resp_prop, response="lower", 
             a=.$a, v=.$v, t0=.$t0, sz = .$sz, z = .$z*.$a, sv=.$sv, st0=.$st0)))) %>% 
  ungroup() %>% gather("quantiles", "dark", V1:V5)
separate_pred_light <- pars_separate_l %>% do(as.data.frame(t(
  qdiffusion(quantiles*(1-.$resp_prop), response="upper", 
             a=.$a, v=.$v, t0=.$t0, sz = .$sz, z = .$z*.$a, sv=.$sv, st0=.$st0)))) %>% 
  ungroup() %>% gather("quantiles", "light", V1:V5)

#separate_pred_light %>% filter(is.na(light))
separate_pred <- inner_join(separate_pred_dark, separate_pred_light)
separate_pred$quantiles <- factor(separate_pred$quantiles, 
                                  levels = c("V5", "V4", "V3", "V2", "V1"), 
                                  labels = c("90%", "70%", "50%", "30%", "10%"))
separate_pred <- separate_pred %>% gather("response", "rt", dark, light)

# get SE for observed quantiles
agg2_rr98_response_se <- rr98  %>% group_by(id, instruction, strength_bin, response) %>% 
  summarise(se_median = sqrt(pi/2)*(sd(rt)/sqrt(n()))) %>%
  ungroup()

# calculate error bars for quantiles.
agg2_rr98_response <- left_join(agg2_rr98_response, agg2_rr98_response_se)
agg2_rr98_response <- agg2_rr98_response %>%
  mutate(lower = rt-se_median, upper = rt+se_median)


p1 <- xyplot(rt ~ strength_bin|id+response, agg2_rr98_response, type = "b", 
             auto.key = list(lines = TRUE), ylab = "RT (in seconds)", 
             subset = instruction == "speed" & quantile == "50%", 
             layout = c(3,2), col = "grey")
p1e <- segplot(strength_bin ~ upper+lower|id+response, agg2_rr98_response, 
               auto.key = list(lines = TRUE), ylab = "Proportion of 'dark' responses", 
               col = "grey", horizontal = FALSE, segments.fun = panel.arrows,  
               draw.bands = FALSE, angle = 90, length = 0.05, ends = "both", 
               subset = instruction == "speed" & quantile == "50%", layout = c(3,2))
p2 <- xyplot(rt ~ strength_bin|id + response, separate_pred, type = "b", 
             auto.key = list(lines = TRUE), ylab = "RT (in seconds)", 
             subset = instruction == "speed" & quantiles == "50%", 
             scales = list(y = list(limits = c(0.25, 0.5))))
p2 + as.layer(p1) + as.layer(p1e)

```

Again, the model seems to be overall able to describe the general pattern quite well. However, there are some visible misfits for participants `jf` and `nh`. 

Next shows the same plot for the accuracy condition. Here we again see that the model is able to predict the pattern in the data quite well. While there also seems to be quite some misfit for participant `nh` this only appears in conditions with very little trials as indicated by the large error bars.

```{r, fig.height=6, fig.width=7}

p1 <- xyplot(rt ~ strength_bin|id+response, agg2_rr98_response, type = "b", 
             auto.key = list(lines = TRUE), ylab = "RT (in seconds)", 
             subset = instruction == "accuracy" & quantile == "50%", 
             layout = c(3,2), col = "grey")
p1e <- segplot(strength_bin ~ upper+lower|id+response, agg2_rr98_response, 
               auto.key = list(lines = TRUE), ylab = "Proportion of 'dark' responses", 
               col = "grey", horizontal = FALSE, segments.fun = panel.arrows,  
               draw.bands = FALSE, angle = 90, length = 0.05, ends = "both", 
               subset = instruction == "accuracy" & quantile == "50%", layout = c(3,2))
p2 <- xyplot(rt ~ strength_bin|id + response, separate_pred, type = "b", 
             auto.key = list(lines = TRUE), ylab = "RT (in seconds)", 
             subset = instruction == "accuracy" & quantiles == "50%", 
             scales = list(y = list(limits = c(0.2, 1.5))))
p2 + as.layer(p1) + as.layer(p1e)

```



### All quantiles

Next, we investigate the full RT distribution by comparing observed and predicted quantiles. The observed quantiles are again displayed in grey and the predictions in black. The first plot shows the sped condition separated by response. 


```{r, fig.height=7, fig.width=7}

p1 <- xyplot(rt ~ strength_bin|id+response, agg2_rr98_response, group = quantile, type = "b", 
             auto.key = list(lines = TRUE), ylab = "RT (in seconds)", 
             subset = instruction == "speed", layout = c(3,2), col = "grey")
p1e <- segplot(strength_bin ~ upper+lower|id+response, agg2_rr98_response, 
               auto.key = list(lines = TRUE), ylab = "Proportion of 'dark' responses", 
               col = "grey", horizontal = FALSE, segments.fun = panel.arrows,  
               draw.bands = FALSE, angle = 90, length = 0.05, ends = "both", 
               subset = instruction == "speed")
p2 <- xyplot(rt ~ strength_bin|id + response, separate_pred, group = quantiles, type = "b", 
             auto.key = list(lines = TRUE), ylab = "RT (in seconds)", 
             subset = instruction == "speed", scales = list(y = list(limits = c(0.2, 0.9))))
p2 + as.layer(p1) + as.layer(p1e)

```

This plots shows some clear misfits for the diffusion model, particularly in the upper quantiles. But there are also misfits in the lower quantiles. 

The next plot shows the accuracy condition separated by response. Here it appears that the diffusion model provides an overall better account. However, it is important to consider the different y-axis scaling of both plots.


```{r, fig.height=7, fig.width=7}

p1 <- xyplot(rt ~ strength_bin|id+response, agg2_rr98_response, group = quantile, type = "b", 
             auto.key = list(lines = TRUE), ylab = "RT (in seconds)", 
             subset = instruction == "accuracy", layout = c(3,2), col = "grey")
p1e <- segplot(strength_bin ~ upper+lower|id+response, agg2_rr98_response, 
               auto.key = list(lines = TRUE), ylab = "Proportion of 'dark' responses", 
               col = "grey", horizontal = FALSE, segments.fun = panel.arrows,  
               draw.bands = FALSE, angle = 90, length = 0.05, ends = "both", 
               subset = instruction == "accuracy")
p2 <- xyplot(rt ~ strength_bin|id + response, separate_pred, group = quantiles, type = "b", 
             auto.key = list(lines = TRUE), ylab = "RT (in seconds)", 
             subset = instruction == "accuracy", scales = list(y = list(limits = c(0.1, 3.0))))
p2 + as.layer(p1) + as.layer(p1e)

```

Overall the diffusion model provides a good account of the data. Only when considering all quantiles we see some clear misfits. Nevertheless, the general trends in the data are well recovered, the only exceptions here are conditions with very low numbers of trials. 

# LBA analysis


Next, we fit the LBA model to the data. For this, we use an LBA model with the same number of parameters. To make the model identifiable, we fix the sum of the drift rates to 1. Specifically, the model has the following 10 parameters per participant and instruction condition:

- 5 drift rates $v$ (i.e., one per strength bin)
- 2 response thresholds $A$ (i.e., one for each accumulator)
- 1 non-decision time $t_0$ (i.e., one per participant)
- 1 drift rate variability $s_v$ 
- 1 starting points $b$ (parameterized as an increment to the max value of the two $A$)

To fit the model we need an objective function wrapping the LBA PDF and a new function for generating the correct starting values.

```{r}

# objective function for diffusion with 1 a. loops over drift to assign drift rates to strength
objective_lba_separate <- function(pars, rt, response, drift, ...) {
  non_v_pars <- grep("^v", names(pars), invert = TRUE, value = TRUE)
  base_par <- length(non_v_pars)  # number of non-drift parameters
  densities <- vector("numeric", length(rt))
  for (i in seq_along(levels(drift))) {
    if (sum(drift == levels(drift)[i]) == 0) next
    densities[drift == levels(drift)[i]] <- dLBA(
      rt[drift == levels(drift)[i]], 
      response=response[drift == levels(drift)[i]],
      A = list(pars["a_1"], pars["a_2"]), 
      b = max(pars["a_1"], pars["a_2"])+pars["b"], 
      t0 = pars["t0"], 
      mean_v = c(pars[i], 1-pars[i]), 
      sd_v = pars["sv"], silent=TRUE)
  }
  if (any(densities == 0)) return(1e6)
  return(-sum(log(densities)))
}

# function that creates random start values
get_start_lba <- function(base_par, n_drift = 10) {
  start1 <- c(
    a = runif(1, 0.5, 3),
    a_1 = runif(1, 0.5, 3), 
    a_2 = runif(1, 0.5, 3),
    t0 = runif(1, 0, 0.5), 
    b = runif(1, 0, 0.5), 
    sv = runif(1, 0.5, 1.5),
    st0 = runif(1, 0, 0.5)
  )
  start2 <- sort(rnorm(n_drift), decreasing = FALSE)
  names(start2) <- paste0("v_", seq_len(n_drift))
  c(start2, start1[base_par])
}

```

With this, we simply need to loop across participants and instructions to estimate the LBA. Again, we need to run several fitting runs to reach the maximum likelihood estimate (i.e., the global optimum).

```{r, eval=FALSE}

fit_lba <- d_nested %>% 
  mutate(fit = 
           map(data, 
               ~ensure_fit(data = ., start_function = get_start_lba, 
                      objective_function = objective_lba_separate, 
                      base_pars = c("a_1", "a_2", "t0", "b", "sv"),
                      lower = c(rep(-Inf, 5), rep(0, 5)),
                      n_drift = 5, n_fits = 10))) %>% 
  unnest(fit)

```

Again, on Unix-like systems (i.e., Linux and Mac) we can use multicore using `parallel::mclapply`:

```{r, eval = FALSE}
require(parallel)

fit_lba <- d_nested
fit_lba$fit <- 
  mclapply(d_nested$data, function(x) 
    ensure_fit(data = x, start_function = get_start_lba, 
                      objective_function = objective_lba_separate, 
                      base_pars = c("a_1", "a_2", "t0", "b", "sv"),
                      lower = c(rep(-Inf, 5), rep(0, 5)),
                      n_drift = 5, n_fits = 10),  
    mc.cores = 2)
fit_lba <- unnest(fit_lba, fit)
```


The following table gives the parameters and the negative summed log-likelihoods obtained from the LBA fit (with $b$ already correctly transformed by the maximum $A$). Note that some of the parameters might differ slightly for for id = `kr` and instruction = `accuracy` although the value of the objective function is identical to the reported one. This suggests that the likelihood surface is either quite shallow near the MLE or there are at least two peaks in the likelihood surface with a similar maximum. The fact that the convergence code for this data set is 1 instead of 0 also suggests some problems in finding the gloval optimum. In any case, running the optimization multiple times and comparing the results should reveal such problems. 

```{r}
knitr::kable(fit_lba)

```

The negtaive log-likelihood (column `objective`) shows that the LBA provides a better account for four of the six data sets (because it is the negative log-likelihood, smaller is better). The diffusion model only provides a better account for the `kr` and `nh` accuracy conditions. In terms of the parameter estimates we see a pattern similar as the one for the diffusion model:

- Instruction shows the expected effect on $b$.
- Instruction also shows an effect on $A$, which is also larger in the accuracy compared to the speed condition.
- Instruction seems to have a small effect on $t_0$ in the same direction.
- Instruction also affected $s_v$ in the same direction.
- Instruction also affected $v$ which seemed to be more extreme in the accuracy compared to the speed condition.

```{r obtain_fits_lba, eval = FALSE, include = FALSE}
fit_lba <- d_nested %>% 
  mutate(fit = 
           map(data, 
               ~ensure_fit(data = ., start_function = get_start_lba, 
                      objective_function = objective_lba_separate, 
                      base_pars = c("a_1", "a_2", "t0", "b", "sv"),
                      lower = c(rep(-Inf, 5), rep(0, 5)),
                      n_drift = 5, n_fits = 10))) %>% 
  unnest(fit)
fit_lba$data <- NULL

fit_lba2 <- d_nested
fit_lba2$fit <- 
  mclapply(d_nested$data, function(x) 
    ensure_fit(data = x, start_function = get_start_lba, 
                      objective_function = objective_lba_separate, 
                      base_pars = c("a_1", "a_2", "t0", "b", "sv"),
                      lower = c(rep(-Inf, 5), rep(0, 5)),
                      n_drift = 5, n_fits = 10),  
    mc.cores = 2)
fit_lba2 <- unnest(fit_lba2, fit)
fit_lba2$data <- NULL

all.equal(as.data.frame(fit_lba), as.data.frame(fit_lba2), tolerance = 0.03)
save(fit_lba, fit_lba2, file = "rr98_full-lba_fits.rda")



# objective function for LBA with 1 a. loops over drift to assign drift rates to strength
objective_lba_separate <- function(pars, rt, response, drift, ...) {
  non_v_pars <- grep("^v", names(pars), invert = TRUE, value = TRUE)
  base_par <- length(non_v_pars)  # number of non-drift parameters
  densities <- vector("numeric", length(rt))
  for (i in seq_along(levels(drift))) {
    if (sum(drift == levels(drift)[i]) == 0) next
    densities[drift == levels(drift)[i]] <- dLBA(
      rt[drift == levels(drift)[i]], 
      response=response[drift == levels(drift)[i]],
      A = pars["a"], b = pars["a"]+pars["b"], 
      t0 = pars["t0"], 
      mean_v = pars[((i-1)*2+1):((i-1)*2+2)], 
      sd_v = c(1, pars["sv"]), silent=TRUE)
  }
  if (any(densities == 0)) return(1e6)
  return(-sum(log(densities)))
}

# objective function for diffusion with 1 a. loops over drift to assign drift rates to strength
objective_lba_separate <- function(pars, rt, response, drift, ...) {
  non_v_pars <- grep("^v", names(pars), invert = TRUE, value = TRUE)
  base_par <- length(non_v_pars)  # number of non-drift parameters
  densities <- vector("numeric", length(rt))
  for (i in seq_along(levels(drift))) {
    if (sum(drift == levels(drift)[i]) == 0) next
    densities[drift == levels(drift)[i]] <- dLBA(
      rt[drift == levels(drift)[i]], 
      response=response[drift == levels(drift)[i]],
      A = list(pars["a_1"], pars["a_2"]), b = list(pars["a_1"]+pars["b"], pars["a_2"]+pars["b"]), 
      t0 = pars["t0"], 
      mean_v = c(pars[i], 1-pars[i]), 
      sd_v = pars["sv"], silent=TRUE)
  }
  if (any(densities == 0)) return(1e6)
  return(-sum(log(densities)))
}

```


## Graphical Model Fit

The fact that the LBA provides a slightly better fit is also visible in the graphical fit assessment.

### Predicted Response Rates

We again first consider the predicted response rates (in black) and they are also highly accurate.

```{r, fig.height=5, fig.width=7, message=FALSE}
# get predicted response proportions
lba_pars_separate_l <- fit_lba %>% gather("strength_bin", "v", starts_with("v"))
lba_pars_separate_l$strength_bin <- factor(substr(lba_pars_separate_l$strength_bin, 3,3), 
                                       levels = as.character(seq_len(length(bins)-1)))
#pars_separate_l <- inner_join(pars_separate_l, agg_rr98_bin)
lba_pars_separate_l <- lba_pars_separate_l  %>% group_by(id, instruction, strength_bin) %>%
  mutate(resp_prop = pLBA(rt=Inf, response=1, A=list(a_1, a_2), sd_v=sv,
                          mean_v=c(v, 1-v), t0=t0, b=max(a_1, a_2)+b, silent=TRUE))

p1 <- xyplot(mean ~ strength_bin|id + instruction, agg_rr98_bin, type = "b", auto.key = 
               list(lines = TRUE), ylab = "Proportion of 'dark' responses", col = "grey")
p2 <- segplot(strength_bin ~ upper+lower|id + instruction, agg_rr98_bin, 
              auto.key = list(lines = TRUE), ylab = "Proportion of 'dark' responses", 
              col = "grey", horizontal = FALSE, segments.fun = panel.arrows,  
              draw.bands = FALSE, angle = 90, length = 0.05, ends = "both")
p3 <- xyplot(resp_prop ~ strength_bin|id + instruction, lba_pars_separate_l, type = "b", 
             auto.key = list(lines = TRUE), ylab = "Proportion of 'dark' responses", 
             col = "black")
p2 + as.layer(p1) + as.layer(p3)

```


### Predicted Median RTs

Again, the data is shown in grey (and error bars show the standard errors of the median) and the predicted RTs in black. We first show the predictions for the speed condition, separated by response.

```{r, fig.height=6, fig.width=7, message=FALSE}

# get predicted quantiles (uses predicted response proportions)
lba_separate_pred_dark <- lba_pars_separate_l %>% do(as.data.frame(t(
  qLBA(quantiles*.$resp_prop, response=1, A=list(.$a_1, .$a_2), sd_v=.$sv,
       mean_v=c(.$v, 1-.$v), t0=.$t0, b=max(.$a_1, .$a_2)+.$b, silent=TRUE)))) %>% 
  ungroup() %>% gather("quantiles", "dark", V1:V5)
lba_separate_pred_light <- lba_pars_separate_l %>% do(as.data.frame(t(
  qLBA(quantiles*(1-.$resp_prop), response=2, A=list(.$a_1, .$a_2), sd_v=.$sv,
       mean_v=c(.$v, 1-.$v), t0=.$t0, b=max(.$a_1, .$a_2)+.$b, silent=TRUE)))) %>% 
  ungroup() %>% gather("quantiles", "light", V1:V5)

#separate_pred_light %>% filter(is.na(light))
lba_separate_pred <- inner_join(lba_separate_pred_dark, lba_separate_pred_light)
lba_separate_pred$quantiles <- factor(lba_separate_pred$quantiles, 
                                  levels = c("V5", "V4", "V3", "V2", "V1"), 
                                  labels = c("90%", "70%", "50%", "30%", "10%"))
lba_separate_pred <- lba_separate_pred %>% gather("response", "rt", dark, light)

p1 <- xyplot(rt ~ strength_bin|id+response, agg2_rr98_response, type = "b", 
             auto.key = list(lines = TRUE), ylab = "RT (in seconds)", 
             subset = instruction == "speed" & quantile == "50%", 
             layout = c(3,2), col = "grey")
p1e <- segplot(strength_bin ~ upper+lower|id+response, agg2_rr98_response, 
               auto.key = list(lines = TRUE), ylab = "Proportion of 'dark' responses", 
               col = "grey", horizontal = FALSE, segments.fun = panel.arrows,  
               draw.bands = FALSE, angle = 90, length = 0.05, ends = "both", 
               subset = instruction == "speed" & quantile == "50%", layout = c(3,2))
p2 <- xyplot(rt ~ strength_bin|id + response, lba_separate_pred, type = "b", 
             auto.key = list(lines = TRUE), ylab = "RT (in seconds)", 
             subset = instruction == "speed" & quantiles == "50%", 
             scales = list(y = list(limits = c(0.25, 0.5))))
p2 + as.layer(p1) + as.layer(p1e)

```



Next shows the same plot for the accuracy condition. 

```{r, fig.height=6, fig.width=7}

p1 <- xyplot(rt ~ strength_bin|id+response, agg2_rr98_response, type = "b", 
             auto.key = list(lines = TRUE), ylab = "RT (in seconds)", 
             subset = instruction == "accuracy" & quantile == "50%", 
             layout = c(3,2), col = "grey")
p1e <- segplot(strength_bin ~ upper+lower|id+response, agg2_rr98_response, 
               auto.key = list(lines = TRUE), ylab = "Proportion of 'dark' responses", 
               col = "grey", horizontal = FALSE, segments.fun = panel.arrows,  
               draw.bands = FALSE, angle = 90, length = 0.05, ends = "both", 
               subset = instruction == "accuracy" & quantile == "50%", layout = c(3,2))
p2 <- xyplot(rt ~ strength_bin|id + response, lba_separate_pred, type = "b", 
             auto.key = list(lines = TRUE), ylab = "RT (in seconds)", 
             subset = instruction == "accuracy" & quantiles == "50%", 
             scales = list(y = list(limits = c(0.2, 1.5))))
p2 + as.layer(p1) + as.layer(p1e)

```

Overall the LBA is able to describe the central tendency relatively well. The only considerable misfit is evident at the extreme bins with few responses (i.e., large error bars).

### All quantiles

Next, we investigate the full RT distribution by comparing observed and predicted quantiles. The observed quantiles are again displayed in grey and the predictions in black. The first plot shows the speed condition separated by response. 


```{r, fig.height=7, fig.width=7}

p1 <- xyplot(rt ~ strength_bin|id+response, agg2_rr98_response, group = quantile, type = "b", 
             auto.key = list(lines = TRUE), ylab = "RT (in seconds)", 
             subset = instruction == "speed", layout = c(3,2), col = "grey")
p1e <- segplot(strength_bin ~ upper+lower|id+response, agg2_rr98_response, 
               auto.key = list(lines = TRUE), ylab = "Proportion of 'dark' responses", 
               col = "grey", horizontal = FALSE, segments.fun = panel.arrows,  
               draw.bands = FALSE, angle = 90, length = 0.05, ends = "both", 
               subset = instruction == "speed")
p2 <- xyplot(rt ~ strength_bin|id + response, lba_separate_pred, group = quantiles, type = "b", 
             auto.key = list(lines = TRUE), ylab = "RT (in seconds)", 
             subset = instruction == "speed", scales = list(y = list(limits = c(0.2, 0.6))))
p2 + as.layer(p1) + as.layer(p1e)

```

The next plot shows the accuracy condition separated by response. 


```{r, fig.height=7, fig.width=7}

p1 <- xyplot(rt ~ strength_bin|id+response, agg2_rr98_response, group = quantile, type = "b", 
             auto.key = list(lines = TRUE), ylab = "RT (in seconds)", 
             subset = instruction == "accuracy", layout = c(3,2), col = "grey")
p1e <- segplot(strength_bin ~ upper+lower|id+response, agg2_rr98_response, 
               auto.key = list(lines = TRUE), ylab = "Proportion of 'dark' responses", 
               col = "grey", horizontal = FALSE, segments.fun = panel.arrows,  
               draw.bands = FALSE, angle = 90, length = 0.05, ends = "both", 
               subset = instruction == "accuracy")
p2 <- xyplot(rt ~ strength_bin|id + response, lba_separate_pred, group = quantiles, type = "b", 
             auto.key = list(lines = TRUE), ylab = "RT (in seconds)", 
             subset = instruction == "accuracy", scales = list(y = list(limits = c(0.1, 3.3))))
p2 + as.layer(p1) + as.layer(p1e)

```

These two plots show not only the central tendency, but the other quantiles are quite well described.


# Comparing Model Fit

Finally, we graphically compare the fit of the two models. Here we can see that while the LBA seems to provide a slightly better fit (as indicated by the lower negative log-likelihoods), the diffusion model seems to somewhat better recover some of the trends in the data.

## Predicted Response Rates

We again first consider the predicted response rates (in black) for the two models. 

```{r, fig.height=6.5, fig.width=7, message=FALSE}

key <- simpleKey(text = c("data", "LBA", "Diffusion"), lines = TRUE)
key$lines$col <- c("grey", "black", "black")
key$lines$lty <- c(1, 1, 2)
key$points$col <- c("grey", "black", "black")
key$points$pch <- c(1, 0, 4)

p1 <- xyplot(mean ~ strength_bin|id + instruction, agg_rr98_bin, type = "b", auto.key = 
               list(lines = TRUE), ylab = "Proportion of 'dark' responses", col = "grey")
p2 <- segplot(strength_bin ~ upper+lower|id + instruction, agg_rr98_bin, 
              auto.key = list(lines = TRUE), ylab = "Proportion of 'dark' responses", 
              col = "grey", horizontal = FALSE, segments.fun = panel.arrows,  
              draw.bands = FALSE, angle = 90, length = 0.05, ends = "both")
p3 <- xyplot(resp_prop ~ strength_bin|id + instruction, lba_pars_separate_l, type = "b", 
             auto.key = list(lines = TRUE), ylab = "Proportion of 'dark' responses", 
             col = "black", pch = 0)
p4 <- xyplot(resp_prop ~ strength_bin|id + instruction, pars_separate_l, type = "b", 
             auto.key = list(lines = TRUE), ylab = "Proportion of 'dark' responses", 
             col = "black", lty = 2, key = key, pch=4)
p4 + as.layer(p2) + as.layer(p1) + as.layer(p3)

```

When comparing the fit of the two models like this, it shows that both models make very similar predictions for the predicted response rates. It is difficult to see huge differences between the models.

## Predicted Median RTs

Next, we also compare the two accounts for the median. As before, data is shown in grey and we begin with a plot for the speed condition, separated by response.

```{r, fig.height=6.5, fig.width=7, message=FALSE}

p1 <- xyplot(rt ~ strength_bin|id+response, agg2_rr98_response, type = "b", 
             auto.key = list(lines = TRUE), ylab = "RT (in seconds)", 
             subset = instruction == "speed" & quantile == "50%", 
             layout = c(3,2), col = "grey")
p1e <- segplot(strength_bin ~ upper+lower|id+response, agg2_rr98_response, 
               auto.key = list(lines = TRUE), ylab = "Proportion of 'dark' responses", 
               col = "grey", horizontal = FALSE, segments.fun = panel.arrows,  
               draw.bands = FALSE, angle = 90, length = 0.05, ends = "both", 
               subset = instruction == "speed" & quantile == "50%", layout = c(3,2))
p2 <- xyplot(rt ~ strength_bin|id + response, lba_separate_pred, type = "b", 
             auto.key = list(lines = TRUE), ylab = "RT (in seconds)", 
             subset = instruction == "speed" & quantiles == "50%", 
             scales = list(y = list(limits = c(0.25, 0.5))), pch = 0)
p3 <- xyplot(rt ~ strength_bin|id + response, separate_pred, type = "b", 
             auto.key = list(lines = TRUE), ylab = "RT (in seconds)", 
             subset = instruction == "speed" & quantiles == "50%", 
             scales = list(y = list(limits = c(0.25, 0.5))),
             col = "black", lty = 2, key = key, pch=4)

p3 + as.layer(p2) + as.layer(p1) + as.layer(p1e)

```

This plot suggests that the diffusion model is better able to predict changes in the median RTs in the speed condition across strength bins than the LBA. While this leads to obvious misfit in certain cases (`dark` responses for `nh`) it appears to provide a generally better recovery of the patterns in the data.

Next shows the same plot for the accuracy condition. 

```{r, fig.height=6.5, fig.width=7, message=FALSE}

p1 <- xyplot(rt ~ strength_bin|id+response, agg2_rr98_response, type = "b", 
             auto.key = list(lines = TRUE), ylab = "RT (in seconds)", 
             subset = instruction == "accuracy" & quantile == "50%", 
             layout = c(3,2), col = "grey")
p1e <- segplot(strength_bin ~ upper+lower|id+response, agg2_rr98_response, 
               auto.key = list(lines = TRUE), ylab = "Proportion of 'dark' responses", 
               col = "grey", horizontal = FALSE, segments.fun = panel.arrows,  
               draw.bands = FALSE, angle = 90, length = 0.05, ends = "both", 
               subset = instruction == "accuracy" & quantile == "50%", layout = c(3,2))
p2 <- xyplot(rt ~ strength_bin|id + response, lba_separate_pred, type = "b", 
             auto.key = list(lines = TRUE), ylab = "RT (in seconds)", 
             subset = instruction == "accuracy" & quantiles == "50%", 
             pch = 0)
p3 <- xyplot(rt ~ strength_bin|id + response, separate_pred, type = "b", 
             auto.key = list(lines = TRUE), ylab = "RT (in seconds)", 
             subset = instruction == "accuracy" & quantiles == "50%", 
             scales = list(y = list(limits = c(0.2, 1.5))),
             col = "black", lty = 2, key = key, pch=4)

p3 + as.layer(p2) + as.layer(p1) + as.layer(p1e)

```

Here we see the same observation as for the speed condition. Considerable changes in median RT across strength bins are better captured by the diffusion model than the LBA. This leads to the situation that in most cases the diffusion model can make more accurate prediction for conditions with low numbers of trials.


# References

- Ratcliff, R., & Rouder, J. N. (1998). Modeling Response Times for Two-Choice Decisions. _Psychological Science_, 9(5), 347--356. https://doi.org/10.1111/1467-9280.00067

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rr98-data.R
\docType{data}
\name{rr98}
\alias{rr98}
\title{Ratcliff and Rouder (1998, Exp. 1) Luminance Discrimination Data}
\format{
A \code{data.frame} with 24,358 obs. and 12 variables:
\describe{
 \item{id}{participant id, factor with three levels}
 \item{session}{session number, integer}
 \item{block}{block number, integer}
 \item{trial}{trial number within a block, integer}
 \item{instruction}{factor with two levels: \code{"accuracy"} for blocks with accuracy instructions; \code{"speed"} for blocks with speed instruction}
 \item{source}{factor with two levels: distribution strength was drawn from, \code{"dark"} and \code{"light"}}
 \item{strength}{proportion of white to black pixels were varied by 33 equally spaced proportions from zero (all 1,024 pixels were black) to 1 (all 1,024 pixels were white). with 0 darkest and 32 lightest. Integer.}
 \item{response}{factor with two levels: \code{"dark"} and \code{"light"}}
 \item{response_num}{numeric response variable such that \code{1="dark"} and \code{2="light"}}
 \item{correct}{boolean indicating whether or not \code{source==response}. (Does not seem to be used in the original analysis.)}
 \item{rt}{response time in seconds}
 \item{outlier}{boolean indicating whether or not the response was considered an outlier by Ratcliff and Rouder (1998), i.e., RTs outside of (200ms, 2500ms)}
}
}
\source{
Ratcliff, R., & Rouder, J. N. (1998). Modeling Response Times for Two-Choice Decisions. \emph{Psychological Science}, 9(5), 347-356. http://doi.org/10.1111/1467-9280.00067
}
\usage{
rr98
}
\description{
Responses and response times from an experiment in which three participants were asked to decide whether the overall brightness of pixel arrays displayed on a computer monitor was "high" or "low". In addition, instruction manipulated speed and accuracy between blocks.
}
\details{
The Experiment is described in the following by Ratcliff and Rouder (1998, pp. 349):

In Experiment 1, subjects were asked to decide whether the overall
brightness of pixel arrays displayed on a computer monitor was "high"
or "low". The brightness of a display was controlled by the
proportion of the pixels that were white. For each trial, the proportion
of white pixels was chosen from one of two distributions, a high
distribution [i.e., light] or a low [i.e., dark] distribution, each with fixed mean and standard
deviation. Feedback was given after each trial to tell the subject whether his or her decision had correctly indicated the distribution
from which the stimulus had been chosen. Other than this feedback, a
subject had no information about the distributions. Because the distributions overlapped substantially, a subject could not be highly accurate. A display with 50% white pixels, for example, might have come
from the high distribution on one trial and the low distribution on
another.

\subsection{Stimuli}{ 
The stimulus display for Experiment 1 was a square that was 64
pixels on each side and subtended 3.8 degree of visual angle on a PC-VGA
monitor. [...] In each square, 3,072 randomly
chosen pixels were neutral gray, like the background, and the remaining 
1,024 pixels were either black or white; the proportion of white to
black pixels provided the brightness manipulation. There were 33
equally spaced proportions from zero (all 1,024 pixels were black) to
1 (all 1,024 pixels were white). The two distributions from which the
bright and dark stimuli were chosen were centered at .375 (low brightness) 
and .625 (high brightness), and they each had a standard deviation of .1875.
}

\subsection{Procedure}{
A subject's task was to decide, on each trial, from which distribution, 
high or low brightness in Experiment 1, the observed
stimulus (stimuli) had been sampled. Subjects made their decision by
pressing one of two response keys. On each trial, a 500-ms foreperiod, 
during which the display consisted solely of neutral gray, was followed 
by presentation of the stimulus; presentation was terminated by
the subject's response. In Experiment 1, speed-versus-accuracy
instructions were manipulated. For some blocks of trials, subjects
were instructed to respond as quickly as possible, and a "too slow"
message followed every response longer than 550 ms. For other
blocks of trials, subjects were instructed to be as accurate as possible,
and a "bad error" message followed incorrect responses to stimuli
from the extreme ends of the distributions. Experiment 1 had ten 35-min 
sessions, and Experiments 2 and 3 had four sessions. In Experiment 1, 
subjects switched from emphasis on speed to emphasis on
accuracy every 204 trials. Each session consisted of eight blocks of
102 trials per block, for a total of 8,160 trials per subject. Each 
session consisted of eight blocks of 102 trials, for a total of 3,264 trials
per subject in each experiment. For all trials in each experiment, subjects 
were instructed to maintain a high level of accuracy while
responding quickly, and an "error" message indicated incorrect
responses. Responses were followed by a 300-ms blank interval, and
the error message was displayed for 300 ms after the blank interval.
}
}
\note{
The data is already prepared following Ratcliff and Rouder (1998) by removing the following trials:
\itemize{
\item{the first session for each participant}
\item{the first 20 trials of each session}
\item{the first trial of each block (each change in speed accuracy starts a new block)}
}
To fully replicate the data used by Ratcliff and Rouder (1998) one only needs to remove the trials that are \code{TRUE} in column \code{outlier} (i.e., RTs outside of (200ms, 2500ms)). The full raw data is also available as part of this package, see:\cr
\code{system.file("extdata", "rr98-data", package = "rtdists")} and \code{system.file("extdata", "rr98-data.codes", package = "rtdists")}
}
\examples{

data(rr98)
rr98 <- rr98[!rr98$outlier,]  #remove outliers
head(rr98)
#   id session block trial instruction source strength response response_num correct    rt outlier
# 1 jf       2     1    21    accuracy   dark        8     dark            1    TRUE 0.801   FALSE
# 2 jf       2     1    22    accuracy   dark        7     dark            1    TRUE 0.680   FALSE
# 3 jf       2     1    23    accuracy  light       19    light            2    TRUE 0.694   FALSE
# 4 jf       2     1    24    accuracy   dark       21    light            2   FALSE 0.582   FALSE
# 5 jf       2     1    25    accuracy  light       19     dark            1   FALSE 0.925   FALSE
# 6 jf       2     1    26    accuracy   dark       10     dark            1    TRUE 0.605   FALSE

## See vignette for more examples.
}
\keyword{dataset}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rtdists-package.R
\docType{package}
\name{rtdists-package}
\alias{rtdists}
\alias{rtdists-package}
\title{The rtdists Package}
\description{
Response Time Distributions.
}
\details{
\tabular{ll}{
Package: \tab rtdists\cr
Type: \tab Package\cr
Version: \tab 0.8-3\cr
Date: \tab 2018-06-23\cr
Depends: \tab R (>= 3.0.0)\cr
License: \tab GPL (>=3)\cr
URL: \tab https://github.com/rtdists/rtdists/\cr
}

Provides response time distributions (density/PDF, distribution function/CDF, quantile
function, and random generation): (a) Ratcliff diffusion model (Ratcliff & McKoon, 2008,
<doi:10.1162/neco.2008.12-06-420>) based on C code by Andreas and Jochen Voss and (b) linear
ballistic accumulator (LBA; Brown & Heathcote, 2008, <doi:10.1016/j.cogpsych.2007.12.002>)
with different distributions underlying the drift rate.
}
\seealso{
Useful links:
\itemize{
  \item \url{https://github.com/rtdists/rtdists/}
  \item Report bugs at \url{https://github.com/rtdists/rtdists/issues}
}

}
\author{
Henrik Singmann, Scott Brown, Matthew Gretton, Andrew Heathcote, with contributions from Andreas Voss, Jochen Voss, Andrew Terry
}
\keyword{package}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/lba.r
\name{LBA}
\alias{LBA}
\alias{dLBA}
\alias{pLBA}
\alias{qLBA}
\alias{rLBA}
\title{The Linear Ballistic Accumulator (LBA)}
\usage{
dLBA(
  rt,
  response,
  A,
  b,
  t0,
  ...,
  st0 = 0,
  distribution = c("norm", "gamma", "frechet", "lnorm"),
  args.dist = list(),
  silent = FALSE
)

pLBA(
  rt,
  response,
  A,
  b,
  t0,
  ...,
  st0 = 0,
  distribution = c("norm", "gamma", "frechet", "lnorm"),
  args.dist = list(),
  silent = FALSE
)

qLBA(
  p,
  response,
  A,
  b,
  t0,
  ...,
  st0 = 0,
  distribution = c("norm", "gamma", "frechet", "lnorm"),
  args.dist = list(),
  silent = FALSE,
  interval = c(0, 10),
  scale_p = FALSE,
  scale_max = Inf
)

rLBA(
  n,
  A,
  b,
  t0,
  ...,
  st0 = 0,
  distribution = c("norm", "gamma", "frechet", "lnorm"),
  args.dist = list(),
  silent = FALSE
)
}
\arguments{
\item{rt}{vector of RTs. Or for convenience also a \code{data.frame} with
columns \code{rt} and \code{response} (such as returned from \code{rLBA} or
\code{\link{rdiffusion}}). See examples.}

\item{response}{integer vector of winning accumulators/responses
corresponding to the vector of RTs/p (i.e., used for specifying the
response for a given RT/probability). Will be recycled if necessary. Cannot
contain values larger than the number of accumulators. First
response/accumulator must receive value 1, second 2, and so forth. For
conmvenience, \code{response} is converted via \code{as.numeric} thereby
allowing factors to be passed as well (such as returned from
\code{\link{rdiffusion}}). Ignored if \code{rt} or \code{p} is a
\code{data.frame}.}

\item{A}{start point interval or evidence in accumulator before beginning of
decision process. Start point varies from trial to trial in the interval
[0, \code{A}] (uniform distribution). Average amount of evidence before
evidence accumulation across trials is \code{A}/2.}

\item{b}{response threshold. (\code{b} - \code{A}/2) is a measure of
"response caution".}

\item{t0}{non-decision time or response time constant (in seconds). Lower
bound for the duration of all non-decisional processes (encoding and
response execution).}

\item{...}{two \emph{named} drift rate parameters depending on
\code{distribution} (e.g., \code{mean_v} and \code{sd_v} for
\code{distribution=="norm"}). The parameters can either be given as a
numeric vector or a list. If a numeric vector is passed each element of the
vector corresponds to one accumulator. If a list is passed each list
element corresponds to one accumulator allowing again trialwise driftrates.
The shorter parameter will be recycled as necessary (and also the elements
of the list to match the length of \code{rt}). See details.}

\item{st0}{variability of non-decision time, such that \code{t0} is uniformly
distributed between \code{t0} and \code{t0} + \code{st0}. Default is 0. Can
be trialwise, and will be recycled to length of \code{rt}.}

\item{distribution}{character specifying the distribution of the drift rate.
Possible values are \code{c("norm", "gamma", "frechet", "lnorm")}, default
is \code{"norm"}.}

\item{args.dist}{list of optional further arguments to the distribution
functions (i.e., \code{posdrift} or \code{robust} for
\code{distribution=="norm"}, see \code{\link{single-LBA}}).}

\item{silent}{logical. Should the number of accumulators used be suppressed?
Default is \code{FALSE} which prints the number of accumulators.}

\item{p}{vector of probabilities. Or for convenience also a \code{data.frame}
with columns \code{p} and \code{response}. See examples.}

\item{interval}{a vector containing the end-points of the interval to be
searched for the desired quantiles (i.e., RTs) in \code{qLBA}. Default is
\code{c(0, 10)}.}

\item{scale_p}{logical. Should entered probabilities automatically be scaled
by maximally predicted probability? Default is \code{FALSE}. Convenience
argument for obtaining predicted quantiles. Can be slow as the maximally
predicted probability is calculated individually for each \code{p}.}

\item{scale_max}{numerical scalar. Value at which maximally predicted RT
should be calculated if \code{scale_p} is \code{TRUE}.}

\item{n}{desired number of observations (scalar integer).}
}
\value{
\code{dLBA} returns the density (PDF), \code{pLBA} returns the
  distribution function (CDF), \code{qLBA} returns the quantile/RT,
  \code{rLBA} return random response times and responses (in a
  \code{data.frame}).
  
  The length of the result is determined by \code{n} for \code{rLBA}, equal
  to the length of \code{rt} for \code{dLBA} and \code{pLBA}, and equal to
  the length of \code{p} for \code{qLBA}.
  
  The distribution parameters (as well as \code{response}) are recycled to
  the length of the result. In other words, the functions are completely
  vectorized for all parameters and even the response.
}
\description{
Density, distribution function, quantile function, and random generation for
the LBA model with the following parameters: \code{A} (upper value of
starting point), \code{b} (response threshold), \code{t0} (non-decision
time), and driftrate (\code{v}). All functions are available with different
distributions underlying the drift rate: Normal (\code{norm}), Gamma
(\code{gamma}), Frechet (\code{frechet}), and log normal (\code{lnorm}). The
functions return their values conditional on the accumulator given in the
response argument winning.
}
\details{
For convenience, all functions (with the exception of \code{rdiffusion})
allow that the first argument is a \code{data.frame} containing the
information of the first and second argument in two columns (i.e.,
\code{rt}/\code{p} and \code{response}). Other columns will be ignored. This
allows, for example, to pass the \code{data.frame} generated by \code{rLBA}
directly to \code{pLBA}. See examples.

\subsection{Parameters}{ The following arguments are allowed as \code{...}
drift rate parameters: \itemize{ \item \code{mean_v,sd_v} mean and standard
deviation of normal distribution for drift rate (\code{norm}). See
\code{\link{Normal}} \item \code{shape_v,rate_v,scale_v} shape, rate, and
scale of gamma (\code{gamma}) and scale and shape of Frechet (\code{frechet})
distributions for drift rate. See \code{\link{GammaDist}} or
\code{\link[evd]{frechet}}. For Gamma, scale = 1/shape and shape = 1/scale. 
\item \code{meanlog_v,sdlog_v} mean and standard deviation of lognormal
distribution on the log scale for drift rate (\code{lnorm}). See
\code{\link{Lognormal}}. }

As described above, the accumulator parameters can either be given as a
numeric vector or a list. If a numeric vector is passed each element of the
vector corresponds to one accumulator. If a list is passed each list element
corresponds to one accumulator allowing trialwise driftrates. The shorter
parameter will be recycled as necessary (and also the elements of the list to
match the length of \code{rt}).

The other LBA parameters (i.e., \code{A}, \code{b}, and \code{t0}, with the
exception of \code{st0}) can either be a single numeric vector (which will be
recycled to reach \code{length(rt)} or \code{length(n)} for trialwise
parameters) \emph{or} a \code{list} of such vectors in which each list
element corresponds to the parameters for this accumulator (i.e., the list
needs to be of the same length as there are accumulators). Each list will
also be recycled to reach \code{length(rt)} for trialwise parameters per
accumulator.

To make the difference between both paragraphs clear: Whereas for the
accumulators both a single vector or a list corresponds to different
accumulators, only the latter is true for the other parameters. For those
(i.e., \code{A}, \code{b}, and \code{t0}) a single vector always corresponds
to trialwise values and a list must be used for accumulator wise values.

\code{st0} can only vary trialwise (via a vector). And it should be noted
that \code{st0} not equal to zero will considerably slow done everything. }

\subsection{Quantile Function}{ Due to the bivariate nature of the LBA,
single accumulators only return defective CDFs that do not reach 1. Only the
sum of all accumulators reaches 1. Therefore, \code{qLBA} can only return
quantiles/RTs for any accumulator up to the maximal probability of that
accumulator's CDF. This can be obtained by evaluating the CDF at \code{Inf}.

As a conveniece for the user, if \code{scale_p = TRUE} in the call to
\code{qLBA} the desired probabilities are automatically scaled by the maximal
probability for the corresponding response. Note that this can be slow as the
maximal probability is calculated separately for each desired probability.
See examples.

Also note that quantiles (i.e., predicted RTs) are obtained by numerically
minimizing the absolute difference between desired probability and the value
returned from \code{pLBA} using \code{\link{optimize}}. If the difference
between the desired probability and probability corresponding to the returned
quantile is above a certain threshold (currently 0.0001) no quantile is
returned but \code{NA}. This can be either because the desired quantile is
above the maximal probability for this accumulator or because the limits for
the numerical integration are too small (default is \code{c(0, 10)}). }

\subsection{RNG}{ For random number generation at least one of the
distribution parameters (i.e., \code{mean_v}, \code{sd_v}, \code{shape_v},
\code{scale_v}, \code{rate_v}, \code{meanlog_v}, and \code{sdlog_v}) should
be of length > 1 to receive RTs from multiple responses. Shorter vectors are
recycled as necessary.\cr Note that for random number generation from a
normal distribution for the driftrate the number of returned samples may be
less than the number of requested samples if \code{posdrifts==FALSE}. }
}
\note{
These are the top-level functions intended for end-users. To obtain the
  density and cumulative density the race functions are called for each
  response time with the corresponding winning accumulator as first
  accumulator (see \code{\link{LBA-race}}).
}
\examples{

## generate random LBA data:
rt1 <- rLBA(500, A=0.5, b=1, t0 = 0.5, mean_v=c(2.4, 1.6), sd_v=c(1,1.2))
head(rt1)
prop.table(table(rt1$response))

# original parameters have 'high' log-likelihood:
sum(log(dLBA(rt1$rt, rt1$response, A=0.5, b=1, t0 = 0.5, mean_v=c(2.4, 1.6), sd_v=c(1,1.2))))

# data can also be passed as data.frame (same is true for pLBA):
sum(log(dLBA(rt1, A=0.5, b=1, t0 = 0.5, mean_v=c(2.4, 1.6), sd_v=c(1,1.2))))

objective_fun <- function(par, rt, response, distribution = "norm") {
  # simple parameters
  spar <- par[!grepl("[12]$", names(par))]  
  
  # distribution parameters:
  dist_par_names <- unique(sub("[12]$", "", grep("[12]$" ,names(par), value = TRUE)))
  dist_par <- vector("list", length = length(dist_par_names))
  names(dist_par) <- dist_par_names
  for (i in dist_par_names) dist_par[[i]] <- as.list(unname(par[grep(i, names(par))]))
  dist_par$sd_v <- c(1, dist_par$sd_v) # fix first sd to 1

  # get summed log-likelihood:
  d <- do.call(dLBA, args = c(rt=list(rt), response=list(response), spar, dist_par, 
                               distribution=distribution, silent=TRUE))
  if (any(d < 0e-10)) return(1e6) 
  else return(-sum(log(d)))
}

# gives same value as manual calculation above:
objective_fun(c(A=0.5, b=1, t0=0.5, mean_v1=2.4, mean_v2=1.6, sd_v2=1.2), 
              rt=rt1$rt, response=rt1$response)

\dontrun{
# can we recover the parameters? 
# should be run several times with different random values of init_par
init_par <- runif(6)
init_par[2] <- sum(init_par[1:2]) # ensures b is larger than A
init_par[3] <- runif(1, 0, min(rt1$rt)) #ensures t0 is mot too large
names(init_par) <- c("A", "b", "t0", "mean_v1", "mean_v2",  "sd_v2")
nlminb(objective_fun, start = init_par, rt=rt1$rt, response=rt1$response, lower = 0)
}


# plot cdf (2 accumulators):
curve(pLBA(x, response = 1, A=0.5, b=1, t0 = 0.5, mean_v=c(2.4, 1.6), sd_v=c(1,1.2)), 
     xlim = c(0, 2), ylim = c(0,1), 
     ylab = "cumulative probability", xlab = "response time",
     main = "Defective CDFs of LBA")
curve(pLBA(x, response = 2, A=0.5, b=1, t0 = 0.5, mean_v=c(2.4, 1.6), sd_v=c(1,1.2)), 
     add=TRUE, lty = 2)
legend("topleft", legend=c("1", "2"), title="Response", lty=1:2)


# plot cdf (3 accumulators):
curve(pLBA(x, response = 1, A=0.5, b=1, t0 = 0.5, mean_v=c(2.4, 1.6, 1.0), sd_v=c(1,1.2, 2.0)), 
     xlim = c(0, 2), ylim = c(0,1), 
     ylab = "cumulative probability", xlab = "response time",
     main = "Defective CDFs of LBA")
curve(pLBA(x, response = 2, A=0.5, b=1, t0 = 0.5,  mean_v=c(2.4, 1.6, 1.0), sd_v=c(1,1.2, 2.0)), 
     add=TRUE, lty = 2)
curve(pLBA(x, response = 3, A=0.5, b=1, t0 = 0.5,  mean_v=c(2.4, 1.6, 1.0), sd_v=c(1,1.2, 2.0)), 
     add=TRUE, lty = 3)
legend("topleft", legend=c("1", "2", "3"), title="Response", lty=1:2)


## qLBA can only return values up to maximal predicted probability:
(max_p <- pLBA(Inf, response = 1, A=0.5, b=1, t0 = 0.5, mean_v=c(2.4, 1.6), sd_v=c(1,1.2)))
# [1] 0.6604696

qLBA(0.66, response = 1, A=0.5, b=1, t0 = 0.5, mean_v=c(2.4, 1.6), sd_v=c(1,1.2))
# 2.559532

qLBA(0.67, response = 1, A=0.5, b=1, t0 = 0.5, mean_v=c(2.4, 1.6), sd_v=c(1,1.2))
# NA

# to get predicted quantiles, scale required quantiles by maximally predicted response rate:
qs <- c(.1, .3, .5, .7, .9)
qLBA(qs*max_p, response = 1, A=0.5, b=1, t0 = 0.5, mean_v=c(2.4, 1.6), sd_v=c(1,1.2))

# or set scale_p to TRUE which scales automatically by maximum p
# (but can be slow as it calculates max_p for each probability separately) 
qLBA(qs, response = 1, A=0.5, b=1, t0 = 0.5, mean_v=c(2.4, 1.6), sd_v=c(1,1.2), scale_p=TRUE)

# qLBA also accepts a data.frame as first argument:
t <- data.frame(p = rep(c(0.05, 0.1, 0.66), 2), response = rep(1:2, each = 3))
#      p response
# 1 0.05        1
# 2 0.10        1
# 3 0.66        1
# 4 0.05        2
# 5 0.10        2
# 6 0.66        2
qLBA(t, A=0.5, b=1, t0 = 0.5, mean_v=c(2.4, 1.6), sd_v=c(1,1.2))


## LBA and diffusion can be used interchangeably:
rt1 <- rLBA(500, A=0.5, b=1, t0 = 0.5, mean_v=c(2.4, 1.6), sd_v=c(1,1.2))
rt2 <- rdiffusion(500, a=1, v=2, t0=0.5)

# data can also be passed as data.frame (same is true for pLBA):
sum(log(dLBA(rt1, A=0.5, b=1, t0 = 0.5, mean_v=c(2.4, 1.6), sd_v=c(1,1.2))))
sum(log(dLBA(rt2, A=0.5, b=1, t0 = 0.5, mean_v=c(2.4, 1.6), sd_v=c(1,1.2))))

sum(log(ddiffusion(rt1, a=1, v=2, t0=0.5)))
sum(log(ddiffusion(rt2, a=1, v=2, t0=0.5)))

### trial wise parameters work as expected (only since package version 0.9):
x1 <- dLBA(rt=c(1,1), response=c(1,2), A=1,b=list(c(1,3),c(2,4)),
           t0=0.1, mean_v=c(3,3), sd_v=c(1,1),distribution="norm")
x2a <- dLBA(rt=c(1), response=c(1), A=1,b=list(c(1),c(2)),
            t0=0.1,mean_v=c(3,3),sd_v=c(1,1),distribution="norm")
x2b <- dLBA(rt=c(1), response=c(2), A=1,b=list(c(3),c(4)),
            t0=0.1,mean_v=c(3,3),sd_v=c(1,1),distribution="norm")
all(x1 == c(x2a, x2b)) ## should be TRUE
}
\references{
Brown, S. D., & Heathcote, A. (2008). The simplest complete model of choice
response time: Linear ballistic accumulation. \emph{Cognitive Psychology},
57(3), 153-178. doi:10.1016/j.cogpsych.2007.12.002

Donkin, C., Averell, L., Brown, S., & Heathcote, A. (2009). Getting more from
accuracy and response time data: Methods for fitting the linear ballistic
accumulator. \emph{Behavior Research Methods}, 41(4), 1095-1110.
doi:10.3758/BRM.41.4.1095

Heathcote, A., & Love, J. (2012). Linear deterministic accumulator models of
simple choice. \emph{Frontiers in Psychology}, 3, 292.
doi:10.3389/fpsyg.2012.00292
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/single-lba.r
\name{single-LBA}
\alias{single-LBA}
\alias{dlba_norm}
\alias{plba_norm}
\alias{rlba_norm}
\alias{dlba_gamma}
\alias{plba_gamma}
\alias{rlba_gamma}
\alias{dlba_frechet}
\alias{plba_frechet}
\alias{rlba_frechet}
\alias{dlba_lnorm}
\alias{plba_lnorm}
\alias{rlba_lnorm}
\title{Single accumulator of linear ballistic accumulator (LBA)}
\usage{
dlba_norm(rt, A, b, t0, mean_v, sd_v, posdrift = TRUE, robust = FALSE)

plba_norm(rt, A, b, t0, mean_v, sd_v, posdrift = TRUE, robust = FALSE)

rlba_norm(n, A, b, t0, mean_v, sd_v, st0 = 0, posdrift = TRUE)

dlba_gamma(rt, A, b, t0, shape_v, rate_v, scale_v)

plba_gamma(rt, A, b, t0, shape_v, rate_v, scale_v)

rlba_gamma(n, A, b, t0, shape_v, rate_v, scale_v, st0 = 0)

dlba_frechet(rt, A, b, t0, shape_v, scale_v)

plba_frechet(rt, A, b, t0, shape_v, scale_v)

rlba_frechet(n, A, b, t0, shape_v, scale_v, st0 = 0)

dlba_lnorm(rt, A, b, t0, meanlog_v, sdlog_v, robust = FALSE)

plba_lnorm(rt, A, b, t0, meanlog_v, sdlog_v, robust = FALSE)

rlba_lnorm(n, A, b, t0, meanlog_v, sdlog_v, st0 = 0)
}
\arguments{
\item{rt}{a vector of RTs.}

\item{A}{start point interval or evidence in accumulator before beginning of decision process. Start point varies from trial to trial in the interval [0, \code{A}] (uniform distribution). Average amount of evidence before evidence accumulation across trials is \code{A}/2.}

\item{b}{response threshold. (\code{b} - \code{A}/2) is a measure of "response caution".}

\item{t0}{non-decision time or response time constant (in seconds). Lower bound for the duration of all non-decisional processes (encoding and response execution).}

\item{mean_v, sd_v}{mean and standard deviation of normal distribution for drift rate (\code{norm}). See \code{\link{Normal}}}

\item{posdrift}{logical. Should driftrates be forced to be positive? Default is \code{TRUE}. (Uses truncated normal for random generation).}

\item{robust}{logical. Should robust normal distributions be used for \code{norm} and \code{lnorm}? Can be helpful in rare cases but is approximately three times slower than the non-robust versions. Default is \code{FALSE}.}

\item{n}{desired number of observations (scalar integer).}

\item{st0}{variability of non-decision time, such that \code{t0} is uniformly distributed between \code{t0} and \code{t0} + \code{st0}. Only available in random number generation functions \code{rlba_}.}

\item{shape_v, rate_v, scale_v}{shape, rate, and scale of gamma (\code{gamma}) and scale and shape of Frechet (\code{frechet}) distributions for drift rate. See \code{\link{GammaDist}} or \code{\link[evd]{frechet}}. For Gamma, scale = 1/shape and shape = 1/scale.}

\item{meanlog_v, sdlog_v}{mean and standard deviation of lognormal distribution on the log scale for drift rate (\code{lnorm}). See \code{\link{Lognormal}}.}
}
\value{
All functions starting with a \code{d} return the density (PDF), all functions starting with \code{p} return the distribution function (CDF), and all functions starting with \code{r} return random response times and responses (in a \code{matrix}).
}
\description{
Density, distribution function, and random generation for a single accumulator of the LBA model with the following parameters: \code{A} (upper value of starting point), \code{b} (response threshold), \code{t0} (non-decision time), and driftrate (\code{v}). All functions are available with different distributions underlying the drift rate: Normal (\code{norm}), Gamma (\code{gamma}), Frechet (\code{frechet}), and log normal (\code{lnorm}).
}
\details{
These functions are mainly for internal purposes. We do not recommend to use them. Use the high-level functions described in \code{/link{LBA}} instead.
}
\note{
Density (i.e., \code{dlba_}), distribution (i.e., \code{plba_}), and random derivative (i.e., \code{rlba_}) functions are vectorized for all parameters (i.e., in case parameters are not of the same length as \code{rt}, parameters are recycled). Furthermore, the random derivative functions also accept a matrix of length \code{n} in which each column corresponds to a accumulator specific value (see \code{\link{rLBA}} for a more user-friendly way).
}
\examples{

## random number generation using different distributions for v:
rlba_norm(10, A=0.5, b=1, t0 = 0.5, mean_v=c(1.2, 1), sd_v=c(0.2,0.3))
rlba_gamma(10, A=0.5, b=1, t0 = 0.5, shape_v=c(1.2, 1), scale_v=c(0.2,0.3))
rlba_frechet(10, A=0.5, b=1, t0 = 0.5, shape_v=c(1.2, 1), scale_v=c(0.2,0.3))
rlba_lnorm(10, A=0.5, b=1, t0 = 0.5, meanlog_v=c(1.2, 1), sdlog_v=c(0.2, 0.3))

# use somewhat plausible values for plotting:
A <- 0.2
b <- 0.5
t0 <- 0.3

# plot density:
curve(dlba_norm(x, A=A, b=b, t0=t0, mean_v = 1.0, sd_v = 0.5), ylim = c(0, 4),
      xlim=c(0,3), main="Density/PDF of LBA versions", ylab="density", xlab="response time")
curve(dlba_gamma(x, A=A, b=b, t0=t0, shape_v=1, scale_v=1), add=TRUE, lty = 2)
curve(dlba_frechet(x, A=A, b=b, t0=t0, shape_v=1,scale_v=1.0), add=TRUE, lty = 3)
curve(dlba_lnorm(x, A=A, b=b, t0=t0, meanlog_v = 0.5, sdlog_v = 0.5), add=TRUE, lty = 4)
legend("topright", legend=c("Normal", "Gamma", "Frechet", "Log-Normal"), 
      title = expression("Distribution of"~~italic(v)), lty = 1:4)


# plot cdf:
curve(plba_norm(x, A=A, b=b, t0=t0, mean_v=1.0, sd_v=1.0), 
      xlim = c(0, 3),ylim = c(0,1), 
      ylab = "cumulative probability", xlab = "response time",
      main = "Distribution/CDF of LBA versions")
curve(plba_gamma(x, A=A, b=b, t0=t0, shape_v=1,scale_v=1), add=TRUE, lty = 2)
curve(plba_frechet(x, A=A, b=b, t0=t0, shape=1, scale=1), add=TRUE, lty = 3)
curve(plba_lnorm(x, A=A, b=b, t0=t0, meanlog_v=0.5, sdlog_v = 0.5), add=TRUE, lty = 4)
legend("bottomright", legend=c("Normal", "Gamma", "Frechet", "Log-Normal"), 
       title = expression("Distribution of"~~italic(v)), lty = 1:4)
}
\references{
Brown, S. D., & Heathcote, A. (2008). The simplest complete model of choice response time: Linear ballistic accumulation. \emph{Cognitive Psychology}, 57(3), 153-178. doi:10.1016/j.cogpsych.2007.12.002

Donkin, C., Averell, L., Brown, S., & Heathcote, A. (2009). Getting more from accuracy and response time data: Methods for fitting the linear ballistic accumulator. \emph{Behavior Research Methods}, 41(4), 1095-1110. doi:10.3758/BRM.41.4.1095

Heathcote, A., & Love, J. (2012). Linear deterministic accumulator models of simple choice. \emph{Frontiers in Psychology}, 3, 292. doi:10.3389/fpsyg.2012.00292
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/lba_race.R
\name{LBA-race}
\alias{LBA-race}
\alias{n1PDF}
\alias{n1CDF}
\title{LBA race functions: Likelihood for first accumulator to win.}
\usage{
n1PDF(
  rt,
  A,
  b,
  t0,
  ...,
  st0 = 0,
  distribution = c("norm", "gamma", "frechet", "lnorm"),
  args.dist = list(),
  silent = FALSE
)

n1CDF(
  rt,
  A,
  b,
  t0,
  ...,
  st0 = 0,
  distribution = c("norm", "gamma", "frechet", "lnorm"),
  args.dist = list(),
  silent = FALSE
)
}
\arguments{
\item{rt}{a vector of RTs.}

\item{A, b, t0}{LBA parameters, see \code{\link{LBA}}. Can either be a single
numeric vector (which will be recycled to reach \code{length(rt)} for
trialwise parameters) \emph{or} a \code{list} of such vectors in which each
list element corresponds to the parameters for this accumulator (i.e., the
list needs to be of the same length as there are accumulators). Each list
will also be recycled to reach \code{length(rt)} for trialwise parameters
per accumulator.}

\item{...}{two \emph{named} drift rate parameters depending on
\code{distribution} (e.g., \code{mean_v} and \code{sd_v} for
\code{distribution=="norm"}). The parameters can either be given as a
numeric vector or a list. If a numeric vector is passed each element of the
vector corresponds to one accumulator. If a list is passed each list
element corresponds to one accumulator allowing again trialwise driftrates.
The shorter parameter will be recycled as necessary (and also the elements
of the list to match the length of \code{rt}). See examples.}

\item{st0}{parameter specifying the variability of \code{t0} (which varies
uniformly from \code{t0} to \code{t0} + \code{st0}). Can be trialwise, and
will be recycled to length of \code{rt}.}

\item{distribution}{character specifying the distribution of the drift rate.
Possible values are \code{c("norm", "gamma", "frechet", "lnorm")}, default
is \code{"norm"}.}

\item{args.dist}{list of optional further arguments to the distribution
functions (i.e., \code{posdrift} or \code{robust} for
\code{distribution=="norm"}).}

\item{silent}{logical. Should the number of accumulators used be suppressed?
Default is \code{FALSE} which prints the number of accumulators.}
}
\description{
n1PDF and n1CDF take RTs, the distribution functions of the \link{LBA}, and
corresponding parameter values and put them throughout the race equations and
return the likelihood for the first accumulator winning (hence n1) in a set
of accumulators.
}
\details{
For a set of \eqn{N} independent accumulators \eqn{i = 1...N}, the
  race likelihood for a given accumulator \eqn{i} is given by 
  \deqn{L(\mbox{unit }i \mbox{ wins}) = f_i(t) \times \prod_{j<>i} [ S_j(t)
  ]}{L(unit i wins) = f_i(t) * prod_j<>i [ S_j(t) ]} where \eqn{f(t)} is the
  PDF (\code{dlba_...}) and \eqn{S_j(t) = 1 - F_j(t)} is the survivor
  function, that is the complement of the CDF \eqn{F(t)} (\code{plba_...}) at
  time \eqn{t}.
  
  In other words, this is just the PDF/CDF for the winning accumulator at
  time \eqn{t} times the probability that no other accumulators have finished
  at time \eqn{t}.
}
\examples{


## check random generated values against race functions:

## 1. Without st0:
r_lba <- rLBA(1e4, A=0.5, b=1, t0 = 0.5, mean_v=c(1.2, 1), sd_v=0.2)
x <- seq(0.5, 4, length.out = 100) # for plotting
# PDF
y <- n1PDF(x, A=0.5, b=1, t0 = 0.5, mean_v=c(1.2, 1.0), sd_v=0.2) # PDF
hist(r_lba$rt[r_lba$response==1],probability = TRUE, breaks = "FD")
lines(x=x,y=y/mean(r_lba$response == 1))
# CDF
plot(ecdf(r_lba$rt[r_lba$response==1]))
y <- n1CDF(x, A=0.5, b=1, t0 = 0.5, st0 = 0, mean_v=c(1.2, 1.0), sd_v=0.2)
lines(x=x,y=y/mean(r_lba$response == 1), col = "red", lwd = 4.5, lty = 2)
# KS test
\dontrun{
normalised_n1CDF = function(rt,...) n1CDF(rt,...)/n1CDF(rt=Inf,...) 
ks.test(r_lba$rt[r_lba$response==1], normalised_n1CDF, A=0.5, b=1, t0 = 0.5, 
        mean_v=c(1.2, 1.0), sd_v=0.2)
}

\dontrun{
## Other examples (don't run to save time):
  
## 2. With st0 = 0.2:
r_lba <- rLBA(1e4, A=0.5, b=1, t0 = 0.5, st0 = 0.2, mean_v=c(1.2, 1), sd_v=0.2)
x <- seq(0.5, 4, length.out = 100) # for plotting
# PDF
y <- n1PDF(x, A=0.5, b=1, t0 = 0.5, st0 = 0.2, mean_v=c(1.2, 1.0), sd_v=0.2) # PDF
hist(r_lba$rt[r_lba$response==1],probability = TRUE, breaks = "FD")
lines(x=x,y=y/mean(r_lba$response == 1))
# CDF
plot(ecdf(r_lba$rt[r_lba$response==1]))
y <- n1CDF(x, A=0.5, b=1, t0 = 0.5, st0 = 0.2, mean_v=c(1.2, 1.0), sd_v=0.2)
lines(x=x,y=y/mean(r_lba$response == 1), col = "red", lwd = 4.5, lty = 2)
# KS test
normalised_n1CDF = function(rt,...) n1CDF(rt,...)/n1CDF(rt=Inf,...) 
ks.test(r_lba$rt[r_lba$response==1], normalised_n1CDF, A=0.5, b=1, t0 = 0.5, 
        st0 = 0.2, mean_v=c(1.2, 1.0), sd_v=0.2)


xx <- rLBA(10, A=0.5, b=1, t0 = 0.5, mean_v=1.2, sd_v=0.2)

# default uses normal distribution for drift rate:
n1PDF(xx$rt, A=0.5, b=1, t0 = 0.5, mean_v=c(1.2, 1.0), sd_v=0.2)

# other distributions:
n1PDF(xx$rt, A=0.5, b=1, t0 = 0.5, shape_v=c(1.2, 1), scale_v=c(0.2,0.3), distribution = "gamma")
n1PDF(xx$rt, A=0.5, b=1, t0 = 0.5, shape_v=c(1.2, 1), scale_v=c(0.2,0.3), distribution = "frechet")
n1PDF(xx$rt, A=0.5, b=1, t0 = 0.5, meanlog_v = c(0.5, 0.8), sdlog_v = 0.5, distribution = "lnorm")

# add st0:
n1PDF(xx$rt, A=0.5, b=1, t0 = 0.5, mean_v=c(1.2, 1.0), sd_v=0.2, st0 = 0.4)


# use different A parameters for each RT:
n1PDF(xx$rt, A=runif(10, 0.4, 0.6), 
      b=1, t0 = 0.5, mean_v=c(1.2, 1.0), sd_v=0.2)

# use different A parameters for each RT and each accumulator:
n1PDF(xx$rt, A=list(runif(10, 0.4, 0.6), runif(10, 0.2, 0.4)), 
      b=1, t0 = 0.5, mean_v=c(1.2, 1.0), sd_v=0.2)


### vectorize drift rates:

# vector versus list:
v1 <- n1PDF(xx$rt, A=0.5, b=1, t0 = 0.5, mean_v=c(1.2, 1.0), sd_v=0.2)
v2 <- n1PDF(xx$rt, A=0.5, b=1, t0 = 0.5, mean_v=list(1.2, 1.0), sd_v=0.2)
identical(v1, v2)  # TRUE

# drift rate per trial:
n1PDF(xx$rt, A=0.5, b=1, t0 = 0.5, mean_v=list(rnorm(10, 1.2), rnorm(10, 1)), sd_v=0.2)

# combine list with vector:
n1PDF(xx$rt, A=0.5, b=1, t0 = 0.5, mean_v=list(rnorm(10, 1.2), rnorm(10, 1)), sd_v=c(0.2, 0.1))

# t0 per trial and accumulator:
n1PDF(xx$rt, A=0.5, b=1, t0 = c(0.5), mean_v=c(1.2, 1.0), sd_v=0.2)
n1PDF(xx$rt, A=0.5, b=1, t0 = c(0.5, 0.6), mean_v=c(1.2, 1.0), sd_v=0.2) # per trial only
n1PDF(xx$rt, A=0.5, b=1, t0 = list(0.5, 0.6), mean_v=c(1.2, 1.0), sd_v=0.2) # per drift rate only
n1PDF(xx$rt, A=0.5, b=1, t0 = list(c(0.4, 0.5), c(0.5, 0.6)), mean_v=c(1.2, 1.0), sd_v=0.2)
}
}
\seealso{
For more user-friendly functions that return the PDF or CDF for the
  corresponding (and not first) accumulator winning see \code{\link{LBA}}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/diffusion.R
\name{Diffusion}
\alias{Diffusion}
\alias{recalc_t0}
\alias{diffusion}
\alias{ddiffusion}
\alias{pdiffusion}
\alias{qdiffusion}
\alias{rdiffusion}
\title{The Ratcliff Diffusion Model}
\usage{
recalc_t0(t0, st0)

ddiffusion(
  rt,
  response = "upper",
  a,
  v,
  t0,
  z = 0.5 * a,
  d = 0,
  sz = 0,
  sv = 0,
  st0 = 0,
  s = 1,
  precision = 3,
  stop_on_error = FALSE
)

pdiffusion(
  rt,
  response = "upper",
  a,
  v,
  t0,
  z = 0.5 * a,
  d = 0,
  sz = 0,
  sv = 0,
  st0 = 0,
  s = 1,
  precision = 3,
  maxt = 20,
  stop_on_error = FALSE,
  use_precise = TRUE
)

qdiffusion(
  p,
  response = "upper",
  a,
  v,
  t0,
  z = 0.5 * a,
  d = 0,
  sz = 0,
  sv = 0,
  st0 = 0,
  s = 1,
  precision = 3,
  maxt = 20,
  interval = c(0, 10),
  scale_p = FALSE,
  scale_max = Inf,
  stop_on_error = FALSE,
  max_diff = 1e-04
)

rdiffusion(
  n,
  a,
  v,
  t0,
  z = 0.5 * a,
  d = 0,
  sz = 0,
  sv = 0,
  st0 = 0,
  s = 1,
  precision = 3,
  stop_on_error = TRUE,
  maxt = 20,
  interval = c(0, 10),
  method = c("fastdm", "qdiffusion")
)
}
\arguments{
\item{t0}{non-decision time or response time constant (in seconds). Lower bound for the duration of all non-decisional processes (encoding and response execution). Typical range: 0.1 < \code{t0} < 0.5}

\item{st0}{inter-trial-variability of non-decisional components. Range of a uniform distribution with mean \code{t0 + st0/2} describing the distribution of actual \code{t0} values across trials. Accounts for response times below \code{t0}. Reduces skew of predicted RT distributions. Values different from 0 can slow computation considerably. Typical range: 0 < \code{st0} < 0.2. Default is 0.}

\item{rt}{a vector of RTs. Or for convenience also a \code{data.frame} with columns \code{rt} and \code{response} (such as returned from \code{rdiffusion} or \code{\link{rLBA}}). See examples.}

\item{response}{character vector. Which response boundary should be tested? Possible values are \code{c("upper", "lower")}, possibly abbreviated and \code{"upper"} being the default. Alternatively, a numeric vector with values 1=lower and 2=upper. For convenience, \code{response} is converted via \code{as.numeric} also allowing factors (see examples). Ignored if the first argument is a \code{data.frame}.}

\item{a}{threshold separation. Amount of information that is considered for a decision. Large values indicate a conservative decisional style. Typical range: 0.5 < \code{a} < 2}

\item{v}{drift rate. Average slope of the information accumulation process. The drift gives information about the speed and direction of the accumulation of information. Large (absolute) values of drift indicate a good performance. If received information supports the response linked to the upper threshold the sign will be positive and vice versa. Typical range: -5 < \code{v} < 5}

\item{z}{starting point. Indicator of an a priori bias in decision making. When the relative starting point \code{z} deviates from \code{0.5*a}, the amount of information necessary for a decision differs between response alternatives. Default is \code{0.5*a} (i.e., no bias).}

\item{d}{differences in speed of response execution (in seconds). Positive values indicate that response execution is faster for responses linked to the upper threshold than for responses linked to the lower threshold. Typical range: -0.1 < \code{d} < 0.1. Default is 0.}

\item{sz}{inter-trial-variability of starting point. Range of a uniform distribution with mean \code{z} describing the distribution of actual starting points from specific trials. Values different from 0 can predict fast errors (but can slow computation considerably). Typical range: 0 < \code{sz} < 0.5. Default is 0.}

\item{sv}{inter-trial-variability of drift rate. Standard deviation of a normal distribution with mean \code{v} describing the distribution of actual drift rates from specific trials. Values different from 0 can predict slow errors. Typical range: 0 < \code{sv} < 2. Default is 0.}

\item{s}{diffusion constant; standard deviation of the random noise of the diffusion process (i.e., within-trial variability), scales \code{a}, \code{v}, and \code{sv}. Needs to be fixed to a constant in most applications. Default is 1. Note that the default used by Ratcliff and in other applications is often 0.1.}

\item{precision}{\code{numerical} scalar value. Precision of calculation. Corresponds roughly to the number of decimals of the predicted CDFs that are calculated accurately. Default is 3.}

\item{stop_on_error}{Should the diffusion functions return 0 if the parameters values are outside the allowed range (= \code{FALSE}) or produce an error in this case (= \code{TRUE}).}

\item{maxt}{maximum \code{rt} allowed, used to stop integration problems. Larger values lead to considerably longer calculation times.}

\item{use_precise}{boolean. Should \code{pdiffusion} use the precise version for calculating the CDF? The default is \code{TRUE} which is highly recommended. Using \code{FALSE} (i.e., the imprecise version) is hardly any faster and produces clearly wrong results for most parameter settings.}

\item{p}{vector of probabilities. Or for convenience also a \code{data.frame} with columns \code{p} and \code{response}. See examples.}

\item{interval}{a vector containing the end-points of the interval to be searched for the desired quantiles (i.e., RTs) in \code{qdiffusion}. Default is \code{c(0, 10)}.}

\item{scale_p}{logical. Should entered probabilities automatically be scaled by maximally predicted probability? Default is \code{FALSE}. Convenience argument for obtaining predicted quantiles. Can be slow as the maximally predicted probability is calculated individually for each \code{p}.}

\item{scale_max}{numerical scalar. Value at which maximally predicted RT should be calculated if \code{scale_p} is \code{TRUE}.}

\item{max_diff}{numeric. Maximum acceptable difference between desired and observed probability of the quantile function (\code{qdiffusion}).}

\item{n}{is a desired number of observations.}

\item{method}{character. Experimentally implementation of an alternative way of generating random variates via the quantile function (\code{qdiffusion}) and random uniform value. For simple calls, the default method \code{"fastdm"} is dramatically faster.}
}
\value{
\code{ddiffusion} gives the density, \code{pdiffusion} gives the distribution function, \code{qdiffusion} gives the quantile function (i.e., predicted RTs), and \code{rdiffusion} generates random response times and decisions (returning a \code{data.frame} with columns \code{rt} (numeric) and \code{response} (factor)).

The length of the result is determined by \code{n} for \code{rdiffusion}, equal to the length of \code{rt} for \code{ddiffusion} and \code{pdiffusion}, and equal to the length of \code{p} for \code{qdiffusion}.

The distribution parameters (as well as \code{response}) are recycled to the length of the result. In other words, the functions are completely vectorized for all parameters and even the response boundary.
}
\description{
Density, distribution function, quantile function, and random generation for the Ratcliff diffusion model with following parameters: \code{a} (threshold separation), \code{z} (starting point), \code{v} (drift rate), \code{t0} (non-decision time/response time constant), \code{d} (differences in speed of response execution), \code{sv} (inter-trial-variability of drift), \code{st0} (inter-trial-variability of non-decisional components), \code{sz} (inter-trial-variability of relative starting point), and \code{s} (diffusion constant). \strong{Note that the parameterization or defaults of non-decision time variability \code{st0} and diffusion constant \code{s} differ from what is often found in the literature and that the parameterization of \code{z} and \code{sz} have changed compared to previous versions (now absolute and not relative).}
}
\details{
The Ratcliff diffusion model (Ratcliff, 1978) is a mathematical model for two-choice discrimination tasks. It is based on the assumption that information is accumulated continuously until one of two decision thresholds is hit. For introductions see Ratcliff and McKoon (2008), Voss, Rothermund, and Voss (2004), Voss, Nagler, and Lerche (2013), or Wagenmakers (2009).

All functions are fully vectorized across all parameters as well as the response to match the length or \code{rt} (i.e., the output is always of length equal to \code{rt}). This allows for trialwise parameters for each model parameter. 

For convenience, all functions (with the exception of \code{rdiffusion}) allow that the first argument is a \code{data.frame} containing the information of the first and second argument in two columns (i.e., \code{rt}/\code{p} and \code{response}). Other columns (as well as passing \code{response} separately argument) will be ignored. This allows, for example, to pass the \code{data.frame} generated by \code{rdiffusion} directly to \code{pdiffusion}. See examples.

\subsection{Quantile Function}{
Due to the bivariate nature of the diffusion model, the diffusion processes reaching each response boundary only return the defective CDF that does not reach 1. Only the sum of the CDF for both boundaries reaches 1. Therefore, \code{qdiffusion} can only return quantiles/RTs for any accumulator up to the maximal probability of that accumulator's CDF. This can be obtained by evaluating the CDF at \code{Inf}. 

As a convenience for the user, if \code{scale_p = TRUE} in the call to \code{qdiffusion} the desired probabilities are automatically scaled by the maximal probability for the corresponding response. Note that this can be slow as the maximal probability is calculated separately for each desired probability. See examples.

Also note that quantiles (i.e., predicted RTs) are obtained by numerically minimizing the absolute difference between desired probability and the value returned from \code{pdiffusion} using \code{\link{optimize}}. If the difference between the desired probability and probability corresponding to the returned quantile is above a certain threshold (currently 0.0001) no quantile is returned but \code{NA}. This can be either because the desired quantile is above the maximal probability for this accumulator or because the limits for the numerical integration are too small (default is \code{c(0, 10)}).
}
}
\note{
The parameterization of the non-decisional components, \code{t0} and \code{st0}, differs from the parameterization used by, for example, Andreas Voss or Roger Ratcliff. In the present case \code{t0} is the lower bound of the uniform distribution of length \code{st0}, but \emph{not} its midpoint. The parameterization employed here is in line with the parametrization for the \link{LBA} code (where \code{t0} is also the lower bound).

The default diffusion constant \code{s} is 1 and not 0.1 as in most applications of Roger Ratcliff and others.

We have changed the parameterization of the start point \code{z} which is now the absolute start point in line with most published literature (it was the relative start point in previous versions of \pkg{rtdists}).
}
\examples{

## identical calls (but different random values)
rt1 <- rdiffusion(500, a=1, v=2, t0=0.5)
head(rt1)
rt2 <- rdiffusion(500, a=1, v=2, t0=0.5, d=0, sz=0, sv=0, st0=0)
head(rt2)
  
# get density for random RTs (possible to specify arguments for pdiffusion in same way):
sum(log(ddiffusion(rt1$rt, rt1$response, a=1, v=2, t0=0.5)))  # response is factor
sum(log(ddiffusion(rt1$rt, as.numeric(rt1$response), a=1, v=2, t0=0.5))) # response is numeric
sum(log(ddiffusion(rt1$rt, as.character(rt1$response), a=1, v=2, t0=0.5))) # response is character
sum(log(ddiffusion(rt1, a=1, v=2, t0=0.5))) # response is data.frame


sum(log(ddiffusion(rt2$rt, rt2$response, a=1, v=2, t0=0.5)))

# can we recover the parameters?
ll_diffusion <- function(pars, rt, response) 
{
  densities <- ddiffusion(rt, response=response, 
               a=pars[1], v=pars[2], t0=pars[3], 
               sz=pars[4], 
               st0=pars[5], sv=pars[6])
  if (any(densities == 0)) return(1e6)
  return(-sum(log(densities)))
}

\dontrun{)
start <- c(runif(2, 0.5, 3), 0.1, runif(3, 0, 0.5))
names(start) <- c("a", "v", "t0", "sz", "st0", "sv")
recov <- nlminb(start, ll_diffusion, lower = 0, rt=rt1$rt, response=rt1$response)
round(recov$par, 3)
#     a     v    t0    sz   st0    sv 
# 1.019 1.879 0.496 0.000 0.000 0.389 
## results of course depend on random seed for rdiffusion and runif
}


\dontrun{
## replicate Table 1 from Wagenmakers et al. (2007) using rdiffusion:

n <- 1e5 # number of samples
# take parameter valeus from Table 2 and set s to 0.1
george <- rdiffusion(n, a = 0.12, v = 0.25, t0 = 0.3, s = 0.1)
rich   <- rdiffusion(n, a = 0.12, v = 0.25, t0 = 0.25, s = 0.1)
amy    <- rdiffusion(n, a = 0.08, v = 0.25, t0 = 0.3, s = 0.1)
mark   <- rdiffusion(n, a = 0.08, v = 0.25, t0 = 0.25, s = 0.1)

george$id <- "george"
rich$id <- "rich"
amy$id <- "amy"
mark$id <- "mark"

wag <- rbind(george, rich, amy, mark)
wag$id <- factor(wag$id, levels = c("george", "rich", "amy", "mark"))

opt <- options()
options(digits = 3)
aggregate(cbind(rt, as.numeric(response)-1) ~ id, wag, mean)
#       id    rt    V2
# 1 george 0.517 0.952
# 2   rich 0.467 0.953
# 3    amy 0.422 0.881
# 4   mark 0.372 0.882
options(digits = 1)
aggregate(rt ~ id, wag, var)
#       id    rt
# 1 george 0.024
# 2   rich 0.024
# 3    amy 0.009
# 4   mark 0.009
options(opt)
}


## plot density:
curve(ddiffusion(x, a=1, v=2, t0=0.5, response = "upper"), 
      xlim=c(0,3), main="Density of upper responses", ylab="density", xlab="response time")
curve(ddiffusion(x, a=1, v=2, t0=0.5, st0=0.2, response = "upper"), 
      add=TRUE, lty = 2)
legend("topright", legend=c("no", "yes"), title = "Starting Point Variability?", lty = 1:2)

# plot cdf:
curve(pdiffusion(x, a=1, v=2, t0=0.5, st0=0.2, response="u"), 
     xlim = c(0, 3),ylim = c(0,1), 
     ylab = "cumulative probability", xlab = "response time",
     main = "CDF of diffusion model with start point variability")
curve(pdiffusion(x, a=1, v=2, t0=0.5, st0=0.2, response="l"), 
     add=TRUE, lty = 2)
legend("topleft", legend=c("upper", "lower"), title="response boundary", lty=1:2)

\dontrun{
### qdiffusion can only return values up to maximal predicted probability:
(max_p <- pdiffusion(Inf, a=1, v=2, t0=0.5, st0=0.2, sz = 0.1, sv = 0.5, response="u"))
# [1] 0.87
# (Note that with the current integration routine for pdiffusion use Inf and not smaller values.)

qdiffusion(0.87, a=1, v=2, t0=0.5, st0=0.2, sz = 0.1, sv = 0.5, response="u")
# [1] 1.945802

qdiffusion(0.88, a=1, v=2, t0=0.5, st0=0.2, sz = 0.1, sv = 0.5, response="u")
# NA with warning.

# to get predicted quantiles, scale required quantiles by maximally predicted response rate:
qs <- c(.1, .3, .5, .7, .9)
qdiffusion(qs*max_p, a=1, v=2, t0=0.5, st0=0.2, sz = 0.1, sv = 0.5, response="u")

# or set scale_p to TRUE which scales automatically by maximum p
# (but can be slow as it calculates max_p for each probability separately) 
qdiffusion(qs, a=1, v=2, t0=0.5, st0=0.2, sz = 0.1, sv = 0.5, response="u", scale_p = TRUE)


# qdiffusion also accepts a data.frame as first argument:
t3 <- data.frame(p = rep(c(0.05, 0.1, 0.87), 2), response = rep(c("upper", "lower"), each = 3))
#      p response
# 1 0.05    upper
# 2 0.10    upper
# 3 0.87    upper
# 4 0.05    lower
# 5 0.10    lower
# 6 0.87    lower
qdiffusion(t3, a=1, v=2, t0=0.5, st0=0.2, sz = 0.1, sv = 0.5, scale_p = TRUE)
}

## LBA and diffusion can be used interchangeably:
rt1 <- rLBA(500, A=0.5, b=1, t0 = 0.5, mean_v=c(2.4, 1.6), sd_v=c(1,1.2))
rt2 <- rdiffusion(500, a=1, v=2, t0=0.5)

# data can also be passed as data.frame (same is true for pLBA):
sum(log(dLBA(rt1, A=0.5, b=1, t0 = 0.5, mean_v=c(2.4, 1.6), sd_v=c(1,1.2))))
sum(log(dLBA(rt2, A=0.5, b=1, t0 = 0.5, mean_v=c(2.4, 1.6), sd_v=c(1,1.2))))

sum(log(ddiffusion(rt1, a=1, v=2, t0=0.5)))
sum(log(ddiffusion(rt2, a=1, v=2, t0=0.5)))

}
\references{
Ratcliff, R. (1978). A theory of memory retrieval. \emph{Psychological Review}, 85(2), 59-108.

Ratcliff, R., & McKoon, G. (2008). The diffusion decision model: Theory and data for two-choice decision tasks. \emph{Neural Computation}, 20(4), 873-922.

Voss, A., Rothermund, K., & Voss, J. (2004). Interpreting the parameters of the diffusion model: An empirical validation. \emph{Memory & Cognition}. Vol 32(7), 32, 1206-1220.

Voss, A., Nagler, M., & Lerche, V. (2013). Diffusion Models in Experimental Psychology: A Practical Introduction. \emph{Experimental Psychology}, 60(6), 385-402. doi:10.1027/1618-3169/a000218

Wagenmakers, E.-J., van der Maas, H. L. J., & Grasman, R. P. P. P. (2007). An EZ-diffusion model for response time and accuracy. \emph{Psychonomic Bulletin & Review}, 14(1), 3-22.

Wagenmakers, E.-J. (2009). Methodological and empirical developments for the Ratcliff diffusion model of response times and accuracy. \emph{European Journal of Cognitive Psychology}, 21(5), 641-671.
}
\author{
Underlying C code by Jochen Voss and Andreas Voss. Porting and R wrapping by Matthew Gretton, Andrew Heathcote, Scott Brown, and Henrik Singmann. \code{qdiffusion} by Henrik Singmann.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/deprecated.R
\name{drd}
\alias{drd}
\alias{rtdists-deprecated}
\alias{prd}
\alias{rrd}
\title{Deprecated functions}
\usage{
drd(...)

prd(...)

rrd(...)
}
\arguments{
\item{...}{arguments passed from the old functions to the new functions}
}
\description{
These functions have been renamed and deprecated in \pkg{rtdists}:
\code{drd()} (use \code{\link{ddiffusion}()}), 
\code{prd()} (use \code{\link{pdiffusion}()}),
\code{rrd()} (use \code{\link{rdiffusion}()}).
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/speed_acc-data.R
\docType{data}
\name{speed_acc}
\alias{speed_acc}
\title{Speed-Accuracy Data from Wagenmakers, Ratcliff, Gomez, & McKoon (2008, Experiment 1)}
\format{
A \code{data.frame} with 31,522 obs. and 9 variables:
\describe{
 \item{id}{participant id}
 \item{block}{block number}
 \item{condition}{\code{accuracy} for blocks with accuracy instructions; \code{speed} for blocks with speed instruction}
 \item{stim}{unique identifier of stimulus, stimuli are nested in frequency conditions}
 \item{stim_cat}{category of stimulus, either word or non-word}
 \item{frequency}{"high frequency word", "low frequency word", "very low frequency word", or non-words derived from the first three categories}
 \item{response}{\code{word}, \code{nonword}, or not interpretable response (\code{error}, i.e., pushed a button, but not the right one and also not the one next to the right button)}
 \item{rt}{response time in seconds}
 \item{censor}{boolean indicating whether or not a response should be eliminated prior to analysis; uninterpretable response, too fast response (<180 ms), too slow response (>3 sec)}
}
}
\source{
Wagenmakers, E.-J., Ratcliff, R., Gomez, P., & McKoon, G. (2008). A diffusion model account of criterion shifts in the lexical decision task. \emph{Journal of Memory and Language}, 58(1), 140-159.
}
\usage{
speed_acc
}
\description{
Responses and response times from an experiment in which instruction manipulated speed and accuracy between blocks. This data was also analyzed by Heathcote and Love (2012) who were the first to use the 17 participants also included here.
}
\details{
The data excludes the practice blocks but includes all trials. Variable \code{censor} can be used for excluding all trials also excluded from the papers using it namely uninterpretable response, too fast response (<180 ms), too slow response (>3 sec). Heathcote and Love (2012, p. 7) describe the data as follows:

We fit the LBA and LNR models to data from Wagenmaker et al.'s (2008) experiment one, where participants made decisions about whether a string of letters constituted a word. These lexical decisions were made about four types of stimuli, non-words (nw) and high-frequency (hf), low-frequency (lf), and very low-frequency (vlf) words. Participants made decisions either under speed or accuracy emphasis instructions in different experimental blocks. Accuracy blocks were preceded by the message "Try to respond accurately" and "ERROR" was displayed after each wrong response. Speed blocks were preceded by the message "Try to respond accurately" and "TOO SLOW" was displayed after each response slower than 0.75 s.We report analyses of data from 17 participants (31,412 data points) in their Experiment 1, including the 15 participants analyzed in Wagenmakers et al. (2008) and two extras (we thank Eric-Jan Wagenmakers for supplying this data).
}
\examples{

data(speed_acc)
str(speed_acc)

# remove excluded trials:
speed_acc <- droplevels(speed_acc[!speed_acc$censor,])

# new factors for obtaining values as in Table 1, Wagenmakers et al. (2008, p. 152)
speed_acc$freq <- with(speed_acc, 
                       factor(ifelse(stim_cat == "nonword", "nonword", 
                                     as.character(frequency)), 
                              levels = c("high", "low", "very_low", "nonword")))
# corr = correct (0 = correct, 1 = error)
speed_acc$corr <- with(speed_acc, 1-as.numeric(stim_cat == response))

str(speed_acc)

## aggregated RTs:
aggregate(rt ~ condition + freq + corr, speed_acc, mean)
## Error Rate:
aggregate(corr ~ condition + freq + corr, speed_acc, mean)

}
\references{
Heathcote, A., & Love, J. (2012). Linear deterministic accumulator models of simple choice. \emph{Frontiers in Psychology: Cognitive Science}, 3, 292. doi:10.3389/fpsyg.2012.00292
}
\keyword{dataset}
